@@ -15,6 +15,10 @@
 #include <omp.h>
 #endif
 
+#if defined(__APPLE__) && defined(__aarch64__) && !defined(C10_MOBILE)
+#include <sys/sysctl.h>
+#endif
+
 namespace at {
 
 namespace {
@@ -28,7 +32,7 @@ const char* get_env_var(
 size_t get_env_num_threads(const char* var_name, size_t def_value = 0) {
   try {
     if (auto* value = std::getenv(var_name)) {
-      int nthreads = c10::stoi(value);
+      int nthreads = std::stoi(value);
       TORCH_CHECK(nthreads > 0);
       return nthreads;
     }
@@ -46,43 +50,41 @@ std::string get_parallel_info() {
   std::ostringstream ss;
 
   ss << "ATen/Parallel:\n\tat::get_num_threads() : "
-     << at::get_num_threads() << std::endl;
+     << at::get_num_threads() << '\n';
   ss << "\tat::get_num_interop_threads() : "
-     << at::get_num_interop_threads() << std::endl;
+     << at::get_num_interop_threads() << '\n';
 
-  ss << at::get_openmp_version() << std::endl;
+  ss << at::get_openmp_version() << '\n';
 #ifdef _OPENMP
-  ss << "\tomp_get_max_threads() : " << omp_get_max_threads() << std::endl;
+  ss << "\tomp_get_max_threads() : " << omp_get_max_threads() << '\n';
 #endif
 
-  ss << at::get_mkl_version() << std::endl;
+  ss << at::get_mkl_version() << '\n';
 #if AT_MKL_ENABLED()
-  ss << "\tmkl_get_max_threads() : " << mkl_get_max_threads() << std::endl;
+  ss << "\tmkl_get_max_threads() : " << mkl_get_max_threads() << '\n';
 #endif
 
-  ss << at::get_mkldnn_version() << std::endl;
+  ss << at::get_mkldnn_version() << '\n';
 
   ss << "std::thread::hardware_concurrency() : "
-     << std::thread::hardware_concurrency() << std::endl;
+     << std::thread::hardware_concurrency() << '\n';
 
-  ss << "Environment variables:" << std::endl;
+  ss << "Environment variables:" << '\n';
   ss << "\tOMP_NUM_THREADS : "
-     << get_env_var("OMP_NUM_THREADS", "[not set]") << std::endl;
+     << get_env_var("OMP_NUM_THREADS", "[not set]") << '\n';
   ss << "\tMKL_NUM_THREADS : "
-     << get_env_var("MKL_NUM_THREADS", "[not set]") << std::endl;
+     << get_env_var("MKL_NUM_THREADS", "[not set]") << '\n';
 
   ss << "ATen parallel backend: ";
   #if AT_PARALLEL_OPENMP
   ss << "OpenMP";
   #elif AT_PARALLEL_NATIVE
   ss << "native thread pool";
-  #elif AT_PARALLEL_NATIVE_TBB
-  ss << "native thread pool and TBB";
   #endif
   #ifdef C10_MOBILE
   ss << " [mobile]";
   #endif
-  ss << std::endl;
+  ss << '\n';
 
   #if AT_EXPERIMENTAL_SINGLE_THREAD_POOL
   ss << "Experimental: single thread pool" << std::endl;
@@ -104,11 +106,23 @@ int intraop_default_num_threads() {
 #if defined(FBCODE_CAFFE2) && defined(__aarch64__)
     nthreads = 1;
 #else
+#if defined(__aarch64__) && defined(__APPLE__)
+    // On Apple Silicon there are efficient and performance core
+    // Restrict parallel algorithms to performance cores by default
+    int32_t num_cores = -1;
+    size_t num_cores_len = sizeof(num_cores);
+    if (sysctlbyname("hw.perflevel0.physicalcpu", &num_cores, &num_cores_len, nullptr, 0) == 0) {
+      if (num_cores > 1) {
+        nthreads = num_cores;
+        return num_cores;
+      }
+    }
+#endif
     nthreads = TaskThreadPoolBase::defaultNumThreads();
 #endif
   }
-  return nthreads;
-#endif
+  return static_cast<int>(nthreads);
+#endif /* !defined(C10_MOBILE) */
 }
 
 } // namespace at
