@@ -1,6 +1,6 @@
 #include <ATen/core/op_registration/infer_schema.h>
 #include <c10/util/irange.h>
-#include <sstream>
+#include <fmt/format.h>
 
 namespace c10 {
 
@@ -8,46 +8,55 @@ namespace detail {
 namespace infer_schema {
 namespace {
 
-std::string fastToString(size_t x) {
-  if (C10_LIKELY(x < 10)) {
-    std::string result;
-    result.push_back('_');
-    result.push_back('0' + x);
-    return result;
-  }
-  return "_" + c10::guts::to_string(x);
-}
-
 std::vector<Argument> createArgumentVector(c10::ArrayRef<ArgumentDef> args) {
   std::vector<Argument> result;
   result.reserve(args.size());
   for (const auto i : c10::irange(args.size())) {
     // Arguments are named "_<index>"
-    result.emplace_back(fastToString(i), (*args[i].getFakeTypeFn)(), (*args[i].getTypeFn)());
+    result.emplace_back(
+        fmt::format("_{}", i),
+        (*args[i].getFakeTypeFn)(),
+        (*args[i].getTypeFn)());
   }
   return result;
 }
-}
+} // namespace
 // This is intentionally a separate function and in a .cpp file
 // because then the template is smaller and that benefits binary size
-FunctionSchema make_function_schema(std::string&& name, std::string&& overload_name, c10::ArrayRef<ArgumentDef> arguments, c10::ArrayRef<ArgumentDef> returns) {
-  return FunctionSchema(std::move(name), std::move(overload_name), createArgumentVector(arguments), createArgumentVector(returns));
+FunctionSchema make_function_schema(
+    std::string&& name,
+    std::string&& overload_name,
+    c10::ArrayRef<ArgumentDef> arguments,
+    c10::ArrayRef<ArgumentDef> returns) {
+  return FunctionSchema(
+      std::move(name),
+      std::move(overload_name),
+      createArgumentVector(arguments),
+      createArgumentVector(returns));
 }
 
-FunctionSchema make_function_schema(c10::ArrayRef<ArgumentDef> arguments, c10::ArrayRef<ArgumentDef> returns) {
+FunctionSchema make_function_schema(
+    c10::ArrayRef<ArgumentDef> arguments,
+    c10::ArrayRef<ArgumentDef> returns) {
   return make_function_schema("", "", arguments, returns);
 }
-}
-}
+} // namespace infer_schema
+} // namespace detail
 
-c10::optional<std::string> findSchemaDifferences(const FunctionSchema& lhs, const FunctionSchema& rhs) {
+std::optional<std::string> findSchemaDifferences(
+    const FunctionSchema& lhs,
+    const FunctionSchema& rhs) {
   if (lhs.arguments().size() != rhs.arguments().size()) {
-    return "The number of arguments is different. " + guts::to_string(lhs.arguments().size()) +
-             " vs " + guts::to_string(rhs.arguments().size()) + ".";
+    return fmt::format(
+        "The number of arguments is different. {} vs {}.",
+        lhs.arguments().size(),
+        rhs.arguments().size());
   }
   if (lhs.returns().size() != rhs.returns().size()) {
-    return "The number of returns is different. " + guts::to_string(lhs.returns().size()) +
-             " vs " + guts::to_string(rhs.returns().size());
+    return fmt::format(
+        "The number of returns is different. {} vs {}.",
+        lhs.returns().size(),
+        rhs.returns().size());
   }
 
   for (const auto i : c10::irange(lhs.arguments().size())) {
@@ -57,8 +66,11 @@ c10::optional<std::string> findSchemaDifferences(const FunctionSchema& lhs, cons
     // cheaper, particularly when one of the types is a singleton like
     // NumberType or AnyType.
     if (leftType.get() != rightType.get() && *leftType != *rightType) {
-      return "Type mismatch in argument " + guts::to_string(i+1) + ": " + lhs.arguments()[i].type()->str() +
-               " vs " + rhs.arguments()[i].type()->str();
+      return fmt::format(
+          "Type mismatch in argument {}: {} vs {}.",
+          i + 1,
+          lhs.arguments()[i].type()->str(),
+          rhs.arguments()[i].type()->str());
     }
   }
 
@@ -67,8 +79,11 @@ c10::optional<std::string> findSchemaDifferences(const FunctionSchema& lhs, cons
     const TypePtr& rightType = rhs.returns()[i].type();
     // See above about comparing pointers first.
     if (leftType.get() != rightType.get() && *leftType != *rightType) {
-      return "Type mismatch in return " + guts::to_string(i+1) + ": " + lhs.returns()[i].type()->str() +
-               " vs " + rhs.returns()[i].type()->str();
+      return fmt::format(
+          "Type mismatch in return {}: {} vs {}.",
+          i + 1,
+          lhs.returns()[i].type()->str(),
+          rhs.returns()[i].type()->str());
     }
   }
 
@@ -76,4 +91,4 @@ c10::optional<std::string> findSchemaDifferences(const FunctionSchema& lhs, cons
   return c10::nullopt;
 }
 
-}
+} // namespace c10
