@@ -1,9 +1,7 @@
 #include <c10/core/impl/DeviceGuardImplInterface.h>
-#include <c10/macros/Macros.h>
 
-namespace at {
-namespace detail {
+namespace at::detail {
 
 C10_REGISTER_GUARD_IMPL(CPU, c10::impl::NoOpDeviceGuardImpl<DeviceType::CPU>);
 
-}} // namespace at::detail
+} // namespace at::detail
