@@ -1,7 +1,6 @@
 #include <ATen/core/TensorBase.h>
-#include <algorithm>
-#include <vector>
 #include <ATen/core/TensorBody.h>
+#include <algorithm>
 #include <c10/util/OptionalArrayRef.h>
 
 #ifdef AT_PER_OPERATOR_HEADERS
@@ -26,7 +25,7 @@ void _assert_match(const O& original, const C& compared, const std::string& name
   }
 }
 
-void _assert_tensor_metadata(at::Tensor const& tensor, at::OptionalIntArrayRef sizes, at::OptionalIntArrayRef strides, c10::optional<c10::ScalarType> dtype) {
+void _assert_tensor_metadata(at::Tensor const& tensor, at::OptionalIntArrayRef sizes, at::OptionalIntArrayRef strides, std::optional<c10::ScalarType> dtype) {
   _assert_match(tensor.sizes(), sizes, "sizes");
   _assert_match(tensor.strides(), strides, "strides");
   _assert_match(tensor.dtype(), dtype, "dtype");
