@@ -47,10 +47,9 @@
 
 #include <utility>
 
-namespace at {
-namespace meta {
+namespace at::meta {
 
-using namespace native;
+using namespace ::at::native;
 
 TORCH_META_FUNC(topk)
 (const Tensor& self, int64_t k, int64_t dim_, bool largest, bool sorted) {
@@ -72,10 +71,7 @@ TORCH_META_FUNC(topk)
 }
 
 TORCH_META_FUNC2(sort, stable)
-(const Tensor& self, c10::optional<bool> stable, int64_t dim, bool descending) {
-  TORCH_INTERNAL_ASSERT(
-      stable.has_value(),
-      "sort(): c10::optional<bool> for stable has to have value.");
+(const Tensor& self, std::optional<bool> stable, int64_t dim, bool descending) {
   maybe_wrap_dim(dim, self.dim());
 
   // See issue: https://github.com/pytorch/pytorch/issues/65863
@@ -89,9 +85,9 @@ TORCH_META_FUNC2(sort, stable)
   set_output_raw_strided(1, self.sizes(), strides, self.options().dtype(kLong), {});
 }
 
-} // namespace meta
+} // namespace at::meta
 
-namespace native {
+namespace at::native {
 
 DEFINE_DISPATCH(sort_stub);
 DEFINE_DISPATCH(topk_stub);
@@ -457,7 +453,7 @@ std::tuple<Tensor&, Tensor&> kthvalue_out_impl_cpu(
     .add_output(indices)
     .build();
 
-  AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, self.scalar_type(), "kthvalue_cpu", [&] {
+  AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, self.scalar_type(), "kthvalue_cpu", [&] {
     auto loop = [&](char** data, const int64_t* strides, int64_t n) {
       for (const auto i : c10::irange(n)) {
         TensorAccessor<scalar_t, 1> tmp_values(
@@ -550,10 +546,10 @@ std::tuple<Tensor&, Tensor&> median_with_indices_impl(
     .declare_static_shape(sizes, /*squash_dims=*/dim)
     .add_output(vals)
     .add_output(inds)
-    .add_input(in)
+    .add_const_input(in)
     .build();
 
-  AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, in.scalar_type(), "median_out", [&] {
+  AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, in.scalar_type(), "median_out", [&] {
     auto loop = [&](char** data, const int64_t* strides, int64_t n) {
       for (const auto i : c10::irange(n)) {
         auto valp = reinterpret_cast<scalar_t*>(data[0] + i * strides[0]);
@@ -621,7 +617,7 @@ Tensor median_impl(const Tensor& self, bool ignore_nan) {
   Tensor in = self.clone();
   Tensor out = at::empty({}, self.options());
 
-  AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, in.scalar_type(), "median_cpu", [&] {
+  AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, in.scalar_type(), "median_cpu", [&] {
     scalar_t* op = out.data_ptr<scalar_t>();
     scalar_t* first = in.data_ptr<scalar_t>();
     scalar_t* last = first + size;
@@ -943,7 +939,7 @@ Tensor nanmedian_cpu(const Tensor& self) {
 
 TORCH_IMPL_FUNC(sort_stable_out)
 (const Tensor& self,
- c10::optional<bool> stable,
+ std::optional<bool> stable,
  int64_t dim,
  bool descending,
  const Tensor& values,
@@ -954,7 +950,7 @@ TORCH_IMPL_FUNC(sort_stable_out)
     indices.zero_();
   } else {
     dim = maybe_wrap_dim(dim, self.dim());
-    sort_stub(self.device().type(), self, values, indices, dim, descending, stable.value());
+    sort_stub(self.device().type(), self, values, indices, dim, descending, stable.value_or(false));
   }
 }
 
@@ -993,5 +989,4 @@ Tensor argsort_stable(const Tensor & self, bool stable, int64_t dim, bool descen
 }
 
 
-} // namespace native
-} // namespace at
+} // namespace at::native
