@@ -5,6 +5,7 @@
 #include <ATen/Dispatch.h>
 #include <ATen/Parallel.h>
 #include <ATen/cpu/vec/vec.h>
+#include <ATen/cpu/vec/functional.h>
 #include <ATen/native/Pool.h>
 #include <ATen/native/cpu/utils.h>
 #include <c10/util/irange.h>
@@ -60,13 +61,15 @@ vec::Vectorized<int64_t> is_nan_vec<int64_t>(vec::Vectorized<int64_t> vec) {
   return ret;
 }
 
-template <typename scalar_t, typename accscalar_t>
-inline void compute_internal(
-  scalar_t* input_data,
+template <typename scalar_t, typename opmath_t>
+inline
+typename std::enable_if<std::is_same<scalar_t, opmath_t>::value, void>::type
+compute_internal(
+  const scalar_t* input_data,
   scalar_t* out_data,
-  accscalar_t* max_ptr,
-  vec::int_same_size_t<accscalar_t>* index_ptr,
-  int64_t*  ind,
+  opmath_t* max_ptr,
+  vec::int_same_size_t<opmath_t>* index_ptr,
+  int64_t* ind,
   int64_t input_depth, int64_t input_height, int64_t input_width, int64_t channels,
   int64_t n,
   int64_t len,
@@ -78,7 +81,7 @@ inline void compute_internal(
   int64_t dilationH,
   int64_t dilationW) {
   using Vec = vec::Vectorized<scalar_t>;
-  using integer_t = vec::int_same_size_t<accscalar_t>;
+  using integer_t = vec::int_same_size_t<opmath_t>;
   using iVec = vec::Vectorized<integer_t>;
   // Pass I: init out lane
   iVec index0_vec = iVec(id0 * input_height * input_width + ih0 * input_width + iw0);
@@ -96,7 +99,7 @@ inline void compute_internal(
   for (int64_t id = id0; id < id1; id += dilationD) {
     for (int64_t ih = ih0; ih < ih1; ih += dilationH) {
       for (int64_t iw = iw0; iw < iw1; iw += dilationW) {
-        scalar_t* in = input_data + (n * input_depth * input_height * input_width +
+        const scalar_t* in = input_data + (n * input_depth * input_height * input_width +
             id * input_height * input_width + ih * input_width + iw) * channels;
 
         int64_t d2 = 0;
@@ -130,13 +133,16 @@ inline void compute_internal(
   }
 }
 
-template <>
-inline void compute_internal(
-  BFloat16* input_data,
-  BFloat16* out_data,
-  float* max_ptr,
-  int32_t* index_ptr,
-  int64_t*  ind,
+// std::is_same<scalar_t, at::BFloat16> || std::is_same<scalar_t, at::Half>
+template <typename scalar_t, typename opmath_t>
+inline
+typename std::enable_if<!std::is_same<scalar_t, opmath_t>::value, void>::type
+compute_internal(
+  const scalar_t* input_data,
+  scalar_t* out_data,
+  opmath_t* max_ptr,
+  vec::int_same_size_t<opmath_t>* index_ptr,
+  int64_t* ind,
   int64_t input_depth, int64_t input_height, int64_t input_width, int64_t channels,
   int64_t n,
   int64_t len,
@@ -147,12 +153,12 @@ inline void compute_internal(
   int64_t dilationD,
   int64_t dilationH,
   int64_t dilationW) {
-  using bVec = vec::Vectorized<BFloat16>;
-  using fVec = vec::Vectorized<float>;
+  using Vec = vec::Vectorized<scalar_t>;
+  using fVec = vec::Vectorized<opmath_t>;
   using iVec = vec::Vectorized<int32_t>;
   // Pass I: init out lane
   iVec index0_vec = iVec(id0 * input_height * input_width + ih0 * input_width + iw0);
-  fVec out_vec = fVec(-std::numeric_limits<float>::infinity());
+  fVec out_vec = fVec(-std::numeric_limits<opmath_t>::infinity());
   int64_t d1 = 0;
   for (; d1 < len; d1 += fVec::size()) {
     index0_vec.store(index_ptr + d1);
@@ -160,21 +166,20 @@ inline void compute_internal(
   }
   for (; d1 < size; d1++) {
     ind[d1] = ih0 * input_width + iw0;
-    max_ptr[d1] = -std::numeric_limits<float>::infinity();
+    max_ptr[d1] = -std::numeric_limits<opmath_t>::infinity();
   }
   // Pass II: compute local max
   for (int64_t id = id0; id < id1; id += dilationD) {
     for (int64_t ih = ih0; ih < ih1; ih += dilationH) {
       for (int64_t iw = iw0; iw < iw1; iw += dilationW) {
-        BFloat16* in = input_data + (n * input_depth * input_height * input_width +
+        const scalar_t* in = input_data + (n * input_depth * input_height * input_width +
             id * input_height * input_width + ih * input_width + iw) * channels;
 
         int64_t d2 = 0;
-        for (; d2 < len; d2 += bVec::size()) {
+        for (; d2 < len; d2 += Vec::size()) {
           iVec index_ivec = iVec(id * input_height * input_width + ih * input_width + iw);
-          bVec val_bvec = bVec::loadu(in + d2);
-          fVec val_fvec0, val_fvec1;
-          std::tie(val_fvec0, val_fvec1) = convert_bfloat16_float(val_bvec);
+          Vec val_bvec = Vec::loadu(in + d2);
+          auto [val_fvec0, val_fvec1] = convert_to_float<scalar_t>(val_bvec);
 
           iVec maxindex_ivec0 = iVec::loadu(index_ptr + d2);
           iVec maxindex_ivec1 = iVec::loadu(index_ptr + d2 + iVec::size());
@@ -200,9 +205,9 @@ inline void compute_internal(
         }
         for (; d2 < size; d2++) {
           int64_t index = id * input_height * input_width + ih * input_width + iw;
-          float val = float(in[d2]);
+          opmath_t val = opmath_t(in[d2]);
           int64_t maxindex = ind[d2];
-          float maxval = max_ptr[d2];
+          opmath_t maxval = max_ptr[d2];
 
           bool mask = (val > maxval) || std::isnan(val);
           max_ptr[d2] = mask ? val : maxval;
@@ -211,16 +216,16 @@ inline void compute_internal(
       }
     }
   }
-  // Convert max values from float to bfloat16
+  // Convert max values from float to bfloat16/half
   int64_t d3 = 0;
-  for (; d3 < len; d3 += bVec::size()) {
+  for (; d3 < len; d3 += Vec::size()) {
     fVec max_fvec0 = fVec::loadu(max_ptr + d3);
     fVec max_fvec1 = fVec::loadu(max_ptr + d3 + fVec::size());
-    bVec max_bvec = convert_float_bfloat16(max_fvec0, max_fvec1);
+    Vec max_bvec = convert_from_float<scalar_t>(max_fvec0, max_fvec1);
     max_bvec.store(out_data + d3);
   }
   for (; d3 < size; d3++) {
-    out_data[d3] = BFloat16(max_ptr[d3]);
+    out_data[d3] = scalar_t(max_ptr[d3]);
   }
 }
 
@@ -254,7 +259,7 @@ void cpu_max_pool(
   auto output = output_.contiguous();
   auto indices = indices_.contiguous();
 
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   auto output_data = output.data_ptr<scalar_t>();
   auto indices_data = indices.data_ptr<int64_t>();
 
@@ -281,11 +286,11 @@ void cpu_max_pool(
   int64_t output_height = output.size(-2);
   int64_t output_width = output.size(-1);
 
-  using accscalar_t = at::opmath_type<scalar_t>;
+  using opmath_t = at::opmath_type<scalar_t>;
   // parallel on dim N, C
   at::parallel_for(0, channels, 0, [&](int64_t begin, int64_t end) {
     for (int64_t c = begin; c < end; c++) {
-      scalar_t* input_ptr = input_data + c * input_depth * input_height * input_width;
+      const scalar_t* input_ptr = input_data + c * input_depth * input_height * input_width;
       scalar_t* output_ptr = output_data + c * output_depth * output_height * output_width;
       int64_t* indices_ptr = indices_data + c * output_depth * output_height * output_width;
 
@@ -306,17 +311,18 @@ void cpu_max_pool(
 
             // compute local max
             int64_t maxindex = id0 * input_height * input_width + ih0 * input_width + iw0;
-            accscalar_t maxval;
-            if (std::numeric_limits<accscalar_t>::has_infinity) {
-              maxval = -std::numeric_limits<accscalar_t>::infinity();
+            opmath_t maxval;
+            if (std::numeric_limits<opmath_t>::has_infinity) {
+              maxval = -std::numeric_limits<opmath_t>::infinity();
             } else {
-              maxval = std::numeric_limits<accscalar_t>::min();
+              maxval = std::numeric_limits<opmath_t>::min();
             }
+
             for (int64_t id = id0; id < id1; id += dilationD) {
               for (int64_t ih = ih0; ih < ih1; ih += dilationH) {
                 for (int64_t iw = iw0; iw < iw1; iw += dilationW) {
                   int64_t index = id * input_height * input_width + ih * input_width + iw;
-                  accscalar_t val = input_ptr[index];
+                  opmath_t val = input_ptr[index];
                   if ((val > maxval) || is_nan(static_cast<double>(val))) {
                     maxval = val;
                     maxindex = index;
@@ -383,7 +389,7 @@ void cpu_max_pool_channels_last(
   auto output = output_.contiguous(memory_format);
   auto indices = indices_.contiguous(memory_format);
 
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   auto output_data = output.data_ptr<scalar_t>();
   auto indices_data = indices.data_ptr<int64_t>();
 
@@ -396,10 +402,10 @@ void cpu_max_pool_channels_last(
   int64_t output_height = output.size(-2);
   int64_t output_width = output.size(-1);
 
-  using accscalar_t = at::opmath_type<scalar_t>;
+  using opmath_t = at::opmath_type<scalar_t>;
   using Vec = vec::Vectorized<scalar_t>;
-  using integer_t = vec::int_same_size_t<accscalar_t>;
-  // for the convience of vectorization, use integer of the same size of scalar_t,
+  using integer_t = vec::int_same_size_t<opmath_t>;
+  // for the convenience of vectorization, use integer of the same size of scalar_t,
   //   e.g. int32_t for float, int64_t for double
   // need to make sure doesn't overflow
   TORCH_CHECK(input_depth * input_height * input_width <= std::numeric_limits<integer_t>::max());
@@ -415,14 +421,13 @@ void cpu_max_pool_channels_last(
     int64_t size = channels;
     int64_t len = size - (size % Vec::size());
     // temp buffer holding index with integer_t
-    // NOLINTNEXTLINE(modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
-    std::unique_ptr<integer_t []> index_buffer(new integer_t[len]);
+    auto index_buffer = std::make_unique<integer_t []>(len);
     integer_t * index_ptr = index_buffer.get();
-    // temp buffer holding max value with accscalar_t
-    std::unique_ptr<accscalar_t []> max_arr;
-    accscalar_t* max_ptr = nullptr;
-    if (!std::is_same<scalar_t, accscalar_t>::value) {
-      max_arr = std::make_unique<accscalar_t[]>(size);
+    // temp buffer holding max value with opmath_t
+    std::unique_ptr<opmath_t []> max_arr;
+    opmath_t* max_ptr = nullptr;
+    if (!std::is_same<scalar_t, opmath_t>::value) {
+      max_arr = std::make_unique<opmath_t[]>(size);
       max_ptr = max_arr.get();
     }
 
@@ -470,8 +475,8 @@ void cpu_max_pool_backward(
   auto indices = indices_.contiguous();
   auto grad_input = grad_input_.contiguous();
 
-  auto grad_output_data = grad_output.data_ptr<scalar_t>();
-  auto indices_data = indices.data_ptr<int64_t>();
+  auto grad_output_data = grad_output.const_data_ptr<scalar_t>();
+  auto indices_data = indices.const_data_ptr<int64_t>();
   auto grad_input_data = grad_input.mutable_data_ptr<scalar_t>();
 
   // treat batch size and channels as one dimension
@@ -502,8 +507,8 @@ void cpu_max_pool_backward(
   at::parallel_for(0, channels, 0, [&](int64_t begin, int64_t end) {
     for (const auto c : c10::irange(begin, end)) {
       scalar_t* grad_input_ptr = grad_input_data + c * input_depth * input_height * input_width;
-      scalar_t* grad_output_ptr = grad_output_data + c * output_depth * output_height * output_width;
-      int64_t * indices_ptr = indices_data + c * output_depth * output_height * output_width;
+      const scalar_t* grad_output_ptr = grad_output_data + c * output_depth * output_height * output_width;
+      const int64_t * indices_ptr = indices_data + c * output_depth * output_height * output_width;
 
       for (int64_t od = 0; od < output_depth; od++) {
         for (int64_t oh = 0; oh < output_height; oh++) {
@@ -544,8 +549,8 @@ void cpu_max_pool_backward_channels_last(
   auto indices = indices_.contiguous(memory_format);
 
   auto grad_input_data = grad_input.mutable_data_ptr<scalar_t>();
-  auto grad_output_data = grad_output.data_ptr<scalar_t>();
-  auto indices_data = indices.data_ptr<int64_t>();
+  auto grad_output_data = grad_output.const_data_ptr<scalar_t>();
+  auto indices_data = indices.const_data_ptr<int64_t>();
 
   // MaxPool2d: NHWC
   // MaxPool3d: NDHWC
@@ -562,14 +567,14 @@ void cpu_max_pool_backward_channels_last(
   at::parallel_for(0, nbatch, 0, [&](int64_t begin, int64_t end) {
     for (const auto n : c10::irange(begin, end)) {
       scalar_t* grad_input_ptr = grad_input_data + n * input_depth * input_height * input_width * channels;
-      scalar_t* grad_output_ptr = grad_output_data + n * output_depth * output_height * output_width * channels;
-      int64_t* indices_ptr = indices_data + n * output_depth * output_height * output_width * channels;
+      const scalar_t* grad_output_ptr = grad_output_data + n * output_depth * output_height * output_width * channels;
+      const int64_t* indices_ptr = indices_data + n * output_depth * output_height * output_width * channels;
 
       for (int64_t od = 0; od < output_depth; od++) {
         for (int64_t oh = 0; oh < output_height; oh++) {
           for (int64_t ow = 0; ow < output_width; ow++) {
-            scalar_t* gout = grad_output_ptr + (od * output_height * output_width + oh * output_width + ow) * channels;
-            int64_t* ind = indices_ptr + (od * output_height * output_width + oh * output_width + ow) * channels;
+            const scalar_t* gout = grad_output_ptr + (od * output_height * output_width + oh * output_width + ow) * channels;
+            const int64_t* ind = indices_ptr + (od * output_height * output_width + oh * output_width + ow) * channels;
             // TODO: gcc vectorization
             for (int64_t c = 0; c < channels; c++) {
               int64_t maxindex = ind[c];
@@ -598,13 +603,13 @@ void max_pool2d_kernel_impl(
     int dilationW, int dilationH) {
   switch (input.suggest_memory_format()) {
     case at::MemoryFormat::Contiguous: {
-      AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, input.scalar_type(), "max_pool2d", [&] {
+      AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, input.scalar_type(), "max_pool2d", [&] {
         cpu_max_pool<scalar_t, /*is 3d*/false>(output, indices, input, {kW, kH}, {dW, dH}, {padW, padH}, {dilationW, dilationH});
       });
       break;
     }
     case at::MemoryFormat::ChannelsLast: {
-      AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, input.scalar_type(), "max_pool2d_channels_last", [&] {
+      AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, input.scalar_type(), "max_pool2d_channels_last", [&] {
         cpu_max_pool_channels_last<scalar_t, false>(output, indices, input, {kW, kH}, {dW, dH}, {padW, padH}, {dilationW, dilationH});
       });
       break;
@@ -637,7 +642,7 @@ void max_pool3d_kernel_impl(
       DimVector indices_sizes(indices.sizes().begin(), indices.sizes().end());
       indices_sizes.insert(indices_sizes.begin(), 1);
       indices.resize_(indices_sizes, at::MemoryFormat::ChannelsLast3d);
-      AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, input.scalar_type(), "max_pool3d_channels_last", [&] {
+      AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, input.scalar_type(), "max_pool3d_channels_last", [&] {
         cpu_max_pool_channels_last<scalar_t, /*is 3d*/true>(output, indices, input_cl_check,
           {kW, kH, kD}, {dW, dH, dD}, {padW, padH, padD}, {dilationW, dilationH, dilationD});
       });
@@ -648,14 +653,14 @@ void max_pool3d_kernel_impl(
   }
   switch (input.suggest_memory_format()) {
     case at::MemoryFormat::Contiguous: {
-      AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, input.scalar_type(), "max_pool3d", [&] {
+      AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, input.scalar_type(), "max_pool3d", [&] {
         cpu_max_pool<scalar_t, /*is 3d*/true>(output, indices, input,
             {kW, kH, kD}, {dW, dH, dD}, {padW, padH, padD}, {dilationW, dilationH, dilationD});
       });
       break;
     }
     case at::MemoryFormat::ChannelsLast3d: {
-      AT_DISPATCH_ALL_TYPES_AND(ScalarType::BFloat16, input.scalar_type(), "max_pool3d_channels_last", [&] {
+      AT_DISPATCH_ALL_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, input.scalar_type(), "max_pool3d_channels_last", [&] {
         cpu_max_pool_channels_last<scalar_t, true>(output, indices, input,
           {kW, kH, kD}, {dW, dH, dD}, {padW, padH, padD}, {dilationW, dilationH, dilationD});
       });
@@ -672,13 +677,13 @@ void max_pool2d_backward_kernel_impl(
     const Tensor& indices) {
   switch (grad_output.suggest_memory_format()) {
     case at::MemoryFormat::Contiguous: {
-      AT_DISPATCH_FLOATING_TYPES_AND(ScalarType::BFloat16, grad_output.scalar_type(), "max_pool2d_backward", [&] {
+      AT_DISPATCH_FLOATING_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, grad_output.scalar_type(), "max_pool2d_backward", [&] {
         cpu_max_pool_backward<scalar_t, /*is 3d*/ false>(grad_input, grad_output, indices);
       });
       break;
     }
     case at::MemoryFormat::ChannelsLast: {
-      AT_DISPATCH_FLOATING_TYPES_AND(ScalarType::BFloat16, grad_output.scalar_type(), "max_pool2d_backward_channels_last", [&] {
+      AT_DISPATCH_FLOATING_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, grad_output.scalar_type(), "max_pool2d_backward_channels_last", [&] {
         cpu_max_pool_backward_channels_last<scalar_t, /*is 3d*/ false>(grad_input, grad_output, indices);
       });
       break;
@@ -705,7 +710,7 @@ void max_pool3d_backward_kernel_impl(
       sizes.insert(sizes.begin(), 1);
       grad_input.resize_(sizes, at::MemoryFormat::ChannelsLast3d);
       auto _indices = indices.unsqueeze(0).contiguous(at::MemoryFormat::ChannelsLast3d);
-      AT_DISPATCH_FLOATING_TYPES_AND(ScalarType::BFloat16, grad_output.scalar_type(), "max_pool3d_backward_channels_last", [&] {
+      AT_DISPATCH_FLOATING_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, grad_output.scalar_type(), "max_pool3d_backward_channels_last", [&] {
         cpu_max_pool_backward_channels_last<scalar_t, /*is_3d*/ true>(grad_input, grad_output_cl_check, _indices);
       });
       grad_input.squeeze_(0);
@@ -714,13 +719,13 @@ void max_pool3d_backward_kernel_impl(
   }
   switch (grad_output.suggest_memory_format()) {
     case at::MemoryFormat::Contiguous: {
-      AT_DISPATCH_FLOATING_TYPES_AND(ScalarType::BFloat16, grad_output.scalar_type(), "max_pool3d_backward", [&] {
+      AT_DISPATCH_FLOATING_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, grad_output.scalar_type(), "max_pool3d_backward", [&] {
         cpu_max_pool_backward<scalar_t, /*is_3d*/ true>(grad_input, grad_output, indices);
       });
       break;
     }
     case at::MemoryFormat::ChannelsLast3d: {
-      AT_DISPATCH_FLOATING_TYPES_AND(ScalarType::BFloat16, grad_output.scalar_type(), "max_pool3d_backward_channels_last", [&] {
+      AT_DISPATCH_FLOATING_TYPES_AND2(ScalarType::BFloat16, ScalarType::Half, grad_output.scalar_type(), "max_pool3d_backward_channels_last", [&] {
         cpu_max_pool_backward_channels_last<scalar_t, /*is_3d*/ true>(grad_input, grad_output, indices);
       });
       break;
