@@ -17,7 +17,7 @@ struct PaddingParams {
   int64_t nbatch;
   int64_t channels;
 
-  // use vectorized logic on width when output index is in [pad, input_witdh + pad),
+  // use vectorized logic on width when output index is in [pad, input_width + pad),
   // applies only to Channels First format when pad_l and pad_r are both positive.
   bool is_padding_positive_width;
 
@@ -136,7 +136,7 @@ void cpu_padding(
   auto input = input_.contiguous();
   auto output = output_.contiguous();
 
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   auto output_data = output.data_ptr<scalar_t>();
 
   // fold nbatch and channels into single dimension for channels first.
@@ -158,7 +158,7 @@ void cpu_padding(
 
   // do vectorized copy whe output is overlapped with input on W,
   // only applies to positive padding
-  auto loop = [=](scalar_t* out, scalar_t* in, bool positive_padding) {
+  auto loop = [=](scalar_t* out, const scalar_t* in, bool positive_padding) {
     if (positive_padding) {
       for (const auto ow : c10::irange(pad_w)) {
         int64_t iw = PaddingType::index(ow, input_width, pad_w, offset_w);
@@ -198,7 +198,7 @@ void cpu_padding(
       for (const auto i : c10::irange(begin, end)) {
         int64_t ih = PaddingType::index(oh, input_height, pad_h, offset_h);
         scalar_t* output_ptr = output_data + i * output_width;
-        scalar_t* input_ptr = input_data + c * input_height * input_width + ih * input_width;
+        const scalar_t* input_ptr = input_data + c * input_height * input_width + ih * input_width;
 
         loop(output_ptr, input_ptr, p.is_padding_positive_width);
         data_index_step(c, channels, oh, output_height);
@@ -214,7 +214,7 @@ void cpu_padding(
         int64_t id = PaddingType::index(od, input_depth, pad_d, offset_d);
         int64_t ih = PaddingType::index(oh, input_height, pad_h, offset_h);
         scalar_t* output_ptr = output_data + i * output_width;
-        scalar_t* input_ptr = input_data + c * input_depth * input_height * input_width +
+        const scalar_t* input_ptr = input_data + c * input_depth * input_height * input_width +
             id * input_height * input_width + ih * input_width;
 
         loop(output_ptr, input_ptr, p.is_padding_positive_width);
@@ -243,7 +243,7 @@ void cpu_padding_channels_last(
   auto input = input_.contiguous(memory_format);
   auto output = output_.contiguous(memory_format);
 
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   auto output_data = output.data_ptr<scalar_t>();
 
   int64_t nbatch = p.nbatch;
@@ -274,7 +274,7 @@ void cpu_padding_channels_last(
         int64_t iw = PaddingType::index(ow, input_width, pad_w, offset_w);
 
         scalar_t* output_ptr = output_data + i * channels;
-        scalar_t* input_ptr = input_data + (n * input_height * input_width + ih * input_width + iw) * channels;
+        const scalar_t* input_ptr = input_data + (n * input_height * input_width + ih * input_width + iw) * channels;
         copy_stub(output_ptr, input_ptr, channels);
 
         data_index_step(n, nbatch, oh, output_height, ow, output_width);
@@ -292,7 +292,7 @@ void cpu_padding_channels_last(
         int64_t iw = PaddingType::index(ow, input_width, pad_w, offset_w);
 
         scalar_t* output_ptr = output_data + i * channels;
-        scalar_t* input_ptr = input_data + (n * input_depth * input_height * input_width +
+        const scalar_t* input_ptr = input_data + (n * input_depth * input_height * input_width +
             id * input_height * input_width + ih * input_width + iw) * channels;
         copy_stub(output_ptr, input_ptr, channels);
 
@@ -317,7 +317,7 @@ void cpu_padding_backward(
   auto grad_output = grad_output_.contiguous();
   auto grad_input = grad_input_.contiguous();
 
-  auto grad_output_data = grad_output.data_ptr<scalar_t>();
+  auto grad_output_data = grad_output.const_data_ptr<scalar_t>();
   auto grad_input_data = grad_input.data_ptr<scalar_t>();
 
   // fold nbatch and channels into single dimension for channels first.
@@ -351,7 +351,7 @@ void cpu_padding_backward(
     // parallel on N,C, sequential on H,W
     at::parallel_for(0, channels, 1, [&](int64_t begin, int64_t end) {
       for (const auto c : c10::irange(begin, end)) {
-        scalar_t* grad_output_ptr = grad_output_data + c * output_height * output_width;
+        const scalar_t* grad_output_ptr = grad_output_data + c * output_height * output_width;
         scalar_t* grad_input_ptr = grad_input_data + c * input_height * input_width;
 
         for (const auto oh : c10::irange(output_height)) {
@@ -367,7 +367,7 @@ void cpu_padding_backward(
     // parallel on N,C, sequential on D,H,W
     at::parallel_for(0, channels, 1, [&](int64_t begin, int64_t end) {
       for (const auto c : c10::irange(begin, end)) {
-        scalar_t* grad_output_ptr = grad_output_data + c * output_depth *output_height * output_width;
+        const scalar_t* grad_output_ptr = grad_output_data + c * output_depth *output_height * output_width;
         scalar_t* grad_input_ptr = grad_input_data + c * input_depth * input_height * input_width;
 
         for (const auto od : c10::irange(output_depth)) {
@@ -406,7 +406,7 @@ void cpu_padding_backward_channels_last(
   auto grad_output = grad_output_.contiguous(memory_format);
 
   auto grad_input_data = grad_input.data_ptr<scalar_t>();
-  auto grad_output_data = grad_output.data_ptr<scalar_t>();
+  auto grad_output_data = grad_output.const_data_ptr<scalar_t>();
 
   int64_t nbatch = p.nbatch;
   int64_t channels = p.channels;
@@ -435,7 +435,7 @@ void cpu_padding_backward_channels_last(
             int64_t iw = PaddingType::index(ow, input_width, pad_w, offset_w);
             scalar_t* grad_input_ptr = grad_input_data +
                 (n * input_height * input_width + ih * input_width + iw) * channels;
-            scalar_t* grad_output_ptr = grad_output_data +
+            const scalar_t* grad_output_ptr = grad_output_data +
                 (n * output_height * output_width + oh * output_width + ow) * channels;
             add_stub(grad_input_ptr, grad_output_ptr, channels);
           }
@@ -455,7 +455,7 @@ void cpu_padding_backward_channels_last(
               scalar_t* grad_input_ptr = grad_input_data +
                   (n * input_depth * input_height * input_width + id * input_height * input_width +
                    ih * input_width + iw) * channels;
-              scalar_t* grad_output_ptr = grad_output_data +
+              const scalar_t* grad_output_ptr = grad_output_data +
                   (n * output_depth * output_height * output_width + od * output_height * output_width +
                    oh * output_width + ow) * channels;
               add_stub(grad_input_ptr, grad_output_ptr, channels);
