@@ -7,6 +7,7 @@
 #include <ATen/Dispatch.h>
 #include <ATen/Parallel.h>
 #include <ATen/TensorIterator.h>
+#include <ATen/OpMathType.h>
 
 #include <ATen/native/cpu/Loops.h>
 #include <ATen/native/cpu/zmath.h>
@@ -26,10 +27,9 @@ inline void reduce_all_impl_vec(
     const scalar_t ident_v,
     func_t op,
     vec_func_t vop) {
-  // TODO: replace vec_scalar_t with at::opmath_type when float16 specialization is done.
-  using Vec = Vectorized<vec_scalar_t<scalar_t>>;
+  using Vec = Vectorized<opmath_type<scalar_t>>;
   const int64_t input_numel = input.numel();
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   // NOTE: parallel_reduce not support bool type
   scalar_t result = at::parallel_reduce(0, input_numel, internal::GRAIN_SIZE, ident_v,
     [&](int64_t start, int64_t end, const scalar_t /*ident*/) -> scalar_t {
@@ -50,7 +50,7 @@ inline void reduce_all_impl(
     const scalar_t ident_v,
     func_t op) {
   const int64_t input_numel = input.numel();
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   scalar_t result = at::parallel_reduce(0, input_numel, internal::GRAIN_SIZE, ident_v,
     [&](int64_t start, int64_t end, const scalar_t ident) -> scalar_t {
       scalar_t partial_out = ident;
@@ -79,8 +79,7 @@ static void min_all_kernel_impl(Tensor& result, const Tensor& input) {
       [=](int64_t a, int64_t b) -> int64_t { return min_impl(a, b); });
   } else {
     AT_DISPATCH_ALL_TYPES_AND2(kHalf, kBFloat16, input.scalar_type(), "min_all", [&] {
-      // TODO: replace vec_scalar_t with at::opmath_type when float16 specialization is done.
-      using Vec = Vectorized<vec_scalar_t<scalar_t>>;
+      using Vec = Vectorized<opmath_type<scalar_t>>;
       reduce_all_impl_vec<scalar_t>(result, input, upper_bound<scalar_t>(),
         [=] (scalar_t a , scalar_t b) -> scalar_t { return min_impl(a, b); },
         [=](Vec a, Vec b) -> Vec { return minimum(a, b); });
@@ -105,8 +104,7 @@ static void max_all_kernel_impl(Tensor& result, const Tensor& input) {
       [=](int64_t a, int64_t b) -> int64_t { return max_impl(a, b); });
   } else {
     AT_DISPATCH_ALL_TYPES_AND2(kHalf, kBFloat16, input.scalar_type(), "max_all", [&] {
-      // TODO: replace vec_scalar_t with at::opmath_type when float16 specialization is done.
-      using Vec = Vectorized<vec_scalar_t<scalar_t>>;
+      using Vec = Vectorized<opmath_type<scalar_t>>;
       reduce_all_impl_vec<scalar_t>(result, input, lower_bound<scalar_t>(),
         [=] (scalar_t a , scalar_t b) -> scalar_t { return max_impl(a, b); },
         [=](Vec a, Vec b) -> Vec { return maximum(a, b); });
@@ -125,7 +123,7 @@ inline void reduce_all_impl_two_outputs(
     func_t2 reduce_acc_func) {
   using scalar_t_pair = std::pair<scalar_t, scalar_t>;
   const int64_t input_numel = input.numel();
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   scalar_t_pair result = at::parallel_reduce(0, input_numel, internal::GRAIN_SIZE, ident_v,
     [&](int64_t start, int64_t end, const scalar_t_pair& ident) -> scalar_t_pair {
       scalar_t_pair partial_out(ident);
@@ -149,11 +147,10 @@ inline void reduce_all_impl_vec_two_outputs(
     func_t reduce_acc_func,
     vec_func_t1 reduce_chunk_func1,
     vec_func_t2 reduce_chunk_func2) {
-  // TODO: replace vec_scalar_t with at::opmath_type when float16 specialization is done.
-  using Vec = Vectorized<vec_scalar_t<scalar_t>>;
+  using Vec = Vectorized<opmath_type<scalar_t>>;
   using scalar_t_pair = std::pair<scalar_t, scalar_t>;
   const int64_t input_numel = input.numel();
-  auto input_data = input.data_ptr<scalar_t>();
+  auto input_data = input.const_data_ptr<scalar_t>();
   // NOTE: parallel_reduce not support bool type
   std::pair<scalar_t, scalar_t> result = at::parallel_reduce(0, input_numel, internal::GRAIN_SIZE, ident_v,
     [&](int64_t start, int64_t end, const scalar_t_pair& /* ident */) -> scalar_t_pair {
@@ -202,9 +199,8 @@ static void aminmax_allreduce_kernel(
       }
     );
   } else {
-    AT_DISPATCH_ALL_TYPES_AND(kBFloat16, input.scalar_type(), "aminmax_cpu", [&] {
-      // TODO: replace vec_scalar_t with at::opmath_type when float16 specialization is done.
-      using Vec = Vectorized<vec_scalar_t<scalar_t>>;
+    AT_DISPATCH_ALL_TYPES_AND2(kBFloat16, kHalf, input.scalar_type(), "aminmax_cpu", [&] {
+      using Vec = Vectorized<opmath_type<scalar_t>>;
       using scalar_t_pair = std::pair<scalar_t, scalar_t>;
       reduce_all_impl_vec_two_outputs<scalar_t>(
         min_result,
