@@ -12,7 +12,18 @@
 #include <ATen/cpu/vec/functional.h>
 #include <ATen/cpu/vec/vec.h>
 #include <c10/util/irange.h>
+#ifdef USE_FBGEMM
+#include <fbgemm/Utils.h>
+#endif
+#include <ATen/OpMathType.h>
 
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/empty.h>
+#include <ATen/ops/zeros.h>
+#endif
 namespace at::native {
 
 namespace {
@@ -21,8 +32,9 @@ namespace {
 class ReduceMultiply {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data *= *src_data;
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data *= opmath_t(*src_data);
   }
 
   constexpr void operator() (bool * self_data, bool * src_data) const {
@@ -34,8 +46,9 @@ static ReduceMultiply reduce_multiply;
 class ReduceAdd {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data += *src_data;
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data += opmath_t(*src_data);
   }
 };
 static ReduceAdd reduce_add;
@@ -43,8 +56,9 @@ static ReduceAdd reduce_add;
 class ReduceMean {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data += *src_data;
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data += opmath_t(*src_data);
   }
 };
 static ReduceMean reduce_mean;
@@ -52,8 +66,9 @@ static ReduceMean reduce_mean;
 class ReduceMaximum {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data = at::_isnan<scalar_t>(*src_data) ? *src_data : std::max(*self_data, *src_data);
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data = at::_isnan<scalar_t>(*src_data) ? opmath_t(*src_data) : std::max(*self_data, opmath_t(*src_data));
   }
 };
 static ReduceMaximum reduce_maximum;
@@ -61,8 +76,9 @@ static ReduceMaximum reduce_maximum;
 class ReduceMinimum {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data = at::_isnan<scalar_t>(*src_data) ? *src_data : std::min(*self_data, *src_data);
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data = at::_isnan<scalar_t>(*src_data) ? opmath_t(*src_data) : std::min(*self_data, opmath_t(*src_data));
   }
 };
 static ReduceMinimum reduce_minimum;
@@ -70,8 +86,9 @@ static ReduceMinimum reduce_minimum;
 class TensorAssign {
 public:
   template <typename scalar_t>
-  constexpr void operator() (scalar_t * self_data, scalar_t * src_data) const {
-    *self_data = *src_data;
+  constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
+    using opmath_t = at::opmath_type<scalar_t>;
+    *self_data = opmath_t(*src_data);
   }
 };
 static TensorAssign tensor_assign;
@@ -80,7 +97,7 @@ template <bool is_scatter_like = true>
 struct _cpu_scatter_gather_dim_loop {
   template <typename scalar_t, typename func_t>
   void operator()(
-    scalar_t* self_data, int64_t self_dim_stride,
+    at::opmath_type<scalar_t>* self_data, int64_t self_dim_stride,
     int64_t* index_data, int64_t index_dim_stride,
     scalar_t* src_data, int64_t src_dim_stride,
     int64_t dim, int64_t index_dim_size,
@@ -107,7 +124,7 @@ struct _cpu_scatter_gather_dim_loop {
 
   template <typename scalar_t, typename func_t>
   void operator()(
-    scalar_t* self_data, int64_t self_dim_stride,
+    at::opmath_type<scalar_t>* self_data, int64_t self_dim_stride,
     int64_t* index_data, int64_t index_dim_stride,
     Scalar value,
     int64_t dim, int64_t index_dim_size,
@@ -132,6 +149,15 @@ struct _cpu_scatter_gather_dim_loop {
   }
 };
 
+inline void create_acc_buffer(Tensor& buffer, const Tensor& self, bool need_acc) {
+  if (need_acc) {
+    auto acc_type = at::toOpMathType(self.scalar_type());
+    buffer = at::empty(self.sizes(), self.options().dtype(acc_type));
+    buffer.copy_(self);
+  } else {
+    buffer = self;
+  }
+}
 
 template <bool is_scatter_like = true>
 struct cpu_scatter_gather_base_kernel {
@@ -140,6 +166,10 @@ struct cpu_scatter_gather_base_kernel {
     const Tensor& index, const Scalar& value,
     const std::string& method_name, func_t& kernel_func) {
 
+    Tensor buffer;
+    bool need_acc = isReducedFloatingType(self.scalar_type());
+    create_acc_buffer(buffer, self, need_acc);
+
     auto index_sizes = ensure_nonempty_vec(index.sizes().vec());
     auto index_strides = ensure_nonempty_vec(index.strides().vec());
 
@@ -155,12 +185,12 @@ struct cpu_scatter_gather_base_kernel {
       .resize_outputs(false)
       // NOLINTNEXTLINE(bugprone-argument-comment)
       .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
-      .add_output(self)
-      .add_input(index)
+      .add_output(buffer)
+      .add_const_input(index)
       .build();
 
-    auto self_dim_stride = ensure_nonempty_stride(self, dim);
-    auto self_dim_size = ensure_nonempty_size(self, dim);
+    auto self_dim_stride = ensure_nonempty_stride(buffer, dim);
+    auto self_dim_size = ensure_nonempty_size(buffer, dim);
 
     auto index_dim_stride = ensure_nonempty_stride(index, dim);
     auto index_dim_size = ensure_nonempty_size(index, dim);
@@ -172,23 +202,24 @@ struct cpu_scatter_gather_base_kernel {
     int64_t grain_size = std::max((int64_t) 1, at::internal::GRAIN_SIZE / index_dim_size);
 
     AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND3(
-      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(),
+      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, self.scalar_type(),
       "scatter_gather_scalar_cpu", [&] {
         constexpr auto SELF_ITER_STRIDE_IDX = 0;
         constexpr auto INDEX_ITER_STRIDE_IDX = 1;
-
+        using opmath_t = at::opmath_type<scalar_t>;
+        _cpu_scatter_gather_dim_loop<is_scatter_like> loop_func;
         auto loop = [&](char** data, const int64_t* strides, int64_t n) {
           auto* self_data_bytes = data[SELF_ITER_STRIDE_IDX];
           auto* index_data_bytes = data[INDEX_ITER_STRIDE_IDX];
           // we change the order of TensorIterator-dim loop
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
-          if (dim== self.dim() - 1) {
+          if (dim== buffer.dim() - 1) {
             for (const auto nelem C10_UNUSED : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
-              _cpu_scatter_gather_dim_loop<is_scatter_like>()(
-                (scalar_t*)self_data_bytes, self_dim_stride,
+              loop_func.template operator()<scalar_t, func_t>(
+                (opmath_t*)self_data_bytes, self_dim_stride,
                 (int64_t*)index_data_bytes, index_dim_stride,
                 value, dim, index_dim_size, index_upper_bound,
                 kernel_func);
@@ -211,7 +242,7 @@ struct cpu_scatter_gather_base_kernel {
                             " with size ", index_upper_bound);
 
                 auto temp = value.to<scalar_t>();
-                kernel_func((scalar_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride, &temp);
+                kernel_func((opmath_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride, &temp);
 
                 self_data += strides[SELF_ITER_STRIDE_IDX];
                 index_data += strides[INDEX_ITER_STRIDE_IDX];
@@ -222,6 +253,9 @@ struct cpu_scatter_gather_base_kernel {
         iter.for_each(loop, grain_size);
       }
     );
+    if (need_acc) {
+      self.copy_(buffer);
+    }
   }
 
   template <typename func_t>
@@ -229,18 +263,22 @@ struct cpu_scatter_gather_base_kernel {
     const Tensor& index, const Tensor& src,
     const std::string& method_name, func_t& kernel_func) {
 
+    Tensor buffer;
+    bool need_acc = isReducedFloatingType(self.scalar_type());
+    create_acc_buffer(buffer, self, need_acc);
+
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
       // NOLINTNEXTLINE(bugprone-argument-comment)
       .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
-      .add_output(self)
-      .add_input(src)
-      .add_input(index)
+      .add_output(buffer)
+      .add_const_input(src)
+      .add_const_input(index)
       .build();
 
-    auto self_dim_stride = ensure_nonempty_stride(self, dim);
-    auto self_dim_size = ensure_nonempty_size(self, dim);
+    auto self_dim_stride = ensure_nonempty_stride(buffer, dim);
+    auto self_dim_size = ensure_nonempty_size(buffer, dim);
 
     auto index_dim_stride = ensure_nonempty_stride(index, dim);
     auto index_dim_size = ensure_nonempty_size(index, dim);
@@ -253,11 +291,13 @@ struct cpu_scatter_gather_base_kernel {
     int64_t grain_size = std::max((int64_t) 1, at::internal::GRAIN_SIZE / index_dim_size);
 
     AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND3(
-      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(),
+      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(1),
       "scatter_gather_tensor_cpu", [&] {
         constexpr auto SELF_ITER_STRIDE_IDX = 0;
         constexpr auto INDEX_ITER_STRIDE_IDX = 2;
         constexpr auto SRC_ITER_STRIDE_IDX = 1;
+        using opmath_t = at::opmath_type<scalar_t>;
+        _cpu_scatter_gather_dim_loop<is_scatter_like> loop_func;
         auto loop = [&](char** data, const int64_t* strides, int64_t n) {
           auto* self_data_bytes = data[SELF_ITER_STRIDE_IDX];
           auto* index_data_bytes = data[INDEX_ITER_STRIDE_IDX];
@@ -265,12 +305,12 @@ struct cpu_scatter_gather_base_kernel {
           // we change the order of TensorIterator-dim loop
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
-          if (dim== self.dim() - 1) {
+          if (dim== buffer.dim() - 1) {
             for (const auto nelem C10_UNUSED : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
-              _cpu_scatter_gather_dim_loop<is_scatter_like>()(
-                 (scalar_t*)self_data_bytes, self_dim_stride,
+              loop_func.template operator()<scalar_t, func_t>(
+                 (opmath_t*)self_data_bytes, self_dim_stride,
                  (int64_t*)index_data_bytes, index_dim_stride,
                  (scalar_t*)src_data_bytes, src_dim_stride,
                  dim, index_dim_size, index_upper_bound,
@@ -297,7 +337,7 @@ struct cpu_scatter_gather_base_kernel {
                             " with size ", index_upper_bound);
 
                 kernel_func(
-                  (scalar_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
+                  (opmath_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
                   (scalar_t*)src_data + (is_scatter_like ? i : idx_dim) * src_dim_stride);
 
                 self_data += strides[SELF_ITER_STRIDE_IDX];
@@ -310,24 +350,31 @@ struct cpu_scatter_gather_base_kernel {
         iter.for_each(loop, grain_size);
       }
     );
+    if (need_acc) {
+      self.copy_(buffer);
+    }
   }
 
   void operator()(const Tensor& self, int64_t dim,
     const Tensor& index, const Tensor& src,
     const std::string& method_name, ReduceMean& kernel_func) {
 
+    Tensor buffer;
+    bool need_acc = isReducedFloatingType(self.scalar_type());
+    create_acc_buffer(buffer, self, need_acc);
+
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
       // NOLINTNEXTLINE(bugprone-argument-comment)
       .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
-      .add_output(self)
-      .add_input(src)
-      .add_input(index)
+      .add_output(buffer)
+      .add_const_input(src)
+      .add_const_input(index)
       .build();
 
-    auto self_dim_stride = ensure_nonempty_stride(self, dim);
-    auto self_dim_size = ensure_nonempty_size(self, dim);
+    auto self_dim_stride = ensure_nonempty_stride(buffer, dim);
+    auto self_dim_size = ensure_nonempty_size(buffer, dim);
 
     auto index_dim_stride = ensure_nonempty_stride(index, dim);
     auto index_dim_size = ensure_nonempty_size(index, dim);
@@ -340,11 +387,13 @@ struct cpu_scatter_gather_base_kernel {
     int64_t grain_size = std::max((int64_t) 1, at::internal::GRAIN_SIZE / index_dim_size);
 
     AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND2(
-      ScalarType::Half, ScalarType::BFloat16, iter.dtype(),
+      ScalarType::Half, ScalarType::BFloat16, iter.dtype(1),
       "scatter_gather_tensor_cpu_reduce_mean", [&] {
         constexpr auto SELF_ITER_STRIDE_IDX = 0;
         constexpr auto INDEX_ITER_STRIDE_IDX = 2;
         constexpr auto SRC_ITER_STRIDE_IDX = 1;
+        using opmath_t = at::opmath_type<scalar_t>;
+        _cpu_scatter_gather_dim_loop<is_scatter_like> loop_func;
         auto loop = [&](char** data, const int64_t* strides, int64_t n) {
           auto* self_data_bytes = data[SELF_ITER_STRIDE_IDX];
           auto* index_data_bytes = data[INDEX_ITER_STRIDE_IDX];
@@ -352,12 +401,12 @@ struct cpu_scatter_gather_base_kernel {
           // we change the order of TensorIterator-dim loop
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
-          if (dim== self.dim() - 1) {
+          if (dim== buffer.dim() - 1) {
             for (const auto nelem C10_UNUSED : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
-              _cpu_scatter_gather_dim_loop<is_scatter_like>()(
-                 (scalar_t*)self_data_bytes, self_dim_stride,
+              loop_func.template operator()<scalar_t, ReduceMean>(
+                 (opmath_t*)self_data_bytes, self_dim_stride,
                  (int64_t*)index_data_bytes, index_dim_stride,
                  (scalar_t*)src_data_bytes, src_dim_stride,
                  dim, index_dim_size, index_upper_bound,
@@ -384,7 +433,7 @@ struct cpu_scatter_gather_base_kernel {
                             " with size ", index_upper_bound);
 
                 kernel_func(
-                  (scalar_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
+                  (opmath_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
                   (scalar_t*)src_data + (is_scatter_like ? i : idx_dim) * src_dim_stride);
 
                 self_data += strides[SELF_ITER_STRIDE_IDX];
@@ -397,24 +446,30 @@ struct cpu_scatter_gather_base_kernel {
         iter.for_each(loop, grain_size);
       }
     );
+    if (need_acc) {
+      self.copy_(buffer);
+    }
   }
 
   void operator()(const Tensor& self, int64_t dim,
     const Tensor& index, const Tensor& src,
     const std::string& method_name, ReduceMaximum& kernel_func) {
+    Tensor buffer;
+    bool need_acc = isReducedFloatingType(self.scalar_type());
+    create_acc_buffer(buffer, self, need_acc);
 
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
       // NOLINTNEXTLINE(bugprone-argument-comment)
       .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
-      .add_output(self)
-      .add_input(src)
-      .add_input(index)
+      .add_output(buffer)
+      .add_const_input(src)
+      .add_const_input(index)
       .build();
 
-    auto self_dim_stride = ensure_nonempty_stride(self, dim);
-    auto self_dim_size = ensure_nonempty_size(self, dim);
+    auto self_dim_stride = ensure_nonempty_stride(buffer, dim);
+    auto self_dim_size = ensure_nonempty_size(buffer, dim);
 
     auto index_dim_stride = ensure_nonempty_stride(index, dim);
     auto index_dim_size = ensure_nonempty_size(index, dim);
@@ -427,11 +482,13 @@ struct cpu_scatter_gather_base_kernel {
     int64_t grain_size = std::max((int64_t) 1, at::internal::GRAIN_SIZE / index_dim_size);
 
     AT_DISPATCH_ALL_TYPES_AND3(
-      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(),
+      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(1),
       "scatter_gather_tensor_cpu_reduce_amax", [&] {
         constexpr auto SELF_ITER_STRIDE_IDX = 0;
         constexpr auto INDEX_ITER_STRIDE_IDX = 2;
         constexpr auto SRC_ITER_STRIDE_IDX = 1;
+        using opmath_t = at::opmath_type<scalar_t>;
+        _cpu_scatter_gather_dim_loop<is_scatter_like> loop_func;
         auto loop = [&](char** data, const int64_t* strides, int64_t n) {
           auto* self_data_bytes = data[SELF_ITER_STRIDE_IDX];
           auto* index_data_bytes = data[INDEX_ITER_STRIDE_IDX];
@@ -439,12 +496,12 @@ struct cpu_scatter_gather_base_kernel {
           // we change the order of TensorIterator-dim loop
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
-          if (dim== self.dim() - 1) {
+          if (dim== buffer.dim() - 1) {
             for (const auto nelem C10_UNUSED : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
-              _cpu_scatter_gather_dim_loop<is_scatter_like>()(
-                 (scalar_t*)self_data_bytes, self_dim_stride,
+              loop_func.template operator()<scalar_t, ReduceMaximum>(
+                 (opmath_t*)self_data_bytes, self_dim_stride,
                  (int64_t*)index_data_bytes, index_dim_stride,
                  (scalar_t*)src_data_bytes, src_dim_stride,
                  dim, index_dim_size, index_upper_bound,
@@ -471,7 +528,7 @@ struct cpu_scatter_gather_base_kernel {
                             " with size ", index_upper_bound);
 
                 kernel_func(
-                  (scalar_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
+                  (opmath_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
                   (scalar_t*)src_data + (is_scatter_like ? i : idx_dim) * src_dim_stride);
 
                 self_data += strides[SELF_ITER_STRIDE_IDX];
@@ -484,24 +541,31 @@ struct cpu_scatter_gather_base_kernel {
         iter.for_each(loop, grain_size);
       }
     );
+    if (need_acc) {
+      self.copy_(buffer);
+    }
   }
 
   void operator()(const Tensor& self, int64_t dim,
     const Tensor& index, const Tensor& src,
     const std::string& method_name, ReduceMinimum& kernel_func) {
 
+    Tensor buffer;
+    bool need_acc = isReducedFloatingType(self.scalar_type());
+    create_acc_buffer(buffer, self, need_acc);
+
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
       // NOLINTNEXTLINE(bugprone-argument-comment)
       .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
-      .add_output(self)
-      .add_input(src)
-      .add_input(index)
+      .add_output(buffer)
+      .add_const_input(src)
+      .add_const_input(index)
       .build();
 
-    auto self_dim_stride = ensure_nonempty_stride(self, dim);
-    auto self_dim_size = ensure_nonempty_size(self, dim);
+    auto self_dim_stride = ensure_nonempty_stride(buffer, dim);
+    auto self_dim_size = ensure_nonempty_size(buffer, dim);
 
     auto index_dim_stride = ensure_nonempty_stride(index, dim);
     auto index_dim_size = ensure_nonempty_size(index, dim);
@@ -514,11 +578,13 @@ struct cpu_scatter_gather_base_kernel {
     int64_t grain_size = std::max((int64_t) 1, at::internal::GRAIN_SIZE / index_dim_size);
 
     AT_DISPATCH_ALL_TYPES_AND3(
-      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(),
+      ScalarType::Bool, ScalarType::Half, ScalarType::BFloat16, iter.dtype(1),
       "scatter_gather_tensor_cpu_reduce_amin", [&] {
         constexpr auto SELF_ITER_STRIDE_IDX = 0;
         constexpr auto INDEX_ITER_STRIDE_IDX = 2;
         constexpr auto SRC_ITER_STRIDE_IDX = 1;
+        using opmath_t = at::opmath_type<scalar_t>;
+        _cpu_scatter_gather_dim_loop<is_scatter_like> loop_func;
         auto loop = [&](char** data, const int64_t* strides, int64_t n) {
           auto* self_data_bytes = data[SELF_ITER_STRIDE_IDX];
           auto* index_data_bytes = data[INDEX_ITER_STRIDE_IDX];
@@ -526,12 +592,12 @@ struct cpu_scatter_gather_base_kernel {
           // we change the order of TensorIterator-dim loop
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
-          if (dim== self.dim() - 1) {
+          if (dim== buffer.dim() - 1) {
             for (const auto nelem C10_UNUSED : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
-              _cpu_scatter_gather_dim_loop<is_scatter_like>()(
-                 (scalar_t*)self_data_bytes, self_dim_stride,
+              loop_func.template operator()<scalar_t, ReduceMinimum>(
+                 (opmath_t*)self_data_bytes, self_dim_stride,
                  (int64_t*)index_data_bytes, index_dim_stride,
                  (scalar_t*)src_data_bytes, src_dim_stride,
                  dim, index_dim_size, index_upper_bound,
@@ -558,7 +624,7 @@ struct cpu_scatter_gather_base_kernel {
                             " with size ", index_upper_bound);
 
                 kernel_func(
-                  (scalar_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
+                  (opmath_t*)self_data + (is_scatter_like ? idx_dim : i) * self_dim_stride,
                   (scalar_t*)src_data + (is_scatter_like ? i : idx_dim) * src_dim_stride);
 
                 self_data += strides[SELF_ITER_STRIDE_IDX];
@@ -571,9 +637,30 @@ struct cpu_scatter_gather_base_kernel {
         iter.for_each(loop, grain_size);
       }
     );
+    if (need_acc) {
+      self.copy_(buffer);
+    }
   }
 };
 
+#ifndef USE_FBGEMM
+namespace fbgemm {
+
+template <typename K, typename V>
+std::pair<K*, V*> radix_sort_parallel(
+    K* const inp_key_buf,
+    V* const inp_value_buf,
+    K* const tmp_key_buf,
+    V* const tmp_value_buf,
+    const int64_t elements_count,
+    const int64_t max_value) {
+  TORCH_INTERNAL_ASSERT(false, "radix_sort_parallel: ATen not compiled with FBGEMM support");
+  return std::make_pair(nullptr, nullptr);
+}
+
+}
+#endif
+
 // Note [scatter reduce optimization]
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
@@ -597,11 +684,12 @@ struct cpu_scatter_gather_base_kernel {
 //
 //   step 2: spmm reduce, parallel on M and vectorize on K
 //
+
 template <typename scalar_t, ReductionType reduce>
 void cpu_scatter_reduce_expanded_index(const Tensor& self, const Tensor& index, const Tensor& src, bool include_self) {
-  int64_t* index_data = index.data_ptr<int64_t>();
+  const int64_t* index_data = index.const_data_ptr<int64_t>();
   scalar_t* self_data = self.data_ptr<scalar_t>();
-  scalar_t* src_data = src.data_ptr<scalar_t>();
+  const scalar_t* src_data = src.const_data_ptr<scalar_t>();
 
   const int64_t M = ensure_nonempty_size(self, 0);
   const int64_t nnz = ensure_nonempty_size(index, 0);
@@ -627,7 +715,7 @@ void cpu_scatter_reduce_expanded_index(const Tensor& self, const Tensor& index,
 
   int64_t* sorted_col_index_keys = nullptr;
   int64_t* sorted_col_index_values = nullptr;
-  std::tie(sorted_col_index_keys, sorted_col_index_values) = radix_sort_parallel(
+  std::tie(sorted_col_index_keys, sorted_col_index_values) = fbgemm::radix_sort_parallel(
       keys.get(),
       values.get(),
       keys_tmp.get(),
@@ -674,21 +762,44 @@ void cpu_scatter_reduce_expanded_index(const Tensor& self, const Tensor& index,
     }
   });
 
+  using opmath_t = at::opmath_type<scalar_t>;
+  Tensor buffer;
+  opmath_t* buffer_data = nullptr;
+  static constexpr bool need_acc = is_reduced_floating_point_v<scalar_t>;
+  if constexpr (need_acc) {
+    auto acc_type = at::toAccumulateType(self.scalar_type(), /*is_cuda=*/true);
+    buffer = at::zeros({num_threads, K}, self.options().dtype(acc_type));
+    buffer_data = buffer.data_ptr<opmath_t>();
+  }
+
   // TODO: do blocking on col dimension to reduce WR bandwidth
   at::parallel_for(0, num_nonzero_rows, 1, [&](int64_t begin, int64_t end) {
+    int tid = at::get_thread_num();
+    TORCH_CHECK(tid < num_threads,
+                "expect thread id smaller than ", num_threads, ", got thread id ", tid);
+    opmath_t* buffer_ptr = nullptr;
+
     for (const auto m : c10::irange(begin, end)) {
       int64_t row = row_index[m];
       int64_t off_start = row_index_offset[m];
       int64_t off_end = row_index_offset[m + 1];
       scalar_t* self_ptr = self_data + row * K;
+      if constexpr (need_acc) {
+        buffer_ptr = buffer_data + tid * K;
+      } else {
+        buffer_ptr = reinterpret_cast<opmath_t*>(self_ptr);
+      }
 
       // step 1: reinit rows in `self` if needed
-      init<scalar_t, reduce>(self_ptr, K, include_self);
+      _init<scalar_t, reduce>(self_ptr, buffer_ptr, K, include_self);
 
       // step 2: reduce
       for (const auto n : c10::irange(off_start, off_end)) {
         int64_t col = sorted_col_index_values[n];
-        update<scalar_t, reduce>(self_ptr, src_data + col * K, K);
+        update<scalar_t, reduce>(buffer_ptr, src_data + col * K, K);
+      }
+      if constexpr (need_acc) {
+        vec::convert(buffer_ptr, self_ptr, K);
       }
 
       // step 3: finalize
@@ -701,9 +812,9 @@ void cpu_scatter_reduce_expanded_index(const Tensor& self, const Tensor& index,
 
 template <typename scalar_t>
 void cpu_gather_expanded_index_kernel(const Tensor& result, const Tensor& index, const Tensor& self) {
-  int64_t* index_data = index.data_ptr<int64_t>();
+  const int64_t* index_data = index.const_data_ptr<int64_t>();
   scalar_t* result_data = result.data_ptr<scalar_t>();
-  scalar_t* self_data = self.data_ptr<scalar_t>();
+  const scalar_t* self_data = self.const_data_ptr<scalar_t>();
 
   const int64_t M = ensure_nonempty_size(result, 0);
   const int64_t N = ensure_nonempty_size(self, 0);
@@ -721,7 +832,7 @@ void cpu_gather_expanded_index_kernel(const Tensor& result, const Tensor& index,
                   "index ", index,
                   " is out of bounds for dimension ", 0,
                   " with size ", index_upper_bound);
-      scalar_t* self_ptr = self_data + index * K;
+      const scalar_t* self_ptr = self_data + index * K;
       int64_t d = 0;
       for (; d < K - (K % Vec::size()); d += Vec::size()) {
         Vec out_vec = Vec::loadu(self_ptr + d);
@@ -738,8 +849,8 @@ void cpu_gather_expanded_index_kernel(const Tensor& result, const Tensor& index,
 }
 
 void scatter_add_expanded_index_kernel(const Tensor& self, const Tensor& index, const Tensor& src) {
-  AT_DISPATCH_FLOATING_TYPES_AND(
-    ScalarType::BFloat16, self.scalar_type(), "scatter_add_expanded_index", [&] {
+  AT_DISPATCH_FLOATING_TYPES_AND2(
+    ScalarType::BFloat16, ScalarType::Half, self.scalar_type(), "scatter_add_expanded_index", [&] {
       cpu_scatter_reduce_expanded_index<scalar_t, ReductionType::SUM>(self, index, src, /*include_self*/true);
   });
 }
@@ -747,8 +858,8 @@ void scatter_add_expanded_index_kernel(const Tensor& self, const Tensor& index,
 void scatter_reduce_expanded_index_kernel(
     const Tensor& self, const Tensor& index, const Tensor& src,
     const ReductionType& reduction, bool include_self) {
-  AT_DISPATCH_FLOATING_TYPES_AND(
-    ScalarType::BFloat16, self.scalar_type(), "scatter_reduce_expanded_index", [&] {
+  AT_DISPATCH_FLOATING_TYPES_AND2(
+    ScalarType::BFloat16, ScalarType::Half, self.scalar_type(), "scatter_reduce_expanded_index", [&] {
     AT_DISPATCH_REDUCTION_TYPES(reduction, [&]() {
       cpu_scatter_reduce_expanded_index<scalar_t, reduce>(self, index, src, include_self);
     });
