@@ -24,6 +24,7 @@
 #include <ATen/ops/randperm_native.h>
 #include <ATen/ops/topk.h>
 #include <ATen/ops/uniform_native.h>
+#include <ATen/ops/view_as_real.h>
 #endif
 
 namespace at::native {
@@ -132,11 +133,7 @@ Tensor& random_mps_impl(Tensor& self,
     }
 
     Placeholder outputPlaceholder = Placeholder(cachedGraph->resultTensor, self);
-    NSDictionary<MPSGraphTensor*, MPSGraphTensorData*>* results = @{
-      outputPlaceholder.getMPSGraphTensor() : outputPlaceholder.getMPSGraphTensorData(),
-    };
-
-    runMPSGraph(stream, cachedGraph->graph(), feeds, results);
+    runMPSGraph(stream, cachedGraph->graph(), feeds, outputPlaceholder);
   }
 
   return self;
@@ -175,6 +172,19 @@ static Tensor& normal_mps_impl(Tensor& self,
     }
     return resultTensor;
   };
+  if (c10::isComplexType(self.scalar_type())) {
+    auto real_view = at::view_as_real(self);
+    random_mps_impl<double>(real_view,
+                            mean_s,
+                            std_s,
+                            mean_opt,
+                            std_opt,
+                            MPSGraphRandomDistributionNormal,
+                            gen,
+                            op_name + getTensorsStringKey({mean_t, std_t}),
+                            random_op_block);
+    return self;
+  }
   return random_mps_impl<double>(self,
                                  mean_s,
                                  std_s,
@@ -216,13 +226,18 @@ static Tensor& bernoulli_mps_impl(Tensor& self,
 } // namespace mps
 
 Tensor& uniform_mps_(Tensor& self, double from, double to, c10::optional<Generator> gen) {
-  AT_DISPATCH_FLOATING_TYPES_AND_HALF(self.scalar_type(), "check_uniform_bounds", [&] {
+  auto scalar_type = self.scalar_type();
+  if (scalar_type == ScalarType::ComplexFloat)
+    scalar_type = ScalarType::Float;
+  else if (scalar_type == ScalarType::ComplexHalf)
+    scalar_type = ScalarType::Half;
+  AT_DISPATCH_FLOATING_TYPES_AND2(kHalf, kBFloat16, scalar_type, "check_uniform_bounds", [&] {
     const auto min = static_cast<double>(std::numeric_limits<scalar_t>::lowest());
     const auto max = static_cast<double>(std::numeric_limits<scalar_t>::max());
     TORCH_CHECK(from <= to, "uniform_ expects to return a [from, to) range, but found from=", from, " > to=", to);
     TORCH_CHECK((to - from) <= std::numeric_limits<scalar_t>::max(),
                 "uniform_ expects to-from <= std::numeric_limits<",
-                toString(self.scalar_type()),
+                toString(scalar_type),
                 ">::max(), but found to=",
                 to,
                 " and from=",
@@ -232,6 +247,12 @@ Tensor& uniform_mps_(Tensor& self, double from, double to, c10::optional<Generat
     to = std::max(std::min(to, max), min);
   });
 
+  if (c10::isComplexType(self.scalar_type())) {
+    auto real_view = at::view_as_real(self);
+    mps::random_mps_impl<double>(
+        real_view, from, to, c10::nullopt, c10::nullopt, MPSGraphRandomDistributionUniform, gen, __func__, nullptr);
+    return self;
+  }
   return mps::random_mps_impl<double>(
       self, from, to, c10::nullopt, c10::nullopt, MPSGraphRandomDistributionUniform, gen, __func__, nullptr);
 }
@@ -300,17 +321,15 @@ Tensor& random_mps_(Tensor& self, int64_t from, c10::optional<int64_t> to_opt, c
     to = *to_opt;
     TORCH_CHECK(from < to, "random_mps_ expects 'from' to be less than 'to', but got from=", from, " >= to=", to);
     if (isFloatingType(input_dtype)) {
-      AT_DISPATCH_FLOATING_TYPES_AND2(
-          at::ScalarType::Half, at::ScalarType::BFloat16, input_dtype, "random_update_from_to", [&] {
-            from = templates::update_from<scalar_t>(from);
-            to = templates::update_to<scalar_t>(to);
-            TORCH_CHECK(
-                from < to,
-                "random_mps_ expects 'from' casted to dtype to be less than 'to' casted to dtype, but got from=",
-                from,
-                " >= to=",
-                to);
-          });
+      AT_DISPATCH_FLOATING_TYPES_AND2(kHalf, kBFloat16, input_dtype, "random_update_from_to", [&] {
+        from = templates::update_from<scalar_t>(from);
+        to = templates::update_to<scalar_t>(to);
+        TORCH_CHECK(from < to,
+                    "random_mps_ expects 'from' casted to dtype to be less than 'to' casted to dtype, but got from=",
+                    from,
+                    " >= to=",
+                    to);
+      });
       templates::check_from_to_in_range(from, to - 1, self.dtype());
     }
   } else if (from != std::numeric_limits<int64_t>::lowest()) {
@@ -550,10 +569,7 @@ static Tensor& multinomial_with_replacement_mps_kernel(const Tensor& self,
       cachedGraph->stateTensor : stateTensorData,
       probPlaceholder.getMPSGraphTensor() : probPlaceholder.getMPSGraphTensorData()
     };
-    NSDictionary<MPSGraphTensor*, MPSGraphTensorData*>* results =
-        @{outputPlaceholder.getMPSGraphTensor() : outputPlaceholder.getMPSGraphTensorData()};
-
-    runMPSGraph(stream, cachedGraph->graph(), feeds, results);
+    runMPSGraph(stream, cachedGraph->graph(), feeds, outputPlaceholder);
   }
 
   return result;
