@@ -1,15 +1,30 @@
-#include <ATen/ATen.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
+#include <ATen/Context.h>
+#include <ATen/Dispatch.h>
+#include <ATen/ExpandUtils.h>
 #include <torch/library.h>
-#include <ATen/cpu/vec/vec.h>
-#include <ATen/native/TensorIterator.h>
-#include <ATen/native/cpu/Loops.h>
 #include <ATen/quantized/Quantizer.h>
+#include <ATen/native/quantized/cpu/BinaryOps.h>
 #include <ATen/native/quantized/cpu/QuantizedOps.h>
 #include <ATen/native/quantized/cpu/init_qnnpack.h>
 #include <ATen/native/quantized/cpu/QnnpackUtils.h>
 #include <ATen/native/quantized/cpu/XnnpackUtils.h>
 #include <caffe2/utils/threadpool/pthreadpool-cpp.h>
 
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/_empty_affine_quantized.h>
+#include <ATen/ops/_empty_affine_quantized_native.h>
+#include <ATen/ops/empty_like.h>
+#include <ATen/ops/relu_native.h>
+#endif
+
+#include <algorithm>
+#include <utility>
+
 namespace at {
 namespace native {
 
@@ -23,10 +38,10 @@ namespace {
 inline void check_inputs(const Tensor& qa, const Tensor& qb) {
   TORCH_CHECK(
       qa.qscheme() == kPerTensorAffine,
-      "Only per tensor quantization is suported in Add.");
+      "Only per tensor quantization is supported in Add.");
   TORCH_CHECK(
       qa.qscheme() == qb.qscheme(),
-      "Both inputs to Add must have the same quantization shceme.");
+      "Both inputs to Add must have the same quantization scheme.");
   TORCH_CHECK(
       qa.scalar_type() == qb.scalar_type(),
       "Add operands should have same data type.");
@@ -245,24 +260,32 @@ enum xnn_status xnnp_create_add_nd(
 }
 
 C10_ALWAYS_INLINE
-enum xnn_status xnnp_setup_add_nd(
+enum xnn_status xnnp_reshape_add_nd(
     xnn_operator_t op,
     const std::vector<size_t>& a_shape,
     const std::vector<size_t>& b_shape,
+    pthreadpool_t pt_pool) {
+  return xnn_reshape_add_nd_qs8(
+      op,             /* xnn_operator_t add_op      */
+      a_shape.size(), /* size_t num_input1_dims     */
+      a_shape.data(), /* const size_t* input1_shape */
+      b_shape.size(), /* size_t num_input2_dims     */
+      b_shape.data(), /* const size_t* input2_shape */
+      pt_pool);       /* pthreadpool_t threadpool   */
+}
+
+C10_ALWAYS_INLINE
+enum xnn_status xnnp_setup_add_nd(
+    xnn_operator_t op,
     const int8_t* da,
     const int8_t* db,
     int8_t* dc,
     pthreadpool_t pt_pool) {
   return xnn_setup_add_nd_qs8(
       op,             /* xnn_operator_t add_op      */
-      a_shape.size(), /* size_t num_input1_dims     */
-      a_shape.data(), /* const size_t* input1_shape */
-      b_shape.size(), /* size_t num_input2_dims     */
-      b_shape.data(), /* const size_t* input2_shape */
       da,             /* const int8_t* input1       */
       db,             /* const int8_t* input2       */
-      dc,             /* int8_t* output             */
-      pt_pool);       /* pthreadpool_t threadpool   */
+      dc);            /* int8_t* output             */
 }
 
 template <typename scalar_t, bool ReLUFused = false>
@@ -304,8 +327,8 @@ Tensor xnnp_add(Tensor qa, Tensor qb, double scale, int64_t zero_point) {
   auto output_min = std::numeric_limits<underlying_t>::min();
   if (ReLUFused) {
     /*
-     * FIXME: use acticationLimits<T>()
-     * With <T>, MSVC runs into "error C3862: indetifier activationLimits not found".
+     * FIXME: use activationLimits<T>()
+     * With <T>, MSVC runs into "error C3862: identifier activationLimits not found".
      */
     constexpr int64_t qmin = std::numeric_limits<underlying_t>::min();
     constexpr int64_t qmax = std::numeric_limits<underlying_t>::max();
@@ -334,11 +357,20 @@ Tensor xnnp_add(Tensor qa, Tensor qb, double scale, int64_t zero_point) {
   const auto qa_shape = xnnp_utils::get_mem_format_aware_shape(qa_contig);
   const auto qb_shape = xnnp_utils::get_mem_format_aware_shape(qb_contig);
 
-  // Setup the operator
-  status = xnnp_setup_add_nd(
+  // Reshape the operator
+  status = xnnp_reshape_add_nd(
       xnnp_add_operator.get(),
       qa_shape,
       qb_shape,
+      caffe2::pthreadpool_());
+
+  TORCH_CHECK(
+      status == xnn_status_success,
+      func_name, ": xnn reshape operator failed(", status,")!");
+
+  // Setup the operator
+  status = xnnp_setup_add_nd(
+      xnnp_add_operator.get(),
       reinterpret_cast<const underlying_t*>(qa_contig.data_ptr<scalar_t>()),
       reinterpret_cast<const underlying_t*>(qb_contig.data_ptr<scalar_t>()),
       reinterpret_cast<underlying_t*>(qy.data_ptr<scalar_t>()),
@@ -374,7 +406,7 @@ Tensor qadd(Tensor qa, Tensor qb, double scale, int64_t zero_point) {
 #endif // USE_XNNPACK
 
 #ifdef USE_PYTORCH_QNNPACK
-    if(qa.sizes() == qb.sizes() && /* qnnpack does not support boradcasting */
+    if(qa.sizes() == qb.sizes() && /* qnnpack does not support boardcasting */
       qa.scalar_type() == kQUInt8) {
     return qnnpack_add<ReLUFused>(qa, qb, scale, zero_point);
     }
@@ -428,7 +460,7 @@ Tensor qadd_scalar_out(Tensor qa, const Scalar& b, Tensor out) {
 // all variations of `quantized::add` is merged into `quantized::add`
 template <bool ReLUFused = false>
 Tensor qadd_scalar_tensor(Tensor qa, Tensor b) {
-  return qadd_scalar(qa, b.item());
+  return qadd_scalar(std::move(qa), b.item());
 }
 
 // `torch.jit.trace` will trace Scalar as Tensor
@@ -436,7 +468,7 @@ Tensor qadd_scalar_tensor(Tensor qa, Tensor b) {
 // all variations of `quantized::add` is merged into `quantized::add`
 template <bool ReLUFused = false>
 Tensor qadd_scalar_tensor_out(Tensor qa, Tensor b, Tensor out) {
-  return qadd_scalar_out(qa, b.item(), out);
+  return qadd_scalar_out(std::move(qa), b.item(), std::move(out));
 }
 
 TORCH_LIBRARY_IMPL(quantized, QuantizedCPU, m) {
@@ -470,7 +502,7 @@ TORCH_LIBRARY_IMPL(_quantized, QuantizedCPU, m) {
 }  // namespace
 
 Tensor quantized_add(Tensor qa, Tensor qb, double scale, int64_t zero_point){
-  return qadd<false>(qa, qb, scale, zero_point);
+  return qadd<false>(std::move(qa), std::move(qb), scale, zero_point);
 }
 
 }}  // namespace at::native
