@@ -35,6 +35,7 @@
 #endif
 
 #include <c10/util/irange.h>
+#include <c10/util/string_utils.h>
 
 namespace {
 // To have a sanity check for maximum matrix size.
@@ -646,7 +647,7 @@ at::Tensor PackedConvWeightsQnnp<kSpatialDim>::apply_impl_xnnp(
     // create an empty tensor for packing the weights
     const at::Tensor weight_contig =
         orig_weight.contiguous(c10::MemoryFormat::ChannelsLast);
-    const float* w_scales_data = w_scales.data_ptr<float>();
+    const float* w_scales_data = w_scales.const_data_ptr<float>();
     underlying_t w_zp = 0;
     at::Tensor weight_tensor;
 
@@ -769,14 +770,12 @@ at::Tensor PackedConvWeightsQnnp<kSpatialDim>::apply_impl_xnnp(
       output_zero_point,
       c10::MemoryFormat::ChannelsLast);
 
-  // Setup the operator
-  status = at::native::xnnp_utils::xnnp_setup_convolution2d_nhwc(
+  // Reshape the operator
+  status = at::native::xnnp_utils::xnnp_reshape_convolution2d_nhwc(
       xnnp_convolution_op.get(),
       N,
       H,
       W,
-      reinterpret_cast<const underlying_t*>(act_nhwc.template data_ptr<scalar_t>()),
-      reinterpret_cast<underlying_t*>(output.template data_ptr<scalar_t>()),
       caffe2::pthreadpool_(),
       per_channel(),
       transpose(),
@@ -790,6 +789,21 @@ at::Tensor PackedConvWeightsQnnp<kSpatialDim>::apply_impl_xnnp(
       status,
       ")");
 
+  // Setup the operator
+  status = at::native::xnnp_utils::xnnp_setup_convolution2d_nhwc(
+      xnnp_convolution_op.get(),
+      reinterpret_cast<const underlying_t*>(act_nhwc.template data_ptr<scalar_t>()),
+      reinterpret_cast<underlying_t*>(output.template data_ptr<scalar_t>()),
+      per_channel(),
+      transpose());
+
+  TORCH_CHECK(
+      status == xnn_status_success,
+      func_name,
+      ": xnn setup operator failed(",
+      status,
+      ")");
+
   // Run the operator
   status = xnn_run_operator(
       xnnp_convolution_op.get(), /* xnn_operator_t op */
@@ -1138,7 +1152,7 @@ template <int kSpatialDim>
 template <bool kReluFused>
 at::Tensor PackedConvWeightsOnednn<kSpatialDim>::apply_impl(
     const at::Tensor& act,
-    const c10::optional<at::Tensor>& accum,
+    const std::optional<at::Tensor>& accum,
     double output_scale,
     int64_t output_zero_point) {
   std::string func_name = "quantized::conv";
@@ -1377,23 +1391,23 @@ static at::Tensor _quantized_convolution_onednn(
     at::Tensor weight, // MKLDNN tensor with quantized values
     at::Tensor weight_scales,
     at::Tensor weight_zero_points,
-    c10::optional<at::Tensor> bias, // Bias is not packed into MKLDNN tensor
+    std::optional<at::Tensor> bias, // Bias is not packed into MKLDNN tensor
     torch::List<int64_t> stride,
     torch::List<int64_t> padding,
     torch::List<int64_t> dilation,
     bool transposed,
     int64_t groups,
-    double inv_output_scale,  // inv_output_scale is the reciprocal of scale in fake quant
+    double output_scale,
     int64_t output_zero_point,
-    c10::optional<at::Tensor> accum, // accum to fused with conv add
+    std::optional<at::Tensor> accum, // accum to fused with conv add
     double accum_scale,
     int64_t accum_zero_point,
-    bool fp32_output,
-    c10::optional<c10::string_view> binary_attr,
-    c10::optional<at::Scalar> binary_alpha,
-    c10::optional<c10::string_view> unary_attr,
-    torch::List<c10::optional<at::Scalar>> unary_scalars,
-    c10::optional<c10::string_view> unary_algorithm) {
+    std::optional<c10::ScalarType> output_dtype,
+    std::optional<c10::string_view> binary_attr,
+    std::optional<at::Scalar> binary_alpha,
+    std::optional<c10::string_view> unary_attr,
+    torch::List<std::optional<at::Scalar>> unary_scalars,
+    std::optional<c10::string_view> unary_algorithm) {
   /*********************************/
   /*          Checks               */
   /*********************************/
@@ -1402,13 +1416,15 @@ static at::Tensor _quantized_convolution_onednn(
   // inv_scale = 1.0 / scale will be folded.
   // So, we can only get inv_scale from quant node which is used as
   // output_scale of this op.
-  if (fp32_output) {
-    // When fp32_output, oneDNN expects op_attr doesn't set_scales and set_zero_points.
-    // So, we will use default inv_output_scale as 1.0 and output_zero_point as 0, since
-    // when inv_output_scale is 1.0, we will skip invoking of op_attr.set_scales in ideep;
+  bool fp32_output = output_dtype.has_value() && (output_dtype.value() == c10::kFloat);
+  bool bfloat16_output = output_dtype.has_value() && (output_dtype.value() == c10::kBFloat16);
+  if (fp32_output || bfloat16_output) {
+    // When fp32 or bf16 output, oneDNN expects op_attr doesn't set_scales and set_zero_points.
+    // So, we will use default output_scale as 1.0 and output_zero_point as 0, since
+    // when output_scale is 1.0, we will skip invoking of op_attr.set_scales in ideep;
     // when output_zero_point is 0, we will skip invoking of op_attr.set_zero_points in ideep.
-    TORCH_CHECK(inv_output_scale == 1.0,  " (ONEDNN): fp32 output, inv_output_scale must be 1.0.");
-    TORCH_CHECK(output_zero_point == 0,  " (ONEDNN): fp32 output, output_zero_point must be 0");
+    TORCH_CHECK(output_scale == 1.0,  " (ONEDNN): fp32 or bf16 output, output_scale must be 1.0.");
+    TORCH_CHECK(output_zero_point == 0,  " (ONEDNN): fp32 or bf16 output, output_zero_point must be 0");
   }
 
   int kSpatialDim = act.dim() - 2;
@@ -1416,8 +1432,26 @@ static at::Tensor _quantized_convolution_onednn(
 
   bool has_binary_post_op = binary_attr.has_value() && binary_attr.value() != "none";
   bool has_unary_post_op = unary_attr.has_value() && unary_attr.value() != "none";
-  // has_accum_postop_sum: extra input besides the conv to do conv add fusion with post op sum.
-  bool has_accum_postop_sum = has_binary_post_op && binary_attr.value() == "add" && !fp32_output;
+  // has_accum_postop_sum: extra input besides the conv to do conv post op sum fusion.
+  bool has_accum_postop_sum = has_binary_post_op && binary_attr.value() == "sum";
+
+  if (has_accum_postop_sum) {
+    TORCH_CHECK(accum.has_value(), "For post op sum, accum tensor should not be empty.");
+    TORCH_CHECK(
+      accum.value().is_contiguous(
+        kSpatialDim == 2
+        ? c10::MemoryFormat::ChannelsLast
+        : c10::MemoryFormat::ChannelsLast3d
+      ),
+      "For post op sum, accum tensor must be contiguous."
+    );
+    if (fp32_output || bfloat16_output) {
+      TORCH_CHECK(accum_scale == 1.0,  " (ONEDNN): fp32 or bf16 output, accum_scale must be 1.0.");
+      TORCH_CHECK(accum_zero_point == 0,  " (ONEDNN): fp32 or bf16 output, accum_zero_point must be 0");
+      TORCH_CHECK((accum.value().scalar_type() == c10::kFloat) || (accum.value().scalar_type() == c10::kBFloat16), "The accum tensor should be KFloat or KBFloat.");
+    }
+  }
+
   std::string func_name = "quantized::packed_weights_conv";
   func_name += std::to_string(kSpatialDim) + "d";
   if (has_binary_post_op) {
@@ -1470,10 +1504,24 @@ static at::Tensor _quantized_convolution_onednn(
     kSpatialDim, "D convolution.");
 
   // Parameters
-  // Scales of ONEDNN and PyTorch are reciprocal
-  const ideep::scale_t& src_scales = ideep::scale_t(1, 1.0 / act_scale);
-
-#if defined(IDEEP_VERSION_MAJOR) && IDEEP_VERSION_MAJOR>=3 && defined(IDEEP_VERSION_REVISION) && IDEEP_VERSION_REVISION == 0
+#if IDEEP_PREREQ(3, 1, 0, 1)
+  // 1. If the weight scale generated by observer should with dtype float32
+  // https://github.com/pytorch/pytorch/blob/d2c24eca8a60c56b31ca967a44d5cc4522802aa6/torch/ao/quantization/observer.py#L323
+  // 2. If the weight scale got from the quantized tensor, like did in the UT. It's with dtype of double.
+  // https://github.com/pytorch/pytorch/blob/d2fa3f608b5e4f582a8aaf752f10efe4ca72a7d0/aten/src/ATen/quantized/Quantizer.cpp#L69
+  TORCH_CHECK(
+    weight_scales.scalar_type() == c10::ScalarType::Double || weight_scales.scalar_type() == c10::ScalarType::Float,
+    "weight_scales should be with data type Double or float");
+  if (weight_scales.scalar_type() == c10::ScalarType::Double) {
+    // For case 2, we will convert it from double to float, since ideep::scale_t is alias of std::vector<float>
+    weight_scales = weight_scales.to(c10::ScalarType::Float);
+  }
+  TORCH_CHECK(
+    weight_scales.ndimension() == 0 ||
+    (weight_scales.strides().size() == 1 || weight_scales.stride(0) == 1),
+    "weight_scales should be scalar tensor or contiguous 1D tensor.");
+  ideep::scale_t weights_scales(weight_scales.data_ptr<float>(), weight_scales.data_ptr<float>()+weight_scales.numel());
+#elif IDEEP_PREREQ(3, 1, 0, 0)
   // TODO (leslie): optimize the performance here:
   // 1. Remove the reciprocal of weight scale, we have done the reciprocal of weight scale back in Ideep:
   // https://github.com/intel/ideep/blob/3c90e365526e19c110371d23831678a7e9d4353d/include/ideep/operators/conv.hpp#L163-L168
@@ -1492,23 +1540,6 @@ static at::Tensor _quantized_convolution_onednn(
       weights_scales[i] = 1.0 / weight_scales[i].item().toDouble();
     }
   }
-#elif defined(IDEEP_VERSION_MAJOR) && IDEEP_VERSION_MAJOR>=3 && defined(IDEEP_VERSION_REVISION) && IDEEP_VERSION_REVISION > 0
-  // 1. If the weight scale generated by observer should with dtype float32
-  // https://github.com/pytorch/pytorch/blob/d2c24eca8a60c56b31ca967a44d5cc4522802aa6/torch/ao/quantization/observer.py#L323
-  // 2. If the weight scale got from the quantized tensor, like did in the UT. It's with dtype of double.
-  // https://github.com/pytorch/pytorch/blob/d2fa3f608b5e4f582a8aaf752f10efe4ca72a7d0/aten/src/ATen/quantized/Quantizer.cpp#L69
-  TORCH_CHECK(
-    weight_scales.scalar_type() == c10::ScalarType::Double || weight_scales.scalar_type() == c10::ScalarType::Float,
-    "weight_scales should be with data type Double or float");
-  if (weight_scales.scalar_type() == c10::ScalarType::Double) {
-    // For case 2, we will convert it from double to float, since ideep::scale_t is alias of std::vector<float>
-    weight_scales = weight_scales.to(c10::ScalarType::Float);
-  }
-  TORCH_CHECK(
-    weight_scales.ndimension() == 0 ||
-    (weight_scales.strides().size() == 1 || weight_scales.stride(0) == 1),
-    "weight_scales should be scalar tensor or contiguous 1D tensor.");
-  ideep::scale_t weights_scales(weight_scales.data_ptr<float>(), weight_scales.data_ptr<float>()+weight_scales.numel());
 #else
   TORCH_CHECK(false, "Unexpected IDeep version to do qconv calculation.");
 #endif
@@ -1523,14 +1554,17 @@ static at::Tensor _quantized_convolution_onednn(
   ideep::tensor onednn_bias;
   const int output_channels = weight.size(0);
   bool with_bias = bias.has_value();
+
+  at::Tensor bias_val_float;
   if (with_bias) {
-    at::Tensor bias_val = bias.value();
-    TORCH_CHECK(bias_val.dim() == 1, "bias should be a vector (1D Tensor)");
+    // For int8-mixed-bf16, we will also use float32 bias
+    bias_val_float = bias.value().to(at::kFloat);
+    TORCH_CHECK(bias_val_float.dim() == 1, "bias should be a vector (1D Tensor)");
     TORCH_CHECK(
-        bias_val.size(0) == output_channels,
+        bias_val_float.size(0) == output_channels,
         "bias should have K elements: " + std::to_string(output_channels));
-    auto bias_desc = ideep::tensor::desc(bias.value().sizes().vec(), dnnl::memory::data_type::f32);
-    onednn_bias.init(bias_desc, bias.value().data_ptr());
+    auto bias_desc = ideep::tensor::desc(bias_val_float.sizes().vec(), dnnl::memory::data_type::f32);
+    onednn_bias.init(bias_desc, bias_val_float.data_ptr());
   }
 
   const auto& expected_bias = with_bias ? onednn_bias : ideep::tensor();
@@ -1555,60 +1589,121 @@ static at::Tensor _quantized_convolution_onednn(
   output_sizes = at::native::conv_output_size(input_size, kernel_size, padding.vec(), stride.vec(), dilation.vec());
   ideep::dims dst_dims = ideep::dims({output_sizes.cbegin(), output_sizes.cend()});
   // Output is not a quantized tensor but data type is uint8
-  at::Tensor output;
-  if (fp32_output) {
-    output = at::empty(
+  at::Tensor output = has_accum_postop_sum ?
+    accum.value() :
+    at::empty(
       dst_dims,
       device(c10::kCPU)
-        .dtype(c10::kFloat)
-        .memory_format(kSpatialDim == 2 ?
-            c10::MemoryFormat::ChannelsLast :
-            c10::MemoryFormat::ChannelsLast3d),
-      c10::nullopt);
-  } else {
-    output = at::empty(
-      dst_dims,
-      device(c10::kCPU)
-          .dtype(c10::kByte)
+          .dtype(fp32_output ? c10::kFloat : (bfloat16_output ? c10::kBFloat16 : c10::kByte))
           .memory_format(kSpatialDim == 2 ?
               c10::MemoryFormat::ChannelsLast :
               c10::MemoryFormat::ChannelsLast3d)
     );
-  }
   if (output.numel() == 0) {
     return output;
   }
-  ideep::tensor dst;
-  at::Tensor accum_contig;
-  if (has_accum_postop_sum) {
-    auto dst_desc = ideep::tensor::desc(dst_dims, src_data_type,
-        kSpatialDim == 2 ? ideep::format_tag::nhwc : ideep::format_tag::ndhwc);
-    accum_contig = accum.value().contiguous(kSpatialDim == 2 ? c10::MemoryFormat::ChannelsLast : c10::MemoryFormat::ChannelsLast3d);
-    TORCH_CHECK(accum_contig.dtype() == output.dtype(), "The output tensor should have same dtype as the accum tensor.");
-    // When fused with sum, the dst tensor will share the data ptr as the accum tensor.
-    dst.init(dst_desc, accum_contig.data_ptr());
-  } else {
-    if (fp32_output) {
-      // Conv without add: int8-in, fp32-output
-      dst = ideep::tensor({dst_dims, ideep::tensor::data_type::f32, {output.strides().cbegin(), output.strides().cend()}},
-                        output.data_ptr());
-    } else {
-      dst = ideep::tensor({dst_dims, ideep::tensor::data_type::u8, {output.strides().cbegin(), output.strides().cend()}},
-                        output.data_ptr());
-    }
+  ideep::tensor dst = at::native::itensor_view_from_dense(output);
+  static ideep::tensor::desc dummy_accum_desc;
+  ideep::attr_t op_attr = onednn_utils::create_attr_by_post_op(
+    binary_attr.has_value() ? binary_attr.value() : "none",
+    binary_alpha.has_value() ? binary_alpha.value().to<double>() : 1.0,
+    accum_scale,
+    accum_zero_point,
+    dummy_accum_desc,
+    unary_attr.has_value() ? unary_attr.value() : "none",
+    unary_scalars,
+    unary_algorithm.has_value() ? unary_algorithm.value() : ""
+  );
+
+#if IDEEP_PREREQ(3, 1, 0, 0)
+  // Use oneDNN's APIs instead of prepare/compute from ideep to reduce integration overhead.
+  // The functions from ideep are heavy because they have complex data structures for unified API
+  // oneDNN version >= 3.1.0 is required.
+  using ideep::tensor;
+  auto weight_grouped = packed_weight.make_grouped_weights(groups, /* is_deconv */false);
+  auto weights_desc = tensor::desc(weight_grouped.get_dims(), ideep::data_type::s8, ideep::format_tag::any);
+  if (groups > 1) {
+    weights_desc = weights_desc.to_grouped(groups);
   }
-  ideep::attr_t op_attr;
-  // attr
+  auto dst_desc = dst.get_desc();
+  auto bias_desc = with_bias ?
+      tensor::desc(expected_bias.get_dims(), ideep::data_type::f32, ideep::format_tag::any) :
+      tensor::desc();
+  if (act_scale != 1.0f) {
+    op_attr.set_scales_mask(DNNL_ARG_SRC, 0);
+  }
+  if (act_zero_point != 0) {
+    op_attr.set_zero_points_mask(DNNL_ARG_SRC, 0);
+  }
+  int oc_per_group = weight_grouped.get_dim(0) / groups;
+  int wei_scale_mask = ideep::utils::conv_weight_scale_mask(weight_scales.numel(), oc_per_group, groups, false);
+  op_attr.set_scales_mask(DNNL_ARG_WEIGHTS, wei_scale_mask);
+  if (output_scale != 1.0f) {
+    op_attr.set_scales_mask(DNNL_ARG_DST, 0);
+  }
+  if (output_zero_point != 0) {
+    op_attr.set_zero_points_mask(DNNL_ARG_DST, 0);
+  }
+  op_attr.set_scratchpad_mode(dnnl::scratchpad_mode::user);
+  auto engine = ideep::engine::cpu_engine();
+  auto dilates_dnnl = ideep::utils::get_compatible_dilates(dilation.vec());
+  auto primitive_desc = with_bias ?
+      dnnl::convolution_forward::primitive_desc(
+        engine, dnnl::prop_kind::forward_inference, dnnl::algorithm::convolution_direct,
+        src_desc, weights_desc, bias_desc, dst_desc,
+        stride.vec(), dilates_dnnl, padding.vec(), padding.vec(), op_attr
+      ) :
+      dnnl::convolution_forward::primitive_desc(
+        engine, dnnl::prop_kind::forward_inference, dnnl::algorithm::convolution_direct,
+        src_desc, weights_desc, dst_desc,
+        stride.vec(), dilates_dnnl, padding.vec(), padding.vec(), op_attr
+      );
+  auto primitive = dnnl::convolution_forward(primitive_desc);
+
+  // Reorder weight if needed
+  auto expected_weight = weight_grouped.reorder_if_differ_in(primitive_desc.weights_desc());
+
+  // Prepare args and execute primitive
+  tensor scratchpad(primitive_desc.scratchpad_desc());
+  ideep::exec_args args;
+  args.insert({DNNL_ARG_SRC, src});
+  args.insert({DNNL_ARG_WEIGHTS, expected_weight});
+  args.insert({DNNL_ARG_DST, dst});
+  args.insert({DNNL_ARG_SCRATCHPAD, scratchpad});
+  if (with_bias) {
+    args.insert({DNNL_ARG_BIAS, expected_bias});
+  }
+  tensor src_scales_t = tensor(ideep::scale_t(1, act_scale));
+  tensor wei_scales_t = tensor(weights_scales);
+  tensor dst_scales_t = tensor(ideep::scale_t(1, output_scale));
+  tensor src_zp_t = tensor(ideep::zero_point_t(1, act_zero_point));
+  tensor dst_zp_t = tensor(ideep::zero_point_t(1, output_zero_point));
+  if (act_scale != 1.0f) {
+    args.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC, src_scales_t});
+  }
+  if (output_scale != 1.0f) {
+    args.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST, dst_scales_t});
+  }
+  args.insert({DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS, wei_scales_t});
+  if (act_zero_point != 0) {
+    args.insert({DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC, src_zp_t});
+  }
+  if (output_zero_point != 0) {
+    args.insert({DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST, dst_zp_t});
+  }
+  primitive.execute(ideep::stream::default_stream(), args);
+#else
+  // Scales of ONEDNN and PyTorch are reciprocal
+  const ideep::scale_t& src_scales = ideep::scale_t(1, 1.0 / act_scale);
+
+  // set accum scale/zero point to dst
   if (has_accum_postop_sum) {
-    op_attr = (has_unary_post_op && unary_attr.value()=="relu") ? ideep::attr_t::residual_with_sum_zero_point() : ideep::attr_t::fuse_sum();
     const ideep::scale_t accum_ideep_scale = ideep::scale_t(1, 1.0/accum_scale);
     const ideep::zero_point_t accum_ideep_zero_points = ideep::zero_point_t(1, accum_zero_point);
     // Set the dst scale and zero point with the value of accum.
-    // The true scale and zero point is stored in ideep::scale_t(scale_size, inv_output_scale) and dst_zero_points.
+    // The true scale and zero point is stored in ideep::scale_t(scale_size, output_scale) and dst_zero_points.
     dst.set_scale(accum_ideep_scale);
     dst.set_zero_point(accum_ideep_zero_points);
-  } else {
-    op_attr = (has_unary_post_op && unary_attr.value()=="relu") ? ideep::attr_t::fuse_relu() : ideep::attr_t();
   }
 
   // Weight Reorder
@@ -1616,7 +1711,7 @@ static at::Tensor _quantized_convolution_onednn(
   ideep::convolution_forward::prepare(
       params, src, packed_weight, expected_bias, dst_dims, dst,
       stride.vec(), dilation.vec(), padding.vec(), padding.vec(), groups,
-      src_scales, weights_scales, ideep::scale_t(1, inv_output_scale),
+      src_scales, weights_scales, ideep::scale_t(1, 1.0f / output_scale),
       src_zero_points, dst_zero_points,
       op_attr, dnnl::algorithm::convolution_direct,
       dnnl::prop_kind::forward_inference,
@@ -1626,13 +1721,14 @@ static at::Tensor _quantized_convolution_onednn(
 
   // Computation
   ideep::convolution_forward::compute<false, false>(params, src, expected_weight, expected_bias, dst);
+#endif
 
   if (is_1d) {
     output.squeeze_(quant_utils::kConv1dSqueezeDim + 2);
     return output;
   }
   if (has_accum_postop_sum) {
-    return accum_contig;
+    return accum.value();
   } else {
     return output;
   }
@@ -1775,17 +1871,17 @@ class QConvoneDNN final {
       at::Tensor weight, // contains quantized values but not QTensor
       at::Tensor weight_scales,
       at::Tensor weight_zero_points,
-      c10::optional<at::Tensor> bias,
+      std::optional<at::Tensor> bias,
       torch::List<int64_t> stride,
       torch::List<int64_t> padding,
       torch::List<int64_t> dilation,
       int64_t groups,
-      double inv_output_scale,  // inv_output_scale is the reciprocal of scale in fake quant
+      double output_scale,
       int64_t output_zero_point,
-      bool fp32_output,
+      std::optional<c10::ScalarType> output_dtype,
       c10::string_view attr,
-      torch::List<c10::optional<at::Scalar>> scalars,
-      c10::optional<c10::string_view> algorithm) {
+      torch::List<std::optional<at::Scalar>> scalars,
+      std::optional<c10::string_view> algorithm) {
 #if AT_MKLDNN_ENABLED()
     if (act.dim() == 3 || act.dim() == 5) {
       // Conv1D/3D post op check
@@ -1799,8 +1895,8 @@ class QConvoneDNN final {
     } else {
       // Conv2D post op check
       TORCH_CHECK(
-        attr == "none" || attr == "relu",
-        "none post_op or post_op relu is supported for quantized pointwise conv2d. Got unary_post_op: ",
+        attr == "none" || attr == "relu" || attr == "hardtanh" || attr == "hardswish" || attr == "swish",
+        "none post_op or post_op relu/hardtanh/hardswish is supported for quantized pointwise conv2d. Got unary_post_op: ",
         attr,
         ".")
     }
@@ -1808,9 +1904,9 @@ class QConvoneDNN final {
         act, act_scale, act_zero_point,
         weight, weight_scales, weight_zero_points,
         bias, stride, padding, dilation, /*transposed*/false,
-        groups, inv_output_scale, output_zero_point,
+        groups, output_scale, output_zero_point,
         /*accum*/c10::nullopt, /*accum_scale*/0.0, /*accum_zero_point*/0,
-        /*fp32_output*/fp32_output, /*binary_attr*/c10::nullopt, /*binary_alpha*/c10::nullopt,
+        /*output_dtype*/output_dtype, /*binary_attr*/c10::nullopt, /*binary_alpha*/c10::nullopt,
         /*unary_attr*/attr, /*unary_scalars*/scalars, /*unary_algorithm*/algorithm
     );
 #else
@@ -1827,23 +1923,23 @@ class QConvoneDNN final {
       at::Tensor weight, // contains quantized values but not QTensor
       at::Tensor weight_scales,
       at::Tensor weight_zero_points,
-      c10::optional<at::Tensor> bias,
+      std::optional<at::Tensor> bias,
       torch::List<int64_t> stride,
       torch::List<int64_t> padding,
       torch::List<int64_t> dilation,
       int64_t groups,
-      double inv_output_scale,  // inv_output_scale is the reciprocal of scale in fake quant
+      double output_scale,
       int64_t output_zero_point,
-      bool fp32_output,
+      std::optional<c10::ScalarType> output_dtype,
       c10::string_view binary_attr,
-      c10::optional<at::Scalar> alpha,
-      c10::optional<c10::string_view> unary_attr,
-      torch::List<c10::optional<at::Scalar>> unary_scalars,
-      c10::optional<c10::string_view> unary_algorithm) {
+      std::optional<at::Scalar> alpha,
+      std::optional<c10::string_view> unary_attr,
+      torch::List<std::optional<at::Scalar>> unary_scalars,
+      std::optional<c10::string_view> unary_algorithm) {
 #if AT_MKLDNN_ENABLED()
     // Conv2D post op check
     TORCH_CHECK(
-      act.dim() == 4 && binary_attr == "add" && (
+      act.dim() == 4 && binary_attr == "sum" && (
         !unary_attr.has_value() ||
         (unary_attr.has_value() &&
           (
@@ -1851,7 +1947,7 @@ class QConvoneDNN final {
           )
         )
       ),
-      "post_op add or post_op add_relu is supported for quantized pointwise conv2d. Got binary_post_op: ",
+      "post_op sum or post_op sum_relu is supported for quantized pointwise conv2d. Got binary_post_op: ",
       binary_attr,
       " unary_post_op: ",
       unary_attr.has_value() ? unary_attr.value() : "none",
@@ -1860,9 +1956,9 @@ class QConvoneDNN final {
         act, act_scale, act_zero_point,
         weight, weight_scales, weight_zero_points,
         bias, stride, padding, dilation, /*transposed*/false,
-        groups, inv_output_scale, output_zero_point,
+        groups, output_scale, output_zero_point,
         accum, accum_scale, accum_zero_point,
-        /*fp32_output*/false, binary_attr, alpha,
+        /*output_dtype*/output_dtype, binary_attr, alpha,
         unary_attr, unary_scalars, unary_algorithm
     );
 #else
