@@ -241,13 +241,13 @@ class FullyConnectedSparseOperatorTester {
       } while (max_elem == min_elem);
 
       std::unique_ptr<qnnpack::BCSRMatrix> bcsr_matrix =
-        qnnpack::generateBlockCSRMatrix(
-            kernel.data(),
-            outputChannels(),
-            inputChannels(),
-            rowBlockSize(),
-            colBlockSize(),
-            kernelZeroPoints.data());
+          qnnpack::generateBlockCSRMatrix<uint32_t>(
+              kernel.data(),
+              outputChannels(),
+              inputChannels(),
+              rowBlockSize(),
+              colBlockSize(),
+              kernelZeroPoints.data());
 
       std::fill(output.begin(), output.end(), 0xA5);
       std::fill(output_dynamic.begin(), output_dynamic.end(), 0.0f);
@@ -320,11 +320,12 @@ class FullyConnectedSparseOperatorTester {
                     outputChannels(),
                     inputZeroPoint,
                     kernelZeroPoints.data(),
-                    bcsr_matrix->col_indices.data(),
-                    bcsr_matrix->row_values.data(),
+                    bcsr_matrix->col_indices_data_ptr(),
+                    bcsr_matrix->row_values_data_ptr(),
                     bcsr_matrix->values.data(),
                     bcsr_matrix->row_block_size,
                     bcsr_matrix->col_block_size,
+                    pytorch_qnnp_sparse_matrix_indices_dtype_uint32_t,
                     outputZeroPoint,
                     qmin(),
                     qmax(),
@@ -441,13 +442,13 @@ class FullyConnectedSparseOperatorTester {
         min_elem = *std::min_element(kernel.cbegin(), kernel.cend());
       } while (max_elem == min_elem);
       std::unique_ptr<qnnpack::BCSRMatrix> bcsr_matrix =
-        qnnpack::generateBlockCSRMatrix(
-            kernel.data(),
-            outputChannels(),
-            inputChannels(),
-            rowBlockSize(),
-            colBlockSize(),
-            kernelZeroPoints.data());
+          qnnpack::generateBlockCSRMatrix<uint32_t>(
+              kernel.data(),
+              outputChannels(),
+              inputChannels(),
+              rowBlockSize(),
+              colBlockSize(),
+              kernelZeroPoints.data());
 
       std::fill(output.begin(), output.end(), 0xA5);
       std::fill(output_dynamic.begin(), output_dynamic.end(), 0.0f);
@@ -520,11 +521,12 @@ class FullyConnectedSparseOperatorTester {
                     outputChannels(),
                     inputZeroPoint,
                     kernelZeroPoints.data(),
-                    bcsr_matrix->col_indices.data(),
-                    bcsr_matrix->row_values.data(),
+                    bcsr_matrix->col_indices_data_ptr(),
+                    bcsr_matrix->row_values_data_ptr(),
                     bcsr_matrix->values.data(),
                     bcsr_matrix->row_block_size,
                     bcsr_matrix->col_block_size,
+                    pytorch_qnnp_sparse_matrix_indices_dtype_uint32_t,
                     outputZeroPoint,
                     qmin(),
                     qmax(),
