@@ -23,8 +23,7 @@
 #define NAME "sparse_binary_op_intersection_cpu"
 #endif
 
-namespace at {
-namespace native {
+namespace at::native {
 
 namespace {
 
@@ -87,7 +86,7 @@ TensorIterator make_value_selection_intersection_iter(
         lhs_values.sizes(),
         // remove nnz dim for smooth broadcasting
         rhs_values.sizes().slice(1));
-    // update nnz dim to be the lenght of an index
+    // update nnz dim to be the length of an index
     sizes[0] = lhs_select_idx.numel();
     return sizes;
   }();
@@ -133,9 +132,9 @@ void _sparse_binary_op_intersection_kernel_impl(
     Tensor& res,
     const Tensor& x_,
     const Tensor& y_,
-    const std::vector<int64_t> broadcasted_shape,
-    const c10::optional<Tensor>& x_hash_opt_ = c10::nullopt,
-    const c10::optional<Tensor>& y_hash_opt_ = c10::nullopt,
+    const std::vector<int64_t>& broadcasted_shape,
+    const std::optional<Tensor>& x_hash_opt_ = c10::nullopt,
+    const std::optional<Tensor>& y_hash_opt_ = c10::nullopt,
     const bool accumulate_matches = true,
     const bool distributive_with_sum = true
 ) {
@@ -149,7 +148,7 @@ void _sparse_binary_op_intersection_kernel_impl(
       " to output ", res.scalar_type());
 
   using KernelLauncher = KernelLauncher<kernel_t>;
-  using OptTensor = c10::optional<Tensor>;
+  using OptTensor = std::optional<Tensor>;
 
   // If the op and sum are not distributive, coalesce is required.
   const auto coalesce_if_not_distributive = [distributive_with_sum](const Tensor& t, const OptTensor& t_hash_opt) -> auto {
@@ -271,13 +270,16 @@ void _sparse_binary_op_intersection_kernel_impl(
       .build();
 
     {
-      const auto* RESTRICT ptr_indices = indices.data_ptr<index_t>();
+      const auto* RESTRICT ptr_indices = indices.const_data_ptr<index_t>();
 
       KernelLauncher::launch(iter,
           // NOTE: capture by value required by CUDA
           [=] FUNCAPI (index_t nnz_idx) -> int64_t {
-          const auto* RESTRICT ptr_indices_dim = ptr_indices ? ptr_indices + nnz_idx * indices_nnz_stride : nullptr;
           int64_t hash = 0;
+          if (!ptr_indices) {
+            return hash;
+          }
+          const auto* RESTRICT ptr_indices_dim = ptr_indices + nnz_idx * indices_nnz_stride;
           for (int64_t dim = 0; dim < sparse_dim; ++dim) {
             const auto dim_hash_coeff = hash_coeffs[dim];
             const auto dim_index = ptr_indices_dim[dim * indices_dim_stride];
@@ -349,8 +351,8 @@ void _sparse_binary_op_intersection_kernel_impl(
       .build();
 
     {
-      const auto* RESTRICT ptr_indices = source_indices.data_ptr<index_t>();
-      const auto* RESTRICT ptr_sorted_hash = sorted_hash.data_ptr<int64_t>();
+      const auto* RESTRICT ptr_indices = source_indices.const_data_ptr<index_t>();
+      const auto* RESTRICT ptr_sorted_hash = sorted_hash.const_data_ptr<int64_t>();
       const auto sorted_hash_len = sorted_hash.numel();
       auto* RESTRICT ptr_intersection_count = intersection_count.data_ptr<int64_t>();
       auto* RESTRICT ptr_intersection_first_idx = intersection_first_idx.data_ptr<int64_t>();
@@ -424,8 +426,8 @@ void _sparse_binary_op_intersection_kernel_out(
     Tensor& res,
     const Tensor& x,
     const Tensor& y,
-    const c10::optional<Tensor>& x_hash_opt = c10::nullopt,
-    const c10::optional<Tensor>& y_hash_opt = c10::nullopt,
+    const std::optional<Tensor>& x_hash_opt = c10::nullopt,
+    const std::optional<Tensor>& y_hash_opt = c10::nullopt,
     // If op distributes with the sum, the arguments are processed as is,
     // without the calls to coalesce().
     const bool distributive_with_sum = true
@@ -440,12 +442,12 @@ void _sparse_binary_op_intersection_kernel_out(
       x._indices().scalar_type() == y._indices().scalar_type(),
       NAME, "(): expects inputs' indices to be of the same dtype (i.e. long or int)");
 
-  const auto check_hash_validity = [](const Tensor& t, const c10::optional<Tensor>& t_hash_opt) {
+  const auto check_hash_validity = [](const Tensor& t, const std::optional<Tensor>& t_hash_opt) {
     if (!t_hash_opt.has_value()) {
       return;
     }
 
-    const auto t_hash = *t_hash_opt;
+    const auto &t_hash = *t_hash_opt;
     TORCH_INTERNAL_ASSERT(
         t_hash.dim() == 1 && t_hash.scalar_type() == kLong && t_hash.size(-1) == t._indices().size(-1),
         NAME, "(): explicit hash values need to be a 1-dim Long tensor with the ",
@@ -478,4 +480,4 @@ void _sparse_binary_op_intersection_kernel_out(
 
 } // anonymous namespace
 
-}} // at::native
+} // at::native
