@@ -5,9 +5,9 @@
 // Always include this file (Allocator.h) instead.
 //
 
-#ifdef USE_VULKAN_API
+#include <ATen/native/vulkan/api/vk_api.h>
 
-#include <ATen/native/vulkan/api/Common.h>
+#ifdef USE_VULKAN_API
 
 #define VMA_VULKAN_VERSION 1000000
 
