@@ -1,3 +1,5 @@
+#include <ATen/native/vulkan/api/Types.h>
+#include <ATen/native/vulkan/api/Utils.h>
 #include <ATen/native/vulkan/impl/Common.h>
 #include <ATen/native/vulkan/impl/Packing.h>
 
@@ -11,93 +13,104 @@ api::ShaderInfo get_nchw_to_image_shader(const vTensor& v_dst) {
     switch (v_dst.storage_type()) {
       case api::StorageType::TEXTURE_3D:
         switch (v_dst.dtype()) {
-          case c10::ScalarType::QUInt8:
+          case api::ScalarType::QUInt8:
             return VK_KERNEL(nchw_to_image_uint8);
-          case c10::ScalarType::QInt8:
+          case api::ScalarType::QInt8:
             return VK_KERNEL(nchw_to_image_int8);
-          case c10::ScalarType::QInt32:
+          case api::ScalarType::QInt32:
             return VK_KERNEL(nchw_to_image_int32);
           default:
-            TORCH_CHECK(
-                false,
+            VK_THROW(
+                "Vulkan quantization currently not supported for dtype ",
+                v_dst.dtype());
+        }
+      case api::StorageType::TEXTURE_2D:
+        switch (v_dst.dtype()) {
+          case api::ScalarType::QUInt8:
+            return VK_KERNEL(nchw_to_image2d_uint8);
+          case api::ScalarType::QInt8:
+            return VK_KERNEL(nchw_to_image2d_int8);
+          case api::ScalarType::QInt32:
+            return VK_KERNEL(nchw_to_image2d_int32);
+          default:
+            VK_THROW(
                 "Vulkan quantization currently not supported for dtype ",
                 v_dst.dtype());
         }
       default:
-        TORCH_CHECK(false, "No kernel available!");
+        VK_THROW("No kernel available!");
       case api::StorageType::BUFFER:
       case api::StorageType::UNKNOWN:
-        TORCH_CHECK(false, "Requested storage type must be a texture type.");
+        VK_THROW("Requested storage type must be a texture type.");
     }
   }
 
-  if (v_dst.dtype() == at::kFloat) {
+  if (v_dst.dtype() == api::kFloat) {
     switch (v_dst.storage_type()) {
       case api::StorageType::TEXTURE_3D:
         return VK_KERNEL(nchw_to_image);
       case api::StorageType::TEXTURE_2D:
         return VK_KERNEL(nchw_to_image2d);
       default:
-        TORCH_CHECK(false, "No kernel available!");
+        VK_THROW("No kernel available!");
     }
-  } else if (v_dst.dtype() == at::kBool) {
+  } else if (v_dst.dtype() == api::kBool) {
     switch (v_dst.storage_type()) {
       case api::StorageType::TEXTURE_3D:
         return VK_KERNEL(nchw_to_image_bool);
       default:
-        TORCH_CHECK(false, "No kernel available!");
+        VK_THROW("No kernel available!");
     }
   } else {
-    TORCH_CHECK(false, "Unsupported dtype!");
+    VK_THROW("Unsupported dtype!");
   }
 }
 
 api::ShaderInfo get_image_to_nchw_shader(const vTensor& v_src) {
-  if (v_src.is_quantized() || v_src.dtype() == at::kBool) {
+  if (v_src.is_quantized() || v_src.dtype() == api::kBool) {
     auto plane_size =
         dim_at<Dim4D::Height>(v_src) * dim_at<Dim4D::Width>(v_src);
     switch (v_src.storage_type()) {
       case api::StorageType::TEXTURE_3D:
         switch (v_src.dtype()) {
-          case c10::ScalarType::QUInt8:
-          case c10::ScalarType::QInt8:
-          case at::kBool:
+          case api::ScalarType::QUInt8:
+          case api::ScalarType::QInt8:
+          case api::kBool:
             return plane_size % 4 == 0 ? VK_KERNEL(image_to_nchw_quantized_mul4)
                                        : VK_KERNEL(image_to_nchw_uint);
-          case c10::ScalarType::QInt32:
+          case api::ScalarType::QInt32:
             return VK_KERNEL(image_to_nchw_int32);
           default:
-            TORCH_CHECK(
-                false,
+            VK_THROW(
                 "Vulkan quantization currently not supported for dtype ",
                 v_src.dtype());
         }
       default:
-        TORCH_CHECK(false, "No kernel available!");
+        VK_THROW("No kernel available!");
       case api::StorageType::BUFFER:
       case api::StorageType::UNKNOWN:
-        TORCH_CHECK(false, "Requested storage type must be a texture type.");
+        VK_THROW("Requested storage type must be a texture type.");
     }
   }
 
-  if (v_src.dtype() == at::kFloat) {
+  if (v_src.dtype() == api::kFloat) {
     switch (v_src.storage_type()) {
       case api::StorageType::TEXTURE_3D:
         return VK_KERNEL(image_to_nchw);
       case api::StorageType::TEXTURE_2D:
         return VK_KERNEL(image2d_to_nchw);
       default:
-        TORCH_CHECK(false, "No kernel available!");
+        VK_THROW("No kernel available!");
     }
   } else {
-    TORCH_CHECK(false, "Unsupported dtype!");
+    VK_THROW("Unsupported dtype!");
   }
 }
 
 struct ToFromTextureParams final {
   api::utils::ivec3 extents;
-  int32_t plane_size;
-  api::utils::ivec2 c_info;
+  int32_t planeSize;
+  api::utils::ivec2 channelInfo;
 };
 
 void record_nchw_to_image_op(
@@ -106,7 +119,7 @@ void record_nchw_to_image_op(
     api::VulkanBuffer& src_buffer,
     vTensor& v_dst,
     api::PipelineBarrier pipeline_barrier,
-    const VkFence fence_handle) {
+    VkFence fence_handle) {
   api::utils::uvec3 global_size = v_dst.extents();
   api::utils::uvec3 local_size = adaptive_work_group_size(global_size);
 
@@ -148,13 +161,13 @@ void record_nchw_to_image_op(
       params.buffer());
 }
 
-void record_image_to_nchw_op(
+bool record_image_to_nchw_op(
     api::Context* const context,
     api::ShaderInfo& compute_shader,
     vTensor& v_src,
     api::VulkanBuffer& dst_buffer,
     api::PipelineBarrier pipeline_barrier,
-    const VkFence fence_handle) {
+    VkFence fence_handle) {
   api::utils::uvec3 global_size = v_src.extents();
   api::utils::uvec3 local_size = adaptive_work_group_size(global_size);
 
@@ -174,8 +187,8 @@ void record_image_to_nchw_op(
       {c_depth, channels},
   };
 
-  if (v_src.dtype() == c10::ScalarType::QUInt8 ||
-      v_src.dtype() == c10::ScalarType::QInt8 || v_src.dtype() == at::kBool) {
+  if (v_src.dtype() == api::ScalarType::QUInt8 ||
+      v_src.dtype() == api::ScalarType::QInt8 || v_src.dtype() == api::kBool) {
     // Special case using optimized shader, image_to_nchw_quantized_mul4
     if (plane_size % 4 == 0) {
       global_size.data[0u] = plane_size / 4;
@@ -192,7 +205,7 @@ void record_image_to_nchw_op(
   }
 
   api::UniformParamsBuffer params(context, block);
-  context->submit_compute_job(
+  return context->submit_compute_job(
       // shader descriptor
       compute_shader,
       // pipeline barrier
@@ -218,7 +231,7 @@ void record_nchw_to_buffer_op(
     api::VulkanBuffer& src_buffer,
     vTensor& v_dst,
     api::PipelineBarrier pipeline_barrier,
-    const VkFence fence_handle) {
+    VkFence fence_handle) {
   uint32_t gpu_buf_len = api::utils::safe_downcast<uint32_t>(v_dst.gpu_numel());
 
   api::utils::uvec3 global_size = {gpu_buf_len, 1u, 1u};
@@ -248,12 +261,12 @@ void record_nchw_to_buffer_op(
       cpu_buffer_metadata.buffer());
 }
 
-void record_buffer_to_nchw_op(
+bool record_buffer_to_nchw_op(
     api::Context* const context,
     vTensor& v_src,
     api::VulkanBuffer& dst_buffer,
     api::PipelineBarrier pipeline_barrier,
-    const VkFence fence_handle) {
+    VkFence fence_handle) {
   uint32_t buf_len = api::utils::safe_downcast<uint32_t>(v_src.numel());
 
   api::utils::uvec3 global_size = {buf_len, 1u, 1u};
@@ -262,7 +275,7 @@ void record_buffer_to_nchw_op(
   api::UniformParamsBuffer cpu_buffer_metadata(
       context, v_src.get_cpu_buffer_metadata());
 
-  context->submit_compute_job(
+  return context->submit_compute_job(
       // shader descriptor
       VK_KERNEL(buffer_to_buffer),
       // pipeline barrier
@@ -283,6 +296,71 @@ void record_buffer_to_nchw_op(
       v_src.buffer_metadata());
 }
 
+vTensor channel_image_repacking(
+    const vTensor& v_input,
+    api::GPUMemoryLayout target_layout,
+    const api::ShaderInfo& shader_descriptor) {
+  api::Context* const context = api::context();
+
+  vTensor v_output{
+      context,
+      v_input.sizes(),
+      v_input.dtype(),
+      v_input.storage_type(),
+      target_layout,
+  };
+
+  // Required to determine how to insert memory barriers in the command buffer
+  api::PipelineBarrier pipeline_barrier{};
+
+  // The shader assumes a 4d nchw to calculate the lookup coordinate.
+  // If the input is not 4d, we need to pad it with 1's on the front.
+  const struct Block final {
+    api::utils::ivec4 sizes;
+  } block{
+      api::utils::make_ivec4_prepadded1(v_input.sizes()),
+  };
+
+  api::UniformParamsBuffer params(context, block);
+
+  context->submit_compute_job(
+      // shader descriptor
+      // VK_KERNEL(packing_channel_to_height),
+      shader_descriptor,
+      // pipeline barrier
+      pipeline_barrier,
+      // global work group size
+      v_output.extents(),
+      // local work group size
+      adaptive_work_group_size(v_output.extents()),
+      // fence handle
+      VK_NULL_HANDLE,
+      // shader arguments
+      v_output.image(
+          pipeline_barrier,
+          api::PipelineStage::COMPUTE,
+          api::MemoryAccessType::WRITE),
+      v_input.image(pipeline_barrier, api::PipelineStage::COMPUTE),
+      // params buffer
+      params.buffer());
+
+  return v_output;
+}
+
+vTensor convert_image_channels_packed_to_height_packed(const vTensor& v_input) {
+  return channel_image_repacking(
+      v_input,
+      api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED,
+      VK_KERNEL(convert_channels_to_height_packed));
+}
+
+vTensor convert_image_channels_packed_to_width_packed(const vTensor& v_input) {
+  return channel_image_repacking(
+      v_input,
+      api::GPUMemoryLayout::TENSOR_WIDTH_PACKED,
+      VK_KERNEL(convert_channels_to_width_packed));
+}
+
 } // namespace packing
 } // namespace vulkan
 } // namespace native
