@@ -170,7 +170,7 @@ const Tensor reorder_weights_for_transpose_conv(const Tensor& weight_nhwc,
 
 ContextConv2D create(
     const Tensor& weight,
-    const c10::optional<Tensor>& bias,
+    const std::optional<Tensor>& bias,
     const IntArrayRef padding,
     const IntArrayRef output_padding,
     const IntArrayRef stride,
@@ -236,6 +236,7 @@ ContextConv2D create(
       output_max,                                                     // output_max
       0u,                                                             // flags
       nullptr,                                                        // xnn_caches_t
+      nullptr,                                                        // xnn_weights_cache_t
       &convolution_op);                                               // operator
   } else {
     for (const auto i : c10::irange(4)) {
@@ -265,6 +266,7 @@ ContextConv2D create(
       output_max,                                                     // output_max
       0u,                                                             // flags
       nullptr,                                                        // xnn_caches_t
+      nullptr,                                                        // xnn_weights_cache_t
       &convolution_op);                                               // operator
   }
 
@@ -338,26 +340,41 @@ Tensor run(
    */
 
   if (context.transposed_) {
-    setup_status = xnn_setup_deconvolution2d_nhwc_f32(
-      context.op.get(),                                      // operator
+    setup_status = xnn_reshape_deconvolution2d_nhwc_f32(
+      context.op.get(),
       padded_input_nhwc.size(Layout::Activation4D::batch),   // batch_size
       padded_input_nhwc.size(Layout::Activation4D::height),  // input_height
       padded_input_nhwc.size(Layout::Activation4D::width),   // input_width
       context.output_padding_[0],                            // adjustment_height
       context.output_padding_[1],                            // adjustment_width
-      padded_input_nhwc.data_ptr<float>(),                   // input
-      output.data_ptr<float>(),                              // output
+      nullptr,                                               // output_height_out
+      nullptr,                                               // output_width_out
       caffe2::pthreadpool_());                               // threadpool
 
-  } else {
-    setup_status = xnn_setup_convolution2d_nhwc_f32(
+    setup_status = xnn_setup_deconvolution2d_nhwc_f32(
       context.op.get(),                                      // operator
+      padded_input_nhwc.data_ptr<float>(),                   // input
+      output.data_ptr<float>());                             // output
+  } else {
+    size_t workspace_size = SIZE_MAX;
+    size_t workspace_alignment = SIZE_MAX;
+
+    setup_status = xnn_reshape_convolution2d_nhwc_f32(
+      context.op.get(),
       padded_input_nhwc.size(Layout::Activation4D::batch),   // batch_size
       padded_input_nhwc.size(Layout::Activation4D::height),  // input_height
       padded_input_nhwc.size(Layout::Activation4D::width),   // input_width
-      padded_input_nhwc.data_ptr<float>(),                   // input
-      output.data_ptr<float>(),                              // output
+      &workspace_size,                                       // workspace_size
+      &workspace_alignment,                                  // workspace_alignment
+      nullptr,                                               // output_height_out
+      nullptr,                                               // output_width_out
       caffe2::pthreadpool_());
+
+    setup_status = xnn_setup_convolution2d_nhwc_f32(
+      context.op.get(),                                      // operator
+      nullptr,                                               // workspace
+      padded_input_nhwc.data_ptr<float>(),                   // input
+      output.data_ptr<float>());                             // output
   }
 
   TORCH_CHECK(
@@ -379,13 +396,13 @@ Tensor run(
 c10::intrusive_ptr<xnnpack::Conv2dOpContext>
     createConv2dClampPrePackOpContext(
         Tensor weight,
-        c10::optional<Tensor> bias,
+        std::optional<Tensor> bias,
         std::vector<int64_t> stride,
         std::vector<int64_t> padding,
         std::vector<int64_t> dilation,
         int64_t groups,
-        const c10::optional<Scalar>& output_min,
-        const c10::optional<Scalar>& output_max) {
+        const std::optional<Scalar>& output_min,
+        const std::optional<Scalar>& output_max) {
       return xnnpack::XNNPackConv2dOpContext::create_context(
           std::move(weight),
           std::move(bias),
@@ -400,14 +417,14 @@ c10::intrusive_ptr<xnnpack::Conv2dOpContext>
 c10::intrusive_ptr<xnnpack::TransposeConv2dOpContext>
     createConv2dTransposeClampPrePackOpContext(
         Tensor weight,
-        c10::optional<Tensor> bias,
+        std::optional<Tensor> bias,
         std::vector<int64_t> stride,
         std::vector<int64_t> padding,
         std::vector<int64_t> output_padding,
         std::vector<int64_t> dilation,
         int64_t groups,
-        const c10::optional<Scalar>& output_min,
-        const c10::optional<Scalar>& output_max) {
+        const std::optional<Scalar>& output_min,
+        const std::optional<Scalar>& output_max) {
       return xnnpack::XNNPackTransposeConv2dOpContext::create_context(
           std::move(weight),
           std::move(bias),
