@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+import functools
 import gc
 import importlib
 import logging
@@ -6,8 +7,11 @@ import os
 import re
 import sys
 import warnings
+from collections import namedtuple
 from os.path import abspath, exists
 
+import yaml
+
 import torch
 
 try:
@@ -22,6 +26,20 @@ from torch._dynamo.utils import clone_inputs
 torch.backends.cuda.matmul.allow_tf32 = True
 
 
+def _reassign_parameters(model):
+    # torch_geometric models register parameter as tensors due to
+    # https://github.com/pyg-team/pytorch_geometric/blob/master/torch_geometric/nn/dense/linear.py#L158-L168
+    # Since it is unusual thing to do, we just reassign them to parameters
+    def state_dict_hook(module, destination, prefix, local_metadata):
+        for name, param in module.named_parameters():
+            if isinstance(destination[name], torch.Tensor) and not isinstance(
+                destination[name], torch.nn.Parameter
+            ):
+                destination[name] = torch.nn.Parameter(destination[name])
+
+    model._register_state_dict_hook(state_dict_hook)
+
+
 def setup_torchbench_cwd():
     original_dir = abspath(os.getcwd())
 
@@ -46,192 +64,52 @@ def setup_torchbench_cwd():
     return original_dir
 
 
-# Some models have large dataset that doesn't fit in memory. Lower the batch
-# size to test the accuracy.
-USE_SMALL_BATCH_SIZE = {
-    "demucs": 4,
-    "dlrm": 1024,
-    "densenet121": 4,
-    "hf_Reformer": 4,
-    "hf_T5_base": 4,
-    "timm_efficientdet": 1,
-    "llama_v2_7b_16h": 1,
-}
+@functools.lru_cache(maxsize=1)
+def load_yaml_file():
+    filename = "torchbench.yaml"
+    filepath = os.path.join(os.path.dirname(__file__), filename)
 
-DETECTRON2_MODELS = {
-    "detectron2_fasterrcnn_r_101_c4",
-    "detectron2_fasterrcnn_r_101_dc5",
-    "detectron2_fasterrcnn_r_101_fpn",
-    "detectron2_fasterrcnn_r_50_c4",
-    "detectron2_fasterrcnn_r_50_dc5",
-    "detectron2_fasterrcnn_r_50_fpn",
-    "detectron2_maskrcnn_r_101_c4",
-    "detectron2_maskrcnn_r_101_fpn",
-    "detectron2_maskrcnn_r_50_fpn",
-}
+    with open(filepath) as f:
+        data = yaml.safe_load(f)
 
-SKIP = {
-    # https://github.com/pytorch/torchdynamo/issues/101
-    "detectron2_maskrcnn",
-    # https://github.com/pytorch/torchdynamo/issues/145
-    "fambench_xlmr",
-    # TIMEOUT, https://github.com/pytorch/pytorch/issues/98467
-    "tacotron2",
-    "hf_Bert",  # Error: RelaxedUnspecConstraint(L['input_ids'].size()[0]) - inferred constant (4)
-    "hf_Bert_large",  # Error: RelaxedUnspecConstraint(L['input_ids'].size()[0]) - inferred constant (4)
-    # takes too long, extreme slowdown (< .001)
-    "maml",
-}
+    def flatten(lst):
+        for item in lst:
+            if isinstance(item, list):
+                yield from flatten(item)
+            else:
+                yield item
 
-SKIP_FOR_CPU = {
-    "hf_T5_generate",  # OOMs
-    "cm3leon_generate",  # model is CUDA only
-    "nanogpt_generate",  # timeout
-    "sam",  # timeout
-    "llama_v2_7b_16h",  # model is CUDA only
-    "stable_diffusion",  # flaky
-    "torchrec_dlrm",  # requires FBGEMM, CUDA only
-}
+    def maybe_list_to_set(obj):
+        if isinstance(obj, dict):
+            return {k: maybe_list_to_set(v) for k, v in obj.items()}
+        if isinstance(obj, list):
+            return set(flatten(obj))
+        return obj
 
-SKIP_FOR_CUDA = {
-    "gat",  # only works on CPU
-    "gcn",  # only works on CPU
-    "sage",  # only works on CPU
-}
-
-# Additional models that are skipped in training
-SKIP_TRAIN = {
-    # not designed for training
-    "pyhpc_equation_of_state",
-    "pyhpc_isoneutral_mixing",
-    "pyhpc_turbulent_kinetic_energy",
-    "maml",
-    "llama",
-    "llama_v2_7b_16h",
-}
-SKIP_TRAIN.update(DETECTRON2_MODELS)
-
-# These models support only train mode. So accuracy checking can't be done in
-# eval mode.
-ONLY_TRAINING_MODE = {
-    "tts_angular",
-    "tacotron2",
-    "demucs",
-    "hf_Reformer",
-    "pytorch_struct",
-    "yolov3",
-}
-ONLY_TRAINING_MODE.update(DETECTRON2_MODELS)
-
-# Need lower tolerance on GPU. GPU kernels have non deterministic kernels for these models.
-REQUIRE_HIGHER_TOLERANCE = {
-    "alexnet",
-    "attention_is_all_you_need_pytorch",
-    "densenet121",
-    "hf_Albert",
-    "vgg16",
-    "mobilenet_v3_large",
-    "nvidia_deeprecommender",
-    "timm_efficientdet",
-}
-
-# These models need >1e-3 tolerance
-REQUIRE_EVEN_HIGHER_TOLERANCE = {
-    "soft_actor_critic",
-    "tacotron2",
-}
-
-REQUIRE_HIGHER_FP16_TOLERANCE = {
-    "drq",
-}
-
-REQUIRE_COSINE_TOLERACE = {
-    # Just keeping it here even though its empty, if we need this in future.
-}
-
-# non-deterministic output / cant check correctness
-NONDETERMINISTIC = {
-    # https://github.com/pytorch/pytorch/issues/98355
-    "mobilenet_v3_large",
-}
-
-# These benchmarks took >600s on an i9-11900K CPU
-VERY_SLOW_BENCHMARKS = {
-    "hf_BigBird",  # 3339s
-    "hf_Longformer",  # 3062s
-    "hf_T5",  # 930s
-}
-
-# These benchmarks took >60s on an i9-11900K CPU
-SLOW_BENCHMARKS = {
-    *VERY_SLOW_BENCHMARKS,
-    "BERT_pytorch",  # 137s
-    "demucs",  # 116s
-    "fastNLP_Bert",  # 242s
-    "hf_Albert",  # 221s
-    "hf_Bart",  # 400s
-    "hf_Bert",  # 334s
-    "hf_DistilBert",  # 187s
-    "hf_GPT2",  # 470s
-    "hf_Reformer",  # 141s
-    "speech_transformer",  # 317s
-    "vision_maskrcnn",  # 99s
-}
-
-TRT_NOT_YET_WORKING = {
-    "alexnet",
-    "resnet18",
-    "resnet50",
-    "mobilenet_v2",
-    "mnasnet1_0",
-    "squeezenet1_1",
-    "shufflenetv2_x1_0",
-    "vgg16",
-    "resnext50_32x4d",
-}
-
-DONT_CHANGE_BATCH_SIZE = {
-    "demucs",
-    "pytorch_struct",
-    "pyhpc_turbulent_kinetic_energy",
-    "vision_maskrcnn",  # https://github.com/pytorch/benchmark/pull/1656
-}
+    return maybe_list_to_set(data)
 
 
-SKIP_ACCURACY_CHECK_MODELS = {
-    # Models too large to have eager, dynamo and fp64_numbers simultaneosuly
-    # even for 40 GB machine. We have tested accuracy for smaller version of
-    # these models
-    "hf_GPT2_large",
-    "hf_T5_large",
-    "timm_vision_transformer_large",
-    "maml",  # accuracy https://github.com/pytorch/pytorch/issues/93847
-    "llama_v2_7b_16h",
-    "Background_Matting",
-}
-
-SKIP_ACCURACY_CHECK_AS_EAGER_NON_DETERMINISTIC_MODELS = {
-    # Models that deterministic algorithms can not be turned on for eager mode.
-    "Background_Matting",
-}
+def process_hf_reformer_output(out):
+    assert isinstance(out, list)
+    # second output is unstable
+    return [elem for i, elem in enumerate(out) if i != 1]
 
 
-MAX_BATCH_SIZE_FOR_ACCURACY_CHECK = {
-    "hf_GPT2": 2,
-    "pytorch_unet": 2,
-}
+def process_hf_whisper_output(out):
+    out_ret = []
+    for i, elem in enumerate(out):
+        if i == 0:
+            assert isinstance(elem, dict)
+            out_ret.append({k: v for k, v in elem.items() if k != "logits"})
+        elif i != 1:
+            out_ret.append(elem)
 
-FORCE_AMP_FOR_FP16_BF16_MODELS = {
-    "DALLE2_pytorch",
-    "doctr_det_predictor",
-    "doctr_reco_predictor",
-    "Super_SloMo",
-    "tts_angular",
-}
+    return out_ret
 
-# models in canary_models that we should run anyway
-CANARY_MODELS = {
-    "torchrec_dlrm",
+
+process_train_model_output = {
+    "hf_Reformer": process_hf_reformer_output,
+    "hf_Whisper": process_hf_whisper_output,
 }
 
 
@@ -241,60 +119,107 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         self.suite_name = "torchbench"
         self.optimizer = None
 
+    @property
+    def _config(self):
+        return load_yaml_file()
+
+    @property
+    def _skip(self):
+        return self._config["skip"]
+
+    @property
+    def _batch_size(self):
+        return self._config["batch_size"]
+
+    @property
+    def _tolerance(self):
+        return self._config["tolerance"]
+
+    @property
+    def _accuracy(self):
+        return self._config["accuracy"]
+
     @property
     def skip_models(self):
-        return SKIP
+        return self._skip["all"]
 
     @property
     def skip_models_for_cpu(self):
-        return SKIP_FOR_CPU
+        return self._skip["device"]["cpu"]
 
     @property
     def skip_models_for_cuda(self):
-        return SKIP_FOR_CUDA
+        return self._skip["device"]["cuda"]
+
+    @property
+    def skip_models_for_freezing(self):
+        return self._skip["freezing"]
 
     @property
     def slow_models(self):
-        return SLOW_BENCHMARKS
+        return self._config["slow"]
 
     @property
     def very_slow_models(self):
-        return VERY_SLOW_BENCHMARKS
+        return self._config["very_slow"]
 
     @property
     def non_deterministic_models(self):
-        return NONDETERMINISTIC
+        return self._config["non_deterministic"]
+
+    @property
+    def get_output_amp_train_process_func(self):
+        return process_train_model_output
 
     @property
     def skip_not_suitable_for_training_models(self):
-        return SKIP_TRAIN
+        return self._skip["test"]["training"]
 
     @property
     def failing_fx2trt_models(self):
-        return TRT_NOT_YET_WORKING
+        return self._config["trt_not_yet_working"]
 
     @property
     def force_amp_for_fp16_bf16_models(self):
-        return FORCE_AMP_FOR_FP16_BF16_MODELS
+        return self._config["dtype"]["force_amp_for_fp16_bf16_models"]
+
+    @property
+    def force_fp16_for_bf16_models(self):
+        return self._config["dtype"]["force_fp16_for_bf16_models"]
 
     @property
     def skip_accuracy_checks_large_models_dashboard(self):
         if self.args.dashboard or self.args.accuracy:
-            return SKIP_ACCURACY_CHECK_MODELS
+            return self._accuracy["skip"]["large_models"]
         return set()
 
     @property
     def skip_accuracy_check_as_eager_non_deterministic(self):
         if self.args.accuracy and self.args.training:
-            return SKIP_ACCURACY_CHECK_AS_EAGER_NON_DETERMINISTIC_MODELS
+            return self._accuracy["skip"]["eager_not_deterministic"]
         return set()
 
+    @property
+    def skip_multiprocess_models(self):
+        return self._skip["multiprocess"]
+
+    @property
+    def skip_models_due_to_control_flow(self):
+        return self._skip["control_flow"]
+
+    @property
+    def guard_on_nn_module_models(self):
+        return {
+            "vision_maskrcnn",
+        }
+
     def load_model(
         self,
         device,
         model_name,
         batch_size=None,
         part=None,
+        extra_args=None,
     ):
         if self.args.enable_activation_checkpointing:
             raise NotImplementedError(
@@ -318,27 +243,46 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         else:
             raise ImportError(f"could not import any of {candidates}")
         benchmark_cls = getattr(module, "Model", None)
+        if benchmark_cls is None:
+            raise NotImplementedError(f"{model_name}.Model is None")
+
         if not hasattr(benchmark_cls, "name"):
             benchmark_cls.name = model_name
 
         cant_change_batch_size = (
             not getattr(benchmark_cls, "ALLOW_CUSTOMIZE_BSIZE", True)
-            or model_name in DONT_CHANGE_BATCH_SIZE
+            or model_name in self._config["dont_change_batch_size"]
         )
         if cant_change_batch_size:
             batch_size = None
-        if batch_size is None and is_training and model_name in USE_SMALL_BATCH_SIZE:
-            batch_size = USE_SMALL_BATCH_SIZE[model_name]
+        if (
+            batch_size is None
+            and is_training
+            and model_name in self._batch_size["training"]
+        ):
+            batch_size = self._batch_size["training"][model_name]
+        elif (
+            batch_size is None
+            and not is_training
+            and model_name in self._batch_size["inference"]
+        ):
+            batch_size = self._batch_size["inference"][model_name]
 
         # Control the memory footprint for few models
-        if self.args.accuracy and model_name in MAX_BATCH_SIZE_FOR_ACCURACY_CHECK:
-            batch_size = min(batch_size, MAX_BATCH_SIZE_FOR_ACCURACY_CHECK[model_name])
+        if self.args.accuracy and model_name in self._accuracy["max_batch_size"]:
+            batch_size = min(batch_size, self._accuracy["max_batch_size"][model_name])
 
         # workaround "RuntimeError: not allowed to set torch.backends.cudnn flags"
         torch.backends.__allow_nonbracketed_mutation_flag = True
-        extra_args = []
+        if extra_args is None:
+            extra_args = []
         if part:
-            extra_args = ["--part", part]
+            extra_args += ["--part", part]
+
+        # sam_fast only runs with amp
+        if model_name == "sam_fast":
+            self.args.amp = True
+            self.setup_amp()
 
         if model_name == "vision_maskrcnn" and is_training:
             # Output of vision_maskrcnn model is a list of bounding boxes,
@@ -346,8 +290,8 @@ class TorchBenchmarkRunner(BenchmarkRunner):
             # comparison hard with torch.compile. torch.compile can cause minor
             # divergences in the output because of how fusion works for amp in
             # TorchInductor compared to eager.  Therefore, instead of looking at
-            # all the bounding boxes, we compare only top 5.
-            model_kwargs = {"box_detections_per_img": 5}
+            # all the bounding boxes, we compare only top 4.
+            model_kwargs = {"box_detections_per_img": 4}
             benchmark = benchmark_cls(
                 test="train",
                 device=device,
@@ -370,15 +314,35 @@ class TorchBenchmarkRunner(BenchmarkRunner):
                 extra_args=extra_args,
             )
         model, example_inputs = benchmark.get_module()
+        if model_name in [
+            "basic_gnn_edgecnn",
+            "basic_gnn_gcn",
+            "basic_gnn_sage",
+            "basic_gnn_gin",
+        ]:
+            _reassign_parameters(model)
 
         # Models that must be in train mode while training
-        if is_training and (not use_eval_mode or model_name in ONLY_TRAINING_MODE):
+        if is_training and (
+            not use_eval_mode or model_name in self._config["only_training"]
+        ):
             model.train()
         else:
             model.eval()
         gc.collect()
         batch_size = benchmark.batch_size
-
+        if model_name == "torchrec_dlrm":
+            batch_namedtuple = namedtuple(
+                "Batch", "dense_features sparse_features labels"
+            )
+            example_inputs = tuple(
+                batch_namedtuple(
+                    dense_features=batch.dense_features,
+                    sparse_features=batch.sparse_features,
+                    labels=batch.labels,
+                )
+                for batch in example_inputs
+            )
         # Torchbench has quite different setup for yolov3, so directly passing
         # the right example_inputs
         if model_name == "yolov3":
@@ -406,7 +370,7 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         models += [
             f
             for f in _list_canary_model_paths()
-            if os.path.basename(f) in CANARY_MODELS
+            if os.path.basename(f) in self._config["canary_models"]
         ]
         models.sort()
 
@@ -437,16 +401,21 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         cosine = self.args.cosine
         # Increase the tolerance for torch allclose
         if self.args.float16 or self.args.amp:
-            if name in REQUIRE_HIGHER_FP16_TOLERANCE:
+            if name in self._tolerance["higher_fp16"]:
                 return 1e-2, cosine
             return 1e-3, cosine
-        if is_training and current_device == "cuda":
+
+        if self.args.bfloat16:
+            if name in self._tolerance["higher_bf16"]:
+                return 1e-2, cosine
+
+        if is_training and (current_device == "cuda" or current_device == "xpu"):
             tolerance = 1e-3
-            if name in REQUIRE_COSINE_TOLERACE:
+            if name in self._tolerance["cosine"]:
                 cosine = True
-            elif name in REQUIRE_HIGHER_TOLERANCE:
+            elif name in self._tolerance["higher"]:
                 tolerance = 1e-3
-            elif name in REQUIRE_EVEN_HIGHER_TOLERANCE:
+            elif name in self._tolerance["even_higher"]:
                 tolerance = 8 * 1e-2
         return tolerance, cosine
 
@@ -454,14 +423,20 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         return reduce_to_scalar_loss(pred)
 
     def forward_pass(self, mod, inputs, collect_outputs=True):
-        with self.autocast():
-            return mod(*inputs)
+        with self.autocast(**self.autocast_arg):
+            if isinstance(inputs, dict):
+                return mod(**inputs)
+            else:
+                return mod(*inputs)
 
     def forward_and_backward_pass(self, mod, inputs, collect_outputs=True):
         cloned_inputs = clone_inputs(inputs)
         self.optimizer_zero_grad(mod)
-        with self.autocast():
-            pred = mod(*cloned_inputs)
+        with self.autocast(**self.autocast_arg):
+            if isinstance(cloned_inputs, dict):
+                pred = mod(**cloned_inputs)
+            else:
+                pred = mod(*cloned_inputs)
             loss = self.compute_loss(pred)
         self.grad_scaler.scale(loss).backward()
         self.optimizer_step()
