@@ -1,10 +1,10 @@
-import torch
-
 from collections import namedtuple
 from typing import List, Tuple
+
+import torch
 from torch import Tensor
 
-from .cells import lstm_cell, premul_lstm_cell, premul_lstm_cell_no_bias, flat_lstm_cell
+from .cells import flat_lstm_cell, lstm_cell, premul_lstm_cell, premul_lstm_cell_no_bias
 
 
 # list[list[T]] -> list[T]
@@ -15,7 +15,7 @@ def flatten_list(lst):
     return result
 
 
-'''
+"""
 Define a creator as a function:
 (options) -> (inputs, params, forward, backward_setup, backward)
 inputs: the inputs to the returned 'forward'. One can call
@@ -30,11 +30,12 @@ backward: Given `output = backward_setup(*forward(*inputs))`, performs
     backpropagation. If None, then nothing happens.
 
 fastrnns.bench times the forward and backward invocations.
-'''
+"""
 
 
-ModelDef = namedtuple('ModelDef', [
-    'inputs', 'params', 'forward', 'backward_setup', 'backward'])
+ModelDef = namedtuple(
+    "ModelDef", ["inputs", "params", "forward", "backward_setup", "backward"]
+)
 
 
 def lstm_backward_setup(lstm_outputs, seed=None):
@@ -61,7 +62,8 @@ def pytorch_lstm_creator(**kwargs):
         params=flatten_list(module.all_weights),
         forward=module,
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lstm_creator(script=True, **kwargs):
@@ -72,51 +74,65 @@ def lstm_creator(script=True, **kwargs):
         params=flatten_list(params),
         forward=lstm_factory(lstm_cell, script),
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lnlstm_creator(script=True, decompose_layernorm=False, **kwargs):
     assert script is True
     from .custom_lstms import script_lnlstm
-    input_size = kwargs['inputSize']
-    hidden_size = kwargs['hiddenSize']
-    seq_len = kwargs['seqLength']
-    batch_size = kwargs['miniBatch']
-    ge = script_lnlstm(input_size, hidden_size, 1,
-                       decompose_layernorm=decompose_layernorm).cuda()
 
-    input = torch.randn(seq_len, batch_size, input_size, device='cuda')
-    states = [(torch.randn(batch_size, hidden_size, device='cuda'),
-               torch.randn(batch_size, hidden_size, device='cuda'))]
+    input_size = kwargs["inputSize"]
+    hidden_size = kwargs["hiddenSize"]
+    seq_len = kwargs["seqLength"]
+    batch_size = kwargs["miniBatch"]
+    ge = script_lnlstm(
+        input_size, hidden_size, 1, decompose_layernorm=decompose_layernorm
+    ).cuda()
+
+    input = torch.randn(seq_len, batch_size, input_size, device="cuda")
+    states = [
+        (
+            torch.randn(batch_size, hidden_size, device="cuda"),
+            torch.randn(batch_size, hidden_size, device="cuda"),
+        )
+    ]
 
     return ModelDef(
         inputs=[input, states],
         params=ge.parameters(),
         forward=ge,
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def dropoutlstm_creator(script=True, **kwargs):
     assert script is True
-    from .custom_lstms import script_lstm, LSTMState
-    input_size = kwargs['inputSize']
-    hidden_size = kwargs['hiddenSize']
-    seq_len = kwargs['seqLength']
-    batch_size = kwargs['miniBatch']
-    num_layers = kwargs['numLayers']
+    from .custom_lstms import LSTMState, script_lstm
+
+    input_size = kwargs["inputSize"]
+    hidden_size = kwargs["hiddenSize"]
+    seq_len = kwargs["seqLength"]
+    batch_size = kwargs["miniBatch"]
+    num_layers = kwargs["numLayers"]
     ge = script_lstm(input_size, hidden_size, num_layers, dropout=True).cuda()
 
-    input = torch.randn(seq_len, batch_size, input_size, device='cuda')
-    states = [LSTMState(torch.randn(batch_size, hidden_size, device='cuda'),
-                        torch.randn(batch_size, hidden_size, device='cuda'))
-              for _ in range(num_layers)]
+    input = torch.randn(seq_len, batch_size, input_size, device="cuda")
+    states = [
+        LSTMState(
+            torch.randn(batch_size, hidden_size, device="cuda"),
+            torch.randn(batch_size, hidden_size, device="cuda"),
+        )
+        for _ in range(num_layers)
+    ]
     return ModelDef(
         inputs=[input, states],
         params=ge.parameters(),
         forward=ge,
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lstm_premul_creator(script=True, **kwargs):
@@ -127,7 +143,8 @@ def lstm_premul_creator(script=True, **kwargs):
         params=flatten_list(params),
         forward=lstm_factory_premul(premul_lstm_cell, script),
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lstm_premul_bias_creator(script=True, **kwargs):
@@ -138,7 +155,8 @@ def lstm_premul_bias_creator(script=True, **kwargs):
         params=flatten_list(params),
         forward=lstm_factory_premul_bias(premul_lstm_cell_no_bias, script),
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lstm_simple_creator(script=True, **kwargs):
@@ -149,7 +167,8 @@ def lstm_simple_creator(script=True, **kwargs):
         params=flatten_list(params),
         forward=lstm_factory_simple(flat_lstm_cell, script),
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def lstm_multilayer_creator(script=True, **kwargs):
@@ -160,11 +179,12 @@ def lstm_multilayer_creator(script=True, **kwargs):
         params=flatten_list(params),
         forward=lstm_factory_multilayer(lstm_cell, script),
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def imagenet_cnn_creator(arch, jit=True):
-    def creator(device='cuda', **kwargs):
+    def creator(device="cuda", **kwargs):
         model = arch().to(device)
         x = torch.randn(32, 3, 224, 224, device=device)
         if jit:
@@ -174,22 +194,30 @@ def imagenet_cnn_creator(arch, jit=True):
             params=list(model.parameters()),
             forward=model,
             backward_setup=simple_backward_setup,
-            backward=simple_backward)
+            backward=simple_backward,
+        )
 
     return creator
 
 
-def varlen_lstm_inputs(minlen=30, maxlen=100,
-                       numLayers=1, inputSize=512, hiddenSize=512,
-                       miniBatch=64, return_module=False, device='cuda',
-                       seed=None, **kwargs):
+def varlen_lstm_inputs(
+    minlen=30,
+    maxlen=100,
+    numLayers=1,
+    inputSize=512,
+    hiddenSize=512,
+    miniBatch=64,
+    return_module=False,
+    device="cuda",
+    seed=None,
+    **kwargs,
+):
     if seed is not None:
         torch.manual_seed(seed)
     lengths = torch.randint(
-        low=minlen, high=maxlen, size=[miniBatch],
-        dtype=torch.long, device=device)
-    x = [torch.randn(length, inputSize, device=device)
-         for length in lengths]
+        low=minlen, high=maxlen, size=[miniBatch], dtype=torch.long, device=device
+    )
+    x = [torch.randn(length, inputSize, device=device) for length in lengths]
     hx = torch.randn(numLayers, miniBatch, hiddenSize, device=device)
     cx = torch.randn(numLayers, miniBatch, hiddenSize, device=device)
     lstm = torch.nn.LSTM(inputSize, hiddenSize, numLayers).to(device)
@@ -214,8 +242,7 @@ def varlen_lstm_backward_setup(forward_output, seed=None):
 
 def varlen_pytorch_lstm_creator(**kwargs):
     rnn_utils = torch.nn.utils.rnn
-    sequences, _, hidden, _, module = varlen_lstm_inputs(
-        return_module=True, **kwargs)
+    sequences, _, hidden, _, module = varlen_lstm_inputs(return_module=True, **kwargs)
 
     def forward(sequences, hidden):
         packed = rnn_utils.pack_sequence(sequences, enforce_sorted=False)
@@ -232,13 +259,19 @@ def varlen_pytorch_lstm_creator(**kwargs):
         params=flatten_list(module.all_weights),
         forward=forward,
         backward_setup=lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 def varlen_lstm_factory(cell, script):
-    def dynamic_rnn(sequences: List[Tensor], hiddens: Tuple[Tensor, Tensor], wih: Tensor,
-                    whh: Tensor, bih: Tensor, bhh: Tensor
-                    ) -> Tuple[List[Tensor], Tuple[List[Tensor], List[Tensor]]]:
+    def dynamic_rnn(
+        sequences: List[Tensor],
+        hiddens: Tuple[Tensor, Tensor],
+        wih: Tensor,
+        whh: Tensor,
+        bih: Tensor,
+        bhh: Tensor,
+    ) -> Tuple[List[Tensor], Tuple[List[Tensor], List[Tensor]]]:
         hx, cx = hiddens
         hxs = hx.unbind(1)
         cxs = cx.unbind(1)
@@ -254,7 +287,8 @@ def varlen_lstm_factory(cell, script):
 
             for seq_idx in range(len(inputs)):
                 hy, cy = cell(
-                    inputs[seq_idx].unsqueeze(0), (hy, cy), wih, whh, bih, bhh)
+                    inputs[seq_idx].unsqueeze(0), (hy, cy), wih, whh, bih, bhh
+                )
                 output += [hy]
             outputs += [torch.stack(output)]
             hx_outs += [hy.unsqueeze(0)]
@@ -270,15 +304,15 @@ def varlen_lstm_factory(cell, script):
 
 
 def varlen_lstm_creator(script=False, **kwargs):
-    sequences, _, hidden, params, _ = varlen_lstm_inputs(
-        return_module=False, **kwargs)
+    sequences, _, hidden, params, _ = varlen_lstm_inputs(return_module=False, **kwargs)
     inputs = [sequences, hidden] + params[0]
     return ModelDef(
         inputs=inputs,
         params=flatten_list(params),
         forward=varlen_lstm_factory(lstm_cell, script),
         backward_setup=varlen_lstm_backward_setup,
-        backward=simple_backward)
+        backward=simple_backward,
+    )
 
 
 # cudnn_layernorm_lstm: since cudnn does not have Layernorm LSTM, we cannot benchmark
@@ -290,12 +324,12 @@ def varlen_lstm_creator(script=False, **kwargs):
 # a faster forward lowerbound though.
 def layernorm_pytorch_lstm_creator(**kwargs):
     input, hidden, _, module = lstm_inputs(return_module=True, **kwargs)
-    batch_size = kwargs['miniBatch']
-    hidden_size = kwargs['hiddenSize']
+    batch_size = kwargs["miniBatch"]
+    hidden_size = kwargs["hiddenSize"]
     ln_i = torch.nn.LayerNorm(4 * hidden_size).cuda()
     ln_h = torch.nn.LayerNorm(4 * hidden_size).cuda()
     ln_c = torch.nn.LayerNorm(hidden_size).cuda()
-    ln_input1 = torch.randn(batch_size, 4 * hidden_size, device='cuda')
+    ln_input1 = torch.randn(batch_size, 4 * hidden_size, device="cuda")
 
     def forward(input, hidden):
         out, new_hidden = module(input, hidden)
@@ -315,7 +349,8 @@ def layernorm_pytorch_lstm_creator(**kwargs):
         params=flatten_list(module.all_weights),
         forward=forward,
         backward_setup=lstm_backward_setup,
-        backward=None)
+        backward=None,
+    )
 
 
 # input: lstm.all_weights format (wih, whh, bih, bhh = lstm.all_weights[layer])
@@ -330,27 +365,34 @@ def stack_weights(weights):
         assert isinstance(mat[0], list)
         layers = len(mat)
         columns = len(mat[0])
-        return [[mat[layer][col] for layer in range(layers)]
-                for col in range(columns)]
+        return [[mat[layer][col] for layer in range(layers)] for col in range(columns)]
 
     # XXX: script fns have problems indexing multidim lists, so we try to
     # avoid them by stacking tensors
     all_weights = weights
-    packed_weights = [torch.stack(param)
-                      for param in unzip_columns(all_weights)]
+    packed_weights = [torch.stack(param) for param in unzip_columns(all_weights)]
     return packed_weights
 
 
 # returns: x, (hx, cx), all_weights, lstm module with all_weights as params
-def lstm_inputs(seqLength=100, numLayers=1, inputSize=512, hiddenSize=512,
-                miniBatch=64, dropout=0.0, return_module=False, device='cuda', seed=None):
+def lstm_inputs(
+    seqLength=100,
+    numLayers=1,
+    inputSize=512,
+    hiddenSize=512,
+    miniBatch=64,
+    dropout=0.0,
+    return_module=False,
+    device="cuda",
+    seed=None,
+):
     if seed is not None:
         torch.manual_seed(seed)
     x = torch.randn(seqLength, miniBatch, inputSize, device=device)
     hx = torch.randn(numLayers, miniBatch, hiddenSize, device=device)
     cx = torch.randn(numLayers, miniBatch, hiddenSize, device=device)
     lstm = torch.nn.LSTM(inputSize, hiddenSize, numLayers, dropout=dropout)
-    if 'cuda' in device:
+    if "cuda" in device:
         lstm = lstm.cuda()
 
     if return_module:
@@ -362,8 +404,14 @@ def lstm_inputs(seqLength=100, numLayers=1, inputSize=512, hiddenSize=512,
 
 
 def lstm_factory(cell, script):
-    def dynamic_rnn(input: Tensor, hidden: Tuple[Tensor, Tensor], wih: Tensor, whh: Tensor,
-                    bih: Tensor, bhh: Tensor) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
+    def dynamic_rnn(
+        input: Tensor,
+        hidden: Tuple[Tensor, Tensor],
+        wih: Tensor,
+        whh: Tensor,
+        bih: Tensor,
+        bhh: Tensor,
+    ) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
         hx, cx = hidden
         outputs = []
         inputs = input.unbind(0)
@@ -382,8 +430,14 @@ def lstm_factory(cell, script):
 
 # premul: we're going to premultiply the inputs & weights
 def lstm_factory_premul(premul_cell, script):
-    def dynamic_rnn(input: Tensor, hidden: Tuple[Tensor, Tensor], wih: Tensor, whh: Tensor,
-                    bih: Tensor, bhh: Tensor) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
+    def dynamic_rnn(
+        input: Tensor,
+        hidden: Tuple[Tensor, Tensor],
+        wih: Tensor,
+        whh: Tensor,
+        bih: Tensor,
+        bhh: Tensor,
+    ) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
         hx, cx = hidden
         outputs = []
         inputs = torch.matmul(input, wih.t()).unbind(0)
@@ -402,8 +456,14 @@ def lstm_factory_premul(premul_cell, script):
 
 # premul: we're going to premultiply the inputs & weights, and add bias
 def lstm_factory_premul_bias(premul_cell, script):
-    def dynamic_rnn(input: Tensor, hidden: Tuple[Tensor, Tensor], wih: Tensor, whh: Tensor,
-                    bih: Tensor, bhh: Tensor) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
+    def dynamic_rnn(
+        input: Tensor,
+        hidden: Tuple[Tensor, Tensor],
+        wih: Tensor,
+        whh: Tensor,
+        bih: Tensor,
+        bhh: Tensor,
+    ) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
         hx, cx = hidden
         outputs = []
         inpSize = input.size()
@@ -445,7 +505,9 @@ def lstm_factory_simple(cell, script):
 
 
 def lstm_factory_multilayer(cell, script):
-    def dynamic_rnn(input: Tensor, hidden: Tuple[Tensor, Tensor], params: List[Tensor]) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
+    def dynamic_rnn(
+        input: Tensor, hidden: Tuple[Tensor, Tensor], params: List[Tensor]
+    ) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:
         params_stride = 4  # NB: this assumes that biases are there
         hx, cx = hidden
         hy, cy = hidden  # for scoping...
