@@ -7,11 +7,13 @@ from collections import namedtuple
 
 import benchmark_utils
 import numpy as np
+
 import torch
 
 # needs to be imported after torch
 import torch.utils.cpp_extension as cpp_extension  # noqa: F401
 
+
 """Performance microbenchmarks.
 
 This module contains core functionalities for performance microbenchmark tests.
@@ -35,7 +37,7 @@ def _register_test(*test_metainfo):
     1) This input when adds single op to the benchmark
      _register_test(configs, pt_bench_op, create_pytorch_op_test_case,
                       run_backward=True)
-    2) This input when addes a list of ops to the benchmark
+    2) This input when adds a list of ops to the benchmark
     _register_test(configs, pt_bench_op, create_pytorch_op_test_case,
                       run_backward=False,
                       op_name_function=op)
@@ -49,7 +51,7 @@ def _create_test(
     """Create tests with the benchmark backend.
     Args:
         bench_op_obj: an object which instantiated from a subclass of
-            Caffe2BenchmarkBase/TorchBenchmarkBase which includes tensor
+            TorchBenchmarkBase which includes tensor
             creation and operator execution.
         orig_test_attrs: a dictionary includes test configs.
         tags: a attribute in test config to filter inputs
@@ -74,7 +76,7 @@ def _build_test(
     """Generate PyTorch/Caffe2 tests of operators with different inputs.
     Args:
         configs: a dictionary that has the input shapes
-        bench_op: a subclass of Caffe2BenchmarkBase/TorchBenchmarkBase which includes tensor
+        bench_op: a subclass of TorchBenchmarkBase which includes tensor
             creation and operator execution
         OperatorTestCase: a named tuple to save the metadata of an test
         run_backward: a bool parameter indicating backward path
@@ -232,9 +234,7 @@ class BenchmarkRunner:
                     )
                 )
         else:
-            if test_case.framework == "PyTorch":
-                print(f"# Mode: {'JIT' if self.use_jit else 'Eager'}")
-
+            print(f"# Mode: {'JIT' if self.use_jit else 'Eager'}")
             print(
                 f"# Name: {test_case.test_config.test_name}\n# Input: {test_case.test_config.input_config}"
             )
@@ -282,8 +282,7 @@ class BenchmarkRunner:
         and the execution time is reported
         """
         test_case.run_forward(num_runs=1, print_per_iter=False, cuda_sync=False)
-        if test_case.framework == "PyTorch":
-            test_case._output_mean()
+        test_case._output_mean()
         backward_time = timeit.timeit(
             functools.partial(test_case.run_backward, iters, print_per_iter), number=1
         )
@@ -356,9 +355,6 @@ class BenchmarkRunner:
         # Currently, this is a sub-string matching.
         op_test_config = test_case.test_config
 
-        if self.args.framework:
-            frameworks = benchmark_utils.process_arg_list(self.args.framework)
-
         operators = (
             benchmark_utils.process_arg_list(self.args.operators)
             if self.args.operators
@@ -369,7 +365,6 @@ class BenchmarkRunner:
         if (
             self._check_keep(op_test_config.test_name, self.args.test_name)
             and self._check_keep_list(test_case.op_bench.module_name(), operators)
-            and self._check_keep_list(test_case.framework, frameworks)
             and self._check_operator_first_char(
                 test_case.op_bench.module_name(), self.operator_range
             )
