@@ -1,10 +1,9 @@
-import torch
-
 import operator_benchmark as op_bench
+import torch
 
 """Microbenchmarks for the quantized interpolate op.
 
-Note: We are not benchmarking `upsample` as it is being depricated, and calls
+Note: We are not benchmarking `upsample` as it is being deprecated, and calls
 the `interpolate` anyway.
 """
 
