@@ -9,11 +9,12 @@ from pprint import pprint
 from typing import List, Optional
 
 import numpy as np
+from prettytable import PrettyTable
+from tqdm import tqdm
+
 import torch
 import torch.utils.benchmark as benchmark
-from prettytable import PrettyTable
 from torch.backends.cuda import sdp_kernel
-from tqdm import tqdm
 
 warnings.filterwarnings("ignore")
 
@@ -29,6 +30,7 @@ class ExperimentConfig:
     enable_math: bool
     enable_flash: bool
     enable_mem_efficient: bool
+    enable_cudnn: bool
 
     def get_entries(self) -> List:
         return [
@@ -41,6 +43,7 @@ class ExperimentConfig:
             self.enable_math,
             self.enable_flash,
             self.enable_mem_efficient,
+            self.enable_cudnn,
         ]
 
     @classmethod
@@ -55,6 +58,7 @@ class ExperimentConfig:
             "enable_math",
             "enable_flash",
             "enable_mem_efficient",
+            "enable_cudnn",
         ]
 
 
@@ -78,10 +82,10 @@ class ExperimentResults:
     @classmethod
     def get_entry_names(cls) -> List[str]:
         return [
-            "nn_mha_time (μs)",
-            "compiled_nn_mha_time (μs)",
-            "composite_mha_time (μs)",
-            "compiled_composite_mha_time (μs)",
+            "nn_mha_time (\u00B5s)",
+            "compiled_nn_mha_time (\u00B5s)",
+            "composite_mha_time (\u00B5s)",
+            "compiled_composite_mha_time (\u00B5s)",
         ]
 
 
@@ -206,6 +210,7 @@ def run_single_experiment(config: ExperimentConfig) -> ExperimentResults:
         enable_math=config.enable_math,
         enable_flash=config.enable_flash,
         enable_mem_efficient=config.enable_mem_efficient,
+        enable_cudnn=config.enable_cudnn,
     ) as kernel_choice, torch.inference_mode() as inference_mode:
         dropout_p = 0.0
         mask = None
@@ -286,6 +291,7 @@ def generate_experiments(
                 enable_math=False,
                 enable_flash=True,
                 enable_mem_efficient=True,
+                enable_cudnn=True,
             )
         )
     return configs
@@ -307,6 +313,7 @@ def main(save_path: Optional[Path]):
         enable_math=False,
         enable_flash=True,
         enable_mem_efficient=True,
+        enable_cudnn=True,
     )
 
     experiment = run_single_experiment(config)
