@@ -3,6 +3,7 @@
 #include <c10/core/SymInt.h>
 #include <c10/core/SymNodeImpl.h>
 #include <c10/util/intrusive_ptr.h>
+#include <c10/util/safe_numerics.h>
 #include <functional>
 
 namespace c10 {
@@ -20,12 +21,14 @@ void SymInt::promote_to_negative() {
 }
 
 SymNode SymInt::toSymNode() const {
-  TORCH_CHECK(is_heap_allocated());
+  TORCH_CHECK_ALWAYS_SHOW_CPP_STACKTRACE(
+      is_heap_allocated(), "SymInt::toSymNode is_heap_allocated");
   return SymNode::reclaim_copy(toSymNodeImplUnowned());
 }
 
 SymInt::SymInt(SymNode sin_sp) {
-  TORCH_CHECK(sin_sp->is_int());
+  TORCH_CHECK_ALWAYS_SHOW_CPP_STACKTRACE(
+      sin_sp->is_int(), "SymInt::SymInt sin_sp->is_int()");
   auto ptr = static_cast<uint64_t>(
       reinterpret_cast<uintptr_t>(static_cast<void*>(sin_sp.release())));
   auto rep = (ptr & ~MASK) | IS_SYM;
@@ -82,6 +85,22 @@ SymInt::operator SymFloat() const {
   }
 }
 
+bool SymInt::is_same(const SymInt& other) const {
+  if (is_heap_allocated() != other.is_heap_allocated()) {
+    return false;
+  }
+  // Both not heap allocated
+  if (!is_heap_allocated() && this->operator!=(other)) {
+    return false;
+  }
+  // Both heap allocated
+  if (is_heap_allocated() &&
+      toSymNodeImplUnowned() != other.toSymNodeImplUnowned()) {
+    return false;
+  }
+  return true;
+}
+
 SymNode SymInt::wrap_node(const SymNode& base) const {
   if (auto ma = maybe_as_int()) {
     return base->wrap_int(*ma);
@@ -106,9 +125,31 @@ int64_t SymInt::guard_int(const char* file, int64_t line) const {
   }
 }
 
+bool SymInt::expect_size(const char* file, int64_t line) const {
+  if (auto ma = maybe_as_int()) {
+    return *ma >= 0;
+  } else {
+    return toSymNodeImplUnowned()->expect_size(file, line);
+  }
+}
+
 SymInt operator-(const SymInt& s) {
   if (auto ma = s.maybe_as_int()) {
-    return SymInt(-*ma);
+    const auto val = *ma;
+    // Note: Result of `-std::numeric_limits<decltype(val)>::min()` is undefined
+    // But on many platforms it equals to self + setting Carry/Overflow flags
+    // Which in opimized code affects results of `check_range` condition
+    // Workaround by using ternary that avoids alterning the flags
+#if C10_HAS_BUILTIN_OVERFLOW()
+    std::decay_t<decltype(val)> out = 0;
+    if (C10_UNLIKELY(__builtin_sub_overflow(out, val, &out))) {
+      return SymInt(val);
+    }
+    return SymInt(out);
+#else
+    constexpr auto val_min = std::numeric_limits<decltype(val)>::min();
+    return SymInt(val != val_min ? -val : val_min);
+#endif
   } else {
     return SymInt(s.toSymNodeImplUnowned()->neg());
   }
