@@ -1,11 +1,13 @@
-import pandas
 import matplotlib.pyplot as plt
+import pandas
 
 df = pandas.read_csv("perf.csv")
 
 ops = pandas.unique(df["operator"])
 nops = len(ops)
-pivot_op_shape = df.pivot_table(values="time", index=["operator", "shape"], columns=["fuser"])
+pivot_op_shape = df.pivot_table(
+    values="time", index=["operator", "shape"], columns=["fuser"]
+)
 pivot_speedups = (pivot_op_shape.T / pivot_op_shape["eager"]).T
 
 plt.rcParams["figure.figsize"] = (20, 100)
