@@ -4,15 +4,15 @@
 
 import torch
 from torch import nn
-from torch.nn.functional import mse_loss
 from torch.func import jacrev, vmap
+from torch.nn.functional import mse_loss
 
 sigma = 0.5
-epsilon = 4.
+epsilon = 4.0
 
 
 def lennard_jones(r):
-    return epsilon * ((sigma / r)**12 - (sigma / r)**6)
+    return epsilon * ((sigma / r) ** 12 - (sigma / r) ** 6)
 
 
 def lennard_jones_force(r):
@@ -29,7 +29,9 @@ norms = torch.norm(drs, dim=1).reshape(-1, 1)
 # Create training energies
 training_energies = torch.stack(list(map(lennard_jones, norms))).reshape(-1, 1)
 # Create forces with random direction vectors
-training_forces = torch.stack([force * dr for force, dr in zip(map(lennard_jones_force, norms), drs)])
+training_forces = torch.stack(
+    [force * dr for force, dr in zip(map(lennard_jones_force, norms), drs)]
+)
 
 model = nn.Sequential(
     nn.Linear(1, 16),
@@ -40,7 +42,7 @@ model = nn.Sequential(
     nn.Tanh(),
     nn.Linear(16, 16),
     nn.Tanh(),
-    nn.Linear(16, 1)
+    nn.Linear(16, 1),
 )
 
 
@@ -54,7 +56,10 @@ def make_prediction(model, drs):
 
 
 def loss_fn(energies, forces, predicted_energies, predicted_forces):
-    return mse_loss(energies, predicted_energies) + 0.01 * mse_loss(forces, predicted_forces) / 3
+    return (
+        mse_loss(energies, predicted_energies)
+        + 0.01 * mse_loss(forces, predicted_forces) / 3
+    )
 
 
 optimiser = torch.optim.Adam(model.parameters(), lr=1e-3)
