@@ -1,11 +1,9 @@
+from torchvision import models
+
 import torch
-import torchvision
+
+from torch.backends._coreml.preprocess import CompileSpec, CoreMLComputeUnit, TensorSpec
 
-from torch.backends._coreml.preprocess import (
-    CompileSpec,
-    TensorSpec,
-    CoreMLComputeUnit,
-)
 
 def mobilenetv2_spec():
     return {
@@ -27,7 +25,7 @@ def mobilenetv2_spec():
 
 
 def main():
-    model = torchvision.models.mobilenet_v2(pretrained=True)
+    model = models.mobilenet_v2(weights=models.MobileNet_V2_Weights.IMAGENET1K_V1)
     model.eval()
     example = torch.rand(1, 3, 224, 224)
     model = torch.jit.trace(model, example)
@@ -35,6 +33,7 @@ def main():
     mlmodel = torch._C._jit_to_backend("coreml", model, compile_spec)
     print(mlmodel._c._get_method("forward").graph)
     mlmodel._save_for_lite_interpreter("../models/model_coreml.ptl")
+    torch.jit.save(mlmodel, "../models/model_coreml.pt")
 
 
 if __name__ == "__main__":
