@@ -1,8 +1,9 @@
+from torchvision import models
+
 import torch
-import torchvision
 from torch.utils.mobile_optimizer import optimize_for_mobile
 
-model = torchvision.models.mobilenet_v2(pretrained=True)
+model = models.mobilenet_v2(weights=models.MobileNet_V2_Weights.IMAGENET1K_V1)
 model.eval()
 example = torch.rand(1, 3, 224, 224)
 traced_script_module = torch.jit.trace(model, example)
