@@ -106,11 +106,11 @@ categories = (
 
 
 topics = [
-    "bc_breaking",
-    "deprecations",
-    "new_features",
+    "bc breaking",
+    "deprecation",
+    "new features",
     "improvements",
-    "bug_fixes",
+    "bug fixes",
     "performance",
     "docs",
     "devs",
@@ -217,11 +217,15 @@ def run_query(query):
     if request.status_code == 200:
         return request.json()
     else:
-        raise Exception(
+        raise Exception(  # noqa: TRY002
             f"Query failed to run by returning code of {request.status_code}. {request.json()}"
         )
 
 
+_ERRORS = []
+_MAX_ERROR_LEN = 20
+
+
 def github_data(pr_number):
     query = (
         """
@@ -248,12 +252,20 @@ def github_data(pr_number):
         }
       }
     }
-    """
+    """  # noqa: UP031
         % pr_number
     )
     query = run_query(query)
     if query.get("errors"):
-        raise Exception(query["errors"])
+        global _ERRORS
+        _ERRORS.append(query.get("errors"))
+        if len(_ERRORS) < _MAX_ERROR_LEN:
+            return [], "None", ()
+        else:
+            raise Exception(  # noqa: TRY002
+                f"Got {_MAX_ERROR_LEN} errors: {_ERRORS}, please check if"
+                " there is something wrong"
+            )
     edges = query["data"]["repository"]["pullRequest"]["labels"]["edges"]
     labels = [edge["node"]["name"] for edge in edges]
     author = query["data"]["repository"]["pullRequest"]["author"]["login"]
@@ -309,7 +321,7 @@ class _CommitDataCache:
         return self.data[commit]
 
     def read_from_disk(self):
-        with open(self.path, "r") as f:
+        with open(self.path) as f:
             data = json.load(f)
             data = {commit: dict_to_features(dct) for commit, dct in data.items()}
         return data
