@@ -6,7 +6,7 @@ import torch.nn as nn
 
 class Model(nn.Module):
     def __init__(self):
-        super(Model, self).__init__()
+        super().__init__()
         self.linear = nn.Linear(20, 20)
 
     def forward(self, input):
@@ -25,5 +25,5 @@ def main():
         optimizer.step()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
