@@ -53,6 +53,23 @@ mobile::Module parse_mobile_module(
 }
 } // namespace
 
+TEST(FlatbufferTest, LoadMalformedModule) {
+  // Manually create some data with Flatbuffer header.
+  std::stringstream bad_data;
+  bad_data << "PK\x03\x04PTMF\x00\x00"
+           << "*}NV\xb3\xfa\xdf\x00pa";
+
+  // Loading module from it should throw an exception.
+  // Check guard at parse_and_initialize_mobile_module_for_jit.
+  ASSERT_THROWS_WITH_MESSAGE(
+      torch::jit::load(bad_data), "Malformed Flatbuffer module");
+
+  // Check guard at parse_and_initialize_mobile_module.
+  ASSERT_THROWS_WITH_MESSAGE(
+      parse_mobile_module(bad_data.str().data(), bad_data.str().size()),
+      "Malformed Flatbuffer module");
+}
+
 TEST(FlatbufferTest, UpsampleNearest2d) {
   Module m("m");
   m.define(R"(
@@ -232,6 +249,22 @@ TEST(FlatbufferTest, ExtraFiles) {
 
   ASSERT_EQ(loaded_extra_files["metadata.json"], "abc");
   ASSERT_EQ(loaded_extra_files["mobile_info.json"], "{\"key\": 23}");
+
+  // Test if flatbuffer does not require any explicit key entries mapping in the
+  // extra file map.
+  std::unordered_map<std::string, std::string>
+      loaded_extra_files_without_explicit_entries;
+  auto mobile_module3 = _load_for_mobile(
+      ss,
+      c10::nullopt,
+      loaded_extra_files_without_explicit_entries,
+      MobileModuleLoadOptions::PARSE_ALL_EXTRA_FILE_MAPS);
+
+  ASSERT_EQ(
+      loaded_extra_files_without_explicit_entries["metadata.json"], "abc");
+  ASSERT_EQ(
+      loaded_extra_files_without_explicit_entries["mobile_info.json"],
+      "{\"key\": 23}");
 }
 
 TEST(FlatbufferTest, Conv) {
