@@ -3,7 +3,9 @@
 import copy
 import functools
 import itertools
+import os
 import sys
+import unittest
 from typing import Any, Dict, List, Optional, Tuple, Type
 
 import torch
@@ -15,11 +17,18 @@ from torch.distributed.fsdp import (
     FullyShardedDataParallel as FSDP,
     MixedPrecision,
     ShardingStrategy,
+    StateDictType,
 )
 from torch.distributed.fsdp._common_utils import clean_tensor_name
+from torch.distributed.fsdp._flat_param import (
+    _FSDP_SKIP_WRITEBACK_CHECK,
+    _FSDP_USE_FULL_PREC_IN_EVAL,
+)
+from torch.distributed.fsdp._init_utils import NO_RESHARD_AFTER_FORWARD_STRATEGIES
 from torch.distributed.fsdp.wrap import always_wrap_policy, ModuleWrapPolicy
 from torch.nn import TransformerDecoderLayer, TransformerEncoderLayer
 from torch.nn.parallel.distributed import DistributedDataParallel as DDP
+from torch.testing._internal.common_cuda import TEST_CUDA
 from torch.testing._internal.common_distributed import skip_if_lt_x_gpu
 from torch.testing._internal.common_fsdp import (
     CUDAInitMode,
@@ -32,7 +41,9 @@ from torch.testing._internal.common_utils import (
     parametrize,
     run_tests,
     TEST_WITH_DEV_DBG_ASAN,
+    TestCase,
 )
+from torch.utils._triton import has_triton
 
 if not dist.is_available():
     print("Distributed not available, skipping tests", file=sys.stderr)
@@ -206,6 +217,59 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
             raise ValueError(f"Invalid string: {sharding_strategy_str}")
         return sharding_strategy
 
+    @unittest.skipIf(not has_triton(), "Inductor+gpu needs triton and recent GPU arch")
+    @skip_if_lt_x_gpu(2)
+    def test_fsdp_compile(self):
+        self.run_subtests(
+            {
+                "sharding_strategy": [
+                    ShardingStrategy.FULL_SHARD,
+                    ShardingStrategy.SHARD_GRAD_OP,
+                    ShardingStrategy.NO_SHARD,
+                ],
+                "skip_fsdp_guards": [True, False],
+            },
+            self._test_fsdp_compile,
+        )
+
+    def _test_fsdp_compile(
+        self, sharding_strategy: ShardingStrategy, skip_fsdp_guards: bool
+    ):
+        torch._dynamo.config.skip_fsdp_guards = skip_fsdp_guards
+        fsdp_kwargs = {
+            "auto_wrap_policy": ModuleWrapPolicy(
+                {
+                    TransformerEncoderLayer,
+                    TransformerDecoderLayer,
+                }
+            ),
+            "use_orig_params": True,
+            "sharding_strategy": sharding_strategy,
+            "backward_prefetch": BackwardPrefetch.BACKWARD_PRE,
+            "cpu_offload": CPUOffload(False),
+        }
+        base_model = TransformerWithSharedParams.init(
+            self.process_group,
+            FSDPInitMode.NO_FSDP,
+            CUDAInitMode.CUDA_BEFORE,
+            deterministic=True,
+        )
+        ref_model = FSDP(copy.deepcopy(base_model), self.process_group, **fsdp_kwargs)
+        ref_optim = torch.optim.Adam(ref_model.parameters(), lr=1e-2)
+        model = FSDP(copy.deepcopy(base_model), self.process_group, **fsdp_kwargs)
+        model = torch.compile(model)
+        optim = torch.optim.Adam(model.parameters(), lr=1e-2)
+        for i in range(10):
+            losses = []
+            inp = ref_model.get_input(torch.device("cuda"))
+            for _model, _optim in ((ref_model, ref_optim), (model, optim)):
+                _optim.zero_grad()
+                loss = _model(*inp).sum()
+                losses.append(loss)
+                loss.backward()
+                _optim.step()
+            self.assertEqual(losses[0], losses[1])
+
     @skip_if_lt_x_gpu(2)
     @parametrize(
         "sharding_strategy_str",
@@ -232,6 +296,7 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
                     BackwardPrefetch.BACKWARD_PRE,
                     BackwardPrefetch.BACKWARD_POST,
                 ],
+                "skip_writeback_check": [False, True],
             },
             self._test_diff_hyperparams,
             cpu_offload=CPUOffload(offload_params=False),
@@ -252,16 +317,18 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
         with ``offload_params=False`` followed by ``True`` but not vice versa).
         """
         sharding_strategy = self._get_sharding_strategy_from_str(sharding_strategy_str)
-        self._test_diff_hyperparams(
-            cuda_init_mode=CUDAInitMode.CUDA_BEFORE,
-            init_optim_before_wrap=False,
-            optim_class=torch.optim.Adam,
-            multi_tensor=False,
-            set_to_none=False,
-            backward_prefetch=BackwardPrefetch.BACKWARD_PRE,
-            cpu_offload=CPUOffload(offload_params=True),
-            sharding_strategy=sharding_strategy,
-        )
+        for skip_writeback_check in (False, True):
+            self._test_diff_hyperparams(
+                cuda_init_mode=CUDAInitMode.CUDA_BEFORE,
+                init_optim_before_wrap=False,
+                optim_class=torch.optim.Adam,
+                multi_tensor=False,
+                set_to_none=False,
+                backward_prefetch=BackwardPrefetch.BACKWARD_PRE,
+                cpu_offload=CPUOffload(offload_params=True),
+                sharding_strategy=sharding_strategy,
+                skip_writeback_check=skip_writeback_check,
+            )
 
     def _test_diff_hyperparams(
         self,
@@ -273,6 +340,7 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
         backward_prefetch: Optional[BackwardPrefetch],
         cpu_offload: CPUOffload,
         sharding_strategy: ShardingStrategy,
+        skip_writeback_check: bool,
     ):
         """
         Args:
@@ -284,6 +352,8 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
         """
         if cuda_init_mode == CUDAInitMode.CUDA_AFTER and cpu_offload.offload_params:
             return  # not supported
+        if skip_writeback_check:
+            os.environ[_FSDP_SKIP_WRITEBACK_CHECK] = "1"
         ddp_model = self._get_ddp_transformer(find_unused_params=False)
         ddp_optim = self._get_optim(ddp_model, optim_class, multi_tensor)
         fsdp_model, fsdp_optim = self._get_fsdp_transformer_and_optim(
@@ -402,17 +472,19 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
         # a bias in this rank's shard
         has_both = False
         for fsdp_module in FSDP.fsdp_modules(fsdp_model):
-            for handle in fsdp_module._handles:
-                flat_param = handle.flat_param
-                assert flat_param._params is not None
-                has_weight = False
-                has_bias = False
-                for param, fqn in zip(flat_param._params, flat_param._fqns):
-                    if "weight" in fqn and param.numel() > 0:
-                        has_weight = True
-                    elif "bias" in fqn and param.numel() > 0:
-                        has_bias = True
-                has_both |= has_weight and has_bias
+            handle = fsdp_module._handle
+            if not handle:
+                continue
+            flat_param = handle.flat_param
+            assert flat_param._params is not None
+            has_weight = False
+            has_bias = False
+            for param, fqn in zip(flat_param._params, flat_param._fqns):
+                if "weight" in fqn and param.numel() > 0:
+                    has_weight = True
+                elif "bias" in fqn and param.numel() > 0:
+                    has_bias = True
+            has_both |= has_weight and has_bias
         assert has_both, (
             f"Rank {self.rank} does not have a `FlatParameter` with both a "
             "weight and a bias in its shard, meaning that this test is vacuous"
@@ -445,6 +517,10 @@ class TestFSDPUseOrigParamsMultipleParamGroups(FSDPTest):
 
         # Check that FSDP correctly exposes gradients even after forward
         # (namely, `None` for weights and non-`None` for biases)
+        if sharding_strategy in NO_RESHARD_AFTER_FORWARD_STRATEGIES:
+            # Skip the check since we do not expose the gradients after forward
+            # for these strategies
+            return
         for (ddp_n, ddp_p), (fsdp_n, fsdp_p) in zip(
             ddp_model.module.named_parameters(),
             fsdp_model.named_parameters(),
@@ -686,8 +762,10 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
                 torch.manual_seed(42)
                 # 5 * 5 = 25 numel -> pad to 26 -> 13 on each rank
                 self.lin1 = nn.Linear(5, 5, bias=False)
-                # 5 * 7 + 7 = 42 numel -> no pad -> 21 on each rank
-                # 21 of weight on rank 0; 14 of weight and 7 of bias on rank 1
+                # 5 * 7 + (1) + 7 = 43 numel -> pad to 44 -> 22 on each rank,
+                # where the (1) is from intra-`FlatParameter` alignment padding
+                # 22 of weight on rank 0; 13 of weight, 1 alignment padding,
+                # and 7 of bias on rank 1
                 self.lin2 = nn.Linear(5, 7)
 
             def forward(self, x: torch.Tensor) -> torch.Tensor:
@@ -702,7 +780,9 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
             def get_loss(self, inp, out):
                 return out.sum()
 
-        def check_parameter_parity(ddp_model, fsdp_model):
+        def check_parameter_parity(
+            ddp_model: DDP, fsdp_model: FSDP, between_fwd_and_bwd: bool
+        ):
             assert self.rank in (
                 0,
                 1,
@@ -716,6 +796,13 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
                     # For `NO_SHARD`, do nothing since the original parameters
                     # are unflattened
                     pass
+                elif (
+                    between_fwd_and_bwd
+                    and sharding_strategy in NO_RESHARD_AFTER_FORWARD_STRATEGIES
+                ):
+                    # For no reshard after forward strategies, do nothing since
+                    # FSDP did not use sharded views after forward
+                    pass
                 # Otherwise, case on the parameter (see the model definition)
                 elif n1 == "lin1.weight":
                     if self.rank == 0:
@@ -724,9 +811,9 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
                         p1 = p1.flatten()[13:]
                 elif n1 == "lin2.weight":
                     if self.rank == 0:
-                        p1 = p1.flatten()[:21]
+                        p1 = p1.flatten()[:22]
                     elif self.rank == 1:
-                        p1 = p1.flatten()[21:]
+                        p1 = p1.flatten()[22:]
                 elif n1 == "lin2.bias":
                     if self.rank == 0:
                         p1 = torch.empty(0, device=p1.device)
@@ -749,7 +836,7 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
         inp = fsdp_model.get_input(device)
         ddp_out = ddp_model(*inp)
         fsdp_out = fsdp_model(*inp)
-        check_parameter_parity(ddp_model, fsdp_model)
+        check_parameter_parity(ddp_model, fsdp_model, True)
 
         ddp_loss = ddp_model.module.get_loss(inp, ddp_out)
         fsdp_loss = fsdp_model.get_loss(inp, fsdp_out)
@@ -757,23 +844,23 @@ class TestFSDPUseOrigParamsParamAccess(FSDPTest):
         fsdp_loss.backward()
         ddp_optim.step()
         fsdp_optim.step()
-        check_parameter_parity(ddp_model, fsdp_model)
+        check_parameter_parity(ddp_model, fsdp_model, False)
 
         inp = fsdp_model.get_input(device)
         ddp_out = ddp_model(*inp)
         fsdp_out = fsdp_model(*inp)
-        check_parameter_parity(ddp_model, fsdp_model)
+        check_parameter_parity(ddp_model, fsdp_model, True)
 
 
 class TestFSDPUseOrigParamsWriteback(FSDPTest):
     """Tests parameter and gradient writeback."""
 
     class Model(nn.Module):
-        def __init__(self):
+        def __init__(self, device: torch.device):
             super().__init__()
             torch.manual_seed(42)
-            self.lin1 = nn.Linear(5, 5, bias=True)
-            self.lin2 = nn.Linear(5, 7, bias=True)
+            self.lin1 = nn.Linear(5, 5, bias=True, device=device)
+            self.lin2 = nn.Linear(5, 7, bias=True, device=device)
 
         def forward(self, x: torch.Tensor) -> torch.Tensor:
             z = self.lin1(x)
@@ -819,10 +906,12 @@ class TestFSDPUseOrigParamsWriteback(FSDPTest):
 
         # Check that the writeback propagates
         ddp_model = DDP(
-            TestFSDPUseOrigParamsWriteback.Model().cuda(), device_ids=[self.rank]
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")),
+            device_ids=[self.rank],
         )
         fsdp_model = FSDP(
-            TestFSDPUseOrigParamsWriteback.Model().cuda(), use_orig_params=True
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")),
+            use_orig_params=True,
         )
         ddp = ddp_model.module  # for brevity
         fsdp = fsdp_model.module
@@ -870,10 +959,12 @@ class TestFSDPUseOrigParamsWriteback(FSDPTest):
             return None if set_to_none else torch.ones_like(param) * 2
 
         ddp_model = DDP(
-            TestFSDPUseOrigParamsWriteback.Model().cuda(), device_ids=[self.rank]
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")),
+            device_ids=[self.rank],
         )
         fsdp_model = FSDP(
-            TestFSDPUseOrigParamsWriteback.Model().cuda(), use_orig_params=True
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")),
+            use_orig_params=True,
         )
         LR = 1e-2
         # TODO: If we add `summon_full_params(with_grads=True)`, then replace
@@ -925,7 +1016,8 @@ class TestFSDPUseOrigParamsWriteback(FSDPTest):
     @skip_if_lt_x_gpu(2)
     def test_writeback_shape_mismatch(self):
         fsdp_model = FSDP(
-            TestFSDPUseOrigParamsWriteback.Model().cuda(), use_orig_params=True
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")),
+            use_orig_params=True,
         )
         # Check that writing back with mismatched shape errors
         fsdp = fsdp_model.module  # for brevity
@@ -962,6 +1054,91 @@ class TestFSDPUseOrigParamsWriteback(FSDPTest):
             with FSDP.summon_full_params(fsdp_model):  # triggers a writeback
                 ...
 
+    @skip_if_lt_x_gpu(2)
+    def test_writeback_between_fwd_and_bwd_for_no_reshard_raises(self):
+        fsdp_kwargs = {
+            "sharding_strategy": ShardingStrategy.SHARD_GRAD_OP,
+            "auto_wrap_policy": ModuleWrapPolicy({nn.Linear}),
+            "use_orig_params": True,
+        }
+        fsdp_wrapper = functools.partial(FSDP, **fsdp_kwargs)
+
+        # Test changing the parameter storage to no longer be a view into the
+        # flat parameter
+        fsdp_model = fsdp_wrapper(
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda"))
+        )
+        inp = fsdp_model.get_input(torch.device("cuda"))
+        loss = fsdp_model(*inp).sum()
+        fsdp_model.lin1.weight.data = fsdp_model.lin1.weight.clone()
+        assert_msg = (
+            "FSDP does not support changing the parameters between forward and backward"
+        )
+        with self.assertRaisesRegex(AssertionError, assert_msg):
+            loss.backward()
+
+        # Test changing the parameter variable itself
+        fsdp_model = fsdp_wrapper(
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda"))
+        )
+        inp = fsdp_model.get_input(torch.device("cuda"))
+        loss = fsdp_model(*inp).sum()
+        fsdp_model.lin1._fsdp_wrapped_module.weight = nn.Parameter(
+            fsdp_model.lin1.weight.clone()
+        )
+        with self.assertRaisesRegex(AssertionError, assert_msg):
+            loss.backward()
+
+    @skip_if_lt_x_gpu(2)
+    def test_no_reshard_and_mixed_precision(self):
+        """
+        Tests that writeback does not falsely get triggered for a few
+        configurations (exercising the sharded view skipping logic):
+        - Train forward -> full-precision unshard -> train forward
+        - Train forward -> eval forward
+        - Train forward/backward -> eval forward -> model checkpoint
+        """
+        self.run_subtests(
+            {"use_full_prec_in_eval": [False, True]},
+            self._test_no_reshard_and_mixed_precision,
+        )
+
+    def _test_no_reshard_and_mixed_precision(self, use_full_prec_in_eval: bool):
+        if use_full_prec_in_eval:
+            os.environ[_FSDP_USE_FULL_PREC_IN_EVAL] = "1"
+        fsdp_kwargs = {
+            "sharding_strategy": ShardingStrategy.SHARD_GRAD_OP,
+            "auto_wrap_policy": ModuleWrapPolicy({nn.Linear}),
+            "mixed_precision": MixedPrecision(param_dtype=torch.float16),
+            "use_orig_params": True,
+        }
+
+        # Train forward -> full-precision unshard -> train forward
+        fsdp_model = FSDP(
+            TestFSDPUseOrigParamsWriteback.Model(torch.device("cuda")), **fsdp_kwargs
+        )
+        inp = fsdp_model.get_input(torch.device("cuda"))
+        fsdp_model(*inp)
+        with FSDP.summon_full_params(fsdp_model):
+            ...
+        fsdp_model(*inp).sum()
+
+        # Train forward -> eval forward
+        fsdp_model.train()
+        fsdp_model(*inp)
+        fsdp_model.eval()
+        fsdp_model(*inp)
+
+        # Train forward/backward -> eval forward -> model checkpoint
+        fsdp_model.train()
+        fsdp_model(*inp).sum().backward()
+        fsdp_model.eval()
+        fsdp_model(*inp)
+        with FSDP.state_dict_type(fsdp_model, StateDictType.SHARDED_STATE_DICT):
+            sd = fsdp_model.state_dict()
+            fsdp_model.load_state_dict(sd)
+        fsdp_model(*inp).sum().backward()
+
 
 class TestFSDPUseOrigParamsFQNs(FSDPTest):
     @skip_if_lt_x_gpu(2)
@@ -1032,7 +1209,7 @@ class TestFSDPUseOrigParamsNoSync(FSDPTest):
         )
 
     def _test_no_sync_correctness(self, sharding_strategy: ShardingStrategy):
-        model = nn.Linear(3, 3, device="cuda")
+        model = nn.Linear(7, 1, bias=False, device="cuda")
         fsdp_kwargs = {
             "sharding_strategy": sharding_strategy,
         }
@@ -1052,11 +1229,13 @@ class TestFSDPUseOrigParamsNoSync(FSDPTest):
             _test_model: nn.Module,
         ):
             """
-            This assumes that the model is ``nn.Linear(3, 3, bias=False)``
-            (i.e. with a single weight parameter) to be able to directly
+            This assumes that the model is ``nn.Linear(7, 1, bias=False)``
+            (i.e. with a single 1D weight parameter) to be able to directly
             compare the baseline and test models. On rank 1, the baseline
             includes 1 element of padding.
             """
+            self.assertEqual(len(list(_baseline_model.parameters())), 1)
+            self.assertEqual(len(list(_test_model.parameters())), 1)
             for flat_param, orig_param in zip(
                 _baseline_model.parameters(), _test_model.parameters()
             ):
@@ -1080,8 +1259,8 @@ class TestFSDPUseOrigParamsNoSync(FSDPTest):
                     orig_param.grad,
                 )
 
-        inp = torch.randn((2, 3), device="cuda")
-        grad = torch.rand_like(inp)
+        inp = torch.randn((2, 7), device="cuda")
+        grad = torch.randn((2, 1), device="cuda")
 
         # Compute some reference gradients using one forward/backward
         out_use_flat_params = model_use_flat_params(inp)
@@ -1178,6 +1357,43 @@ class TestFSDPUseOrigParamsNoSync(FSDPTest):
                 self.assertEqual(param.grad.dtype, torch.float32)
 
 
+class TestFSDPUseOrigParamsInit(FSDPTest):
+    @skip_if_lt_x_gpu(2)
+    def test_non_uniform_requires_grad(self):
+        model = nn.Sequential(
+            nn.Linear(3, 3, device="cuda"),
+            nn.Linear(3, 3, device="cuda"),
+        )
+        # Freeze biases only and flatten both weights and biases into the same
+        # `FlatParameter` to exercise non-uniform `requires_grad`
+        model[0].bias.requires_grad = False
+        model[1].bias.requires_grad = False
+        fsdp_model = FSDP(model, use_orig_params=True)
+        self.assertTrue(fsdp_model[0].weight.requires_grad)
+        self.assertFalse(fsdp_model[0].bias.requires_grad)
+        self.assertTrue(fsdp_model[1].weight.requires_grad)
+        self.assertFalse(fsdp_model[1].bias.requires_grad)
+
+
+# Define this to be large enough to trigger stack corruption
+NUM_SIZE0_TENSORS = 1000
+
+
+class TestMultiTensorApply(TestCase):
+    def test_multi_tensor_apply_size0_tensors_cpu(self):
+        size0_tensors = [torch.empty(0, device="cpu") for _ in range(NUM_SIZE0_TENSORS)]
+        # Check that this does not segfault
+        torch._foreach_mul_(size0_tensors, 0.1)
+
+    @unittest.skipIf(not TEST_CUDA, "no cuda")
+    def test_multi_tensor_apply_size0_tensors_cuda(self):
+        size0_tensors = [
+            torch.empty(0, device="cuda") for _ in range(NUM_SIZE0_TENSORS)
+        ]
+        # Check that this does not segfault
+        torch._foreach_mul_(size0_tensors, 0.1)
+
+
 instantiate_parametrized_tests(TestFSDPUseOrigParamsMultipleParamGroups)
 instantiate_parametrized_tests(TestFSDPUseOrigParamsUnshardReshard)
 instantiate_parametrized_tests(TestFSDPUseOrigParamsParamAccess)
