@@ -1,20 +1,19 @@
 # Owner(s): ["oncall: distributed"]
 
 import sys
+
 import test_c10d_spawn
+from test_c10d_spawn import _torch_dist_nn_available, TestDistributedNNFunctions
+
 import torch
 import torch.distributed as c10d
-from test_c10d_spawn import _torch_dist_nn_available, TestDistributedNNFunctions
 from torch.testing._internal.common_cuda import TEST_MULTIGPU
-from torch.testing._internal.common_distributed import (
-    requires_nccl,
-    skip_if_lt_x_gpu,
-)
+from torch.testing._internal.common_distributed import requires_nccl, skip_if_lt_x_gpu
 from torch.testing._internal.common_utils import (
-    TestCase,
     run_tests,
     skip_but_pass_in_sandcastle_if,
     TEST_WITH_DEV_DBG_ASAN,
+    TestCase,
 )
 
 NO_NCCL = not hasattr(c10d, "ProcessGroupNCCL")
@@ -30,7 +29,9 @@ if sys.version_info < (3, 9):
             store = c10d.FileStore(filename, world_size)
             return c10d.ProcessGroupNCCL(store, rank, world_size)
 
-        @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "At least 2 CUDA GPUS needed")
+        @skip_but_pass_in_sandcastle_if(
+            not TEST_MULTIGPU, "At least 2 CUDA GPUS needed"
+        )
         @skip_but_pass_in_sandcastle_if(NO_NCCL, "NCCL needed")
         def test_shared_broadcast_nccl(self):
             self._test_multiprocess(
@@ -40,7 +41,9 @@ if sys.version_info < (3, 9):
                 1,
             )
 
-        @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "At least 2 CUDA GPUS needed")
+        @skip_but_pass_in_sandcastle_if(
+            not TEST_MULTIGPU, "At least 2 CUDA GPUS needed"
+        )
         @skip_but_pass_in_sandcastle_if(NO_NCCL, "NCCL needed")
         def test_shared_allreduce_nccl(self):
             self._test_multiprocess(
@@ -63,7 +66,9 @@ if sys.version_info < (3, 9):
                 c2p.put((rank, torch.ones(2, 2), x.to("cpu")))
             p2c.get()
 
-        @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "At least 2 CUDA GPUS needed")
+        @skip_but_pass_in_sandcastle_if(
+            not TEST_MULTIGPU, "At least 2 CUDA GPUS needed"
+        )
         @skip_but_pass_in_sandcastle_if(NO_NCCL, "NCCL needed")
         def test_shared_reduce_nccl(self):
             self._test_multiprocess(
@@ -73,7 +78,9 @@ if sys.version_info < (3, 9):
                 1,
             )
 
-        @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "At least 2 CUDA GPUS needed")
+        @skip_but_pass_in_sandcastle_if(
+            not TEST_MULTIGPU, "At least 2 CUDA GPUS needed"
+        )
         @skip_but_pass_in_sandcastle_if(NO_NCCL, "NCCL needed")
         def test_shared_allgather_nccl(self):
             self._test_multiprocess(
@@ -99,50 +106,66 @@ if not TEST_WITH_DEV_DBG_ASAN:
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_reduce(self):
             self._test_reduce("nccl")
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_allreduce(self):
             self._test_allreduce("nccl")
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_all_gather(self):
             self._test_all_gather("nccl")
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_all_to_all(self):
             self._test_all_to_all("nccl")
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_all_to_all_single(self):
             self._test_all_to_all_single("nccl")
 
         # Test Ops only supported in NCCL.
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_reduce_scatter(self):
             store = c10d.FileStore(self.file_name, self.world_size)
             # This is required because these functions calls directly to the .dist and needs
             # the world to be initialized
-            c10d.init_process_group(store=store, rank=self.rank, world_size=self.world_size, backend='nccl')
+            c10d.init_process_group(
+                store=store, rank=self.rank, world_size=self.world_size, backend="nccl"
+            )
             device = torch.device(f"cuda:{self.rank}")
             x0 = torch.ones(5, 5, device=device) + self.rank
             x1 = torch.ones(5, 5, device=device) + self.rank + 1
             x0.requires_grad = True
             x1.requires_grad = True
             y = torch.empty_like(x0)
-            expected = (1 + self.world_size) * self.world_size / 2 + self.world_size * self.rank
+            expected = (
+                1 + self.world_size
+            ) * self.world_size / 2 + self.world_size * self.rank
             y = torch.distributed.nn.reduce_scatter(y, [x0, x1])
             self.assertEqual(y, torch.ones(5, 5, device=device) * expected)
             z = y.sin().sum()
@@ -156,16 +179,19 @@ if not TEST_WITH_DEV_DBG_ASAN:
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_reduce_scatter_non_contiguous(self):
             store = c10d.FileStore(self.file_name, self.world_size)
             # This is required because these functions calls directly to the .dist and needs
             # the world to be initialized
-            c10d.init_process_group(store=store, rank=self.rank, world_size=self.world_size, backend='nccl')
+            c10d.init_process_group(
+                store=store, rank=self.rank, world_size=self.world_size, backend="nccl"
+            )
             device = torch.device(f"cuda:{self.rank}")
 
             class NonContiguousGrad(torch.autograd.Function):
-
                 @staticmethod
                 def forward(ctx, input):
                     return input
@@ -184,10 +210,14 @@ if not TEST_WITH_DEV_DBG_ASAN:
 
         @requires_nccl()
         @skip_if_lt_x_gpu(2)
-        @skip_but_pass_in_sandcastle_if(not _torch_dist_nn_available, "torch.distributed.nn is not available")
+        @skip_but_pass_in_sandcastle_if(
+            not _torch_dist_nn_available, "torch.distributed.nn is not available"
+        )
         def test_all_gather_base(self):
             store = c10d.FileStore(self.file_name, self.world_size)
-            c10d.init_process_group(store=store, rank=self.rank, world_size=self.world_size, backend='nccl')
+            c10d.init_process_group(
+                store=store, rank=self.rank, world_size=self.world_size, backend="nccl"
+            )
 
             device = torch.device(f"cuda:{self.rank}")
             x = torch.ones(5, 5, device=device) + self.rank
@@ -198,7 +228,10 @@ if not TEST_WITH_DEV_DBG_ASAN:
             self.assertEqual(output.size(), torch.Size((5 * self.world_size, 5)))
 
             for idx in range(self.world_size):
-                self.assertEqual(output[5 * idx : 5 * (idx + 1)], torch.ones(5, 5, device=device) + idx)
+                self.assertEqual(
+                    output[5 * idx : 5 * (idx + 1)],
+                    torch.ones(5, 5, device=device) + idx,
+                )
 
             y = torch.sum(output.view(self.world_size, 5, 5), axis=0)
             z = y.sin().sum()
