@@ -18,15 +18,16 @@ if not c10d.is_available() or not c10d.is_ucc_available():
     sys.exit(0)
 
 import test_c10d_common
+from test_c10d_common import (
+    gpus_for_rank,
+    ModuleForDdpCommHook,
+    SparseGradientModule,
+    Task,
+)
+
 import torch.distributed as dist
 import torch.nn.functional as F
 import torch.testing._internal.common_utils as common
-from test_c10d_common import (
-    gpus_for_rank,
-    Task,
-    ModuleForDdpCommHook,
-    SparseGradientModule,
-)
 from torch import nn
 from torch.nn.parallel import DistributedDataParallel
 from torch.testing._internal.common_distributed import (
@@ -36,10 +37,10 @@ from torch.testing._internal.common_distributed import (
     verify_ddp_error_logged,
 )
 from torch.testing._internal.common_utils import (
-    TestCase,
-    run_tests,
     retry_on_connect_failures,
+    run_tests,
     skip_but_pass_in_sandcastle,
+    TestCase,
 )
 
 
@@ -207,7 +208,7 @@ class ProcessGroupUCCTest(MultiProcessTestCase):
 
         # Single input tests
         tests = simple_reduce_tests(self.rank, self.world_size)
-        for (op, input, expected) in tests:
+        for op, input, expected in tests:
             opts = c10d.AllreduceOptions()
             opts.reduceOp = op
             tensor = fn(input)
@@ -260,7 +261,7 @@ class ProcessGroupUCCTest(MultiProcessTestCase):
 
     def _test_reduce_basics(self, fn):
         pg = self._create_process_group_ucc()
-        for (op, input, output) in simple_reduce_tests(self.rank, self.world_size):
+        for op, input, output in simple_reduce_tests(self.rank, self.world_size):
             for root in range(self.world_size):
                 opts = c10d.ReduceOptions()
                 opts.reduceOp = op
@@ -346,7 +347,9 @@ class DistributedDataParallelTest(
 
     def _get_process_group(self):
         store = self._get_store()
-        c10d.init_process_group("ucc", store=store, rank=self.rank, world_size=self.world_size)
+        c10d.init_process_group(
+            "ucc", store=store, rank=self.rank, world_size=self.world_size
+        )
         return c10d.distributed_c10d._get_default_group()
 
     def _test_ucc_backend(
@@ -383,7 +386,9 @@ class DistributedDataParallelTest(
 
     # TODO: test_ucc_backend_2gpu_module and test_ucc_backend_4gpu_module
     # require broadcast_coalesced which is not supported by ucc currently
-    @skip_but_pass_in_sandcastle("requires broadcast coalesced, which is not supported by ucc currently")
+    @skip_but_pass_in_sandcastle(
+        "requires broadcast coalesced, which is not supported by ucc currently"
+    )
     @requires_ucc()
     @skip_if_lt_x_gpu(4)
     def test_ucc_backend_2gpu_module(self):
@@ -391,7 +396,9 @@ class DistributedDataParallelTest(
         devices = [torch.device("cuda:" + str(i)) for i in int_devices]
         self._test_ucc_backend(devices, None, multi_device=True)
 
-    @skip_but_pass_in_sandcastle("requires broadcast coalesced, which is not supported by ucc currently")
+    @skip_but_pass_in_sandcastle(
+        "requires broadcast coalesced, which is not supported by ucc currently"
+    )
     @requires_ucc()
     @skip_if_lt_x_gpu(8)
     def test_ucc_backend_4gpu_module(self):
@@ -646,7 +653,9 @@ class DistributedDataParallelTest(
         # Check that the gradients are sparse and identical
         vanilla_parameter = next(vanilla_model.parameters())
         ddp_parameter = next(ddp_model.parameters())
-        self.assertEqual(vanilla_parameter.grad.coalesce(), ddp_parameter.grad.coalesce())
+        self.assertEqual(
+            vanilla_parameter.grad.coalesce(), ddp_parameter.grad.coalesce()
+        )
 
     @requires_ucc()
     @skip_if_lt_x_gpu(2)
@@ -874,7 +883,9 @@ class DistributedDataParallelTest(
             ModuleForDdpCommHook(), process_group=process_group
         )
 
-        expected_err = "Communication hook: return annotation should be torch.futures.Future"
+        expected_err = (
+            "Communication hook: return annotation should be torch.futures.Future"
+        )
         with self.assertRaisesRegex(
             ValueError,
             expected_err,
@@ -1035,7 +1046,6 @@ class CommTest(test_c10d_common.AbstractCommTest, MultiProcessTestCase):
 
 
 class CompilerTest(test_c10d_common.CompilerTest):
-
     @property
     def world_size(self):
         return 2
@@ -1058,15 +1068,11 @@ class CompilerTest(test_c10d_common.CompilerTest):
 
     @skip_if_lt_x_gpu(2)
     def test_allgather_work_wait_gpu(self):
-        self._test_allgather_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
+        self._test_allgather_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
 
     @skip_if_lt_x_gpu(2)
     def test_broadcast_work_wait_gpu(self):
-        self._test_broadcast_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
+        self._test_broadcast_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
 
     @skip_if_lt_x_gpu(2)
     def test_nested_comm_tensor_wrapping_gpu(self):
@@ -1074,6 +1080,7 @@ class CompilerTest(test_c10d_common.CompilerTest):
             torch.ones(2, 2, device=self.rank) * self.rank
         )
 
+    @skip_if_lt_x_gpu(2)
     def test_consecutive_comm_work_wait_gpu(self):
         self._test_consecutive_comm_work_wait(
             torch.ones(2, 2, device=self.rank) * self.rank
@@ -1085,28 +1092,21 @@ class CompilerTest(test_c10d_common.CompilerTest):
         )
 
     def test_allgather_work_wait_cpu(self):
-        self._test_allgather_work_wait(
-            torch.ones(2, 2) * self.rank
-        )
+        self._test_allgather_work_wait(torch.ones(2, 2) * self.rank)
 
     def test_broadcast_work_wait_cpu(self):
-        self._test_broadcast_work_wait(
-            torch.ones(2, 2) * self.rank
-        )
+        self._test_broadcast_work_wait(torch.ones(2, 2) * self.rank)
 
     def test_nested_comm_tensor_wrapping_cpu(self):
-        self._test_nested_comm_tensor_wrapping(
-            torch.ones(2, 2) * self.rank
-        )
+        self._test_nested_comm_tensor_wrapping(torch.ones(2, 2) * self.rank)
 
     def test_consecutive_comm_work_wait_cpu(self):
-        self._test_consecutive_comm_work_wait(
-            torch.ones(2, 2) * self.rank
-        )
+        self._test_consecutive_comm_work_wait(torch.ones(2, 2) * self.rank)
 
 
-class UccProcessGroupWithDispatchedCollectivesTests(test_c10d_common.ProcessGroupWithDispatchedCollectivesTests):
-
+class UccProcessGroupWithDispatchedCollectivesTests(
+    test_c10d_common.ProcessGroupWithDispatchedCollectivesTests
+):
     @skip_but_pass_in_sandcastle("Fails on M60")
     @requires_ucc()
     @skip_if_lt_x_gpu(1)
