@@ -2,54 +2,25 @@
 import typing
 import unittest
 
-from torch.testing._internal.common_utils import (
-    TestCase,
-    run_tests,
-    instantiate_parametrized_tests,
-    parametrize,
-    subtest
-)
-
 from torch._C import (
     _dispatch_get_registrations_for_dispatch_key as get_registrations_for_dispatch_key,
 )
 
+from torch.testing._internal.common_utils import (
+    instantiate_parametrized_tests,
+    parametrize,
+    run_tests,
+    subtest,
+    TestCase,
+)
+
 xfail_functorch_batched = {
-    "aten::flatten.using_ints",
-    "aten::imag",
     "aten::is_nonzero",
-    "aten::isfinite",
-    "aten::isreal",
     "aten::item",
-    "aten::linalg_pinv",
-    "aten::linalg_pinv.atol_rtol_float",
     "aten::linalg_slogdet",
-    "aten::linalg_lu_factor",
-    "aten::linear",
-    "aten::log_sigmoid",
-    "aten::log_softmax.int",
-    "aten::logdet",
     "aten::masked_select_backward",
-    "aten::movedim.intlist",
     "aten::one_hot",
-    "aten::real",
     "aten::silu_backward",
-    "aten::special_xlogy",
-    "aten::special_xlogy.other_scalar",
-    "aten::special_xlogy.self_scalar",
-    "aten::tensor_split.indices",
-    "aten::tensor_split.sections",
-    "aten::to.device",
-    "aten::to.dtype",
-    "aten::to.dtype_layout",
-    "aten::to.other",
-    "aten::upsample_bicubic2d.vec",
-    "aten::upsample_bilinear2d.vec",
-    "aten::upsample_linear1d.vec",
-    "aten::upsample_nearest1d.vec",
-    "aten::upsample_nearest2d.vec",
-    "aten::upsample_nearest3d.vec",
-    "aten::upsample_trilinear3d.vec",
     "aten::where",
 }
 
@@ -60,7 +31,6 @@ xfail_functorch_batched_decomposition = {
 }
 
 xfail_not_implemented = {
-    "aten::absolute_",
     "aten::affine_grid_generator_backward",
     "aten::align_as",
     "aten::align_tensors",
@@ -68,13 +38,6 @@ xfail_not_implemented = {
     "aten::align_to.ellipsis_idx",
     "aten::alpha_dropout",
     "aten::alpha_dropout_",
-    "aten::arccos_",
-    "aten::arccosh_",
-    "aten::arcsin_",
-    "aten::arcsinh_",
-    "aten::arctan2_",
-    "aten::arctan_",
-    "aten::arctanh_",
     "aten::argwhere",
     "aten::bilinear",
     "aten::can_cast",
@@ -111,7 +74,6 @@ xfail_not_implemented = {
     "aten::fill_diagonal_",
     "aten::fix_",
     "aten::flatten.named_out_dim",
-    "aten::flatten.using_ints",
     "aten::flatten.using_names",
     "aten::flatten_dense_tensors",
     "aten::float_power_.Scalar",
@@ -130,11 +92,8 @@ xfail_not_implemented = {
     "aten::histogramdd",
     "aten::histogramdd.TensorList_bins",
     "aten::histogramdd.int_bins",
-    "aten::imag",
     "aten::infinitely_differentiable_gelu_backward",
     "aten::isclose",
-    "aten::isfinite",
-    "aten::isreal",
     "aten::istft",
     "aten::item",
     "aten::kl_div",
@@ -144,23 +103,15 @@ xfail_not_implemented = {
     "aten::less_equal_.Scalar",
     "aten::less_equal_.Tensor",
     "aten::linalg_cond.p_str",
-    "aten::linalg_eigh",
     "aten::linalg_eigh.eigvals",
-    "aten::linalg_lu_factor",
     "aten::linalg_matrix_rank",
     "aten::linalg_matrix_rank.out_tol_tensor",
     "aten::linalg_matrix_rank.tol_tensor",
-    "aten::linalg_pinv",
-    "aten::linalg_pinv.atol_rtol_float",
     "aten::linalg_pinv.out_rcond_tensor",
     "aten::linalg_pinv.rcond_tensor",
     "aten::linalg_slogdet",
     "aten::linalg_svd.U",
     "aten::linalg_tensorsolve",
-    "aten::linear",
-    "aten::log_sigmoid",
-    "aten::log_softmax.int",
-    "aten::logdet",
     "aten::logsumexp.names",
     "aten::lstm.data",
     "aten::lstm.input",
@@ -179,7 +130,6 @@ xfail_not_implemented = {
     "aten::min.names_dim_min",
     "aten::mish_backward",
     "aten::moveaxis.int",
-    "aten::movedim.intlist",
     "aten::multilabel_margin_loss",
     "aten::nanmedian.names_dim",
     "aten::nanmedian.names_dim_values",
@@ -204,7 +154,6 @@ xfail_not_implemented = {
     "aten::qr.Q",
     "aten::quantile",
     "aten::quantile.scalar",
-    "aten::real",
     "aten::refine_names",
     "aten::rename",
     "aten::rename_",
@@ -246,9 +195,6 @@ xfail_not_implemented = {
     "aten::special_shifted_chebyshev_polynomial_v.x_scalar",
     "aten::special_shifted_chebyshev_polynomial_w.n_scalar",
     "aten::special_shifted_chebyshev_polynomial_w.x_scalar",
-    "aten::special_xlogy",
-    "aten::special_xlogy.other_scalar",
-    "aten::special_xlogy.self_scalar",
     "aten::square_",
     "aten::sspaddmm",
     "aten::std.correction_names",
@@ -266,45 +212,35 @@ xfail_not_implemented = {
     "aten::sym_stride.int",
     "aten::sym_numel",
     "aten::sym_storage_offset",
-    "aten::tensor_split.indices",
-    "aten::tensor_split.sections",
     "aten::tensor_split.tensor_indices_or_sections",
     "aten::thnn_conv2d",
-    "aten::to.device",
-    "aten::to.dtype",
-    "aten::to.dtype_layout",
-    "aten::to.other",
     "aten::to_dense",
     "aten::to_dense_backward",
     "aten::to_mkldnn_backward",
     "aten::trace_backward",
     "aten::triplet_margin_loss",
     "aten::unflatten_dense_tensors",
-    "aten::unsafe_chunk",
-    "aten::upsample_bicubic2d.vec",
-    "aten::upsample_bilinear2d.vec",
-    "aten::upsample_linear1d.vec",
-    "aten::upsample_nearest1d.vec",
-    "aten::upsample_nearest2d.vec",
-    "aten::upsample_nearest3d.vec",
-    "aten::upsample_trilinear3d.vec",
     "aten::vander",
     "aten::var.correction_names",
     "aten::var.names_dim",
     "aten::var_mean.correction_names",
     "aten::var_mean.names_dim",
     "aten::where",
-
 }
 
 
 def dispatch_registrations(
-        dispatch_key: str, xfails: set, filter_func: typing.Callable = lambda reg: True):
+    dispatch_key: str, xfails: set, filter_func: typing.Callable = lambda reg: True
+):
     registrations = sorted(get_registrations_for_dispatch_key(dispatch_key))
     subtests = [
-        subtest(reg, name=f"[{reg}]",
-                decorators=([unittest.expectedFailure] if reg in xfails else []))
-        for reg in registrations if filter_func(reg)
+        subtest(
+            reg,
+            name=f"[{reg}]",
+            decorators=([unittest.expectedFailure] if reg in xfails else []),
+        )
+        for reg in registrations
+        if filter_func(reg)
     ]
     return parametrize("registration", subtests)
 
@@ -330,17 +266,17 @@ def filter_vmap_implementable(reg):
         return False
     if reg.endswith("_out"):
         return False
-    if '.dimname' in reg:
+    if ".dimname" in reg:
         return False
     if "_dimname" in reg:
         return False
-    if 'fbgemm' in reg:
+    if "fbgemm" in reg:
         return False
-    if 'quantize' in reg:
+    if "quantize" in reg:
         return False
-    if 'sparse' in reg:
+    if "sparse" in reg:
         return False
-    if '::is_' in reg:
+    if "::is_" in reg:
         return False
     return True
 
