@@ -1,19 +1,24 @@
 # Owner(s): ["module: fx"]
 
 import unittest
-import torch
-from torch.fx import symbolic_trace
-from torch.fx.experimental.unify_refinements import infer_symbolic_types
-from torch.fx.experimental.refinement_types import Equality
-from torch.fx.tensor_type import TensorType, Dyn, is_consistent, is_more_precise
-from torch.fx.annotate import annotate
-from torch.fx.experimental.graph_gradual_typechecker import GraphTypeChecker, broadcast_types, Refine
-from torch.fx.experimental.rewriter import RewritingTracer
-from torch.fx import GraphModule
-from torch.fx.passes.shape_prop import ShapeProp
-from torch.testing._internal.common_utils import TestCase
+
 import sympy
 
+import torch
+from torch.fx import GraphModule, symbolic_trace
+from torch.fx.annotate import annotate
+from torch.fx.experimental.graph_gradual_typechecker import (
+    broadcast_types,
+    GraphTypeChecker,
+    Refine,
+)
+from torch.fx.experimental.refinement_types import Equality
+from torch.fx.experimental.rewriter import RewritingTracer
+from torch.fx.experimental.unify_refinements import infer_symbolic_types
+from torch.fx.passes.shape_prop import ShapeProp
+from torch.fx.tensor_type import Dyn, is_consistent, is_more_precise, TensorType
+from torch.testing._internal.common_utils import TestCase
+
 
 try:
     from torchvision.models import resnet50
@@ -23,24 +28,33 @@ except ImportError:
     HAS_TORCHVISION = False
 skipIfNoTorchVision = unittest.skipIf(not HAS_TORCHVISION, "no torchvision")
 
+
 def conv3x3(in_planes, out_planes, stride=1, groups=1, dilation=1):
     """3x3 convolution with padding"""
-    return torch.nn.Conv2d(in_planes, out_planes, kernel_size=3, stride=stride,
-                           padding=dilation, groups=groups, bias=False, dilation=dilation)
+    return torch.nn.Conv2d(
+        in_planes,
+        out_planes,
+        kernel_size=3,
+        stride=stride,
+        padding=dilation,
+        groups=groups,
+        bias=False,
+        dilation=dilation,
+    )
+
 
 class AnnotationsTest(TestCase):
-
     def test_annotations(self):
         """
         Test type annotations in the forward function.
-        The annoation should appear in the n.graph
-        where n is the corresoinding node in the resulting graph.
+        The annotation should appear in the n.graph
+        where n is the corresponding node in the resulting graph.
         """
+
         class M(torch.nn.Module):
-            def forward(self,
-                        x: TensorType((1, 2, 3, Dyn)),
-                        y: Dyn,
-                        z: TensorType[Dyn, 3, Dyn]):
+            def forward(
+                self, x: TensorType((1, 2, 3, Dyn)), y: Dyn, z: TensorType[Dyn, 3, Dyn]
+            ):
                 return torch.add(x, y) + z
 
         module = M()
@@ -50,20 +64,19 @@ class AnnotationsTest(TestCase):
         expected_iter = iter(expected_ph_types)
 
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == next(expected_iter)
 
     def test_annotate(self):
         class M(torch.nn.Module):
-
             def forward(self, x):
                 y = annotate(x, TensorType((1, 2, 3, Dyn)))
                 return torch.add(x, y)
 
         module = M()
-        symbolic_traced : torch.fx.GraphModule = symbolic_trace(module)
+        symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == TensorType((1, 2, 3, Dyn))
 
     def test_consistency(self):
@@ -83,7 +96,9 @@ class AnnotationsTest(TestCase):
         self.assertTrue(is_more_precise(TensorType((1, 2, 3)), TensorType((1, Dyn, 3))))
         self.assertTrue(is_more_precise(int, Dyn))
         self.assertTrue(is_more_precise(int, int))
-        self.assertFalse(is_more_precise(TensorType((1, 2, 3)), TensorType((1, 2, 3, 5))))
+        self.assertFalse(
+            is_more_precise(TensorType((1, 2, 3)), TensorType((1, 2, 3, 5)))
+        )
         self.assertFalse(is_more_precise(TensorType((1, 2, 3)), int))
 
     def test_broadcasting1(self):
@@ -94,7 +109,10 @@ class AnnotationsTest(TestCase):
         t5 = TensorType((4, 4, 4))
         # todo switch all code to use list instead of tuple
         t6 = TensorType([1])
-        assert broadcast_types(t1, t2) == (TensorType((1, 2, 3, 4)), TensorType((1, 2, 3, 4)))
+        assert broadcast_types(t1, t2) == (
+            TensorType((1, 2, 3, 4)),
+            TensorType((1, 2, 3, 4)),
+        )
         assert broadcast_types(t3, t4) == (t4, t4)
         assert broadcast_types(t5, t6) == (t5, t5)
 
@@ -102,46 +120,58 @@ class AnnotationsTest(TestCase):
         t1 = TensorType((2, 3, 4))
         t2 = TensorType((1, 2, 1, 4))
 
-        assert broadcast_types(t1, t2) == (TensorType((1, 2, 3, 4)), TensorType((1, 2, 3, 4)))
+        assert broadcast_types(t1, t2) == (
+            TensorType((1, 2, 3, 4)),
+            TensorType((1, 2, 3, 4)),
+        )
 
     def test_broadcasting3(self):
         t1 = TensorType((1, 2, 3, Dyn))
         t2 = TensorType((2, 3, 4))
-        assert broadcast_types(t1, t2) == (TensorType((1, 2, 3, Dyn)), TensorType((1, 2, 3, 4)))
+        assert broadcast_types(t1, t2) == (
+            TensorType((1, 2, 3, Dyn)),
+            TensorType((1, 2, 3, 4)),
+        )
+
 
 class TypeCheckerTest(TestCase):
-
     def test_type_check_add_with_broadcast(self):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2, 3, Dyn)), y: TensorType((2, 3, 4))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
         tc.type_check()
-        expected_ph_types = [TensorType((1, 2, 3, Dyn)),
-                             TensorType((2, 3, 4)),
-                             TensorType((1, 2, 3, Dyn)),
-                             TensorType((1, 2, 3, Dyn))]
+        expected_ph_types = [
+            TensorType((1, 2, 3, Dyn)),
+            TensorType((2, 3, 4)),
+            TensorType((1, 2, 3, Dyn)),
+            TensorType((1, 2, 3, Dyn)),
+        ]
         expected_iter = iter(expected_ph_types)
 
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'call_function':
-                assert n.meta['broadcast']
+            if n.op == "call_function":
+                assert n.meta["broadcast"]
             assert n.type == next(expected_iter)
 
     def test_type_check_add_with_scalar(self):
         class M(torch.nn.Module):
             def forward(self, x: int, y: TensorType((2, 3, 4))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
         tc.type_check()
-        expected_ph_types = [int,
-                             TensorType((2, 3, 4)),
-                             TensorType((2, 3, 4)),
-                             TensorType((2, 3, 4))]
+        expected_ph_types = [
+            int,
+            TensorType((2, 3, 4)),
+            TensorType((2, 3, 4)),
+            TensorType((2, 3, 4)),
+        ]
         expected_iter = iter(expected_ph_types)
 
         for n in symbolic_traced.graph.nodes:
@@ -151,6 +181,7 @@ class TypeCheckerTest(TestCase):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2, 3, Dyn)), y: TensorType((1, 2, 3))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
@@ -161,6 +192,7 @@ class TypeCheckerTest(TestCase):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2, Dyn)), y: TensorType((1, 2, 3))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
@@ -170,9 +202,9 @@ class TypeCheckerTest(TestCase):
         expected_iter = iter(expected_ph_types)
 
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == next(expected_iter)
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((1, 2, Dyn))
 
     def test_type_check_reshape_true(self):
@@ -186,13 +218,13 @@ class TypeCheckerTest(TestCase):
         self.assertTrue(tc.type_check())
 
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == TensorType((1, 6))
 
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == TensorType((1, 2, 3))
 
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((1, 2, 3))
 
     def test_type_check_reshape_false(self):
@@ -244,16 +276,16 @@ class TypeCheckerTest(TestCase):
                 return torch.transpose(x, 0, 1)
 
         module = M()
-        symbolic_traced : torch.fx.GraphModule = symbolic_trace(module)
+        symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
         self.assertTrue(tc.type_check())
 
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == TensorType([2, 1, 3, 5])
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType([2, 1, 3, 5])
-            if n.op == 'x':
+            if n.op == "x":
                 assert n.placeholder == TensorType([1, 2, 3, 5])
 
     def test_type_check_transpose_False(self):
@@ -269,7 +301,6 @@ class TypeCheckerTest(TestCase):
 
     def test_type_check_batch_norm_2D(self):
         class BasicBlock(torch.nn.Module):
-
             def __init__(self, inplanes, planes):
                 super().__init__()
                 norm_layer = torch.nn.BatchNorm2d
@@ -289,18 +320,17 @@ class TypeCheckerTest(TestCase):
         tc.type_check()
 
         for n in graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == TensorType((2, 2, 5, 4))
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((2, 2, 5, 4))
-            if n.op == 'call_module':
+            if n.op == "call_module":
                 assert n.type == TensorType((2, 2, 5, 4))
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == TensorType((2, 2, 5, 4))
 
     def test_type_check_batch_norm_2D_false(self):
         class BasicBlock(torch.nn.Module):
-
             def __init__(self, inplanes, planes):
                 super().__init__()
                 norm_layer = torch.nn.BatchNorm2d
@@ -322,7 +352,6 @@ class TypeCheckerTest(TestCase):
 
     def test_type_check_batch_norm_2D_broadcast(self):
         class BasicBlock(torch.nn.Module):
-
             def __init__(self, inplanes, planes):
                 super().__init__()
                 norm_layer = torch.nn.BatchNorm2d
@@ -341,13 +370,13 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, traced)
         tc.type_check()
         for n in graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'call_module':
+            if n.op == "call_module":
                 assert n.type == TensorType((2, 2, Dyn, 4))
 
         B = BasicBlock(1, 1)
@@ -379,13 +408,13 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, traced)
         tc.type_check()
         for n in graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((Dyn, Dyn, Dyn, Dyn))
-            if n.op == 'call_module':
+            if n.op == "call_module":
                 assert n.type == TensorType((2, 2, Dyn, 4))
 
     def test_type_check_conv2D_2(self):
@@ -412,13 +441,13 @@ class TypeCheckerTest(TestCase):
         tc.type_check()
         t = TensorType((5, 2, 3, 4))
         for n in graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == t
-            if n.op == 'call_function':
+            if n.op == "call_function":
                 assert n.type == t
-            if n.op == 'output':
+            if n.op == "output":
                 assert torch.Size(n.type.__args__) == b.shape
-            if n.op == 'call_module':
+            if n.op == "call_module":
                 assert n.type == t
 
         B = BasicBlock(1, 2)
@@ -430,12 +459,27 @@ class TypeCheckerTest(TestCase):
             tc.type_check()
 
     def test_type_check_conv2D_2_fully_static(self):
-        annotation_list = [(1, 2, 3, 5), (2, 5, 6, 9), (10, 15, 13, 14),
-                           (10, Dyn, 13, 14), (Dyn, Dyn, Dyn, 3)]
-        input_list = [(1, 2, 3, 5), (2, 5, 6, 9), (10, 15, 13, 14),
-                      (10, 15, 13, 14), (1, 2, 2, 3)]
-        intermediate_types = [(1, Dyn, Dyn, 7), (2, Dyn, 4, 6), (10, 15, Dyn, 5),
-                              (10, 15, 7, 7), (1, Dyn, Dyn, Dyn)]
+        annotation_list = [
+            (1, 2, 3, 5),
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, Dyn, 13, 14),
+            (Dyn, Dyn, Dyn, 3),
+        ]
+        input_list = [
+            (1, 2, 3, 5),
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, 15, 13, 14),
+            (1, 2, 2, 3),
+        ]
+        intermediate_types = [
+            (1, Dyn, Dyn, 7),
+            (2, Dyn, 4, 6),
+            (10, 15, Dyn, 5),
+            (10, 15, 7, 7),
+            (1, Dyn, Dyn, Dyn),
+        ]
         in_planes_list = [2, 5, 15, 15, 2]
         stride_list = [1, 2, 3, 2, 2]
         out_planes_list = [2, 5, 15, 15, 2]
@@ -443,7 +487,13 @@ class TypeCheckerTest(TestCase):
         dilation_list = [1, 2, 3, 3, 3]
         padding_list = [1, 2, 3, 3, 3]
         kernel_size_list = [1, 2, 3, 3, 3]
-        output_types = [(1, 2, Dyn, 7), (2, 5, 4, 6), (10, 15, Dyn, 5), (10, 15, 7, 7), (1, 2, Dyn, Dyn)]
+        output_types = [
+            (1, 2, Dyn, 7),
+            (2, 5, 4, 6),
+            (10, 15, Dyn, 5),
+            (10, 15, 7, 7),
+            (1, 2, Dyn, Dyn),
+        ]
 
         for i in range(5):
             annotation = annotation_list[i]
@@ -458,24 +508,42 @@ class TypeCheckerTest(TestCase):
             intermediate_type = intermediate_types[i]
 
             class BasicBlock(torch.nn.Module):
-                def __init__(self, in_planes, out_planes, kernel_size, stride, padding, groups, dilation):
+                def __init__(
+                    self,
+                    in_planes,
+                    out_planes,
+                    kernel_size,
+                    stride,
+                    padding,
+                    groups,
+                    dilation,
+                ):
                     super().__init__()
-                    self.conv1 = torch.nn.Conv2d(in_channels=in_planes, out_channels=out_planes,
-                                                 kernel_size=kernel_size, stride=stride,
-                                                 padding=padding, groups=groups, bias=False, dilation=dilation)
+                    self.conv1 = torch.nn.Conv2d(
+                        in_channels=in_planes,
+                        out_channels=out_planes,
+                        kernel_size=kernel_size,
+                        stride=stride,
+                        padding=padding,
+                        groups=groups,
+                        bias=False,
+                        dilation=dilation,
+                    )
 
                 def forward(self, x):
                     out = self.conv1(x)
                     return out
 
-            B = BasicBlock(in_planes, out_planes, kernel_size, stride, padding, groups, dilation)
+            B = BasicBlock(
+                in_planes, out_planes, kernel_size, stride, padding, groups, dilation
+            )
             ast_rewriter = RewritingTracer()
             graph = ast_rewriter.trace(B)
             traced = GraphModule(ast_rewriter.root, graph, "gm")
 
             # annotate our argument
             for n in graph.nodes:
-                if n.op == 'placeholder':
+                if n.op == "placeholder":
                     n.type = TensorType(annotation)
 
             b = B.forward(torch.rand(input))
@@ -483,36 +551,54 @@ class TypeCheckerTest(TestCase):
             tc.type_check()
 
             for n in graph.nodes:
-                if n.op == 'output':
+                if n.op == "output":
                     assert is_consistent(n.type, TensorType(b.size()))
 
             # test with intermediate annotations
             class BasicBlock(torch.nn.Module):
-                def __init__(self, in_planes, out_planes, kernel_size, stride, padding, groups, dilation):
+                def __init__(
+                    self,
+                    in_planes,
+                    out_planes,
+                    kernel_size,
+                    stride,
+                    padding,
+                    groups,
+                    dilation,
+                ):
                     super().__init__()
-                    self.conv1 = torch.nn.Conv2d(in_channels=in_planes, out_channels=out_planes,
-                                                 kernel_size=kernel_size, stride=stride,
-                                                 padding=padding, groups=groups, bias=False, dilation=dilation)
+                    self.conv1 = torch.nn.Conv2d(
+                        in_channels=in_planes,
+                        out_channels=out_planes,
+                        kernel_size=kernel_size,
+                        stride=stride,
+                        padding=padding,
+                        groups=groups,
+                        bias=False,
+                        dilation=dilation,
+                    )
 
                 def forward(self, x):
                     out = self.conv1(x)
                     return out
 
-            B = BasicBlock(in_planes, out_planes, kernel_size, stride, padding, groups, dilation)
+            B = BasicBlock(
+                in_planes, out_planes, kernel_size, stride, padding, groups, dilation
+            )
             ast_rewriter = RewritingTracer()
             graph = ast_rewriter.trace(B)
             traced = GraphModule(ast_rewriter.root, graph, "gm")
 
             # populate our intermediate notes
             for n in traced.graph.nodes:
-                if n.op == 'call_module':
+                if n.op == "call_module":
                     n.type = TensorType(intermediate_type)
 
             tc = GraphTypeChecker({}, traced)
             tc.type_check()
 
             for n in traced.graph.nodes:
-                if n.op == 'output':
+                if n.op == "output":
                     assert n.type == TensorType(output_types[i])
                     assert is_consistent(n.type, TensorType(b.size()))
 
@@ -520,14 +606,26 @@ class TypeCheckerTest(TestCase):
         class BasicBlock(torch.nn.Module):
             expansion = 1
 
-            def __init__(self, inplanes, planes, stride=1, downsample=None, groups=1,
-                         base_width=64, dilation=1):
+            def __init__(
+                self,
+                inplanes,
+                planes,
+                stride=1,
+                downsample=None,
+                groups=1,
+                base_width=64,
+                dilation=1,
+            ):
                 super().__init__()
                 norm_layer = torch.nn.BatchNorm2d
                 if groups != 1 or base_width != 64:
-                    raise ValueError('BasicBlock only supports groups=1 and base_width=64')
+                    raise ValueError(
+                        "BasicBlock only supports groups=1 and base_width=64"
+                    )
                 if dilation > 1:
-                    raise NotImplementedError("Dilation > 1 not supported in BasicBlock")
+                    raise NotImplementedError(
+                        "Dilation > 1 not supported in BasicBlock"
+                    )
                 # Both self.conv1 and self.downsample layers downsample the input when stride != 1
                 self.conv1 = conv3x3(inplanes, planes, stride)
                 self.bn1 = norm_layer(planes)
@@ -565,12 +663,14 @@ class TypeCheckerTest(TestCase):
         tc.type_check()
 
         for n in traced.graph.nodes:
-            if n.target == 'output':
+            if n.target == "output":
                 assert isinstance(n.type, TensorType)
-                assert torch.Size(n.type.__args__) == B.forward(torch.rand(2, 2, 4, 5)).size()
+                assert (
+                    torch.Size(n.type.__args__)
+                    == B.forward(torch.rand(2, 2, 4, 5)).size()
+                )
 
     def test_type_check_conv2D_maxpool2d_flatten(self):
-
         class BasicBlock(torch.nn.Module):
             def __init__(self):
                 super().__init__()
@@ -581,7 +681,7 @@ class TypeCheckerTest(TestCase):
                 self.fc1 = torch.nn.Linear(5, 120)
                 self.pool2 = torch.nn.AdaptiveAvgPool2d((6, 7))
 
-            def forward(self, x : TensorType((4, 3, 32, 32))):
+            def forward(self, x: TensorType((4, 3, 32, 32))):
                 out = self.conv1(x)
                 out = self.pool(out)
                 out = self.conv2(out)
@@ -598,10 +698,17 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, traced)
         tc.type_check()
 
-        expected_ph_types = [TensorType((4, 3, 32, 32)), TensorType((4, 6, 28, 28)),
-                             TensorType((4, 6, 14, 14)), TensorType((4, 16, 10, 10)),
-                             TensorType((4, 16, 5, 5)), TensorType((4, 16, 5, 120)),
-                             TensorType((4, 16, 6, 7)), TensorType((4, 672)), TensorType((4, 672))]
+        expected_ph_types = [
+            TensorType((4, 3, 32, 32)),
+            TensorType((4, 6, 28, 28)),
+            TensorType((4, 6, 14, 14)),
+            TensorType((4, 16, 10, 10)),
+            TensorType((4, 16, 5, 5)),
+            TensorType((4, 16, 5, 120)),
+            TensorType((4, 16, 6, 7)),
+            TensorType((4, 672)),
+            TensorType((4, 672)),
+        ]
 
         expected_iter = iter(expected_ph_types)
         traced.graph.eliminate_dead_code()
@@ -619,10 +726,9 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, symbolic_traced)
         tc.type_check()
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((1, 6, 5, Dyn))
 
-
     def test_type_check_flatten_2(self):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, Dyn, 3, 5, Dyn))):
@@ -633,7 +739,7 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, symbolic_traced)
         tc.type_check()
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((1, Dyn, 5, Dyn))
 
     def test_type_check_flatten3(self):
@@ -646,7 +752,7 @@ class TypeCheckerTest(TestCase):
         tc = GraphTypeChecker({}, symbolic_traced)
         tc.type_check()
         for n in symbolic_traced.graph.nodes:
-            if n.op == 'output':
+            if n.op == "output":
                 assert n.type == TensorType((2, 60))
         r = Refine(symbolic_traced)
         r.refine()
@@ -654,13 +760,12 @@ class TypeCheckerTest(TestCase):
         assert c == [Equality(2, 2)]
 
     def test_type_typechecl_maxpool2d_3dinput(self):
-
         class BasicBlock(torch.nn.Module):
             def __init__(self):
                 super().__init__()
                 self.pool = torch.nn.MaxPool2d(5, 8)
 
-            def forward(self, x : TensorType((64, 8, 8))):
+            def forward(self, x: TensorType((64, 8, 8))):
                 out = self.pool(x)
                 return out
 
@@ -672,21 +777,42 @@ class TypeCheckerTest(TestCase):
         tc.type_check()
 
         for n in traced.graph.nodes:
-            if n.target == 'output':
+            if n.target == "output":
                 assert n.type == TensorType((64, 1, 1))
 
     def test_type_maxpool2d_fully_static(self):
-        annotation_list = [(Dyn, Dyn, 3, 5), (2, 5, 6, 9), (10, 15, 13, 14),
-                           (10, Dyn, 13, 14), (Dyn, Dyn, Dyn, 10)]
-        input_list = [(1, 2, 3, 5), (2, 5, 6, 9), (10, 15, 13, 14),
-                      (10, 15, 13, 14), (2, 2, 10, 10)]
-        intermediate_types = [(1, 2, Dyn, Dyn), (2, Dyn, 2, 4), (10, 15, Dyn, 2),
-                              (10, 15, 2, 3), (2, Dyn, Dyn, Dyn)]
+        annotation_list = [
+            (Dyn, Dyn, 3, 5),
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, Dyn, 13, 14),
+            (Dyn, Dyn, Dyn, 10),
+        ]
+        input_list = [
+            (1, 2, 3, 5),
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, 15, 13, 14),
+            (2, 2, 10, 10),
+        ]
+        intermediate_types = [
+            (1, 2, Dyn, Dyn),
+            (2, Dyn, 2, 4),
+            (10, 15, Dyn, 2),
+            (10, 15, 2, 3),
+            (2, Dyn, Dyn, Dyn),
+        ]
         stride_list = [1, 2, 3, 2, 1]
         dilation_list = [1, 2, 3, 3, 2]
         padding_list = [1, 2, 3, 3, 1]
         kernel_size_list = [2, 4, 6, 6, 3]
-        output_types = [(1, 2, 4, 6), (2, 5, 2, 4), (10, 15, 2, 2), (10, 15, 2, 3), (2, Dyn, Dyn, 8)]
+        output_types = [
+            (1, 2, 4, 6),
+            (2, 5, 2, 4),
+            (10, 15, 2, 2),
+            (10, 15, 2, 3),
+            (2, Dyn, Dyn, 8),
+        ]
 
         for i in range(5):
             annotation = annotation_list[i]
@@ -700,9 +826,14 @@ class TypeCheckerTest(TestCase):
             class BasicBlock(torch.nn.Module):
                 def __init__(self, kernel_size, stride, padding, dilation):
                     super().__init__()
-                    self.pool = torch.nn.MaxPool2d(kernel_size, stride=stride,
-                                                   padding=padding, dilation=dilation,
-                                                   return_indices=False, ceil_mode=False)
+                    self.pool = torch.nn.MaxPool2d(
+                        kernel_size,
+                        stride=stride,
+                        padding=padding,
+                        dilation=dilation,
+                        return_indices=False,
+                        ceil_mode=False,
+                    )
 
                 def forward(self, x):
                     out = self.pool(x)
@@ -715,7 +846,7 @@ class TypeCheckerTest(TestCase):
 
             # annotate our argument
             for n in graph.nodes:
-                if n.op == 'placeholder':
+                if n.op == "placeholder":
                     n.type = TensorType(annotation)
 
             b = B.forward(torch.rand(input))
@@ -723,16 +854,21 @@ class TypeCheckerTest(TestCase):
             tc.type_check()
 
             for n in graph.nodes:
-                if n.op == 'output':
+                if n.op == "output":
                     assert is_consistent(n.type, TensorType(b.size()))
 
             # test with intermediate annotations
             class BasicBlock(torch.nn.Module):
                 def __init__(self, kernel_size, stride, padding, dilation):
                     super().__init__()
-                    self.pool = torch.nn.MaxPool2d(kernel_size, stride=stride,
-                                                   padding=padding, dilation=dilation,
-                                                   return_indices=False, ceil_mode=False)
+                    self.pool = torch.nn.MaxPool2d(
+                        kernel_size,
+                        stride=stride,
+                        padding=padding,
+                        dilation=dilation,
+                        return_indices=False,
+                        ceil_mode=False,
+                    )
 
                 def forward(self, x):
                     out = self.pool(x)
@@ -745,30 +881,45 @@ class TypeCheckerTest(TestCase):
 
             # annotate our argument
             for n in graph.nodes:
-                if n.op == 'placeholder':
+                if n.op == "placeholder":
                     n.type = TensorType(annotation)
 
             # populate our intermediate notes
             for n in traced.graph.nodes:
-                if n.op == 'call_module':
+                if n.op == "call_module":
                     n.type = TensorType(intermediate_type)
 
             tc = GraphTypeChecker({}, traced)
             tc.type_check()
 
             for n in traced.graph.nodes:
-                if n.op == 'output':
+                if n.op == "output":
                     assert n.type == TensorType(output_types[i])
                     assert is_consistent(n.type, TensorType(b.size()))
 
     def test_flatten_fully_static(self):
-        annotation_list = [Dyn, TensorType((2, 5, 6, 9)), TensorType((10, 15, 13, 14)),
-                           TensorType((10, Dyn, 13, 14)), TensorType((Dyn, Dyn, Dyn, 10))]
-        input_list = [(1, 2, 3, 5), (2, 5, 6, 9), (10, 15, 13, 14),
-                      (10, 15, 13, 14), (2, 2, 10, 10)]
+        annotation_list = [
+            Dyn,
+            TensorType((2, 5, 6, 9)),
+            TensorType((10, 15, 13, 14)),
+            TensorType((10, Dyn, 13, 14)),
+            TensorType((Dyn, Dyn, Dyn, 10)),
+        ]
+        input_list = [
+            (1, 2, 3, 5),
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, 15, 13, 14),
+            (2, 2, 10, 10),
+        ]
 
-        intermediate_list = [Dyn, (2, 5, 6, 9), (10, 15, 13, 14),
-                             (10, 15, 13, 14), (2, 2, 10, 10)]
+        intermediate_list = [
+            Dyn,
+            (2, 5, 6, 9),
+            (10, 15, 13, 14),
+            (10, 15, 13, 14),
+            (2, 2, 10, 10),
+        ]
 
         start_dim = [1, 2, 1, 2, 0]
         end_dim = [1, 3, 3, 3, -2]
@@ -795,7 +946,7 @@ class TypeCheckerTest(TestCase):
 
             # annotate our argument
             for n in graph.nodes:
-                if n.op == 'placeholder':
+                if n.op == "placeholder":
                     n.type = annotation
 
             b = B.forward(torch.rand(input))
@@ -803,7 +954,7 @@ class TypeCheckerTest(TestCase):
             tc.type_check()
 
             for n in graph.nodes:
-                if n.op == 'output':
+                if n.op == "output":
                     assert is_consistent(n.type, TensorType(b.size()))
 
     @skipIfNoTorchVision
@@ -825,36 +976,34 @@ class TypeCheckerTest(TestCase):
         gm_run.graph.eliminate_dead_code()
         # here we are checking for consistency with fully dynamic nodes
         for n1, n2 in zip(gm_static.graph.nodes, gm_run.graph.nodes):
-            assert is_consistent(n1.type, TensorType(n2.meta['tensor_meta'].shape))
+            assert is_consistent(n1.type, TensorType(n2.meta["tensor_meta"].shape))
 
-        # here we give the same input as to runtume
+        # here we give the same input as to runtime
         gm_static_with_types = symbolic_trace(resnet50())
 
         # we initialize our placeholder
         for n in gm_static_with_types.graph.nodes:
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 n.type = TensorType((1, 3, 224, 224))
 
         g = GraphTypeChecker({}, gm_static_with_types)
         g.type_check()
         for n1, n2 in zip(gm_static_with_types.graph.nodes, gm_run.graph.nodes):
-            assert n1.type == TensorType(n2.meta['tensor_meta'].shape)
+            assert n1.type == TensorType(n2.meta["tensor_meta"].shape)
 
         # apply shape inference to graph and check
         # that the batch size is equal across all layers
         infer_symbolic_types(gm_static)
 
-
         batch_sizes = set()
         gm_static.graph.eliminate_dead_code()
         for n in gm_static.graph.nodes:
             assert isinstance(n.type, TensorType)
             batch_sizes.add(n.type.__args__[0])
-        assert (len(batch_sizes) == 1)
+        assert len(batch_sizes) == 1
 
     def test_type_check_batch_norm_symbolic(self):
         class BasicBlock(torch.nn.Module):
-
             def __init__(self, inplanes, planes):
                 super().__init__()
                 norm_layer = torch.nn.BatchNorm2d
@@ -875,10 +1024,14 @@ class TypeCheckerTest(TestCase):
 
         infer_symbolic_types(traced)
 
-        my_types = iter([TensorType[(2, 2, sympy.symbols('~7'), 4)],
-                         TensorType[(2, 2, sympy.symbols('~7'), 4)],
-                         TensorType[(2, 2, sympy.symbols('~7'), 4)],
-                         TensorType[(2, 2, sympy.symbols('~7'), 4)]])
+        my_types = iter(
+            [
+                TensorType[(2, 2, sympy.symbols("~7"), 4)],
+                TensorType[(2, 2, sympy.symbols("~7"), 4)],
+                TensorType[(2, 2, sympy.symbols("~7"), 4)],
+                TensorType[(2, 2, sympy.symbols("~7"), 4)],
+            ]
+        )
 
         for n in graph.nodes:
             assert n.type == next(my_types)
@@ -887,6 +1040,7 @@ class TypeCheckerTest(TestCase):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2, 3, Dyn)), y: TensorType((2, 3, 4))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
@@ -901,13 +1055,14 @@ class TypeCheckerTest(TestCase):
 
         infer_symbolic_types(symbolic_traced)
 
-        expected_ph_types = [TensorType((1, 2, 3, sympy.symbols('~0'))),
-                             TensorType((2, 3, 4)),
-                             TensorType((1, 2, 3, sympy.symbols('~1'))),
-                             TensorType((1, 2, 3, sympy.symbols('~1')))]
+        expected_ph_types = [
+            TensorType((1, 2, 3, sympy.symbols("~0"))),
+            TensorType((2, 3, 4)),
+            TensorType((1, 2, 3, sympy.symbols("~1"))),
+            TensorType((1, 2, 3, sympy.symbols("~1"))),
+        ]
         expected_iter = iter(expected_ph_types)
 
-
         for n in symbolic_traced.graph.nodes:
             assert n.type == next(expected_iter)
 
@@ -915,6 +1070,7 @@ class TypeCheckerTest(TestCase):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2)), y: TensorType((Dyn, 2))):
                 return torch.add(x, y)
+
         module = M()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         tc = GraphTypeChecker({}, symbolic_traced)
@@ -923,10 +1079,12 @@ class TypeCheckerTest(TestCase):
         r = Refine(symbolic_traced)
         r.refine()
 
-        expected_ph_types = [TensorType((1, 2)),
-                             TensorType((sympy.symbols('~1'), 2)),
-                             TensorType((sympy.symbols('~1'), 2)),
-                             TensorType((sympy.symbols('~1'), 2))]
+        expected_ph_types = [
+            TensorType((1, 2)),
+            TensorType((sympy.symbols("~1"), 2)),
+            TensorType((sympy.symbols("~1"), 2)),
+            TensorType((sympy.symbols("~1"), 2)),
+        ]
         expected_iter = iter(expected_ph_types)
 
         for n in symbolic_traced.graph.nodes:
@@ -955,12 +1113,11 @@ class TypeCheckerTest(TestCase):
         infer_symbolic_types(traced)
 
         for n in traced.graph.nodes:
-            if n.op == 'call_module':
+            if n.op == "call_module":
                 assert isinstance(n.type.__args__[2], sympy.floor)
                 assert isinstance(n.type.__args__[3], sympy.floor)
 
     def test_type_check_symbolic_inferenceconv2D_maxpool2d_flatten(self):
-
         class BasicBlock(torch.nn.Module):
             def __init__(self):
                 super().__init__()
@@ -971,7 +1128,7 @@ class TypeCheckerTest(TestCase):
                 self.fc1 = torch.nn.Linear(5, 120)
                 self.pool2 = torch.nn.AdaptiveAvgPool2d((6, 7))
 
-            def forward(self, x : TensorType((4, 3, Dyn, Dyn))):
+            def forward(self, x: TensorType((4, 3, Dyn, Dyn))):
                 out = self.conv1(x)
                 out = self.pool(out)
                 out = self.conv2(out)
@@ -989,13 +1146,26 @@ class TypeCheckerTest(TestCase):
         infer_symbolic_types(traced)
 
         for n in traced.graph.nodes:
-            if n.target == 'conv1':
-                assert n.type == TensorType((4, 6, sympy.floor(sympy.symbols('~0') - 4),
-                                             sympy.floor(sympy.symbols('~1') - 4)))
+            if n.target == "conv1":
+                assert n.type == TensorType(
+                    (
+                        4,
+                        6,
+                        sympy.floor(sympy.symbols("~0") - 4),
+                        sympy.floor(sympy.symbols("~1") - 4),
+                    )
+                )
 
-            elif n.target == 'conv2':
-                assert n.type == TensorType((4, 16, sympy.floor(sympy.symbols('~4') - 4),
-                                             sympy.floor(sympy.symbols('~5') - 4)))
+            elif n.target == "conv2":
+                assert n.type == TensorType(
+                    (
+                        4,
+                        16,
+                        sympy.floor(sympy.symbols("~4") - 4),
+                        sympy.floor(sympy.symbols("~5") - 4),
+                    )
+                )
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     unittest.main()
