@@ -6,21 +6,29 @@ import unittest
 from typing import Tuple
 
 import torch
+
 from jit.test_hooks_modules import (
-    ModuleDirectforwardSubmodCall, ModuleForwardSingleInput,
-    ModuleForwardTupleInput, create_forward_tuple_input,
-    create_module_forward_multiple_inputs, create_module_forward_single_input,
+    create_forward_tuple_input,
+    create_module_forward_multiple_inputs,
+    create_module_forward_single_input,
     create_module_hook_return_nothing,
     create_module_multiple_hooks_multiple_inputs,
-    create_module_multiple_hooks_single_input, create_module_no_forward_input,
-    create_module_same_hook_repeated, create_submodule_forward_multiple_inputs,
+    create_module_multiple_hooks_single_input,
+    create_module_no_forward_input,
+    create_module_same_hook_repeated,
+    create_submodule_forward_multiple_inputs,
     create_submodule_forward_single_input,
     create_submodule_forward_single_input_return_not_tupled,
     create_submodule_hook_return_nothing,
     create_submodule_multiple_hooks_multiple_inputs,
     create_submodule_multiple_hooks_single_input,
-    create_submodule_no_forward_input, create_submodule_same_hook_repeated,
-    create_submodule_to_call_directly_with_hooks)
+    create_submodule_no_forward_input,
+    create_submodule_same_hook_repeated,
+    create_submodule_to_call_directly_with_hooks,
+    ModuleDirectforwardSubmodCall,
+    ModuleForwardSingleInput,
+    ModuleForwardTupleInput,
+)
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
@@ -37,7 +45,6 @@ if __name__ == "__main__":
 
 # Tests for JIT forward hooks and pre-hooks
 class TestHooks(JitTestCase):
-
     def test_module_no_forward_input(self):
         self.checkModule(create_module_no_forward_input(), ())
 
@@ -73,7 +80,8 @@ class TestHooks(JitTestCase):
 
     def test_submodule_multiple_hooks_multiple_inputs(self):
         self.checkModule(
-            create_submodule_multiple_hooks_multiple_inputs(), (["a"], "no_pre_hook"),
+            create_submodule_multiple_hooks_multiple_inputs(),
+            (["a"], "no_pre_hook"),
         )
 
     def test_submodule_forward_single_input(self):
@@ -242,7 +250,8 @@ class TestHooks(JitTestCase):
         m.register_forward_pre_hook(pre_hook_wrong_input1)
 
         with self.assertRaisesRegex(
-            RuntimeError, "has the wrong inner types for the input tuple argument",
+            RuntimeError,
+            "has the wrong inner types for the input tuple argument",
         ):
             torch.jit.script(m)
 
@@ -278,7 +287,8 @@ class TestHooks(JitTestCase):
         m.register_forward_pre_hook(pre_hook_wrong_output)
 
         with self.assertRaisesRegex(
-            RuntimeError, "returned the wrong type of: 'int'",
+            RuntimeError,
+            "returned the wrong type of: 'int'",
         ):
             torch.jit.script(m)
 
