@@ -4,10 +4,13 @@ import torch
 from torch.testing import FileCheck
 from torch.testing._internal.jit_utils import JitTestCase
 
-if __name__ == '__main__':
-    raise RuntimeError("This test file is not meant to be run directly, use:\n\n"
-                       "\tpython test/test_jit.py TESTNAME\n\n"
-                       "instead.")
+if __name__ == "__main__":
+    raise RuntimeError(
+        "This test file is not meant to be run directly, use:\n\n"
+        "\tpython test/test_jit.py TESTNAME\n\n"
+        "instead."
+    )
+
 
 class TestOpDecompositions(JitTestCase):
     def test_op_decomposition(self):
@@ -31,7 +34,9 @@ class TestOpDecompositions(JitTestCase):
         def square_decomp(x):
             return torch.pow(x, 2)
 
-        torch.jit._register_decomposition(torch.ops.aten.square.default, square_decomp.graph)
+        torch.jit._register_decomposition(
+            torch.ops.aten.square.default, square_decomp.graph
+        )
         torch._C._jit_pass_run_decompositions(foo.graph)
         FileCheck().check_not("aten::square").check("aten::pow").run(foo.graph)
         x = torch.rand([4])
