@@ -4,18 +4,21 @@ import fnmatch
 import io
 import shutil
 import tempfile
+from pathlib import Path
+
 import torch
 import torch.utils.show_pickle
+
 # from torch.utils.mobile_optimizer import optimize_for_mobile
 from torch.jit.mobile import (
-    _load_for_lite_interpreter,
+    _backport_for_mobile,
+    _backport_for_mobile_to_buffer,
     _get_mobile_model_contained_types,
     _get_model_bytecode_version,
     _get_model_ops_and_info,
-    _backport_for_mobile_to_buffer,
-    _backport_for_mobile)
-from torch.testing._internal.common_utils import TestCase, run_tests
-from pathlib import Path
+    _load_for_lite_interpreter,
+)
+from torch.testing._internal.common_utils import run_tests, TestCase
 
 pytorch_test_dir = Path(__file__).resolve().parents[1]
 
@@ -35,7 +38,7 @@ pytorch_test_dir = Path(__file__).resolve().parents[1]
 # exported_optimized_scripted_module = optimized_scripted_module._save_for_lite_interpreter(
 #   str(output_model_path))
 
-SCRIPT_MODULE_V4_BYTECODE_PKL = '''
+SCRIPT_MODULE_V4_BYTECODE_PKL = """
 (4,
  ('__torch__.*.TestModule.forward',
   (('instructions',
@@ -66,9 +69,9 @@ SCRIPT_MODULE_V4_BYTECODE_PKL = '''
      (('name', 'y'), ('type', 'int'), ('default_value', None)))),
    ('returns',
     ((('name', ''), ('type', 'Tensor'), ('default_value', None)),)))))
-        '''
+        """
 
-SCRIPT_MODULE_V5_BYTECODE_PKL = '''
+SCRIPT_MODULE_V5_BYTECODE_PKL = """
 (5,
  ('__torch__.*.TestModule.forward',
   (('instructions',
@@ -99,9 +102,9 @@ SCRIPT_MODULE_V5_BYTECODE_PKL = '''
      (('name', 'y'), ('type', 'int'), ('default_value', None)))),
    ('returns',
     ((('name', ''), ('type', 'Tensor'), ('default_value', None)),)))))
-        '''
+        """
 
-SCRIPT_MODULE_V6_BYTECODE_PKL = '''
+SCRIPT_MODULE_V6_BYTECODE_PKL = """
 (6,
  ('__torch__.*.TestModule.forward',
   (('instructions',
@@ -131,23 +134,26 @@ SCRIPT_MODULE_V6_BYTECODE_PKL = '''
      (('name', 'y'), ('type', 'int'), ('default_value', None)))),
    ('returns',
     ((('name', ''), ('type', 'Tensor'), ('default_value', None)),)))))
-    '''
+    """
 
 SCRIPT_MODULE_BYTECODE_PKL = {
     4: {
         "bytecode_pkl": SCRIPT_MODULE_V4_BYTECODE_PKL,
-        "model_name": "script_module_v4.ptl"},
+        "model_name": "script_module_v4.ptl",
+    },
 }
 
 # The minimum version a model can be backported to
 # Need to be updated when a bytecode version is completely retired
 MINIMUM_TO_VERSION = 4
 
+
 class testVariousModelVersions(TestCase):
     def test_get_model_bytecode_version(self):
         def check_model_version(model_path, expect_version):
             actual_version = _get_model_bytecode_version(model_path)
-            assert(actual_version == expect_version)
+            assert actual_version == expect_version
+
         for version, model_info in SCRIPT_MODULE_BYTECODE_PKL.items():
             model_path = pytorch_test_dir / "cpp" / "jit" / model_info["model_name"]
             check_model_version(model_path, version)
@@ -165,29 +171,44 @@ class testVariousModelVersions(TestCase):
         with tempfile.TemporaryDirectory() as tmpdirname:
             while current_from_version > MINIMUM_TO_VERSION:
                 # Load model v5 and run forward method
-                model_name = SCRIPT_MODULE_BYTECODE_PKL[current_from_version]["model_name"]
+                model_name = SCRIPT_MODULE_BYTECODE_PKL[current_from_version][
+                    "model_name"
+                ]
                 input_model_path = pytorch_test_dir / "cpp" / "jit" / model_name
 
                 # A temporary model file will be export to this path, and run through bytecode.pkl
                 # content check.
-                tmp_output_model_path_backport = Path(tmpdirname, "tmp_script_module_backport.ptl")
+                tmp_output_model_path_backport = Path(
+                    tmpdirname, "tmp_script_module_backport.ptl"
+                )
 
                 current_to_version = current_from_version - 1
-                backport_success = _backport_for_mobile(input_model_path, tmp_output_model_path_backport, current_to_version)
-                assert(backport_success)
+                backport_success = _backport_for_mobile(
+                    input_model_path, tmp_output_model_path_backport, current_to_version
+                )
+                assert backport_success
 
-                expect_bytecode_pkl = SCRIPT_MODULE_BYTECODE_PKL[current_to_version]["bytecode_pkl"]
+                expect_bytecode_pkl = SCRIPT_MODULE_BYTECODE_PKL[current_to_version][
+                    "bytecode_pkl"
+                ]
 
                 buf = io.StringIO()
                 torch.utils.show_pickle.main(
-                    ["", tmpdirname + "/" + tmp_output_model_path_backport.name + "@*/bytecode.pkl"],
-                    output_stream=buf)
+                    [
+                        "",
+                        tmpdirname
+                        + "/"
+                        + tmp_output_model_path_backport.name
+                        + "@*/bytecode.pkl",
+                    ],
+                    output_stream=buf,
+                )
                 output = buf.getvalue()
 
                 acutal_result_clean = "".join(output.split())
                 expect_result_clean = "".join(expect_bytecode_pkl.split())
                 isMatch = fnmatch.fnmatch(acutal_result_clean, expect_result_clean)
-                assert(isMatch)
+                assert isMatch
 
                 current_from_version -= 1
             shutil.rmtree(tmpdirname)
@@ -243,55 +264,78 @@ class testVariousModelVersions(TestCase):
     # Check just the test_backport_bytecode_from_file_to_file mechanism but not the function implementations
     def test_backport_bytecode_from_file_to_file(self):
         maximum_checked_in_model_version = max(SCRIPT_MODULE_BYTECODE_PKL.keys())
-        script_module_v5_path = pytorch_test_dir / "cpp" / "jit" / SCRIPT_MODULE_BYTECODE_PKL[
-            maximum_checked_in_model_version]["model_name"]
+        script_module_v5_path = (
+            pytorch_test_dir
+            / "cpp"
+            / "jit"
+            / SCRIPT_MODULE_BYTECODE_PKL[maximum_checked_in_model_version]["model_name"]
+        )
 
-        if (maximum_checked_in_model_version > MINIMUM_TO_VERSION):
+        if maximum_checked_in_model_version > MINIMUM_TO_VERSION:
             with tempfile.TemporaryDirectory() as tmpdirname:
-                tmp_backport_model_path = Path(tmpdirname, "tmp_script_module_v5_backported_to_v4.ptl")
+                tmp_backport_model_path = Path(
+                    tmpdirname, "tmp_script_module_v5_backported_to_v4.ptl"
+                )
                 # backport from file
                 success = _backport_for_mobile(
                     script_module_v5_path,
                     tmp_backport_model_path,
-                    maximum_checked_in_model_version - 1)
-                assert(success)
+                    maximum_checked_in_model_version - 1,
+                )
+                assert success
 
                 buf = io.StringIO()
                 torch.utils.show_pickle.main(
-                    ["", tmpdirname + "/" + tmp_backport_model_path.name + "@*/bytecode.pkl"],
-                    output_stream=buf)
+                    [
+                        "",
+                        tmpdirname
+                        + "/"
+                        + tmp_backport_model_path.name
+                        + "@*/bytecode.pkl",
+                    ],
+                    output_stream=buf,
+                )
                 output = buf.getvalue()
 
                 expected_result = SCRIPT_MODULE_V4_BYTECODE_PKL
                 acutal_result_clean = "".join(output.split())
                 expect_result_clean = "".join(expected_result.split())
                 isMatch = fnmatch.fnmatch(acutal_result_clean, expect_result_clean)
-                assert(isMatch)
+                assert isMatch
 
                 # Load model v4 and run forward method
                 mobile_module = _load_for_lite_interpreter(str(tmp_backport_model_path))
                 module_input = 1
                 mobile_module_result = mobile_module(module_input)
-                expected_mobile_module_result = 3 * torch.ones([2, 4], dtype=torch.float64)
-                torch.testing.assert_close(mobile_module_result, expected_mobile_module_result)
+                expected_mobile_module_result = 3 * torch.ones(
+                    [2, 4], dtype=torch.float64
+                )
+                torch.testing.assert_close(
+                    mobile_module_result, expected_mobile_module_result
+                )
                 shutil.rmtree(tmpdirname)
 
     # Check just the _backport_for_mobile_to_buffer mechanism but not the function implementations
     def test_backport_bytecode_from_file_to_buffer(self):
         maximum_checked_in_model_version = max(SCRIPT_MODULE_BYTECODE_PKL.keys())
-        script_module_v5_path = pytorch_test_dir / "cpp" / "jit" / SCRIPT_MODULE_BYTECODE_PKL[
-            maximum_checked_in_model_version]["model_name"]
+        script_module_v5_path = (
+            pytorch_test_dir
+            / "cpp"
+            / "jit"
+            / SCRIPT_MODULE_BYTECODE_PKL[maximum_checked_in_model_version]["model_name"]
+        )
 
-        if (maximum_checked_in_model_version > MINIMUM_TO_VERSION):
+        if maximum_checked_in_model_version > MINIMUM_TO_VERSION:
             # Backport model to v4
             script_module_v4_buffer = _backport_for_mobile_to_buffer(
-                script_module_v5_path, maximum_checked_in_model_version - 1)
+                script_module_v5_path, maximum_checked_in_model_version - 1
+            )
             buf = io.StringIO()
 
             # Check version of the model v4 from backport
             bytesio = io.BytesIO(script_module_v4_buffer)
             backport_version = _get_model_bytecode_version(bytesio)
-            assert(backport_version == maximum_checked_in_model_version - 1)
+            assert backport_version == maximum_checked_in_model_version - 1
 
             # Load model v4 from backport and run forward method
             bytesio = io.BytesIO(script_module_v4_buffer)
@@ -299,15 +343,16 @@ class testVariousModelVersions(TestCase):
             module_input = 1
             mobile_module_result = mobile_module(module_input)
             expected_mobile_module_result = 3 * torch.ones([2, 4], dtype=torch.float64)
-            torch.testing.assert_close(mobile_module_result, expected_mobile_module_result)
-
+            torch.testing.assert_close(
+                mobile_module_result, expected_mobile_module_result
+            )
 
     def test_get_model_ops_and_info(self):
         # TODO update this to be more in the style of the above tests after a backport from 6 -> 5 exists
         script_module_v6 = pytorch_test_dir / "cpp" / "jit" / "script_module_v6.ptl"
         ops_v6 = _get_model_ops_and_info(script_module_v6)
-        assert(ops_v6["aten::add.int"].num_schema_args == 2)
-        assert(ops_v6["aten::add.Scalar"].num_schema_args == 2)
+        assert ops_v6["aten::add.int"].num_schema_args == 2
+        assert ops_v6["aten::add.Scalar"].num_schema_args == 2
 
     def test_get_mobile_model_contained_types(self):
         class MyTestModule(torch.nn.Module):
@@ -322,7 +367,8 @@ class testVariousModelVersions(TestCase):
         buffer = io.BytesIO(script_module._save_to_buffer_for_lite_interpreter())
         buffer.seek(0)
         type_list = _get_mobile_model_contained_types(buffer)
-        assert(len(type_list) >= 0)
+        assert len(type_list) >= 0
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     run_tests()
