@@ -1,34 +1,44 @@
 # Owner(s): ["oncall: mobile"]
 
+import inspect
+import io
+from tempfile import TemporaryFileName
+from typing import Dict, List
+
 import torch
 import torch.utils.bundled_inputs
-import io
-from typing import Dict, List
-import inspect
-from torch.testing import FileCheck
 
-from torch.jit.mobile import _load_for_lite_interpreter, _export_operator_list
-from torch.testing._internal.common_utils import TestCase, run_tests
+from torch.jit.mobile import _export_operator_list, _load_for_lite_interpreter
+from torch.testing import FileCheck
 from torch.testing._internal.common_quantization import (
+    AnnotatedNestedModel,
     AnnotatedSingleLayerLinearModel,
+    QuantizationLiteTestCase,
     TwoLayerLinearModel,
-    AnnotatedNestedModel
 )
-from torch.testing._internal.common_quantization import QuantizationLiteTestCase
+from torch.testing._internal.common_utils import run_tests, TestCase
+
 
 class TestLiteScriptModule(TestCase):
-
-    def getScriptExportImportCopy(self, m, save_mobile_debug_info=True, also_test_file=False):
+    def getScriptExportImportCopy(
+        self, m, save_mobile_debug_info=True, also_test_file=False
+    ):
         m_scripted = torch.jit.script(m)
 
         if not also_test_file:
-            buffer = io.BytesIO(m_scripted._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=save_mobile_debug_info))
+            buffer = io.BytesIO(
+                m_scripted._save_to_buffer_for_lite_interpreter(
+                    _save_mobile_debug_info=save_mobile_debug_info
+                )
+            )
             buffer.seek(0)
             mobile_module = _load_for_lite_interpreter(buffer)
             return mobile_module
 
         with TemporaryFileName() as fname:
-            m_scripted._save_for_lite_interpreter(fname, _save_mobile_debug_info=save_mobile_debug_info)
+            m_scripted._save_for_lite_interpreter(
+                fname, _save_mobile_debug_info=save_mobile_debug_info
+            )
             mobile_module = _load_for_lite_interpreter(fname)
             return mobile_module
 
@@ -53,7 +63,9 @@ class TestLiteScriptModule(TestCase):
         torch.testing.assert_close(script_module_result, mobile_module_forward_result)
 
         mobile_module_run_method_result = mobile_module.run_method("forward", input)
-        torch.testing.assert_close(script_module_result, mobile_module_run_method_result)
+        torch.testing.assert_close(
+            script_module_result, mobile_module_run_method_result
+        )
 
     def test_save_mobile_module_with_debug_info_with_trace(self):
         class A(torch.nn.Module):
@@ -69,27 +81,35 @@ class TestLiteScriptModule(TestCase):
             def forward(self, x, y, z):
                 return self.A0(x, y) + self.A1(y, z)
 
-        for export_method in ['trace', 'script']:
+        for export_method in ["trace", "script"]:
             x = torch.rand((2, 3))
             y = torch.rand((2, 3))
             z = torch.rand((2, 3))
-            if export_method == 'trace':
+            if export_method == "trace":
                 trace_module = torch.jit.trace(B(), [x, y, z])
             else:
                 trace_module = torch.jit.script(B())
-            exported_module = trace_module._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=True)
+            exported_module = trace_module._save_to_buffer_for_lite_interpreter(
+                _save_mobile_debug_info=True
+            )
             buffer = io.BytesIO(exported_module)
             buffer.seek(0)
 
-            assert(b"callstack_debug_map.pkl" in exported_module)
+            assert b"callstack_debug_map.pkl" in exported_module
 
             mobile_module = _load_for_lite_interpreter(buffer)
-            with self.assertRaisesRegex(RuntimeError, r"Module hierarchy:top\(B\)::<unknown>.A0\(A\)::forward.aten::mul"):
+            with self.assertRaisesRegex(
+                RuntimeError,
+                r"Module hierarchy:top\(B\)::<unknown>.A0\(A\)::forward.aten::mul",
+            ):
                 x = torch.rand((2, 3))
                 y = torch.rand((8, 10))
                 z = torch.rand((8, 10))
                 mobile_module(x, y, z)
-            with self.assertRaisesRegex(RuntimeError, r"Module hierarchy:top\(B\)::<unknown>.A1\(A\)::forward.aten::mul"):
+            with self.assertRaisesRegex(
+                RuntimeError,
+                r"Module hierarchy:top\(B\)::<unknown>.A1\(A\)::forward.aten::mul",
+            ):
                 x = torch.rand((2, 3))
                 y = torch.rand((2, 3))
                 z = torch.rand((8, 10))
@@ -105,7 +125,11 @@ class TestLiteScriptModule(TestCase):
         script_module = torch.jit.script(MyTestModule())
         script_module_result = script_module(input)
 
-        buffer = io.BytesIO(script_module._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=True))
+        buffer = io.BytesIO(
+            script_module._save_to_buffer_for_lite_interpreter(
+                _save_mobile_debug_info=True
+            )
+        )
         buffer.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer)
 
@@ -116,14 +140,16 @@ class TestLiteScriptModule(TestCase):
         torch.testing.assert_close(script_module_result, mobile_module_forward_result)
 
         mobile_module_run_method_result = mobile_module.run_method("forward", input)
-        torch.testing.assert_close(script_module_result, mobile_module_run_method_result)
+        torch.testing.assert_close(
+            script_module_result, mobile_module_run_method_result
+        )
 
     def test_find_and_run_method(self):
         class MyTestModule(torch.nn.Module):
             def forward(self, arg):
                 return arg
 
-        input = (torch.tensor([1]), )
+        input = (torch.tensor([1]),)
 
         script_module = torch.jit.script(MyTestModule())
         script_module_result = script_module(*input)
@@ -136,7 +162,8 @@ class TestLiteScriptModule(TestCase):
         self.assertFalse(has_bundled_inputs)
 
         torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-            script_module, [input], [])
+            script_module, [input], []
+        )
 
         buffer = io.BytesIO(script_module._save_to_buffer_for_lite_interpreter())
         buffer.seek(0)
@@ -168,32 +195,26 @@ class TestLiteScriptModule(TestCase):
                 return self.A0(x) + one
 
         script_module = torch.jit.script(B())
-        buffer = io.BytesIO(
-            script_module._save_to_buffer_for_lite_interpreter()
-        )
+        buffer = io.BytesIO(script_module._save_to_buffer_for_lite_interpreter())
         mobile_module = _load_for_lite_interpreter(buffer)
 
         input = torch.tensor([5])
         script_module_forward_result = script_module.forward(input)
         mobile_module_forward_result = mobile_module.forward(input)
         torch.testing.assert_close(
-            script_module_forward_result,
-            mobile_module_forward_result
+            script_module_forward_result, mobile_module_forward_result
         )
 
         # change ref only
         script_module_forward_result = script_module.forward(input, 2)
         self.assertFalse(
-            (script_module_forward_result == mobile_module_forward_result)
-            .all()
-            .item()
+            (script_module_forward_result == mobile_module_forward_result).all().item()
         )
 
         # now both match again
         mobile_module_forward_result = mobile_module.forward(input, 2)
         torch.testing.assert_close(
-            script_module_forward_result,
-            mobile_module_forward_result
+            script_module_forward_result, mobile_module_forward_result
         )
 
     def test_unsupported_classtype(self):
@@ -210,10 +231,12 @@ class TestLiteScriptModule(TestCase):
                 return f.func(1, 2)
 
         script_module = torch.jit.script(MyTestModule())
-        with self.assertRaisesRegex(RuntimeError,
-                                    r"Workaround: instead of using arbitrary class type \(class Foo\(\)\), "
-                                    r"define a pytorch class \(class Foo\(torch\.nn\.Module\)\)\. "
-                                    r"The problematic type is: "):
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"Workaround: instead of using arbitrary class type \(class Foo\(\)\), "
+            r"define a pytorch class \(class Foo\(torch\.nn\.Module\)\)\. "
+            r"The problematic type is: ",
+        ):
             script_module._save_to_buffer_for_lite_interpreter()
 
     def test_unsupported_return_list_with_module_class(self):
@@ -230,12 +253,14 @@ class TestLiteScriptModule(TestCase):
                 return my_list
 
         script_module = torch.jit.script(MyTestModuleForListWithModuleClass())
-        with self.assertRaisesRegex(RuntimeError,
-                                    r"^Returning a list or dictionary with pytorch class type "
-                                    r"is not supported in mobile module "
-                                    r"\(List\[Foo\] or Dict\[int\, Foo\] for class Foo\(torch\.nn\.Module\)\)\. "
-                                    r"Workaround\: instead of using pytorch class as their element type\, "
-                                    r"use a combination of list\, dictionary\, and single types\.$"):
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"^Returning a list or dictionary with pytorch class type "
+            r"is not supported in mobile module "
+            r"\(List\[Foo\] or Dict\[int\, Foo\] for class Foo\(torch\.nn\.Module\)\)\. "
+            r"Workaround\: instead of using pytorch class as their element type\, "
+            r"use a combination of list\, dictionary\, and single types\.$",
+        ):
             script_module._save_to_buffer_for_lite_interpreter()
 
     def test_unsupported_return_dict_with_module_class(self):
@@ -252,12 +277,14 @@ class TestLiteScriptModule(TestCase):
                 return my_dict
 
         script_module = torch.jit.script(MyTestModuleForDictWithModuleClass())
-        with self.assertRaisesRegex(RuntimeError,
-                                    r"^Returning a list or dictionary with pytorch class type "
-                                    r"is not supported in mobile module "
-                                    r"\(List\[Foo\] or Dict\[int\, Foo\] for class Foo\(torch\.nn\.Module\)\)\. "
-                                    r"Workaround\: instead of using pytorch class as their element type\, "
-                                    r"use a combination of list\, dictionary\, and single types\.$"):
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"^Returning a list or dictionary with pytorch class type "
+            r"is not supported in mobile module "
+            r"\(List\[Foo\] or Dict\[int\, Foo\] for class Foo\(torch\.nn\.Module\)\)\. "
+            r"Workaround\: instead of using pytorch class as their element type\, "
+            r"use a combination of list\, dictionary\, and single types\.$",
+        ):
             script_module._save_to_buffer_for_lite_interpreter()
 
     def test_module_export_operator_list(self):
@@ -303,7 +330,6 @@ class TestLiteScriptModule(TestCase):
         self.assertEqual(actual_ops, expected_ops)
 
     def test_source_range_simple(self):
-
         class FooTest(torch.jit.ScriptModule):
             @torch.jit.script_method
             def forward(self, x, w):
@@ -313,15 +339,16 @@ class TestLiteScriptModule(TestCase):
         loaded = self.getScriptExportImportCopy(ft)
         _, lineno = inspect.getsourcelines(FooTest)
 
-        with self.assertRaisesRegex(RuntimeError, f'test_lite_script_module.py\", line {lineno + 3}'):
+        with self.assertRaisesRegex(
+            RuntimeError, f'test_lite_script_module.py", line {lineno + 3}'
+        ):
             loaded(torch.rand(3, 4), torch.rand(30, 40))
 
     def test_source_range_raise_exception(self):
-
         class FooTest2(torch.jit.ScriptModule):
             @torch.jit.script_method
             def forward(self):
-                raise RuntimeError('foo')
+                raise RuntimeError("foo")
 
         _, lineno = inspect.getsourcelines(FooTest2)
 
@@ -332,7 +359,7 @@ class TestLiteScriptModule(TestCase):
         # the debug handle string to show where in the Python code the exception
         # occured w/o first changing
         # torch::jit::JITException to extend c10::Error.
-        with self.assertRaisesRegex(torch.jit.Error, 'foo'):
+        with self.assertRaisesRegex(torch.jit.Error, "foo"):
             ft = FooTest2()
             loaded = self.getScriptExportImportCopy(ft)
             loaded()
@@ -357,12 +384,15 @@ class TestLiteScriptModule(TestCase):
             loaded(torch.rand(3, 4), torch.rand(3, 4), torch.rand(30, 40))
         except RuntimeError as e:
             error_message = f"{e}"
-        self.assertTrue(f'test_lite_script_module.py\", line {lineno + 3}' in error_message)
-        self.assertTrue(f'test_lite_script_module.py\", line {lineno + 9}' in error_message)
-        self.assertTrue('top(FooTest3)' in error_message)
+        self.assertTrue(
+            f'test_lite_script_module.py", line {lineno + 3}' in error_message
+        )
+        self.assertTrue(
+            f'test_lite_script_module.py", line {lineno + 9}' in error_message
+        )
+        self.assertTrue("top(FooTest3)" in error_message)
 
     def test_source_range_no_debug_info(self):
-
         class FooTest4(torch.jit.ScriptModule):
             @torch.jit.script_method
             def forward(self, x, w):
@@ -385,8 +415,8 @@ class TestLiteScriptModule(TestCase):
 
             @torch.jit.script_method
             def add_method(self, val: int, x, w):
-                if (val == self.val):
-                    raise RuntimeError('self.val and val are same')
+                if val == self.val:
+                    raise RuntimeError("self.val and val are same")
                 return x + w
 
             @torch.jit.script_method
@@ -411,7 +441,7 @@ class TestLiteScriptModule(TestCase):
         # the debug handle string to show where in the Python code the exception
         # occured w/o first changing
         # torch::jit::JITException to extend c10::Error.
-        self.assertTrue('self.val and val are same' in error_message)
+        self.assertTrue("self.val and val are same" in error_message)
 
     def test_stacktrace_interface_call(self):
         @torch.jit.interface
@@ -433,7 +463,7 @@ class TestLiteScriptModule(TestCase):
                 return torch.ones(-1)
 
         class A(torch.nn.Module):
-            b : Forward
+            b: Forward
 
             def __init__(self):
                 super().__init__()
@@ -445,26 +475,38 @@ class TestLiteScriptModule(TestCase):
 
         a = torch.jit.script(A())
         torch._C._enable_mobile_interface_call_export()
-        buffer = io.BytesIO(a._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=True))
+        buffer = io.BytesIO(
+            a._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=True)
+        )
         buffer.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer)
         try:
             mobile_module()
             self.assertTrue(False)
         except RuntimeError as exp:
-            FileCheck().check("Trying to create tensor with negative dimension") \
-                .check("Traceback of TorchScript") \
-                .check("self.b.forwardError").check_next("~~~~~~~~~~~~~~~~~~~ <--- HERE") \
-                .check("return self.call").check_next("~~~~~~~~~ <--- HERE") \
-                .check("return torch.ones").check_next("~~~~~~~~~~ <--- HERE").run(str(exp))
-
+            FileCheck().check("Trying to create tensor with negative dimension").check(
+                "Traceback of TorchScript"
+            ).check("self.b.forwardError").check_next(
+                "~~~~~~~~~~~~~~~~~~~ <--- HERE"
+            ).check(
+                "return self.call"
+            ).check_next(
+                "~~~~~~~~~ <--- HERE"
+            ).check(
+                "return torch.ones"
+            ).check_next(
+                "~~~~~~~~~~ <--- HERE"
+            ).run(
+                str(exp)
+            )
 
 
 class TestLiteScriptQuantizedModule(QuantizationLiteTestCase):
-
     def test_single_layer(self):
         input = torch.rand(2, 5, dtype=torch.float)
-        quantized_model = self._create_quantized_model(model_class=AnnotatedSingleLayerLinearModel, qengine="qnnpack")
+        quantized_model = self._create_quantized_model(
+            model_class=AnnotatedSingleLayerLinearModel, qengine="qnnpack"
+        )
         self._compare_script_and_mobile(model=quantized_model, input=input)
 
     def test_two_layer(self):
@@ -474,11 +516,12 @@ class TestLiteScriptQuantizedModule(QuantizationLiteTestCase):
 
     def test_annotated_nested(self):
         input = torch.rand(2, 5, dtype=torch.float)
-        quantized_model = self._create_quantized_model(model_class=AnnotatedNestedModel, qengine="qnnpack")
+        quantized_model = self._create_quantized_model(
+            model_class=AnnotatedNestedModel, qengine="qnnpack"
+        )
         self._compare_script_and_mobile(model=quantized_model, input=input)
 
     def test_quantization_example(self):
-
         # From the example in Static Quantization section of https://pytorch.org/docs/stable/quantization.html
         class M(torch.nn.Module):
             def __init__(self):
@@ -498,8 +541,10 @@ class TestLiteScriptQuantizedModule(QuantizationLiteTestCase):
         model_fp32 = M()
 
         model_fp32.eval()
-        model_fp32.qconfig = torch.ao.quantization.get_default_qconfig('qnnpack')
-        model_fp32_fused = torch.ao.quantization.fuse_modules(model_fp32, [['conv', 'relu']])
+        model_fp32.qconfig = torch.ao.quantization.get_default_qconfig("qnnpack")
+        model_fp32_fused = torch.ao.quantization.fuse_modules(
+            model_fp32, [["conv", "relu"]]
+        )
         model_fp32_prepared = torch.ao.quantization.prepare(model_fp32_fused)
         input_fp32 = torch.randn(4, 1, 4, 4)
         model_fp32_prepared(input_fp32)
@@ -549,5 +594,6 @@ class TestLiteScriptQuantizedModule(QuantizationLiteTestCase):
             ),
         )
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     run_tests()
