@@ -1,20 +1,22 @@
 # Owner(s): ["oncall: mobile"]
 
+import io
+import unittest
+from collections import namedtuple
+from typing import Dict, List, NamedTuple
+
 import torch
 import torch.utils.bundled_inputs
-import io
-from typing import Dict, List, NamedTuple
-import unittest
 
 from torch.jit.mobile import _load_for_lite_interpreter
-from torch.testing._internal.common_utils import TestCase, run_tests
-from collections import namedtuple
+from torch.testing._internal.common_utils import run_tests, TestCase
 
 
 class TestLiteScriptModule(TestCase):
-
     def test_typing_namedtuple(self):
-        myNamedTuple = NamedTuple('myNamedTuple', [('a', List[torch.Tensor])])
+        myNamedTuple = NamedTuple(  # noqa: UP014
+            "myNamedTuple", [("a", List[torch.Tensor])]
+        )
 
         class MyTestModule(torch.nn.Module):
             def forward(self, a: torch.Tensor):
@@ -25,15 +27,15 @@ class TestLiteScriptModule(TestCase):
         script_module = torch.jit.script(MyTestModule())
         script_module_result = script_module(sample_input).a
 
-        buffer = io.BytesIO(script_module._save_to_buffer_for_lite_interpreter(_save_mobile_debug_info=True))
+        buffer = io.BytesIO(
+            script_module._save_to_buffer_for_lite_interpreter(
+                _save_mobile_debug_info=True
+            )
+        )
         buffer.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer)  # Error here
         mobile_module_result = mobile_module(sample_input).a
-        torch.testing.assert_close(
-            script_module_result,
-            mobile_module_result
-        )
-
+        torch.testing.assert_close(script_module_result, mobile_module_result)
 
     @unittest.skip("T137512434")
     def test_typing_dict_with_namedtuple(self):
@@ -93,10 +95,7 @@ class TestLiteScriptModule(TestCase):
         buffer_mobile.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer_mobile)
         mobile_module_result = mobile_module(sample_input)
-        torch.testing.assert_close(
-            script_module_result,
-            mobile_module_result
-        )
+        torch.testing.assert_close(script_module_result, mobile_module_result)
 
     def test_typing_namedtuple_custom_classtype(self):
         class Foo(NamedTuple):
@@ -119,13 +118,10 @@ class TestLiteScriptModule(TestCase):
         buffer_mobile.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer_mobile)
         mobile_module_result = mobile_module(sample_input)
-        torch.testing.assert_close(
-            script_module_result,
-            mobile_module_result
-        )
+        torch.testing.assert_close(script_module_result, mobile_module_result)
 
     def test_return_collections_namedtuple(self):
-        myNamedTuple = namedtuple('myNamedTuple', [('a')])
+        myNamedTuple = namedtuple("myNamedTuple", [("a")])
 
         class MyTestModule(torch.nn.Module):
             def forward(self, a: torch.Tensor):
@@ -138,10 +134,7 @@ class TestLiteScriptModule(TestCase):
         buffer_mobile.seek(0)
         mobile_module = _load_for_lite_interpreter(buffer_mobile)
         mobile_module_result = mobile_module(sample_input)
-        torch.testing.assert_close(
-            script_module_result,
-            mobile_module_result
-        )
+        torch.testing.assert_close(script_module_result, mobile_module_result)
 
     def test_nest_typing_namedtuple_custom_classtype(self):
         class Baz(NamedTuple):
@@ -169,9 +162,9 @@ class TestLiteScriptModule(TestCase):
         mobile_module = _load_for_lite_interpreter(buffer_mobile)
         mobile_module_result = mobile_module(sample_input)
         torch.testing.assert_close(
-            script_module_result.baz.di,
-            mobile_module_result.baz.di
+            script_module_result.baz.di, mobile_module_result.baz.di
         )
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     run_tests()
