@@ -4,23 +4,22 @@ import itertools
 import random
 
 import torch
-from torch.testing._internal.common_utils import TestCase, run_tests
 import torch.nn.utils.rnn as rnn_utils
+from torch.testing._internal.common_utils import run_tests, TestCase
 
 
 class PackedSequenceTest(TestCase):
-
     _type_by_name = {
-        'torch.DoubleTensor': (torch.DoubleTensor, 'double'),
-        'torch.FloatTensor': (torch.FloatTensor, 'float'),
+        "torch.DoubleTensor": (torch.DoubleTensor, "double"),
+        "torch.FloatTensor": (torch.FloatTensor, "float"),
         # We leave out `'torch.HalfTensor': (torch.HalfTensor, 'half'),`
         # because of an error in `pad_packed_sequence`
         # > AttributeError: 'torch.HalfTensor' object has no attribute 'fill_'
-        'torch.LongTensor': (torch.LongTensor, 'long'),
-        'torch.IntTensor': (torch.IntTensor, 'int'),
-        'torch.ShortTensor': (torch.ShortTensor, 'short'),
-        'torch.CharTensor': (torch.CharTensor, 'char'),
-        'torch.ByteTensor': (torch.ByteTensor, 'byte'),
+        "torch.LongTensor": (torch.LongTensor, "long"),
+        "torch.IntTensor": (torch.IntTensor, "int"),
+        "torch.ShortTensor": (torch.ShortTensor, "short"),
+        "torch.CharTensor": (torch.CharTensor, "char"),
+        "torch.ByteTensor": (torch.ByteTensor, "byte"),
     }
 
     def __init__(self, *args, **kwargs):
@@ -30,8 +29,10 @@ class PackedSequenceTest(TestCase):
 
     def _ordered_sequence(self, tensor_type):
         """Create ordered list of random sequences"""
-        seqs = [tensor_type(random.randint(1, self.max_length))
-                for _ in range(self.batch_size)]
+        seqs = [
+            tensor_type(random.randint(1, self.max_length))
+            for _ in range(self.batch_size)
+        ]
         if tensor_type == torch.ByteTensor:
             seqs = [s.random_(0, 256) for s in seqs]
         else:
@@ -48,12 +49,13 @@ class PackedSequenceTest(TestCase):
 
     def test_type_casts(self):
         """Test type casting of `PackedSequence` against type casting of tensor"""
-        for (input_type, _) in self._type_by_name.values():
+        for input_type, _ in self._type_by_name.values():
             for expected_type_str, (_, cast_str) in self._type_by_name.items():
                 for enforce_sorted in [True, False]:
                     padded, lengths = self._padded_sequence(input_type)
                     packed = rnn_utils.pack_padded_sequence(
-                        padded, lengths, enforce_sorted=enforce_sorted)
+                        padded, lengths, enforce_sorted=enforce_sorted
+                    )
                     # Apply cast to `PackedSequence` instance and unpack
                     masked = getattr(packed, cast_str)()
                     unpacked, lengths_out = rnn_utils.pad_packed_sequence(masked)
@@ -65,7 +67,8 @@ class PackedSequenceTest(TestCase):
         b_a = rnn_utils.pad_sequence([b, a])
         self.assertRaises(
             RuntimeError,
-            lambda: rnn_utils.pack_padded_sequence(b_a, [22, 25], enforce_sorted=True))
+            lambda: rnn_utils.pack_padded_sequence(b_a, [22, 25], enforce_sorted=True),
+        )
 
     def test_pad_sequence_with_tensor_sequences(self):
         seq_tuple_input = torch.nn.utils.rnn.pad_sequence(
@@ -89,29 +92,41 @@ class PackedSequenceTest(TestCase):
         # test ValueError if total_length < max_length
         for total_length in (-1, 0, max_length - 1):
             for batch_first in (True, False):
+
                 def err_fn():
-                    rnn_utils.pad_packed_sequence(packed, batch_first=batch_first,
-                                                  total_length=total_length)
-            self.assertRaisesRegex(ValueError,
-                                   r'Expected total_length to be at least the '
-                                   r'length of the longest sequence in input',
-                                   err_fn)
+                    rnn_utils.pad_packed_sequence(
+                        packed, batch_first=batch_first, total_length=total_length
+                    )
+
+            self.assertRaisesRegex(
+                ValueError,
+                r"Expected total_length to be at least the "
+                r"length of the longest sequence in input",
+                err_fn,
+            )
         # test that pad_packed_sequence returns results of correct length
         for batch_first in (True, False):
-            no_extra_pad, _ = rnn_utils.pad_packed_sequence(packed, batch_first=batch_first)
+            no_extra_pad, _ = rnn_utils.pad_packed_sequence(
+                packed, batch_first=batch_first
+            )
             for total_length_delta in (0, 1, 8):
                 total_length = max_length + total_length_delta
-                unpacked, lengths_out = rnn_utils.pad_packed_sequence(packed, batch_first=batch_first,
-                                                                      total_length=total_length)
+                unpacked, lengths_out = rnn_utils.pad_packed_sequence(
+                    packed, batch_first=batch_first, total_length=total_length
+                )
                 self.assertEqual(lengths, lengths_out)
                 self.assertEqual(unpacked.size(1 if batch_first else 0), total_length)
                 if total_length_delta == 0:
                     ref_output = no_extra_pad
                 elif batch_first:
-                    extra_pad = no_extra_pad.new_zeros(self.batch_size, total_length_delta)
+                    extra_pad = no_extra_pad.new_zeros(
+                        self.batch_size, total_length_delta
+                    )
                     ref_output = torch.cat([no_extra_pad, extra_pad], 1)
                 else:
-                    extra_pad = no_extra_pad.new_zeros(total_length_delta, self.batch_size)
+                    extra_pad = no_extra_pad.new_zeros(
+                        total_length_delta, self.batch_size
+                    )
                     ref_output = torch.cat([no_extra_pad, extra_pad], 0)
                 self.assertEqual(unpacked, ref_output)
 
@@ -119,21 +134,25 @@ class PackedSequenceTest(TestCase):
         for enforce_sorted in (True, False):
             padded, lengths = self._padded_sequence(torch.IntTensor)
             a = rnn_utils.pack_padded_sequence(
-                padded, lengths, enforce_sorted=enforce_sorted).cpu()
+                padded, lengths, enforce_sorted=enforce_sorted
+            ).cpu()
 
-            self.assertIs(a, a.to('cpu'))
+            self.assertIs(a, a.to("cpu"))
             self.assertIs(a, a.cpu())
-            self.assertIs(a, a.to('cpu', dtype=torch.int32))
+            self.assertIs(a, a.to("cpu", dtype=torch.int32))
             self.assertEqual(a.long(), a.to(torch.int64))
 
             if torch.cuda.is_available():
-                for cuda in ['cuda', 'cuda:0' if torch.cuda.device_count() == 1 else 'cuda:1']:
+                for cuda in [
+                    "cuda",
+                    "cuda:0" if torch.cuda.device_count() == 1 else "cuda:1",
+                ]:
                     b = a.cuda(device=cuda)
                     self.assertIs(b, b.to(cuda))
                     self.assertIs(b, b.cuda())
-                    self.assertEqual(a, b.to('cpu'))
+                    self.assertEqual(a, b.to("cpu"))
                     self.assertEqual(b, a.to(cuda))
-                    self.assertEqual(a, b.to('cpu', dtype=torch.int32))
+                    self.assertEqual(a, b.to("cpu", dtype=torch.int32))
                     self.assertIs(b, b.to(dtype=torch.int32))
                     self.assertEqual(b.long(), b.to(dtype=torch.int64))
 
@@ -147,8 +166,13 @@ class PackedSequenceTest(TestCase):
     def test_pad_sequence(self):
         def pad(tensor, length):
             return torch.cat(
-                [tensor.data, tensor.data.new(
-                    length - tensor.size(0), *tensor.size()[1:]).zero_()])
+                [
+                    tensor.data,
+                    tensor.data.new(
+                        length - tensor.size(0), *tensor.size()[1:]
+                    ).zero_(),
+                ]
+            )
 
         # single dimensional
         a = torch.tensor([1, 2, 3])
@@ -196,7 +220,6 @@ class PackedSequenceTest(TestCase):
             self.assertEqual(padded, expected.transpose(0, 1))
 
     def test_unpad_sequence(self):
-
         # single dimensional
         a = torch.tensor([1, 2, 3])
         b = torch.tensor([4, 5])
@@ -205,8 +228,12 @@ class PackedSequenceTest(TestCase):
 
         lengths = torch.as_tensor([v.size(0) for v in sequences])
         for batch_first in [True, False]:
-            padded_sequences = rnn_utils.pad_sequence(sequences, batch_first=batch_first)
-            unpadded_sequences = rnn_utils.unpad_sequence(padded_sequences, lengths, batch_first=batch_first)
+            padded_sequences = rnn_utils.pad_sequence(
+                sequences, batch_first=batch_first
+            )
+            unpadded_sequences = rnn_utils.unpad_sequence(
+                padded_sequences, lengths, batch_first=batch_first
+            )
             self.assertEqual(sequences, unpadded_sequences)
 
         # more dimensions
@@ -220,8 +247,12 @@ class PackedSequenceTest(TestCase):
             random.shuffle(sequences)
 
             lengths = torch.as_tensor([v.size(0) for v in sequences])
-            padded_sequences = rnn_utils.pad_sequence(sequences, batch_first=batch_first)
-            unpadded_sequences = rnn_utils.unpad_sequence(padded_sequences, lengths, batch_first=batch_first)
+            padded_sequences = rnn_utils.pad_sequence(
+                sequences, batch_first=batch_first
+            )
+            unpadded_sequences = rnn_utils.unpad_sequence(
+                padded_sequences, lengths, batch_first=batch_first
+            )
             self.assertEqual(sequences, unpadded_sequences)
 
     def test_pack_sequence(self):
@@ -231,7 +262,8 @@ class PackedSequenceTest(TestCase):
             unpacked = rnn_utils.pad_packed_sequence(packed, batch_first)
             self.assertEqual(padded, unpacked[0])
             pack_padded = rnn_utils.pack_padded_sequence(
-                padded, lengths, batch_first, enforce_sorted)
+                padded, lengths, batch_first, enforce_sorted
+            )
             self.assertEqual(packed, pack_padded)
 
         # single dimensional
@@ -258,10 +290,12 @@ class PackedSequenceTest(TestCase):
         self.assertTrue(packed_enforce_sorted.sorted_indices is None)
         self.assertTrue(packed_enforce_sorted.unsorted_indices is None)
 
-        with self.assertRaisesRegex(RuntimeError, 'must be sorted in decreasing order'):
+        with self.assertRaisesRegex(RuntimeError, "must be sorted in decreasing order"):
             rnn_utils.pack_sequence([b, c, a], enforce_sorted=True)
 
-        with self.assertRaisesRegex(RuntimeError, 'You can pass `enforce_sorted=False`'):
+        with self.assertRaisesRegex(
+            RuntimeError, "You can pass `enforce_sorted=False`"
+        ):
             rnn_utils.pack_sequence([b, c, a], enforce_sorted=True)
 
         # more dimensions
@@ -282,11 +316,11 @@ class PackedSequenceTest(TestCase):
             for batch_first in (True, False):
                 for enforce_sorted in (True, False):
                     _compatibility_test(sequences, lengths, batch_first, enforce_sorted)
-                _compatibility_test(unsorted_sequences, unsorted_sequences_lengths,
-                                    batch_first)
+                _compatibility_test(
+                    unsorted_sequences, unsorted_sequences_lengths, batch_first
+                )
 
     def test_unpack_sequence(self):
-
         # single dimensional
         a = torch.tensor([1, 2, 3])
         b = torch.tensor([4, 5])
@@ -314,16 +348,35 @@ class PackedSequenceTest(TestCase):
     def test_pack_padded_sequence(self):
         def generate_test_case(sorted_lengths, should_shuffle):
             def pad(tensor, length):
-                return torch.cat([tensor, tensor.new(length - tensor.size(0), *tensor.size()[1:]).zero_()])
+                return torch.cat(
+                    [
+                        tensor,
+                        tensor.new(length - tensor.size(0), *tensor.size()[1:]).zero_(),
+                    ]
+                )
 
             max_length = sorted_lengths[0]
-            batch_sizes = [sum(map(bool, filter(lambda x: x >= i, sorted_lengths)))
-                           for i in range(1, max_length + 1)]
+            batch_sizes = [
+                sum(map(bool, filter(lambda x: x >= i, sorted_lengths)))
+                for i in range(1, max_length + 1)
+            ]
             offset = 0
-            padded = torch.cat([pad(i * 100 + torch.arange(1., 5 * l + 1).view(l, 1, 5), max_length)
-                                for i, l in enumerate(sorted_lengths, 1)], 1)
-            expected_data = [[torch.arange(1., 6) + (i + 1) * 100 + 5 * n for i in range(batch_size)]
-                             for n, batch_size in enumerate(batch_sizes)]
+            padded = torch.cat(
+                [
+                    pad(
+                        i * 100 + torch.arange(1.0, 5 * l + 1).view(l, 1, 5), max_length
+                    )
+                    for i, l in enumerate(sorted_lengths, 1)
+                ],
+                1,
+            )
+            expected_data = [
+                [
+                    torch.arange(1.0, 6) + (i + 1) * 100 + 5 * n
+                    for i in range(batch_size)
+                ]
+                for n, batch_size in enumerate(batch_sizes)
+            ]
             expected_data = list(itertools.chain.from_iterable(expected_data))
             expected_data = torch.stack(expected_data, dim=0)
 
@@ -339,7 +392,13 @@ class PackedSequenceTest(TestCase):
                 unsorted_indices = None
                 lengths = sorted_lengths
 
-            return padded.requires_grad_(), lengths, expected_data, batch_sizes, unsorted_indices
+            return (
+                padded.requires_grad_(),
+                lengths,
+                expected_data,
+                batch_sizes,
+                unsorted_indices,
+            )
 
         test_cases = [
             # sorted_lengths, should_shuffle
@@ -350,22 +409,30 @@ class PackedSequenceTest(TestCase):
 
         for test_case, batch_first in itertools.product(test_cases, (True, False)):
             sorted_lengths, should_shuffle = test_case
-            padded, lengths, expected_data, batch_sizes, unsorted_indices = generate_test_case(
-                sorted_lengths, should_shuffle)
+            (
+                padded,
+                lengths,
+                expected_data,
+                batch_sizes,
+                unsorted_indices,
+            ) = generate_test_case(sorted_lengths, should_shuffle)
 
             src = padded
             if batch_first:
                 src = src.transpose(0, 1)
 
             # check output
-            packed = rnn_utils.pack_padded_sequence(src, lengths, batch_first=batch_first,
-                                                    enforce_sorted=not should_shuffle)
+            packed = rnn_utils.pack_padded_sequence(
+                src, lengths, batch_first=batch_first, enforce_sorted=not should_shuffle
+            )
             self.assertEqual(packed.data.data, expected_data)
             self.assertEqual(packed.batch_sizes, batch_sizes)
             self.assertEqual(packed.unsorted_indices, unsorted_indices)
 
             # test inverse
-            unpacked, unpacked_len = rnn_utils.pad_packed_sequence(packed, batch_first=batch_first)
+            unpacked, unpacked_len = rnn_utils.pad_packed_sequence(
+                packed, batch_first=batch_first
+            )
             self.assertEqual(unpacked, src)
             self.assertEqual(unpacked_len, lengths)
 
@@ -382,14 +449,17 @@ class PackedSequenceTest(TestCase):
                     self.assertEqual(padded.grad.data[l:, i].abs().sum(), 0)
 
         # test error messages
-        with self.assertRaisesRegex(RuntimeError, 'You can pass `enforce_sorted=False`'):
+        with self.assertRaisesRegex(
+            RuntimeError, "You can pass `enforce_sorted=False`"
+        ):
             packed = rnn_utils.pack_padded_sequence(torch.randn(3, 3), [1, 3, 2])
-        with self.assertRaisesRegex(RuntimeError, 'empty tensor'):
+        with self.assertRaisesRegex(RuntimeError, "empty tensor"):
             packed = rnn_utils.pack_padded_sequence(torch.randn(0, 0), [])
-        with self.assertRaisesRegex(RuntimeError, 'empty tensor'):
-            packed = rnn_utils.pack_padded_sequence(torch.randn([0, 1, 10]),
-                                                    torch.randn([11, 14, 14, 2]), True)
+        with self.assertRaisesRegex(RuntimeError, "empty tensor"):
+            packed = rnn_utils.pack_padded_sequence(
+                torch.randn([0, 1, 10]), torch.randn([11, 14, 14, 2]), True
+            )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     run_tests()
