@@ -30,15 +30,18 @@ import numpy as np
 import onnxruntime
 import pytest
 import pytorch_test_common
+
 import torch
+from torch import export as torch_export
 from torch.onnx import _constants, verification
 from torch.onnx._internal import _beartype
 from torch.onnx._internal.fx import diagnostics
+from torch.testing._internal import common_utils
 from torch.testing._internal.opinfo import core as opinfo_core
 from torch.types import Number
 
 _NumericType = Union[Number, torch.Tensor, np.ndarray]
-_ModelType = Union[torch.nn.Module, Callable]
+_ModelType = Union[torch.nn.Module, Callable, torch_export.ExportedProgram]
 _InputArgsType = Optional[
     Union[torch.Tensor, int, float, bool, Sequence[Any], Mapping[str, Any]]
 ]
@@ -84,11 +87,11 @@ def run_model_test(test_suite: _TestONNXRuntime, *args, **kwargs):
     return verification.verify(*args, options=options, **kwargs)
 
 
-def assert_dynamic_shapes(export_output: torch.onnx.ExportOutput, dynamic_shapes: bool):
+def assert_dynamic_shapes(onnx_program: torch.onnx.ONNXProgram, dynamic_shapes: bool):
     """Assert whether the exported model has dynamic shapes or not.
 
     Args:
-        export_output (torch.onnx.ExportOutput): The output of torch.onnx.dynamo_export.
+        onnx_program (torch.onnx.ONNXProgram): The output of torch.onnx.dynamo_export.
         dynamic_shapes (bool): Whether the exported model has dynamic shapes or not.
             When True, raises if graph inputs don't have at least one dynamic dimension
             When False, raises if graph inputs have at least one dynamic dimension.
@@ -100,7 +103,7 @@ def assert_dynamic_shapes(export_output: torch.onnx.ExportOutput, dynamic_shapes
     if dynamic_shapes is None:
         return
 
-    model_proto = export_output.model_proto
+    model_proto = onnx_program.model_proto
     # Process graph inputs
     dynamic_inputs = []
     for inp in model_proto.graph.input:
@@ -213,7 +216,6 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
         rtol: Optional[float] = 1e-3,
         atol: Optional[float] = 1e-7,
         has_mutation: bool = False,
-        verbose: bool = False,
         additional_test_inputs: Optional[
             List[
                 Union[
@@ -235,8 +237,6 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
             has_mutation (bool, optional): Whether the model mutates its input or state.
                 `mutation` as `True` incurs extra overhead of cloning the inputs and model.
                 Defaults to False.
-            verbose (bool, optional): Whether to save diagnostics as Sarif log and print
-                verbose information. Defaults to False.
             additional_test_inputs: Test the models with another dataset input, which
                 is designed for dynamic axes testing. Defaults to None. It's a list of
                 different input sets in tuples. Inside tuple, the first element is a tuple
@@ -254,7 +254,11 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
         if input_kwargs is None:
             input_kwargs = {}
 
-        if has_mutation:
+        if (
+            has_mutation
+            and self.model_type
+            != pytorch_test_common.TorchModelType.TORCH_EXPORT_EXPORTEDPROGRAM
+        ):
             ref_model = _try_clone_model(model)
             ref_input_args, ref_input_kwargs = _try_clone_inputs(
                 input_args, input_kwargs
@@ -264,12 +268,28 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
             ref_input_args = input_args
             ref_input_kwargs = input_kwargs
 
+        assert isinstance(ref_model, torch.nn.Module) or callable(
+            ref_model
+        ), "Model must be a torch.nn.Module or callable"
+        if (
+            self.model_type
+            == pytorch_test_common.TorchModelType.TORCH_EXPORT_EXPORTEDPROGRAM
+        ):
+            ref_model = torch.export.export(ref_model, args=ref_input_args)
+            if (
+                self.dynamic_shapes
+            ):  # TODO: Support dynamic shapes for torch.export.ExportedProgram
+                #       https://github.com/pytorch/pytorch/issues/113705
+                pytest.xfail(
+                    reason="torch.export.ExportedProgram does not support dynamic shapes"
+                )
+
         # Feed args and kwargs into exporter.
         # Note that exporter should flatten kwargs into positional args the exported model;
         # since ONNX doesn't represent kwargs.
         export_error: Optional[torch.onnx.OnnxExporterError] = None
         try:
-            export_output = torch.onnx.dynamo_export(
+            onnx_program = torch.onnx.dynamo_export(
                 ref_model,
                 *ref_input_args,
                 **ref_input_kwargs,
@@ -283,13 +303,14 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
             )
         except torch.onnx.OnnxExporterError as e:
             export_error = e
-            export_output = e.export_output
+            onnx_program = e.onnx_program
 
-        if verbose and diagnostics.is_onnx_diagnostics_log_artifact_enabled():
-            export_output.save_diagnostics(
+        if diagnostics.is_onnx_diagnostics_log_artifact_enabled():
+            onnx_program.save_diagnostics(
                 f"test_report_{self._testMethodName}"
                 f"_op_level_debug_{self.op_level_debug}"
                 f"_dynamic_axes_{self.dynamic_shapes}"
+                f"_model_type_{self.model_type}"
                 ".sarif"
             )
 
@@ -297,11 +318,14 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
             raise export_error
 
         if not skip_dynamic_shapes_check:
-            assert_dynamic_shapes(export_output, self.dynamic_shapes)
+            assert_dynamic_shapes(onnx_program, self.dynamic_shapes)
+
+        if isinstance(ref_model, torch.export.ExportedProgram):
+            ref_model = ref_model.module()
 
         _compare_pytorch_onnx_with_ort(
-            export_output,
-            model,
+            onnx_program,
+            ref_model,
             input_args,
             input_kwargs,
             atol,
@@ -323,8 +347,8 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
                     else {}
                 )
                 _compare_pytorch_onnx_with_ort(
-                    export_output,
-                    model,
+                    onnx_program,
+                    ref_model,
                     additional_input_args,
                     additional_input_kwargs,
                     atol,
@@ -335,7 +359,7 @@ class _TestONNXRuntime(pytorch_test_common.ExportTestCase):
 
 @_beartype.beartype
 def run_ort(
-    onnx_model: Union[str, torch.onnx.ExportOutput],
+    onnx_model: Union[str, torch.onnx.ONNXProgram],
     pytorch_inputs: Sequence[_InputArgsType],
 ) -> _OutputsType:
     """Run ORT on the given ONNX model and inputs
@@ -343,7 +367,7 @@ def run_ort(
     Used in test_fx_to_onnx_with_onnxruntime.py
 
     Args:
-        onnx_model (Union[str, torch.onnx.ExportOutput]): Converter ONNX model
+        onnx_model (Union[str, torch.onnx.ONNXProgram]): Converter ONNX model
         pytorch_inputs (Sequence[_InputArgsType]): The given torch inputs
 
     Raises:
@@ -352,7 +376,7 @@ def run_ort(
     Returns:
         _OutputsType: ONNX model predictions
     """
-    if isinstance(onnx_model, torch.onnx.ExportOutput):
+    if isinstance(onnx_model, torch.onnx.ONNXProgram):
         buffer = io.BytesIO()
         onnx_model.save(buffer)
         ort_model = buffer.getvalue()
@@ -372,7 +396,10 @@ def run_ort(
             f"Expected {len(input_names)} inputs, got {len(pytorch_inputs)}"
         )
 
-    ort_input = {k: v.cpu().numpy() for k, v in zip(input_names, pytorch_inputs)}
+    ort_input = {
+        k: torch.Tensor.numpy(v, force=True)
+        for k, v in zip(input_names, pytorch_inputs)
+    }
     return session.run(None, ort_input)
 
 
@@ -397,7 +424,7 @@ def _try_clone_inputs(input_args, input_kwargs):
 
 @_beartype.beartype
 def _compare_pytorch_onnx_with_ort(
-    export_output: torch.onnx.ExportOutput,
+    onnx_program: torch.onnx.ONNXProgram,
     model: _ModelType,
     input_args: Sequence[_InputArgsType],
     input_kwargs: Mapping[str, _InputArgsType],
@@ -413,20 +440,19 @@ def _compare_pytorch_onnx_with_ort(
         ref_input_args = input_args
         ref_input_kwargs = input_kwargs
 
-    # Format original model inputs into the format expected by exported ONNX model.
-    onnx_format_args = export_output.adapt_torch_inputs_to_onnx(
-        *input_args, **input_kwargs
-    )
-
-    ref_outputs = export_output.adapt_torch_outputs_to_onnx(
-        ref_model(*ref_input_args, **ref_input_kwargs)
-    )
-    ort_outputs = run_ort(export_output, onnx_format_args)
+    # NOTE: ONNXProgram holds a reference (not copy) to the original ref_model, including its state_dict.
+    # Thus, ONNXProgram() must run before ref_model() to prevent ref_model.forward() from changing the state_dict.
+    # Otherwise, the ref_model can change buffers on state_dict which would be used by ONNXProgram.__call__()
+    # NOTE: `model_with_state_dict=ref_model` is specified to cover runs with FakeTensor support
+    ort_outputs = onnx_program(*input_args, **input_kwargs)
+    ref_outputs = ref_model(*ref_input_args, **ref_input_kwargs)
+    ref_outputs = onnx_program.adapt_torch_outputs_to_onnx(ref_outputs)
 
     if len(ref_outputs) != len(ort_outputs):
         raise AssertionError(
             f"Expected {len(ref_outputs)} outputs, got {len(ort_outputs)}"
         )
+
     for ref_output, ort_output in zip(ref_outputs, ort_outputs):
         torch.testing.assert_close(
             ref_output, torch.tensor(ort_output), rtol=rtol, atol=atol
@@ -439,7 +465,6 @@ MIN_ONNX_OPSET_VERSION = 9
 MAX_ONNX_OPSET_VERSION = _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
 TESTED_OPSETS = range(MIN_ONNX_OPSET_VERSION, MAX_ONNX_OPSET_VERSION + 1)
 
-# TODO(titaiwang): Change this when more versions are supported
 # The min onnx opset version to test for
 FX_MIN_ONNX_OPSET_VERSION = 18
 # The max onnx opset version to test for
@@ -449,11 +474,11 @@ FX_TESTED_OPSETS = range(FX_MIN_ONNX_OPSET_VERSION, FX_MAX_ONNX_OPSET_VERSION +
 BOOL_TYPES = (torch.bool,)
 
 INT_TYPES = (
-    torch.int8,
-    torch.int16,
+    # torch.int8,
+    # torch.int16,
     torch.int32,
     torch.int64,
-    torch.uint8,
+    # torch.uint8,
 )
 
 QINT_TYPES = (
@@ -480,6 +505,8 @@ TESTED_DTYPES = (
     *INT_TYPES,
     # Floating types
     *FLOAT_TYPES,
+    # Complex types
+    *COMPLEX_TYPES,
 )
 
 
@@ -499,6 +526,7 @@ class DecorateMeta:
         test_behavior: The behavior of the test case. [skip or xfail]
         matcher: The matcher to apply to the test case.
         enabled_if: Whether to enable test behavior. Usually used on onnx/ort version control
+        model_type: The type of the torch model. Defaults to None.
     """
 
     op_name: str
@@ -510,6 +538,7 @@ class DecorateMeta:
     test_behavior: str
     matcher: Optional[Callable[[Any], bool]] = None
     enabled_if: bool = True
+    model_type: Optional[pytorch_test_common.TorchModelType] = None
 
     def contains_opset(self, opset: int) -> bool:
         if self.opsets is None:
@@ -529,6 +558,7 @@ def xfail(
     dtypes: Optional[Collection[torch.dtype]] = None,
     matcher: Optional[Callable[[Any], bool]] = None,
     enabled_if: bool = True,
+    model_type: Optional[pytorch_test_common.TorchModelType] = None,
 ):
     """Expects a OpInfo test to fail.
 
@@ -541,6 +571,7 @@ def xfail(
         matcher: A function that matches the test sample input. It is used only when
             xfail is in the SKIP_XFAIL_SUBTESTS list.
         enabled_if: Whether to enable xfail. Usually used on onnx/ort version control
+        model_type: The type of the torch model. Defaults to None.
     """
     return DecorateMeta(
         op_name=op_name,
@@ -552,6 +583,7 @@ def xfail(
         matcher=matcher,
         reason=reason,
         test_behavior="xfail",
+        model_type=model_type,
     )
 
 
@@ -564,6 +596,7 @@ def skip(
     dtypes: Optional[Collection[torch.dtype]] = None,
     matcher: Optional[Callable[[Any], Any]] = None,
     enabled_if: bool = True,
+    model_type: Optional[pytorch_test_common.TorchModelType] = None,
 ):
     """Skips a test case in OpInfo that we don't care about.
 
@@ -578,6 +611,7 @@ def skip(
         matcher: A function that matches the test sample input. It is used only when
             skip is in the SKIP_XFAIL_SUBTESTS list.
         enabled_if: Whether to enable skip. Usually used on onnx/ort version control
+        model_type: The type of the torch model. Defaults to None.
     """
     return DecorateMeta(
         op_name=op_name,
@@ -589,6 +623,45 @@ def skip(
         matcher=matcher,
         enabled_if=enabled_if,
         test_behavior="skip",
+        model_type=model_type,
+    )
+
+
+def skip_slow(
+    op_name: str,
+    variant_name: str = "",
+    *,
+    reason: str,
+    opsets: Optional[Collection[Union[int, Callable[[int], bool]]]] = None,
+    dtypes: Optional[Collection[torch.dtype]] = None,
+    matcher: Optional[Callable[[Any], Any]] = None,
+    model_type: Optional[pytorch_test_common.TorchModelType] = None,
+):
+    """Skips a test case in OpInfo that is too slow.
+
+    It needs further investigation to understand why it is slow.
+
+    Args:
+        op_name: The name of the operator.
+        variant_name: The name of the variant.
+        opsets: The opsets to expect the failure. e.g. [9, 10] or [opsets_before(11)]
+        dtypes: The dtypes to expect the failure.
+        reason: The reason for the failure.
+        matcher: A function that matches the test sample input. It is used only when
+            skip is in the SKIP_XFAIL_SUBTESTS list.
+        model_type: The type of the torch model. Defaults to None.
+    """
+    return DecorateMeta(
+        op_name=op_name,
+        variant_name=variant_name,
+        decorator=common_utils.slowTest,
+        opsets=opsets,
+        dtypes=dtypes,
+        reason=reason,
+        matcher=matcher,
+        enabled_if=not common_utils.TEST_WITH_SLOW,
+        test_behavior="skip",
+        model_type=model_type,
     )
 
 
@@ -617,6 +690,11 @@ def add_decorate_info(
         assert (
             opinfo is not None
         ), f"Couldn't find OpInfo for {decorate_meta}. Did you need to specify variant_name?"
+        assert decorate_meta.model_type is None, (
+            f"Tested op: {decorate_meta.op_name} in wrong position! "
+            "If model_type needs to be specified, it should be "
+            "put under SKIP_XFAIL_SUBTESTS_WITH_MATCHER_AND_MODEL_TYPE."
+        )
         decorators = list(opinfo.decorators)
         new_decorator = opinfo_core.DecorateInfo(
             decorate_meta.decorator,
