@@ -5,9 +5,11 @@ import io
 import numpy as np
 import onnx
 import pytorch_test_common
-import torch
 from pytorch_test_common import skipIfUnsupportedMinOpsetVersion
-from torch.onnx import _constants, symbolic_helper, utils
+
+import torch
+from torch.onnx import _constants, utils
+from torch.onnx._globals import GLOBALS
 from torch.onnx._internal import jit_utils
 from torch.testing._internal import common_utils
 
@@ -31,6 +33,7 @@ def as_graphcontext(graph: torch.Graph) -> jit_utils.GraphContext:
         original_node=None,  # type: ignore[arg-type]
         params_dict={},
         env={},
+        values_in_env=set(),
     )
 
 
@@ -41,8 +44,7 @@ def g_op(graph: torch.Graph, op_name: str, *args, **kwargs):
 class TestONNXShapeInference(pytorch_test_common.ExportTestCase):
     def setUp(self):
         self.opset_version = _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
-        symbolic_helper._set_onnx_shape_inference(True)
-        symbolic_helper._set_opset_version(self.opset_version)
+        GLOBALS.export_onnx_opset_version = self.opset_version
 
     def run_test(self, g, n, type_assertion_funcs):
         if not isinstance(type_assertion_funcs, list):
