@@ -43,7 +43,7 @@ from hypothesis import strategies as st
 import torch.testing._internal.hypothesis_utils as hu
 hu.assert_deadline_disabled()
 from torch.testing._internal.common_cuda import TEST_MULTIGPU, TEST_CUDA
-from torch.testing._internal.common_utils import TestCase
+from torch.testing._internal.common_utils import TestCase, skipIfTorchDynamo
 from torch.testing._internal.common_quantization import (
     QuantizationTestCase,
     AnnotatedSingleLayerLinearModel,
@@ -137,16 +137,17 @@ class TestObserver(QuantizationTestCase):
             state_dict = myobs.state_dict()
             b = io.BytesIO()
             torch.save(state_dict, b)
-            b.seek(0)
-            loaded_dict = torch.load(b)
-            for key in state_dict:
-                self.assertEqual(state_dict[key], loaded_dict[key])
-            loaded_obs = MinMaxObserver(dtype=qdtype, qscheme=qscheme, reduce_range=reduce_range)
-            loaded_obs.load_state_dict(loaded_dict)
-            loaded_qparams = loaded_obs.calculate_qparams()
-            self.assertEqual(myobs.min_val, loaded_obs.min_val)
-            self.assertEqual(myobs.max_val, loaded_obs.max_val)
-            self.assertEqual(myobs.calculate_qparams(), loaded_obs.calculate_qparams())
+            for weights_only in [True, False]:
+                b.seek(0)
+                loaded_dict = torch.load(b, weights_only=weights_only)
+                for key in state_dict:
+                    self.assertEqual(state_dict[key], loaded_dict[key])
+                loaded_obs = MinMaxObserver(dtype=qdtype, qscheme=qscheme, reduce_range=reduce_range)
+                loaded_obs.load_state_dict(loaded_dict)
+                loaded_qparams = loaded_obs.calculate_qparams()
+                self.assertEqual(myobs.min_val, loaded_obs.min_val)
+                self.assertEqual(myobs.max_val, loaded_obs.max_val)
+                self.assertEqual(myobs.calculate_qparams(), loaded_obs.calculate_qparams())
 
 
     @given(qdtype=st.sampled_from((torch.qint8, torch.quint8)),
@@ -327,6 +328,61 @@ class TestObserver(QuantizationTestCase):
         self.assertEqual(min_shape_before, obs.min_val.shape)
         self.assertEqual(max_shape_before, obs.max_val.shape)
 
+    def test_histogram_observer_ignore_infinity(self):
+        """
+        Ensures that HistogramObserver doesn't record values of infinity
+        """
+        obs = HistogramObserver()
+        obs2 = HistogramObserver()
+        x = torch.randn(4, 4, 4, 4)
+        obs(x * torch.inf)
+        obs(x)
+        obs2(x)
+        obs(x * torch.inf)
+        self.assertTrue(obs.min_val != -torch.inf and obs.max_val != torch.inf)
+        self.assertEqual(obs.histogram, obs2.histogram)
+
+    def test_histogram_observer_handle_close_to_infinity(self):
+        for sign in [-1, 1]:
+            obser = HistogramObserver.with_args(reduce_range=False)()
+            mask = torch.tensor([-3.4028234663852886 * 10**30, 0, 0, 0]) * sign
+            obser(mask)
+            obser(mask - sign)
+            scale, zp = obser.calculate_qparams()
+
+            input = torch.randn(1, 4)
+            ref_result = torch.softmax(input + mask, dim=1)
+
+            quant_mask = torch.quantize_per_tensor(mask, scale, zp, torch.quint8)
+            dequant_mask = quant_mask.dequantize()
+            result = torch.softmax(input + dequant_mask, dim=1)
+            self.assertEqual(result, ref_result)
+
+    def test_histogram_observer_handle_OOM_due_to_close_min_max_value(self):
+        obser = HistogramObserver.with_args(reduce_range=False)()
+        # close min and max value in the 1st forward() pass of observer tends
+        # to cause OOM in the following pass.
+        # This is due to the allocation of histogram tensor during _combine_histograms().
+        # With sanity check on the size of histogram tensor, we expect the histogram observer
+        # can still work by resetting the histogram
+        x1 = torch.tensor([0, 1e-9])
+        obser(x1)
+
+        x2 = torch.tensor([2.0, 3.0])
+        obser(x2)
+
+    def test_histogram_observer_handle_OOM_due_to_large_upsample_rate(self):
+        # a large upsample rate leads to OOM due to the allocation of histogram tensor
+        # during _combine_histograms(). With sanity check on the size of histogram tensor,
+        # we expect the histogram observer can still work by resetting the histogram
+        obser = HistogramObserver.with_args(upsample_rate=(8000**2), reduce_range=False)()
+
+        x1 = torch.tensor([0, 1.0])
+        obser(x1)
+
+        x2 = torch.tensor([2, 2 + 1e-9])
+        obser(x2)
+
     def test_histogram_observer_save_load_state_dict(self):
         """
         Smoke test on saving/loading state_dict
@@ -399,8 +455,8 @@ class TestObserver(QuantizationTestCase):
             # verify no crash
             x = obs(x)
 
-    def _test_memoryless(self, obs_class):
-        obs = obs_class(averaging_constant=1)
+    def test_dynamic_quant_observer(self):
+        obs = MovingAverageMinMaxObserver(averaging_constant=1, is_dynamic=True)
         x = torch.randn((3, 3))
         obs(x)
         params = obs.calculate_qparams()
@@ -410,11 +466,26 @@ class TestObserver(QuantizationTestCase):
             obs(x)
             self.assertEqual(params, obs.calculate_qparams())
 
-    def test_memoryless_minmaxobserver(self):
-        self._test_memoryless(MovingAverageMinMaxObserver)
+    def test_dynamic_quant_observer_matching_choose_qparams(self):
+        obs = MovingAverageMinMaxObserver(averaging_constant=1, is_dynamic=True)
+        for x in [torch.randn(3, 3), torch.rand(3, 3, 3), torch.randn(3, 3, 3, 3)]:
+            obs(x)
+            params = obs.calculate_qparams()
+            scale, zero_point = torch._choose_qparams_per_tensor(x)
+            self.assertEqual(scale, params[0])
+            self.assertEqual(zero_point, params[1])
 
-    def test_memoryless_perchannelminmaxobserver(self):
-        self._test_memoryless(MovingAveragePerChannelMinMaxObserver)
+    def test_per_channel_observers_load_state_dict(self):
+        observer_list = [PerChannelMinMaxObserver, MovingAveragePerChannelMinMaxObserver]
+
+        for obs_cls in observer_list:
+            obs = obs_cls()
+            obs(torch.randn((32, 32)))
+            new_obs = obs_cls()
+            # make sure the state_dict can be loaded
+            new_obs.load_state_dict(obs.state_dict())
+            self.assertTrue(torch.equal(obs.min_val, new_obs.min_val))
+            self.assertTrue(torch.equal(obs.max_val, new_obs.max_val))
 
 # HistogramObserver that works like it does on master
 class _ReferenceHistogramObserver(HistogramObserver):
@@ -692,6 +763,7 @@ class TestHistogramObserver(QuantizationTestCase):
         self.assertEqual(myobs.max_val, 8.0)
         self.assertEqual(myobs.histogram, [2., 3., 3.])
 
+    @skipIfTorchDynamo("too slow")
     @given(N=st.sampled_from([10, 1000]),
            bins=st.sampled_from([256, 512, 1024, 2048]),
            dtype=st.sampled_from([torch.qint8, torch.quint8]),
