@@ -1,13 +1,14 @@
 #!/usr/bin/env python3
 # Owner(s): ["oncall: mobile"]
+# mypy: allow-untyped-defs
 
 import io
 import textwrap
-from typing import List, Optional, Dict
+from typing import Dict, List, Optional
 
 import torch
 import torch.utils.bundled_inputs
-from torch.testing._internal.common_utils import TestCase, run_tests
+from torch.testing._internal.common_utils import run_tests, TestCase
 
 
 def model_size(sm):
@@ -24,7 +25,6 @@ def save_and_load(sm):
 
 
 class TestBundledInputs(TestCase):
-
     def test_single_tensors(self):
         class SingleTensorModel(torch.nn.Module):
             def forward(self, arg):
@@ -32,7 +32,7 @@ class TestBundledInputs(TestCase):
 
         sm = torch.jit.script(SingleTensorModel())
         original_size = model_size(sm)
-        get_expr : List[str] = []
+        get_expr: List[str] = []
         samples = [
             # Tensor with small numel and small storage.
             (torch.tensor([1]),),
@@ -50,7 +50,8 @@ class TestBundledInputs(TestCase):
             (torch.quantize_per_tensor(torch.zeros(4, 8, 32, 32), 1, 0, torch.qint8),),
         ]
         torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-            sm, samples, get_expr)
+            sm, samples, get_expr
+        )
         # print(get_expr[0])
         # print(sm._generate_bundled_inputs.code)
 
@@ -80,18 +81,17 @@ class TestBundledInputs(TestCase):
         self.assertEqual(inflated[5][0].mean().item(), 0, atol=0.025, rtol=0)
         self.assertEqual(inflated[5][0].std().item(), 1, atol=0.02, rtol=0)
 
-
     def test_large_tensor_with_inflation(self):
         class SingleTensorModel(torch.nn.Module):
             def forward(self, arg):
                 return arg
+
         sm = torch.jit.script(SingleTensorModel())
         sample_tensor = torch.randn(1 << 16)
         # We can store tensors with custom inflation functions regardless
         # of size, even if inflation is just the identity.
         sample = torch.utils.bundled_inputs.bundle_large_tensor(sample_tensor)
-        torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-            sm, [(sample,)])
+        torch.utils.bundled_inputs.augment_model_with_bundled_inputs(sm, [(sample,)])
 
         loaded = save_and_load(sm)
         inflated = loaded.get_all_bundled_inputs()
@@ -99,17 +99,18 @@ class TestBundledInputs(TestCase):
 
         self.assertEqual(inflated[0][0], sample_tensor)
 
-
     def test_rejected_tensors(self):
         def check_tensor(sample):
             # Need to define the class in this scope to get a fresh type for each run.
             class SingleTensorModel(torch.nn.Module):
                 def forward(self, arg):
                     return arg
+
             sm = torch.jit.script(SingleTensorModel())
             with self.assertRaisesRegex(Exception, "Bundled input argument"):
                 torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-                    sm, [(sample,)])
+                    sm, [(sample,)]
+                )
 
         # Plain old big tensor.
         check_tensor(torch.randn(1 << 16))
@@ -120,7 +121,6 @@ class TestBundledInputs(TestCase):
         self.assertEqual(small_sparse.numel(), 2)
         check_tensor(small_sparse)
 
-
     def test_non_tensors(self):
         class StringAndIntModel(torch.nn.Module):
             def forward(self, fmt: str, num: int):
@@ -131,8 +131,7 @@ class TestBundledInputs(TestCase):
             ("first {}", 1),
             ("second {}", 2),
         ]
-        torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-            sm, samples)
+        torch.utils.bundled_inputs.augment_model_with_bundled_inputs(sm, samples)
 
         loaded = save_and_load(sm)
         inflated = loaded.get_all_bundled_inputs()
@@ -162,23 +161,17 @@ class TestBundledInputs(TestCase):
             (torch.ones(4, 8, 32, 32).contiguous(memory_format=torch.channels_last),),
         ]
         info = [
-            'Tensor with small numel and small storage.',
-            'Tensor with large numel and small storage.',
-            'Tensor with small numel and large storage.',
-            'Large zero tensor.',
-            'Large channels-last ones tensor.',
-            'Special encoding of random tensor.',
+            "Tensor with small numel and small storage.",
+            "Tensor with large numel and small storage.",
+            "Tensor with small numel and large storage.",
+            "Large zero tensor.",
+            "Large channels-last ones tensor.",
+            "Special encoding of random tensor.",
         ]
         torch.utils.bundled_inputs.augment_many_model_functions_with_bundled_inputs(
             mm,
-            inputs={
-                mm.forward : samples,
-                mm.foo : samples
-            },
-            info={
-                mm.forward : info,
-                mm.foo : info
-            }
+            inputs={mm.forward: samples, mm.foo: samples},
+            info={mm.forward: info, mm.foo: info},
         )
         loaded = save_and_load(mm)
         inflated = loaded.get_all_bundled_inputs()
@@ -194,15 +187,21 @@ class TestBundledInputs(TestCase):
 
         # Check helper that work on all functions
         all_info = loaded.get_bundled_inputs_functions_and_info()
-        self.assertEqual(set(all_info.keys()), {'forward', 'foo'})
-        self.assertEqual(all_info['forward']['get_inputs_function_name'], ['get_all_bundled_inputs_for_forward'])
-        self.assertEqual(all_info['foo']['get_inputs_function_name'], ['get_all_bundled_inputs_for_foo'])
-        self.assertEqual(all_info['forward']['info'], info)
-        self.assertEqual(all_info['foo']['info'], info)
+        self.assertEqual(set(all_info.keys()), {"forward", "foo"})
+        self.assertEqual(
+            all_info["forward"]["get_inputs_function_name"],
+            ["get_all_bundled_inputs_for_forward"],
+        )
+        self.assertEqual(
+            all_info["foo"]["get_inputs_function_name"],
+            ["get_all_bundled_inputs_for_foo"],
+        )
+        self.assertEqual(all_info["forward"]["info"], info)
+        self.assertEqual(all_info["foo"]["info"], info)
 
         # example of how to turn the 'get_inputs_function_name' into the actual list of bundled inputs
         for func_name in all_info.keys():
-            input_func_name = all_info[func_name]['get_inputs_function_name'][0]
+            input_func_name = all_info[func_name]["get_inputs_function_name"][0]
             func_to_run = getattr(loaded, input_func_name)
             self.assertEqual(func_to_run(), samples)
 
@@ -220,16 +219,18 @@ class TestBundledInputs(TestCase):
         # inputs defined 2 ways so should fail
         with self.assertRaises(Exception):
             mm = torch.jit.script(MultipleMethodModel())
-            definition = textwrap.dedent("""
+            definition = textwrap.dedent(
+                """
                 def _generate_bundled_inputs_for_forward(self):
                     return []
-                """)
+                """
+            )
             mm.define(definition)
             torch.utils.bundled_inputs.augment_many_model_functions_with_bundled_inputs(
                 mm,
                 inputs={
-                    mm.forward : samples,
-                    mm.foo : samples,
+                    mm.forward: samples,
+                    mm.foo: samples,
                 },
             )
 
@@ -251,8 +252,8 @@ class TestBundledInputs(TestCase):
             torch.utils.bundled_inputs.augment_many_model_functions_with_bundled_inputs(
                 mm,
                 inputs={
-                    mm.forward : None,
-                    mm.foo : samples,
+                    mm.forward: None,
+                    mm.foo: samples,
                 },
             )
 
@@ -265,8 +266,7 @@ class TestBundledInputs(TestCase):
         with self.assertRaises(TypeError):
             m = torch.jit.script(SingleTensorModel())
             torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-                m,
-                inputs="foo"  # type: ignore[arg-type]
+                m, inputs="foo"  # type: ignore[arg-type]
             )
 
         # List of non tuples. Most common error using the api.
@@ -274,7 +274,9 @@ class TestBundledInputs(TestCase):
             m = torch.jit.script(SingleTensorModel())
             torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
                 m,
-                inputs=[torch.ones(1, 2), ]  # type: ignore[list-item]
+                inputs=[
+                    torch.ones(1, 2),  # type: ignore[list-item]
+                ],
             )
 
     def test_double_augment_fail(self):
@@ -284,13 +286,13 @@ class TestBundledInputs(TestCase):
 
         m = torch.jit.script(SingleTensorModel())
         torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-            m,
-            inputs=[(torch.ones(1),)]
+            m, inputs=[(torch.ones(1),)]
         )
-        with self.assertRaisesRegex(Exception, "Models can only be augmented with bundled inputs once."):
+        with self.assertRaisesRegex(
+            Exception, "Models can only be augmented with bundled inputs once."
+        ):
             torch.utils.bundled_inputs.augment_model_with_bundled_inputs(
-                m,
-                inputs=[(torch.ones(1),)]
+                m, inputs=[(torch.ones(1),)]
             )
 
     def test_double_augment_non_mutator(self):
@@ -300,8 +302,7 @@ class TestBundledInputs(TestCase):
 
         m = torch.jit.script(SingleTensorModel())
         bundled_model = torch.utils.bundled_inputs.bundle_inputs(
-            m,
-            inputs=[(torch.ones(1),)]
+            m, inputs=[(torch.ones(1),)]
         )
         with self.assertRaises(AttributeError):
             m.get_all_bundled_inputs()
@@ -315,18 +316,15 @@ class TestBundledInputs(TestCase):
 
         m = torch.jit.script(SingleTensorModel())
         bundled_model = torch.utils.bundled_inputs.bundle_inputs(
-            m,
-            inputs={m.forward : [(torch.ones(1),)]}
+            m, inputs={m.forward: [(torch.ones(1),)]}
         )
         self.assertEqual(bundled_model.get_all_bundled_inputs(), [(torch.ones(1),)])
 
         bundled_model2 = torch.utils.bundled_inputs.bundle_inputs(
-            bundled_model,
-            inputs=[(torch.ones(2),)]
+            bundled_model, inputs=[(torch.ones(2),)]
         )
         self.assertEqual(bundled_model2.get_all_bundled_inputs(), [(torch.ones(2),)])
 
-
     def test_dict_args(self):
         class MyModel(torch.nn.Module):
             def forward(
@@ -396,7 +394,7 @@ class TestBundledInputs(TestCase):
                 """,
             )
 
-        out : List[str] = []
+        out: List[str] = []
         sm = torch.jit.script(MyModel())
         original_size = model_size(sm)
         small_inputs = (
@@ -426,7 +424,10 @@ class TestBundledInputs(TestCase):
         inflated = loaded.get_all_bundled_inputs()
         self.assertEqual(len(inflated[0]), len(small_inputs))
 
-        methods, _ = torch.utils.bundled_inputs._get_bundled_inputs_attributes_and_methods(
+        (
+            methods,
+            _,
+        ) = torch.utils.bundled_inputs._get_bundled_inputs_attributes_and_methods(
             loaded
         )
 
@@ -435,9 +436,9 @@ class TestBundledInputs(TestCase):
         # two args which have InflatableArg with fmt_fn
         # 1 * 2 * 2 = 4
         self.assertEqual(
-            sum([method.startswith("_inflate_helper") for method in methods]), 4
+            sum(method.startswith("_inflate_helper") for method in methods), 4
         )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     run_tests()
