@@ -4,16 +4,16 @@ import collections
 import unittest
 
 import torch
-from torch.testing._internal.common_utils import (
-    TestCase, run_tests, TEST_WITH_ASAN)
+from torch.testing._internal.common_utils import run_tests, TEST_WITH_ASAN, TestCase
 
 try:
     import psutil
+
     HAS_PSUTIL = True
 except ImportError:
     HAS_PSUTIL = False
 
-device = torch.device('cpu')
+device = torch.device("cpu")
 
 
 class Network(torch.nn.Module):
@@ -50,21 +50,21 @@ class TestOpenMP_ParallelFor(TestCase):
             if idx == 0:
                 continue
             is_increasing = is_increasing and (last_rss[idx] > last_rss[idx - 1])
-        self.assertTrue(not is_increasing,
-                        msg=f'memory usage is increasing, {str(last_rss)}')
+        self.assertTrue(
+            not is_increasing, msg=f"memory usage is increasing, {str(last_rss)}"
+        )
 
     def test_one_thread(self):
-        """Make sure there is no memory leak with one thread: issue gh-32284
-        """
+        """Make sure there is no memory leak with one thread: issue gh-32284"""
         torch.set_num_threads(1)
         self.func_rss(300)
 
     def test_n_threads(self):
-        """Make sure there is no memory leak with many threads
-        """
+        """Make sure there is no memory leak with many threads"""
         ncores = min(5, psutil.cpu_count(logical=False))
         torch.set_num_threads(ncores)
         self.func_rss(300)
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     run_tests()
