@@ -253,9 +253,13 @@ class TestStatelessFunctionalAPI(TestCase):
         parameters = {'l1.parametrizations.weight.original': torch.nn.Parameter(torch.tensor([[1.0]])),
                       'l1.bias': torch.tensor([0.0]),
                       'buffer': torch.tensor([0.0])}
+
         with self.assertRaisesRegex(RuntimeError, "shapes cannot be multiplied"):
-            x = torch.rand((4, 5))  # to work, it should be of size (1, 1)
-            functional_call(module, parameters, x)  # this call will fail because x is the wrong size
+            @torch._dynamo.disable
+            def _error_case():
+                x = torch.rand((4, 5))  # to work, it should be of size (1, 1)
+                functional_call(module, parameters, x)  # this call will fail because x is the wrong size
+            _error_case()
 
         # verify that the spectral normalization is still applied
         self.assertTrue('l1.parametrizations.weight.original' in dict(module.named_parameters()))
@@ -897,7 +901,7 @@ exit(len(w))
         m = torch.nn.Linear(1, 1)
         params = dict(m.named_parameters())
         x = torch.randn(3, 1)
-        with self.assertWarnsRegex(UserWarning, "Please use torch.func.functional_call"):
+        with self.assertWarnsRegex(FutureWarning, "Please use `torch.func.functional_call`"):
             stateless.functional_call(m, params, x)
 
 class TestPythonOptimizeMode(TestCase):
