@@ -3,10 +3,17 @@
 """ Test printing of scalar types.
 
 """
+import functools
+
+from unittest import skipIf as skipif
+
 import pytest
 
 import torch._numpy as np
 from torch._numpy.testing import assert_
+from torch.testing._internal.common_utils import run_tests, TestCase
+
+skip = functools.partial(skipif, True)
 
 
 class A:
@@ -42,8 +49,8 @@ class B1(np.float64, HasNew):
     pass
 
 
-@pytest.mark.skip(reason="scalar repr: numpy plans to make it more explicit")
-class TestInherit:
+@skip(reason="scalar repr: numpy plans to make it more explicit")
+class TestInherit(TestCase):
     def test_init(self):
         x = B(1.0)
         assert_(str(x) == "1.0")
@@ -66,3 +73,7 @@ class TestInherit:
         # previously caused RecursionError!?
         with pytest.raises(TypeError):
             B1(1.0, 2.0)
+
+
+if __name__ == "__main__":
+    run_tests()
