@@ -1,26 +1,51 @@
 # Owner(s): ["module: dynamo"]
 
-import pytest
+from unittest import skipIf, SkipTest
 
-import torch._numpy as np
+import numpy
+
+import pytest
 from pytest import raises as assert_raises
 
-from torch._numpy import _util
-from torch._numpy.testing import (
-    assert_allclose,
-    assert_almost_equal,
-    assert_array_equal,
-    assert_equal,
+from torch.testing._internal.common_utils import (
+    instantiate_parametrized_tests,
+    parametrize,
+    run_tests,
+    TEST_WITH_TORCHDYNAMO,
+    TestCase,
+    xpassIfTorchDynamo,
 )
 
 
-class TestFlatnonzero:
+# If we are going to trace through these, we should use NumPy
+# If testing on eager mode, we use torch._numpy
+if TEST_WITH_TORCHDYNAMO:
+    import numpy as np
+    import numpy.core.numeric as _util  # for normalize_axis_tuple
+    from numpy.testing import (
+        assert_allclose,
+        assert_almost_equal,
+        assert_array_equal,
+        assert_equal,
+    )
+else:
+    import torch._numpy as np
+    from torch._numpy import _util
+    from torch._numpy.testing import (
+        assert_allclose,
+        assert_almost_equal,
+        assert_array_equal,
+        assert_equal,
+    )
+
+
+class TestFlatnonzero(TestCase):
     def test_basic(self):
         x = np.arange(-2, 3)
         assert_equal(np.flatnonzero(x), [0, 1, 3, 4])
 
 
-class TestAny:
+class TestAny(TestCase):
     def test_basic(self):
         y1 = [0, 0, 1, 0]
         y2 = [0, 0, 0, 0]
@@ -43,7 +68,7 @@ class TestAny:
         assert_equal(np.any(y), y.any())
 
 
-class TestAll:
+class TestAll(TestCase):
     def test_basic(self):
         y1 = [0, 1, 1, 0]
         y2 = [0, 0, 0, 0]
@@ -65,7 +90,7 @@ class TestAll:
         assert_equal(np.all(y), y.all())
 
 
-class TestMean:
+class TestMean(TestCase):
     def test_mean(self):
         A = [[1, 2, 3], [4, 5, 6]]
         assert np.mean(A) == 3.5
@@ -103,7 +128,7 @@ class TestMean:
         # of float32.
         assert np.mean(np.ones(100000, dtype="float16")) == 1
 
-    @pytest.mark.xfail(reason="XXX: mean(..., where=...) not implemented")
+    @xpassIfTorchDynamo  # (reason="XXX: mean(..., where=...) not implemented")
     def test_mean_where(self):
         a = np.arange(16).reshape((4, 4))
         wh_full = np.array(
@@ -141,7 +166,8 @@ class TestMean:
             assert_equal(np.mean(a, where=False), np.nan)
 
 
-class TestSum:
+@instantiate_parametrized_tests
+class TestSum(TestCase):
     def test_sum(self):
         m = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
         tgt = [[6], [15], [24]]
@@ -168,7 +194,8 @@ class TestSum:
         assert_allclose(res_float, 4.0, atol=1e-15)
         assert res_float.dtype == "float64"
 
-    @pytest.mark.xfail(reason="sum: does not warn on overflow")
+    @skipIf(numpy.__version__ < "1.24", reason="NP_VER: fails on NumPy 1.23.x")
+    @xpassIfTorchDynamo  # (reason="sum: does not warn on overflow")
     def test_sum_dtypes_warnings(self):
         for dt in (int, np.float16, np.float32, np.float64):
             for v in (0, 1, 2, 7, 8, 9, 15, 16, 19, 127, 128, 1024, 1235):
@@ -209,7 +236,7 @@ class TestSum:
             d += d
             assert_almost_equal(d, 2.0)
 
-    @pytest.mark.parametrize("dt", [np.complex64, np.complex128])
+    @parametrize("dt", [np.complex64, np.complex128])
     def test_sum_complex_1(self, dt):
         for v in (0, 1, 2, 7, 8, 9, 15, 16, 19, 127, 128, 1024, 1235):
             tgt = dt(v * (v + 1) / 2) - dt((v * (v + 1) / 2) * 1j)
@@ -219,7 +246,7 @@ class TestSum:
             assert_allclose(np.sum(d), tgt, atol=1.5e-5)
             assert_allclose(np.sum(np.flip(d)), tgt, atol=1.5e-7)
 
-    @pytest.mark.parametrize("dt", [np.complex64, np.complex128])
+    @parametrize("dt", [np.complex64, np.complex128])
     def test_sum_complex_2(self, dt):
         d = np.ones(500, dtype=dt) + 1j
         assert_allclose(np.sum(d[::2]), 250.0 + 250j, atol=1.5e-7)
@@ -235,7 +262,7 @@ class TestSum:
         d += d
         assert_allclose(d, 2.0 + 2j, atol=1.5e-7)
 
-    @pytest.mark.xfail(reason="initial=... need implementing")
+    @xpassIfTorchDynamo  # (reason="initial=... need implementing")
     def test_sum_initial(self):
         # Integer, single axis
         assert_equal(np.sum([3], initial=2), 5)
@@ -249,7 +276,7 @@ class TestSum:
             [12, 12, 12],
         )
 
-    @pytest.mark.xfail(reason="where=... need implementing")
+    @xpassIfTorchDynamo  # (reason="where=... need implementing")
     def test_sum_where(self):
         # More extensive tests done in test_reduction_with_where.
         assert_equal(np.sum([[1.0, 2.0], [3.0, 4.0]], where=[True, False]), 4.0)
@@ -259,7 +286,46 @@ class TestSum:
         )
 
 
-class _GenericReductionsTestMixin:
+parametrize_axis = parametrize(
+    "axis", [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
+)
+parametrize_func = parametrize(
+    "func",
+    [
+        np.any,
+        np.all,
+        np.argmin,
+        np.argmax,
+        np.min,
+        np.max,
+        np.mean,
+        np.sum,
+        np.prod,
+        np.std,
+        np.var,
+        np.count_nonzero,
+    ],
+)
+
+fails_axes_tuples = {
+    np.any,
+    np.all,
+    np.argmin,
+    np.argmax,
+    np.prod,
+}
+
+fails_out_arg = {
+    np.count_nonzero,
+}
+
+restricts_dtype_casts = {np.var, np.std}
+
+fails_empty_tuple = {np.argmin, np.argmax}
+
+
+@instantiate_parametrized_tests
+class TestGenericReductions(TestCase):
     """Run a set of generic tests to verify that self.func acts like a
     reduction operation.
 
@@ -269,64 +335,76 @@ class _GenericReductionsTestMixin:
     To use: subclass, define self.func and self.allowed_axes.
     """
 
-    def test_bad_axis(self):
+    @parametrize_func
+    def test_bad_axis(self, func):
         # Basic check of functionality
         m = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
 
-        assert_raises(TypeError, self.func, m, axis="foo")
-        assert_raises(np.AxisError, self.func, m, axis=3)
-        assert_raises(TypeError, self.func, m, axis=np.array([[1], [2]]))
-        assert_raises(TypeError, self.func, m, axis=1.5)
+        assert_raises(TypeError, func, m, axis="foo")
+        assert_raises(np.AxisError, func, m, axis=3)
+        assert_raises(TypeError, func, m, axis=np.array([[1], [2]]))
+        assert_raises(TypeError, func, m, axis=1.5)
 
         # TODO: add tests with np.int32(3) etc, when implemented
 
-    def test_array_axis(self):
+    @parametrize_func
+    def test_array_axis(self, func):
         a = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
-        assert_equal(self.func(a, axis=np.array(-1)), self.func(a, axis=-1))
+        assert_equal(func(a, axis=np.array(-1)), func(a, axis=-1))
 
         with assert_raises(TypeError):
-            self.func(a, axis=np.array([1, 2]))
+            func(a, axis=np.array([1, 2]))
+
+    @parametrize_func
+    def test_axis_empty_generic(self, func):
+        if func in fails_empty_tuple:
+            raise SkipTest("func(..., axis=()) is not valid")
 
-    def test_axis_empty_generic(self):
         a = np.array([[0, 0, 1], [1, 0, 1]])
-        assert_array_equal(
-            self.func(a, axis=()), self.func(np.expand_dims(a, axis=0), axis=0)
-        )
+        assert_array_equal(func(a, axis=()), func(np.expand_dims(a, axis=0), axis=0))
 
-    def test_axis_bad_tuple(self):
+    @parametrize_func
+    def test_axis_bad_tuple(self, func):
         # Basic check of functionality
         m = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
-        with assert_raises(ValueError):
-            self.func(m, axis=(1, 1))
 
-    def _check_keepdims_generic(self, axis):
+        if func in fails_axes_tuples:
+            raise SkipTest(f"{func.__name__} does not allow tuple axis.")
+
+        with assert_raises(ValueError):
+            func(m, axis=(1, 1))
+
+    @parametrize_axis
+    @parametrize_func
+    def test_keepdims_generic(self, axis, func):
+        if func in fails_axes_tuples:
+            raise SkipTest(f"{func.__name__} does not allow tuple axis.")
+
         a = np.arange(2 * 3 * 4).reshape((2, 3, 4))
-        with_keepdims = self.func(a, axis, keepdims=True)
-        expanded = np.expand_dims(self.func(a, axis=axis), axis=axis)
+        with_keepdims = func(a, axis, keepdims=True)
+        expanded = np.expand_dims(func(a, axis=axis), axis=axis)
         assert_array_equal(with_keepdims, expanded)
 
-    def test_keepdims_generic(self):
-        for axis in self.allowed_axes:
-            self._check_keepdims_generic(axis)
-
-    def test_keepdims_generic_axis_none(self):
+    @skipIf(numpy.__version__ < "1.24", reason="NP_VER: fails on CI w/old numpy")
+    @parametrize_func
+    def test_keepdims_generic_axis_none(self, func):
         a = np.arange(2 * 3 * 4).reshape((2, 3, 4))
-        with_keepdims = self.func(a, axis=None, keepdims=True)
-        scalar = self.func(a, axis=None)
+        with_keepdims = func(a, axis=None, keepdims=True)
+        scalar = func(a, axis=None)
         expanded = np.full((1,) * a.ndim, fill_value=scalar)
         assert_array_equal(with_keepdims, expanded)
 
-
-class _GenericHasOutTestMixin:
-    """Tests for reduction functions which support out=... parameter."""
-
-    def test_out_scalar(self):
+    @parametrize_func
+    def test_out_scalar(self, func):
         # out no axis: scalar
+        if func in fails_out_arg:
+            raise SkipTest(f"{func.__name__} does not have out= arg.")
+
         a = np.arange(2 * 3 * 4).reshape((2, 3, 4))
 
-        result = self.func(a)
+        result = func(a)
         out = np.empty_like(result)
-        result_with_out = self.func(a, out=out)
+        result_with_out = func(a, out=out)
 
         assert result_with_out is out
         assert_array_equal(result, result_with_out)
@@ -352,14 +430,46 @@ class _GenericHasOutTestMixin:
         # Here we follow pytorch, since the result is a superset
         # of the numpy functionality
 
-    @pytest.mark.parametrize("keepdims", [True, False, None])
-    @pytest.mark.parametrize("dtype", [bool, "int32", "float64"])
-    def test_out_axis(self, dtype, keepdims):
-        for axis in self.allowed_axes + [None]:
-            self._check_out_axis(axis, dtype, keepdims)
+    @parametrize("keepdims", [True, False])
+    @parametrize("dtype", [bool, "int32", "float64"])
+    @parametrize_func
+    @parametrize_axis
+    def test_out_axis(self, func, axis, dtype, keepdims):
+        # out with axis
+        if func in fails_out_arg:
+            raise SkipTest(f"{func.__name__} does not have out= arg.")
+        if func in fails_axes_tuples:
+            raise SkipTest(f"{func.__name__} does not hangle tuple axis.")
+        if func in restricts_dtype_casts:
+            raise SkipTest(f"{func.__name__}: test implies float->int casts")
+
+        a = np.arange(2 * 3 * 4).reshape((2, 3, 4))
+        result = func(a, axis=axis, keepdims=keepdims).astype(dtype)
+
+        out = np.empty_like(result, dtype=dtype)
+        result_with_out = func(a, axis=axis, keepdims=keepdims, out=out)
+
+        assert result_with_out is out
+        assert result_with_out.dtype == dtype
+        assert_array_equal(result, result_with_out)
+
+        # TODO: what if result.dtype != out.dtype; does out typecast the result?
+
+        # out of wrong shape (any/out does not broadcast)
+        # np.any(m, out=np.empty_like(m)) raises a ValueError (wrong number
+        # of dimensions.)
+        # pytorch.any emits a warning and resizes the out array.
+        # Here we follow pytorch, since the result is a superset
+        # of the numpy functionality
+
+    @parametrize_func
+    @parametrize_axis
+    def test_keepdims_out(self, func, axis):
+        if func in fails_out_arg:
+            raise SkipTest(f"{func.__name__} does not have out= arg.")
+        if func in fails_axes_tuples:
+            raise SkipTest(f"{func.__name__} does not hangle tuple axis.")
 
-    def _check_keepdims_out(self, axis):
-        """Check the expicit shape of out w/keepdims=True"""
         d = np.ones((3, 5, 7, 11))
         if axis is None:
             shape_out = (1,) * d.ndim
@@ -369,165 +479,48 @@ class _GenericHasOutTestMixin:
                 1 if i in axis_norm else d.shape[i] for i in range(d.ndim)
             )
         out = np.empty(shape_out)
-        result = self.func(d, axis=axis, keepdims=True, out=out)
+
+        result = func(d, axis=axis, keepdims=True, out=out)
         assert result is out
         assert_equal(result.shape, shape_out)
 
-    def test_keepdims_out(self):
-        allowed_axes = [ax for ax in self.allowed_axes if ax != ()]
-        for axis in allowed_axes + [
-            None,
-        ]:
-            self._check_keepdims_out(axis)
 
+@instantiate_parametrized_tests
+class TestGenericCumSumProd(TestCase):
+    """Run a set of generic tests to verify that cumsum/cumprod are sane."""
 
-class TestAnyGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.any
-        self.allowed_axes = [
-            0,
-            1,
-            2,
-            -1,
-            -2,
-        ]
-
-
-class TestAllGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.all
-        self.allowed_axes = [
-            0,
-            1,
-            2,
-            -1,
-            -2,
-        ]
-
-
-class TestCountNonzeroGeneric(_GenericReductionsTestMixin):
-    # count_nonzero does not have the out=... argument
-    def setup_method(self):
-        self.func = np.count_nonzero
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestArgminGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.argmin
-        self.allowed_axes = [
-            0,
-            1,
-            2,
-            -1,
-            -2,
-        ]
-
-
-class TestArgmaxGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.argmax
-        self.allowed_axes = [
-            0,
-            1,
-            2,
-            -1,
-            -2,
-        ]
-
-
-class TestAmaxGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.amax
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestAminGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.amin
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestMeanGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.mean
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestSumGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.sum
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestProdGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.prod
-        self.allowed_axes = [
-            0,
-            1,
-            2,
-            -1,
-            -2,
-        ]
-
-
-class TestStdGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.std
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class TestVarGeneric(_GenericReductionsTestMixin, _GenericHasOutTestMixin):
-    def setup_method(self):
-        self.func = np.var
-        self.allowed_axes = [0, 1, 2, -1, -2, (0, 1), (1, 0), (0, 1, 2), (1, -1, 0)]
-
-
-class _GenericCumSumProdTestMixin:
-    """Run a set of generic tests to verify that self.func acts like a
-    reduction operation.
-
-    Specifically, this class checks axis=... and keepdims=... parameters.
-    To check the out=... parameter, see the _GenericHasOutTestMixin class below.
-
-    To use: subclass, define self.func and self.allowed_axes.
-    """
-
-    def test_bad_axis(self):
+    @parametrize("func", [np.cumsum, np.cumprod])
+    def test_bad_axis(self, func):
         # Basic check of functionality
         m = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
 
-        assert_raises(TypeError, self.func, m, axis="foo")
-        assert_raises(np.AxisError, self.func, m, axis=3)
-        assert_raises(TypeError, self.func, m, axis=np.array([[1], [2]]))
-        assert_raises(TypeError, self.func, m, axis=1.5)
+        assert_raises(TypeError, func, m, axis="foo")
+        assert_raises(np.AxisError, func, m, axis=3)
+        assert_raises(TypeError, func, m, axis=np.array([[1], [2]]))
+        assert_raises(TypeError, func, m, axis=1.5)
 
         # TODO: add tests with np.int32(3) etc, when implemented
 
-    def test_array_axis(self):
+    @parametrize("func", [np.cumsum, np.cumprod])
+    def test_array_axis(self, func):
         a = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
-        assert_equal(self.func(a, axis=np.array(-1)), self.func(a, axis=-1))
+        assert_equal(func(a, axis=np.array(-1)), func(a, axis=-1))
 
         with assert_raises(TypeError):
-            self.func(a, axis=np.array([1, 2]))
+            func(a, axis=np.array([1, 2]))
 
-    def test_axis_empty_generic(self):
+    @parametrize("func", [np.cumsum, np.cumprod])
+    def test_axis_empty_generic(self, func):
         a = np.array([[0, 0, 1], [1, 0, 1]])
-        assert_array_equal(self.func(a, axis=None), self.func(a.ravel(), axis=0))
+        assert_array_equal(func(a, axis=None), func(a.ravel(), axis=0))
 
-    def test_axis_bad_tuple(self):
+    @parametrize("func", [np.cumsum, np.cumprod])
+    def test_axis_bad_tuple(self, func):
         # Basic check of functionality
         m = np.array([[0, 1, 7, 0, 0], [3, 0, 0, 2, 19]])
         with assert_raises(TypeError):
-            self.func(m, axis=(1, 1))
+            func(m, axis=(1, 1))
 
 
-class TestCumProdGeneric(_GenericCumSumProdTestMixin):
-    def setup_method(self):
-        self.func = np.cumprod
-
-
-class TestCumSumGeneric(_GenericCumSumProdTestMixin):
-    def setup_method(self):
-        self.func = np.cumsum
+if __name__ == "__main__":
+    run_tests()
