@@ -90,7 +90,14 @@ includes = [
     "aten/src/ATen/native/nested/cuda/*",
     "aten/src/ATen/native/sparse/cuda/*",
     "aten/src/ATen/native/quantized/cuda/*",
-    "aten/src/ATen/native/transformers/cuda/*",
+    "aten/src/ATen/native/transformers/cuda/attention_backward.cu",
+    "aten/src/ATen/native/transformers/cuda/attention.cu",
+    "aten/src/ATen/native/transformers/cuda/sdp_utils.cpp",
+    "aten/src/ATen/native/transformers/cuda/sdp_utils.h",
+    "aten/src/ATen/native/transformers/cuda/mem_eff_attention/debug_utils.h",
+    "aten/src/ATen/native/transformers/cuda/mem_eff_attention/gemm_kernel_utils.h",
+    "aten/src/ATen/native/transformers/cuda/mem_eff_attention/pytorch_utils.h",
+    "aten/src/ATen/native/transformers/cuda/flash_attn/flash_api.h",
     "aten/src/THC/*",
     "aten/src/ATen/test/*",
     # CMakeLists.txt isn't processed by default, but there are a few
@@ -144,57 +151,60 @@ def is_hip_clang() -> bool:
         return False
 
 
-# TODO Remove once gloo submodule is recent enough to contain upstream fix.
-if is_hip_clang():
-    gloo_cmake_file = "third_party/gloo/cmake/Hip.cmake"
+# TODO Remove once the following submodules are updated
+hip_platform_files = [
+    "third_party/fbgemm/fbgemm_gpu/CMakeLists.txt",
+    "third_party/fbgemm/fbgemm_gpu/cmake/Hip.cmake",
+    "third_party/fbgemm/fbgemm_gpu/codegen/embedding_backward_dense_host.cpp",
+    "third_party/fbgemm/fbgemm_gpu/codegen/embedding_backward_split_host_template.cpp",
+    "third_party/fbgemm/fbgemm_gpu/codegen/embedding_backward_split_template.cu",
+    "third_party/fbgemm/fbgemm_gpu/codegen/embedding_forward_quantized_split_lookup.cu",
+    "third_party/fbgemm/fbgemm_gpu/include/fbgemm_gpu/fbgemm_cuda_utils.cuh",
+    "third_party/fbgemm/fbgemm_gpu/include/fbgemm_gpu/sparse_ops.cuh",
+    "third_party/fbgemm/fbgemm_gpu/src/jagged_tensor_ops.cu",
+    "third_party/fbgemm/fbgemm_gpu/src/quantize_ops.cu",
+    "third_party/fbgemm/fbgemm_gpu/src/sparse_ops.cu",
+    "third_party/fbgemm/fbgemm_gpu/src/split_embeddings_cache_cuda.cu",
+    "third_party/fbgemm/fbgemm_gpu/src/topology_utils.cpp",
+    "third_party/fbgemm/src/EmbeddingSpMDM.cc",
+    "third_party/gloo/cmake/Dependencies.cmake",
+    "third_party/gloo/gloo/cuda.cu",
+    "third_party/kineto/libkineto/CMakeLists.txt",
+    "third_party/nvfuser/CMakeLists.txt",
+    "third_party/tensorpipe/cmake/Hip.cmake",
+]
+
+
+def remove_hcc(line: str) -> str:
+    line = line.replace("HIP_PLATFORM_HCC", "HIP_PLATFORM_AMD")
+    line = line.replace("HIP_HCC_FLAGS", "HIP_CLANG_FLAGS")
+    return line
+
+
+for hip_platform_file in hip_platform_files:
     do_write = False
-    if os.path.exists(gloo_cmake_file):
-        with open(gloo_cmake_file) as sources:
+    if os.path.exists(hip_platform_file):
+        with open(hip_platform_file) as sources:
             lines = sources.readlines()
-        newlines = [line.replace(" hip_hcc ", " amdhip64 ") for line in lines]
+        newlines = [remove_hcc(line) for line in lines]
         if lines == newlines:
-            print(f"{gloo_cmake_file} skipped")
+            print(f"{hip_platform_file} skipped")
         else:
-            with open(gloo_cmake_file, "w") as sources:
+            with open(hip_platform_file, "w") as sources:
                 for line in newlines:
                     sources.write(line)
-            print(f"{gloo_cmake_file} updated")
-
-gloo_cmake_file = "third_party/gloo/cmake/Modules/Findrccl.cmake"
-if os.path.exists(gloo_cmake_file):
-    do_write = False
-    with open(gloo_cmake_file) as sources:
-        lines = sources.readlines()
-    newlines = [line.replace("RCCL_LIBRARY", "RCCL_LIB_PATH") for line in lines]
-    if lines == newlines:
-        print(f"{gloo_cmake_file} skipped")
-    else:
-        with open(gloo_cmake_file, "w") as sources:
-            for line in newlines:
-                sources.write(line)
-        print(f"{gloo_cmake_file} updated")
-
-# TODO Remove once gloo submodule is recent enough to contain upstream fix.
-if is_hip_clang():
-    gloo_cmake_file = "third_party/gloo/cmake/Dependencies.cmake"
-    do_write = False
-    if os.path.exists(gloo_cmake_file):
-        with open(gloo_cmake_file) as sources:
-            lines = sources.readlines()
-        newlines = [line.replace("HIP_HCC_FLAGS", "HIP_CLANG_FLAGS") for line in lines]
-        if lines == newlines:
-            print(f"{gloo_cmake_file} skipped")
-        else:
-            with open(gloo_cmake_file, "w") as sources:
-                for line in newlines:
-                    sources.write(line)
-            print(f"{gloo_cmake_file} updated")
+            print(f"{hip_platform_file} updated")
 
 hipify_python.hipify(
     project_directory=proj_dir,
     output_directory=out_dir,
     includes=includes,
     ignores=ignores,
+    extra_files=[
+        "torch/_inductor/codegen/cpp_wrapper_cpu.py",
+        "torch/_inductor/codegen/cpp_wrapper_cuda.py",
+        "torch/_inductor/codegen/wrapper.py",
+    ],
     out_of_place_only=args.out_of_place_only,
     hip_clang_launch=is_hip_clang(),
 )
