@@ -5,10 +5,8 @@ import sys
 import traceback
 import warnings
 
-from pkg_resources import packaging
-
-MIN_CUDA_VERSION = packaging.version.parse("11.6")
-MIN_ROCM_VERSION = packaging.version.parse("5.4")
+MIN_CUDA_VERSION = "11.6"
+MIN_ROCM_VERSION = "5.4"
 MIN_PYTHON_VERSION = (3, 8)
 
 
@@ -28,11 +26,12 @@ def check_python():
 def check_torch():
     import torch
 
-    return packaging.version.parse(torch.__version__)
+    return torch.__version__
 
 
 # based on torch/utils/cpp_extension.py
 def get_cuda_version():
+    from torch.torch_version import TorchVersion
     from torch.utils import cpp_extension
 
     CUDA_HOME = cpp_extension._find_cuda_home()
@@ -50,10 +49,11 @@ def get_cuda_version():
         raise VerifyDynamoError("CUDA version not found in `nvcc --version` output")
 
     cuda_str_version = cuda_version.group(1)
-    return packaging.version.parse(cuda_str_version)
+    return TorchVersion(cuda_str_version)
 
 
 def get_rocm_version():
+    from torch.torch_version import TorchVersion
     from torch.utils import cpp_extension
 
     ROCM_HOME = cpp_extension._find_rocm_home()
@@ -75,16 +75,17 @@ def get_rocm_version():
 
     hip_str_version = hip_version.group(1)
 
-    return packaging.version.parse(hip_str_version)
+    return TorchVersion(hip_str_version)
 
 
 def check_cuda():
     import torch
+    from torch.torch_version import TorchVersion
 
     if not torch.cuda.is_available() or torch.version.hip is not None:
         return None
 
-    torch_cuda_ver = packaging.version.parse(torch.version.cuda)
+    torch_cuda_ver = TorchVersion(torch.version.cuda)
 
     # check if torch cuda version matches system cuda version
     cuda_ver = get_cuda_version()
@@ -112,14 +113,13 @@ def check_cuda():
 
 def check_rocm():
     import torch
+    from torch.torch_version import TorchVersion
 
     if not torch.cuda.is_available() or torch.version.hip is None:
         return None
 
     # Extracts main ROCm version from full string
-    torch_rocm_ver = packaging.version.parse(
-        ".".join(list(torch.version.hip.split(".")[0:2]))
-    )
+    torch_rocm_ver = TorchVersion(".".join(list(torch.version.hip.split(".")[0:2])))
 
     # check if torch rocm version matches system rocm version
     rocm_ver = get_rocm_version()
@@ -152,9 +152,9 @@ def check_dynamo(backend, device, err_msg):
         import torch._dynamo as dynamo
 
         if device == "cuda":
-            import torch._inductor.utils as utils
+            from torch.utils._triton import has_triton
 
-            if not utils.has_triton():
+            if not has_triton():
                 print(
                     f"WARNING: CUDA available but triton cannot be used. "
                     f"Your GPU may not be supported. "
@@ -215,8 +215,8 @@ def main():
         f"ROCM version: {rocm_ver}\n"
     )
     for args in _SANITY_CHECK_ARGS:
-        if sys.version_info >= (3, 12):
-            warnings.warn("Dynamo not yet supported in Python 3.12. Skipping check.")
+        if sys.version_info >= (3, 13):
+            warnings.warn("Dynamo not yet supported in Python 3.13. Skipping check.")
             continue
         check_dynamo(*args)
     print("All required checks passed")
