@@ -64,14 +64,36 @@ at::Tensor wrapper_CPU_Tensor_foo(const at::Tensor & self) {
         self._test_function_schema_generates_correct_kernel(obj, expected)
 
     def test_function_schema_generates_correct_kernel_no_return(self) -> None:
-        obj = {"func": "custom::foo(Tensor self, *, Tensor(a!)[] out) -> ()"}
+        obj = {"func": "custom::foo.out(Tensor self, *, Tensor(a!)[] out) -> ()"}
         expected = f"""
-void wrapper_CPU__foo_out(const at::Tensor & self, at::TensorList out) {{
+void wrapper_CPU_out_foo_out(const at::Tensor & self, at::TensorList out) {{
 {SPACES}
 }}
     """
         self._test_function_schema_generates_correct_kernel(obj, expected)
 
+    def test_function_schema_generates_correct_kernel_3_returns(self) -> None:
+        obj = {
+            "func": "custom::foo(Tensor self, Tensor[] other) -> (Tensor, Tensor, Tensor)"
+        }
+        expected = """
+::std::tuple<at::Tensor,at::Tensor,at::Tensor> wrapper_CPU__foo(const at::Tensor & self, at::TensorList other) {
+    return ::std::tuple<at::Tensor, at::Tensor, at::Tensor>(
+                at::Tensor(), at::Tensor(), at::Tensor()
+            );
+}
+    """
+        self._test_function_schema_generates_correct_kernel(obj, expected)
+
+    def test_function_schema_generates_correct_kernel_1_return_no_out(self) -> None:
+        obj = {"func": "custom::foo(Tensor[] a) -> Tensor"}
+        expected = """
+at::Tensor wrapper_CPU__foo(at::TensorList a) {
+    return at::Tensor();
+}
+    """
+        self._test_function_schema_generates_correct_kernel(obj, expected)
+
     def test_schema_has_no_return_type_argument_throws(self) -> None:
         func = _get_native_function_from_yaml(
             {"func": "custom::foo.bool(Tensor self) -> bool"}
