@@ -1,13 +1,15 @@
+# mypy: allow-untyped-defs
 import inspect
 from collections import defaultdict
 from functools import wraps
 from itertools import chain
-from typing import Callable, Dict, Sequence, Union
+from typing import Callable, Dict, List, Sequence, Union
 
 import torch
 import torch.library
-from torch._ops import OpOverload, OpOverloadPacket
-from torch.utils._pytree import tree_map
+from torch._ops import HigherOrderOperator, OpOverload, OpOverloadPacket
+from torch._prims_common import CustomOutParamAnnotation
+from torch.utils import _pytree as pytree
 
 __all__ = [
     "decomposition_table",
@@ -21,7 +23,9 @@ __all__ = [
 
 # TODO: relax key type here; torch registrations should be possible to; but
 # right now this type is accurate
-global_decomposition_table: Dict[str, Dict[OpOverload, Callable]] = defaultdict(dict)
+global_decomposition_table: Dict[
+    str, Dict[torch._ops.OperatorBase, Callable]
+] = defaultdict(dict)
 
 decomposition_table = global_decomposition_table["post_autograd"]
 pre_autograd_decomposition_table = global_decomposition_table["pre_autograd"]
@@ -35,8 +39,12 @@ def _add_op_to_registry(registry, op, fn):
     If op is OpOverload, it will be added to the registry directly.
     If op is OpOverloadPacket, all the valid op_overloads in the packet will be added to the registry.
     """
-    overloads = []
-    if isinstance(op, OpOverload):
+    overloads: List[Union[torch._ops.OperatorBase]] = []
+    if isinstance(op, HigherOrderOperator):
+        # There's no concept of overloads for HigherOrderOperator
+        registry[op] = fn
+        return
+    elif isinstance(op, OpOverload):
         overloads.append(op)
     else:
         assert isinstance(op, OpOverloadPacket)
@@ -46,7 +54,6 @@ def _add_op_to_registry(registry, op, fn):
     for op_overload in overloads:
         if op_overload in registry:
             raise RuntimeError(f"duplicate registrations for {op_overload}")
-
         # TorchScript dumps a bunch of extra nonsense overloads
         # which don't have corresponding dispatcher entries, we need
         # to filter those out, e.g aten.add.float_int
@@ -55,11 +62,15 @@ def _add_op_to_registry(registry, op, fn):
 
 
 def _convert_out_params(f):
-    sig = inspect.signature(f)
     out_annotation = f.__annotations__.get("out")
+
+    # If there are no out params, do not wrap the function.
+    if not out_annotation:
+        return f
+
     # Hack to detect when out is a Tuple. There seems to be no pretty way of doing this
-    fn = f
-    if out_annotation and getattr(out_annotation, "__origin__", None) is tuple:
+    if getattr(out_annotation, "__origin__", None) is tuple:
+        sig = inspect.signature(f)
         out_names = sig.return_annotation._fields
         # If out is a tuple, we need to register a function that unpacks all the out
         # elements as this is what native_functions.yaml expects
@@ -91,12 +102,50 @@ def _convert_out_params(f):
         for o in out_params:
             _fn.__annotations__[o.name] = o.annotation
 
-        fn = _fn
+        # Propagate that this function is wrapped by `out_wrapper`
+        _fn._torch_decompositions_out_wrapper = f._torch_decompositions_out_wrapper  # type: ignore[attr-defined]
 
-    return fn
+        return _fn
+
+    # Alternatively, there may be a single tensor out parameter with a name
+    # other than "out". This will need special treatment and is indicated by an
+    # annotation, which we will remove here so it is not exposed after wrapping.
+    custom_out_param_name = f.__annotations__.pop(CustomOutParamAnnotation, None)
+    if custom_out_param_name:
+
+        @wraps(f)
+        def _fn(*args, **kwargs):
+            out_kwarg = kwargs.pop(custom_out_param_name, None)
+            return f(*args, **kwargs, out=out_kwarg)
+
+        out_param = inspect.Parameter(
+            custom_out_param_name,
+            kind=inspect.Parameter.KEYWORD_ONLY,
+            default=None,
+            annotation=out_annotation,
+        )
+
+        # Drop the out parameter and concatenate the new kwarg in the signature
+        sig = inspect.signature(f)
+        params = chain(
+            (v for k, v in sig.parameters.items() if k != "out"), (out_param,)
+        )
+        _fn.__signature__ = inspect.Signature(  # type: ignore[attr-defined]
+            parameters=params, return_annotation=sig.return_annotation  # type: ignore[arg-type]
+        )
+
+        # Drop the out parameter and concatenate the new kwargs in the annotations
+        _fn.__annotations__ = {k: v for k, v in f.__annotations__.items() if k != "out"}
+        _fn.__annotations__[out_param.name] = out_param.annotation
+
+        return _fn
+
+    return f
 
 
-def register_decomposition(aten_op, registry=None, *, type="post_autograd"):
+def register_decomposition(
+    aten_op, registry=None, *, type="post_autograd", unsafe=False
+):
     """
     A decorator to register a function as a decomposition to the Python
     decomposition table.  Use it like this::
@@ -115,12 +164,17 @@ def register_decomposition(aten_op, registry=None, *, type="post_autograd"):
 
     By default, we also will register it to the Meta key of dispatcher,
     and replace the c++ Meta implementation if there is already one.
+
+    unsafe kwarg is for reuse of this function for registering non-function
+    things
     """
 
     assert type in {"post_autograd", "pre_autograd", "meta"}
 
     def decomposition_decorator(fn: Callable) -> Callable:
-        fn = _convert_out_params(fn)
+        orig_fn = fn
+        if not unsafe:
+            fn = _convert_out_params(fn)
 
         nonlocal registry
         if registry is None:
@@ -130,16 +184,16 @@ def register_decomposition(aten_op, registry=None, *, type="post_autograd"):
             _add_op_to_registry(registry, op, fn)
 
         # To handle allowing multiple aten_ops at once
-        tree_map(register, aten_op)
-        return fn
+        pytree.tree_map_(register, aten_op)
+        return orig_fn
 
     return decomposition_decorator
 
 
 def get_decompositions(
-    aten_ops: Sequence[Union[OpOverload, OpOverloadPacket]],
+    aten_ops: Sequence[Union[torch._ops.OperatorBase, OpOverloadPacket]],
     type: str = "post_autograd",
-) -> Dict[OpOverload, Callable]:
+) -> Dict[torch._ops.OperatorBase, Callable]:
     """
     Retrieve a dictionary of decompositions corresponding to the list of
     operator overloads and overload packets passed as input.  Overload
@@ -156,19 +210,20 @@ def get_decompositions(
     registry = global_decomposition_table[type]
     packets_to_overloads = defaultdict(list)
     for opo in registry:
-        packets_to_overloads[opo.overloadpacket].append(opo)
-    decompositions = {}
+        if isinstance(opo, (OpOverload, OpOverloadPacket)):
+            packets_to_overloads[opo.overloadpacket].append(opo)
+    decompositions: Dict[torch._ops.OperatorBase, Callable] = {}
     for op in aten_ops:
         if isinstance(op, OpOverloadPacket) and op in packets_to_overloads:
             for op_overload in packets_to_overloads[op]:
                 decompositions[op_overload] = registry[op_overload]
-        elif isinstance(op, OpOverload) and op in registry:
+        elif isinstance(op, (torch._ops.OperatorBase)) and op in registry:
             decompositions[op] = registry[op]
     return decompositions
 
 
 def remove_decompositions(
-    decompositions: Dict[OpOverload, Callable],
+    decompositions: Dict[torch._ops.OperatorBase, Callable],
     aten_ops: Sequence[Union[OpOverload, OpOverloadPacket]],
 ) -> None:
     """
@@ -196,7 +251,7 @@ import torch._refs
 # list was copied from torch/_inductor/decomposition.py
 # excluding decompositions that results in prim ops
 # Resulting opset of decomposition is core aten ops
-def core_aten_decompositions() -> Dict[OpOverload, Callable]:
+def core_aten_decompositions() -> Dict[torch._ops.OperatorBase, Callable]:
     aten = torch.ops.aten
     return get_decompositions(
         [
@@ -206,25 +261,33 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.addcmul_,
             aten.addr,
             aten.affine_grid_generator,
+            aten.all,
             aten.aminmax,
             aten.arange.default,
             aten.arange.start,
             aten.avg_pool2d_backward,
+            aten.baddbmm,
             aten.binary_cross_entropy,
             aten.binary_cross_entropy_backward,
             aten.binary_cross_entropy_with_logits,
+            aten.block_diag,
             aten.celu,
             aten.celu_,
+            aten.clamp_max,
+            aten.clamp_min,
             aten.col2im,
             aten.count_nonzero,
+            aten.linalg_cross,
             aten.cudnn_batch_norm,
             aten.cudnn_batch_norm_backward,
+            aten.miopen_batch_norm_backward,
             aten.deg2rad,
             aten.deg2rad_,
             aten.detach,
             aten.diag_embed,
             aten.diagonal_backward,
             aten.dot,
+            aten.vdot,
             aten.elu,
             aten.elu_,
             aten.elu_backward,
@@ -236,13 +299,14 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.eye,
             aten.fill,
             aten.fill_,
+            aten.floor_divide,
             aten.frac,
             aten.frac_,
             aten._fused_moving_avg_obs_fq_helper,
             aten.gelu_,
             aten.gelu_backward,
+            aten.glu,
             aten.glu_backward,
-            aten.grid_sampler_2d,
             aten.hardshrink,
             aten.hardsigmoid,
             aten.hardsigmoid_,
@@ -263,9 +327,12 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.index_copy_,
             aten.index_fill,
             aten.index_fill_,
+            aten.isin,
             aten.isneginf,
             aten.isposinf,
             aten.l1_loss,
+            aten._lazy_clone,
+            aten._test_parallel_materialize,
             aten.leaky_relu_,
             aten.leaky_relu_backward,
             aten.lerp,
@@ -309,18 +376,29 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.norm,
             aten.ones,
             aten.ones_like,
+            aten.pixel_shuffle,
+            aten.pixel_unshuffle,
             aten._prelu_kernel,
             aten._prelu_kernel_backward,
             aten._reshape_alias,
             aten.rad2deg,
             aten.rad2deg_,
+            aten.reflection_pad1d,
+            aten.reflection_pad2d,
+            aten.reflection_pad3d,
+            aten.replication_pad1d,
+            aten.replication_pad2d,
+            aten.replication_pad3d,
             aten.renorm,
             aten.renorm_,
+            aten.replication_pad2d,
+            aten.resize_as,
+            aten.roll,
             aten.rot90,
             aten.rrelu_with_noise,
             aten.rrelu_with_noise_,
-            aten.rsub.Scalar,
-            aten.rsub.Tensor,
+            aten.rsub,
+            aten._scaled_dot_product_flash_attention_for_cpu.default,
             aten.select_backward,
             aten.select_scatter,
             aten.sgn,
@@ -343,8 +421,17 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.special_entr,
             aten.special_log_ndtr,
             aten.special_xlog1py,
+            aten.split.Tensor,
+            aten.split_with_sizes_copy,
+            aten.squeeze.default,
+            aten.squeeze.dim,
+            aten.std,
+            aten.std_mean,
             aten.stack,
+            aten.sum.default,
+            aten.sum.out,
             aten.t,
+            aten.take,
             aten.tanh_backward,
             aten.threshold,
             aten.threshold_,
@@ -355,16 +442,25 @@ def core_aten_decompositions() -> Dict[OpOverload, Callable]:
             aten.tril_,
             aten.triu,
             aten.triu_,
+            aten.unbind,
             aten.unfold_backward,
             aten.unfold_copy,
             aten._unsafe_index,
+            aten.unsafe_split.Tensor,
+            aten.unsafe_split_with_sizes,
+            aten._unsafe_view,
+            aten.upsample_linear1d,
             aten.upsample_bilinear2d,
+            aten.upsample_trilinear3d,
             aten.upsample_nearest2d_backward,
+            aten.view_as_complex,
             aten.xlogy,
             aten.xlogy_,
             aten.zero,
             aten.zero_,
             aten.zeros,
             aten.zeros_like,
+            aten._chunk_cat,
+            aten._weight_norm_interface,
         ]
     )
