@@ -8,6 +8,8 @@
 #include <torch/csrc/utils/python_strings.h>
 #include <torch/csrc/utils/tensor_dtypes.h>
 
+#include <ATen/Dispatch_v2.h>
+
 #include <c10/util/Exception.h>
 
 #include <structmember.h>
@@ -107,98 +109,105 @@ PyObject* THPDTypeInfo_compare(THPDTypeInfo* a, THPDTypeInfo* b, int op) {
 }
 
 static PyObject* THPDTypeInfo_bits(THPDTypeInfo* self, void*) {
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-magic-numbers)
-  uint64_t bits = elementSize(self->type) * 8;
+  uint64_t bits = elementSize(self->type) * CHAR_BIT;
   return THPUtils_packUInt64(bits);
 }
 
+#define _AT_DISPATCH_FINFO_TYPES(TYPE, NAME, ...) \
+  AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND6(    \
+      at::kHalf,                                  \
+      at::ScalarType::BFloat16,                   \
+      at::ScalarType::Float8_e5m2,                \
+      at::ScalarType::Float8_e5m2fnuz,            \
+      at::ScalarType::Float8_e4m3fn,              \
+      at::ScalarType::Float8_e4m3fnuz,            \
+      TYPE,                                       \
+      NAME,                                       \
+      __VA_ARGS__)
+
 static PyObject* THPFInfo_eps(THPFInfo* self, void*) {
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "epsilon",
-      [] {
-        return PyFloat_FromDouble(
-            std::numeric_limits<
-                at::scalar_value_type<scalar_t>::type>::epsilon());
-      });
+  HANDLE_TH_ERRORS
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "epsilon", [] {
+    return PyFloat_FromDouble(
+        std::numeric_limits<at::scalar_value_type<scalar_t>::type>::epsilon());
+  });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPFInfo_max(THPFInfo* self, void*) {
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "max",
-      [] {
-        return PyFloat_FromDouble(
-            std::numeric_limits<at::scalar_value_type<scalar_t>::type>::max());
-      });
+  HANDLE_TH_ERRORS
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "max", [] {
+    return PyFloat_FromDouble(
+        std::numeric_limits<at::scalar_value_type<scalar_t>::type>::max());
+  });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPFInfo_min(THPFInfo* self, void*) {
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "lowest",
-      [] {
-        return PyFloat_FromDouble(
-            std::numeric_limits<
-                at::scalar_value_type<scalar_t>::type>::lowest());
-      });
+  HANDLE_TH_ERRORS
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "lowest", [] {
+    return PyFloat_FromDouble(
+        std::numeric_limits<at::scalar_value_type<scalar_t>::type>::lowest());
+  });
+  END_HANDLE_TH_ERRORS
 }
 
+#define AT_DISPATCH_IINFO_TYPES(TYPE, NAME, ...) \
+  AT_DISPATCH_V2(                                \
+      TYPE, NAME, AT_WRAP(__VA_ARGS__), AT_EXPAND(AT_INTEGRAL_TYPES_V2))
+
 static PyObject* THPIInfo_max(THPIInfo* self, void*) {
+  HANDLE_TH_ERRORS
   if (at::isIntegralType(self->type, /*includeBool=*/false)) {
-    return AT_DISPATCH_INTEGRAL_TYPES(self->type, "max", [] {
-      return THPUtils_packInt64(std::numeric_limits<scalar_t>::max());
+    return AT_DISPATCH_IINFO_TYPES(self->type, "max", [] {
+      if (std::is_unsigned_v<scalar_t>) {
+        return THPUtils_packUInt64(std::numeric_limits<scalar_t>::max());
+      } else {
+        return THPUtils_packInt64(std::numeric_limits<scalar_t>::max());
+      }
     });
   }
   // Quantized Type
   return AT_DISPATCH_QINT_AND_SUB_BYTE_TYPES(self->type, "max", [] {
     return THPUtils_packInt64(std::numeric_limits<underlying_t>::max());
   });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPIInfo_min(THPIInfo* self, void*) {
+  HANDLE_TH_ERRORS
   if (at::isIntegralType(self->type, /*includeBool=*/false)) {
-    return AT_DISPATCH_INTEGRAL_TYPES(self->type, "min", [] {
-      return THPUtils_packInt64(std::numeric_limits<scalar_t>::lowest());
+    return AT_DISPATCH_IINFO_TYPES(self->type, "min", [] {
+      if (std::is_unsigned_v<scalar_t>) {
+        return THPUtils_packUInt64(std::numeric_limits<scalar_t>::lowest());
+      } else {
+        return THPUtils_packInt64(std::numeric_limits<scalar_t>::lowest());
+      }
     });
   }
   // Quantized Type
   return AT_DISPATCH_QINT_AND_SUB_BYTE_TYPES(self->type, "min", [] {
     return THPUtils_packInt64(std::numeric_limits<underlying_t>::lowest());
   });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPIInfo_dtype(THPIInfo* self, void*) {
+  HANDLE_TH_ERRORS
   auto primary_name = torch::utils::getDtypeNames(self->type).first;
-  return AT_DISPATCH_INTEGRAL_TYPES(self->type, "dtype", [&primary_name] {
+  return AT_DISPATCH_IINFO_TYPES(self->type, "dtype", [&primary_name] {
     return PyUnicode_FromString(primary_name.data());
   });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPFInfo_smallest_normal(THPFInfo* self, void*) {
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "smallest",
-      [] {
-        return PyFloat_FromDouble(
-            std::numeric_limits<at::scalar_value_type<scalar_t>::type>::min());
-      });
+  HANDLE_TH_ERRORS
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "min", [] {
+    return PyFloat_FromDouble(
+        std::numeric_limits<at::scalar_value_type<scalar_t>::type>::min());
+  });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPFInfo_tiny(THPFInfo* self, void*) {
@@ -207,35 +216,27 @@ static PyObject* THPFInfo_tiny(THPFInfo* self, void*) {
 }
 
 static PyObject* THPFInfo_resolution(THPFInfo* self, void*) {
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "digits10",
-      [] {
-        return PyFloat_FromDouble(std::pow(
-            10,
-            -std::numeric_limits<
-                at::scalar_value_type<scalar_t>::type>::digits10));
-      });
+  HANDLE_TH_ERRORS
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "digits10", [] {
+    return PyFloat_FromDouble(std::pow(
+        10,
+        -std::numeric_limits<at::scalar_value_type<scalar_t>::type>::digits10));
+  });
+  END_HANDLE_TH_ERRORS
 }
 
 static PyObject* THPFInfo_dtype(THPFInfo* self, void*) {
+  HANDLE_TH_ERRORS
   auto primary_name = torch::utils::getDtypeNames(self->type).first;
-  return AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND4(
-      at::kHalf,
-      at::ScalarType::BFloat16,
-      at::ScalarType::Float8_e4m3fn,
-      at::ScalarType::Float8_e5m2,
-      self->type,
-      "dtype",
-      [&primary_name] { return PyUnicode_FromString(primary_name.data()); });
+  return _AT_DISPATCH_FINFO_TYPES(self->type, "dtype", [&primary_name] {
+    return PyUnicode_FromString(primary_name.data());
+  });
+  END_HANDLE_TH_ERRORS
 }
 
 PyObject* THPFInfo_str(THPFInfo* self) {
   std::ostringstream oss;
+  const auto dtypeStr = THPFInfo_dtype(self, nullptr);
   oss << "finfo(resolution="
       << PyFloat_AsDouble(THPFInfo_resolution(self, nullptr));
   oss << ", min=" << PyFloat_AsDouble(THPFInfo_min(self, nullptr));
@@ -244,19 +245,23 @@ PyObject* THPFInfo_str(THPFInfo* self) {
   oss << ", smallest_normal="
       << PyFloat_AsDouble(THPFInfo_smallest_normal(self, nullptr));
   oss << ", tiny=" << PyFloat_AsDouble(THPFInfo_tiny(self, nullptr));
-  oss << ", dtype=" << PyUnicode_AsUTF8(THPFInfo_dtype(self, nullptr)) << ")";
-
-  return THPUtils_packString(oss.str().c_str());
+  if (dtypeStr != nullptr) {
+    oss << ", dtype=" << PyUnicode_AsUTF8(dtypeStr) << ")";
+  }
+  return !PyErr_Occurred() ? THPUtils_packString(oss.str().c_str()) : nullptr;
 }
 
 PyObject* THPIInfo_str(THPIInfo* self) {
   std::ostringstream oss;
 
+  const auto dtypeStr = THPIInfo_dtype(self, nullptr);
   oss << "iinfo(min=" << PyLong_AsDouble(THPIInfo_min(self, nullptr));
   oss << ", max=" << PyLong_AsDouble(THPIInfo_max(self, nullptr));
-  oss << ", dtype=" << PyUnicode_AsUTF8(THPIInfo_dtype(self, nullptr)) << ")";
+  if (dtypeStr) {
+    oss << ", dtype=" << PyUnicode_AsUTF8(dtypeStr) << ")";
+  }
 
-  return THPUtils_packString(oss.str().c_str());
+  return !PyErr_Occurred() ? THPUtils_packString(oss.str().c_str()) : nullptr;
 }
 
 // NOLINTNEXTLINE(modernize-avoid-c-arrays,cppcoreguidelines-avoid-non-const-global-variables,cppcoreguidelines-avoid-c-arrays)
