@@ -4,17 +4,6 @@
 namespace torch {
 namespace nn {
 
-template <size_t D, typename Derived>
-void InstanceNormImpl<D, Derived>::pretty_print(std::ostream& stream) const {
-  stream << std::boolalpha << "torch::nn::InstanceNorm" << D << "d("
-         << this->options.num_features() << ", "
-         << "eps=" << this->options.eps() << ", "
-         << "momentum=" << this->options.momentum() << ", "
-         << "affine=" << this->options.affine() << ", "
-         << "track_running_stats=" << this->options.track_running_stats()
-         << ")";
-}
-
 void InstanceNorm1dImpl::_check_input_dim(const Tensor& input) {
   if (input.dim() != 3 && input.dim() != 2) {
     TORCH_CHECK(
