@@ -25,15 +25,15 @@ void UpsampleImpl::pretty_print(std::ostream& stream) const {
 
 Tensor UpsampleImpl::forward(const Tensor& input) {
   F::InterpolateFuncOptions::mode_t mode;
-  if (c10::get_if<enumtype::kNearest>(&options.mode())) {
+  if (std::holds_alternative<enumtype::kNearest>(options.mode())) {
     mode = torch::kNearest;
-  } else if (c10::get_if<enumtype::kLinear>(&options.mode())) {
+  } else if (std::holds_alternative<enumtype::kLinear>(options.mode())) {
     mode = torch::kLinear;
-  } else if (c10::get_if<enumtype::kBilinear>(&options.mode())) {
+  } else if (std::holds_alternative<enumtype::kBilinear>(options.mode())) {
     mode = torch::kBilinear;
-  } else if (c10::get_if<enumtype::kBicubic>(&options.mode())) {
+  } else if (std::holds_alternative<enumtype::kBicubic>(options.mode())) {
     mode = torch::kBicubic;
-  } else if (c10::get_if<enumtype::kTrilinear>(&options.mode())) {
+  } else if (std::holds_alternative<enumtype::kTrilinear>(options.mode())) {
     mode = torch::kTrilinear;
   }
 
