@@ -7,13 +7,14 @@
 #include <torch/csrc/autograd/grad_mode.h>
 #include <torch/csrc/autograd/variable.h>
 #include <torch/csrc/dynamo/compiled_autograd.h>
-#include <torch/csrc/utils/memory.h>
 
+#include <ATen/DeviceAccelerator.h>
 #include <ATen/DeviceGuard.h>
 #include <ATen/ExpandUtils.h>
 #include <ATen/Parallel.h>
 #include <ATen/SparseCsrTensorUtils.h>
 #include <ATen/detail/CUDAHooksInterface.h>
+#include <ATen/detail/PrivateUse1HooksInterface.h>
 
 #ifndef AT_PER_OPERATOR_HEADERS
 #include <ATen/Functions.h>
@@ -25,10 +26,12 @@
 #include <c10/core/Event.h>
 #include <c10/core/Stream.h>
 #include <c10/core/StreamGuard.h>
+#include <c10/util/AbortHandler.h>
 #include <c10/util/Exception.h>
 #include <c10/util/Optional.h>
 #include <c10/util/ThreadLocal.h>
 #include <c10/util/irange.h>
+#include <c10/util/thread_name.h>
 
 #include <atomic>
 #include <chrono>
@@ -39,16 +42,13 @@
 #include <memory>
 #include <mutex>
 #include <queue>
-#include <set>
 #include <sstream>
 #include <string>
 #include <thread>
-#include <typeinfo>
 #include <unordered_set>
 #include <utility>
 
-namespace torch {
-namespace autograd {
+namespace torch::autograd {
 
 namespace {
 static bool in_bad_autograd_fork =
@@ -82,7 +82,7 @@ std::atomic<Engine::compiled_autograd_fn> the_compiled_autograd = nullptr;
   reinterpret_cast<Engine::compiled_autograd_fn>(1)
 std::atomic<int32_t> num_threads_in_backwards;
 struct CompiledAutogradThreadingDebugCheck {
-  CompiledAutogradThreadingDebugCheck() : incremented(true) {
+  CompiledAutogradThreadingDebugCheck() {
     num_threads_in_backwards++;
   }
   ~CompiledAutogradThreadingDebugCheck() {
@@ -95,7 +95,7 @@ struct CompiledAutogradThreadingDebugCheck {
   }
 
  private:
-  bool incremented;
+  bool incremented{true};
 };
 
 } // namespace
@@ -181,8 +181,8 @@ C10_DEFINE_TLS_static(std::shared_ptr<ReadyQueue>, tls_local_ready_queue);
 
 // Note [Streaming backwards]
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~
-// On CUDA devices the autograd engine's device operations are run on the
-// same stream that ran them in forward. This requires automatically
+// On CUDA/privateuse1 devices the autograd engine's device operations are run
+// on the same stream that ran them in forward. This requires automatically
 // syncing the streams so that function A finishes producing its
 // output before function B consumes it.
 //
@@ -191,9 +191,10 @@ C10_DEFINE_TLS_static(std::shared_ptr<ReadyQueue>, tls_local_ready_queue);
 // recording their streams from forward, and during backward this
 // data is used to sync the producer's stream with the consumer's.
 //
-// When a CUDA function is run either all its inputs were accumulated on the
-// stream used to run the function OR the inputs are on different devices
-// and the function is responsible for properly acquiring them.
+// When a CUDA/privateuse1 function is run either all its inputs were
+// accumulated on the stream used to run the function OR the inputs are on
+// different devices and the function is responsible for properly acquiring
+// them.
 //
 // User-facing stream semantics of a backward() (or torch.autograd.grad())
 // call with respect to surrounding ops are the same as for any other call.
@@ -202,7 +203,7 @@ C10_DEFINE_TLS_static(std::shared_ptr<ReadyQueue>, tls_local_ready_queue);
 //
 // Internally, backward() runs ops (including leaf nodes) on side threads.
 // And streams are thread local. So GraphTask achieves the above semantics by
-//  1. remembering the current streams on all active CUDA devices
+//  1. remembering the current streams on all active CUDA/privateuse1 devices
 //     in the user-facing thread (aka, the thread that called execute() to
 //     launch the GraphTask)
 //  2. remembering the "leaf streams" (streams each backward leaf node ran on)
@@ -223,8 +224,8 @@ int NodeTask::getReentrantDepth() const {
 }
 
 CheckpointValidGuard::CheckpointValidGuard(
-    const std::shared_ptr<const GraphTask>& graph_task) {
-  prev_checkpoint_valid_state = checkpoint_valid;
+    const std::shared_ptr<const GraphTask>& graph_task)
+    : prev_checkpoint_valid_state(checkpoint_valid) {
   checkpoint_valid =
       graph_task->can_checkpoint() && prev_checkpoint_valid_state;
 }
@@ -347,6 +348,12 @@ void Engine::thread_init(
     int device,
     const std::shared_ptr<ReadyQueue>& ready_queue,
     bool should_increment) {
+  // pthread_setname_np restricts the name to 16 characters including
+  // the null byte.
+  std::string thread_name = "pt_autograd_" + std::to_string(device);
+  c10::setThreadName(thread_name);
+
+  c10::set_terminate_handler();
   if (should_increment) {
     increment_non_reentrant_thread_count();
   }
@@ -384,8 +391,8 @@ void Engine::thread_init(
   }
 }
 
-GraphTaskGuard::GraphTaskGuard(std::shared_ptr<GraphTask> graph_task) {
-  last_graph_task_ = std::move(current_graph_task);
+GraphTaskGuard::GraphTaskGuard(std::shared_ptr<GraphTask> graph_task)
+    : last_graph_task_(std::move(current_graph_task)) {
   current_graph_task = std::move(graph_task);
 }
 GraphTaskGuard::~GraphTaskGuard() {
@@ -539,7 +546,8 @@ auto Engine::thread_main(const std::shared_ptr<GraphTask>& graph_task) -> void {
         break;
       }
 
-      if (!(local_graph_task = task.base_.lock())) {
+      local_graph_task = task.base_.lock();
+      if (!local_graph_task) {
         // GraphTask for function is no longer valid, skipping further
         // execution.
         continue;
@@ -575,6 +583,7 @@ auto Engine::thread_main(const std::shared_ptr<GraphTask>& graph_task) -> void {
                 local_graph_task->cpu_ready_queue_);
           }
         } catch (std::exception& e) {
+          // See Note [ Persisting PyErr state across autograd engine threads ]
           thread_on_exception(local_graph_task, task.fn_, e);
         }
       }
@@ -612,6 +621,7 @@ auto Engine::thread_main(const std::shared_ptr<GraphTask>& graph_task) -> void {
 // thread, but sharing the same cpu_ready_queue with parent thread is a
 // performance improvement and cuda thread still have to do the same thing.
 void Engine::reentrant_thread_init() {
+  c10::set_terminate_handler();
   at::init_num_threads();
   auto tp_shared = thread_pool_shared_;
   while (true) {
@@ -623,8 +633,8 @@ void Engine::reentrant_thread_init() {
     auto task = tp_shared->graphtasks_queue_.front();
     tp_shared->graphtasks_queue_.pop();
     lk.unlock();
-    std::shared_ptr<GraphTask> graph_task;
-    if (!(graph_task = task.lock())) {
+    std::shared_ptr<GraphTask> graph_task = task.lock();
+    if (!graph_task) {
       LOG(INFO) << "GraphTask has expired, skipping reentrant execution";
       continue;
     }
@@ -639,12 +649,36 @@ void Engine::reentrant_thread_init() {
 }
 
 void Engine::thread_on_exception(
+    // NOLINTNEXTLINE(performance-unnecessary-value-param)
     std::shared_ptr<GraphTask> graph_task,
     const std::shared_ptr<Node>& fn,
     std::exception& e) {
   graph_task->set_exception(std::current_exception(), fn);
 }
 
+namespace {
+std::atomic<uint64_t> graph_task_id{0};
+}
+
+GraphTask::GraphTask(
+    bool keep_graph,
+    bool grad_mode,
+    int reentrant_depth,
+    std::shared_ptr<ReadyQueue> cpu_ready_queue,
+    c10::SmallVector<Node*, 4> graph_roots,
+    bool exit_on_error)
+    : keep_graph_(keep_graph),
+      graph_roots_(std::move(graph_roots)),
+      owner_(NO_DEVICE),
+      reentrant_depth_(reentrant_depth),
+      exit_on_error_(exit_on_error),
+      cpu_ready_queue_(std::move(cpu_ready_queue)),
+      future_result_(c10::make_intrusive<at::ivalue::Future>(
+          c10::ListType::create(c10::TensorType::get()))),
+      id_(graph_task_id.fetch_add(1, std::memory_order_relaxed)) {
+  thread_locals_.set_grad_mode(grad_mode);
+}
+
 bool GraphTask::completed() {
   return outstanding_tasks_.load() == 0 ||
       (exit_on_error_ && has_error_.load());
@@ -671,8 +705,8 @@ void GraphTask::mark_as_completed_and_run_post_processing() {
     // Need to unlock before we call markCompleted to avoid holding locks
     // when the callbacks are called.
     lock.unlock();
-    future_result_->markCompleted(std::move(vars));
-  } catch (std::exception& e) {
+    future_result_->markCompleted(vars);
+  } catch (std::exception&) {
     future_result_->setErrorIfNeeded(std::current_exception());
   }
 }
@@ -699,17 +733,19 @@ void GraphTask::exec_post_processing() {
   // any grad on its device's current stream.
   if (!leaf_streams.empty()) {
     for (const auto& leaf_stream : leaf_streams) {
-      // stash_current_streams() stashed streams for all device IDs that already
-      // had a CUDA context before the GraphTask executed. For inactive devices,
-      // it stashed a c10::nullopt. I don't expect GraphTask's backward pass ran
-      // leaf nodes on any new devices, so the stashed streams should be enough.
-      // If leaf_stream.device_index() happens to be for a new device,
-      // operator* on the c10::nullopt should throw an error.
+      // stash_current_cuda/privateuse1_streams() stashed streams for all device
+      // IDs that already had a CUDA/privateuse1 context before the GraphTask
+      // executed. For inactive devices, it stashed a c10::nullopt. I don't
+      // expect GraphTask's backward pass ran leaf nodes on any new devices, so
+      // the stashed streams should be enough. If leaf_stream.device_index()
+      // happens to be for a new device, operator* on the c10::nullopt should
+      // throw an error.
       const auto caller_current_stream =
+          // NOLINTNEXTLINE(bugprone-unchecked-optional-access)
           *caller_current_streams_[leaf_stream.device_index()];
 
       if (caller_current_stream != leaf_stream) {
-        auto event = c10::Event{c10::DeviceType::CUDA};
+        auto event = c10::Event{leaf_stream.device_type()};
         event.record(leaf_stream);
         caller_current_stream.wait(event);
       }
@@ -809,7 +845,9 @@ void set_device(int device) {
              c10::DeviceType::COMPILE_TIME_MAX_DEVICE_TYPES))) {
       auto* impl = c10::impl::device_guard_impl_registry[i].load();
       if (impl && device < impl->deviceCount()) {
-        impl->setDevice(at::Device(static_cast<c10::DeviceType>(i), device));
+        impl->setDevice(at::Device(
+            static_cast<c10::DeviceType>(i),
+            static_cast<c10::DeviceIndex>(device)));
       }
     }
   }
@@ -841,14 +879,7 @@ void validate_outputs(
       continue;
     }
 
-    if (!metadata.is_same_shape(grad)) {
-      if (metadata.is_expandable_to_shape(grad)) {
-        grad = metadata.reduce_grad(grad);
-      } else {
-        const auto message = metadata.incompatible_shape_error_message(i, grad);
-        TORCH_CHECK(false, format_error(message.str()));
-      }
-    }
+    grad = metadata.maybe_reduce(i, std::move(grad), format_error);
 
     bool input_is_complex =
         isComplexType(c10::typeMetaToScalarType(metadata.options().dtype()));
@@ -951,6 +982,7 @@ static variable_list call_function(
   });
 
   if (has_post_hooks) {
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     return call_post_hooks(fn, std::move(outputs), inputs);
   }
   return outputs;
@@ -965,7 +997,7 @@ void Engine::evaluate_function(
   // ensure they're safe to consume in the context of the present
   // func's stream (if applicable). So we guard onto that stream
   // before working with the grads in any capacity.
-  const auto opt_parent_stream = (*func).stream(c10::DeviceType::CUDA);
+  auto opt_parent_stream = (*func).stream();
   c10::OptionalStreamGuard parent_stream_guard{opt_parent_stream};
 
   // If exec_info_ is not empty, we have to instrument the execution
@@ -983,7 +1015,7 @@ void Engine::evaluate_function(
           *func, InputBuffer::variables(std::move(inputs)));
     }
     if (auto* capture_vec = fn_info.captures_.get()) {
-      const auto opt_parent_stream = (*func).stream(c10::DeviceType::CUDA);
+      auto opt_parent_stream = (*func).stream();
       // Lock mutex for writing to graph_task->captured_vars_.
       std::lock_guard<std::mutex> lock(graph_task->mutex_);
       for (const auto& capture : *capture_vec) {
@@ -1013,7 +1045,7 @@ void Engine::evaluate_function(
     fn.release_variables();
   }
 
-  int num_outputs = outputs.size();
+  auto num_outputs = outputs.size();
   if (num_outputs == 0) { // Note: doesn't acquire the mutex
     // Records leaf stream (if applicable)
     // See Note [Streaming backwards]
@@ -1075,7 +1107,7 @@ void Engine::evaluate_function(
       InputBuffer input_buffer(next.function->num_inputs());
 
       // Accumulates into buffer
-      const auto opt_next_stream = next.function->stream(c10::DeviceType::CUDA);
+      auto opt_next_stream = next.function->stream();
       input_buffer.add(
           next.input_nr, std::move(output), opt_parent_stream, opt_next_stream);
 
@@ -1091,7 +1123,7 @@ void Engine::evaluate_function(
       auto& input_buffer = not_ready_it->second;
 
       // Accumulates into buffer
-      const auto opt_next_stream = next.function->stream(c10::DeviceType::CUDA);
+      auto opt_next_stream = next.function->stream();
       input_buffer.add(
           next.input_nr, std::move(output), opt_parent_stream, opt_next_stream);
       if (is_ready) {
@@ -1111,7 +1143,7 @@ inline static uint64_t compute_min_topological_nr(const edge_list& outputs) {
   }
   auto min_topo_nr = std::numeric_limits<uint64_t>::max();
   for (auto& output_edge : outputs) {
-    auto topo_nr = output_edge.function.get()->topological_nr();
+    auto topo_nr = output_edge.function->topological_nr();
     min_topo_nr = (min_topo_nr < topo_nr) ? min_topo_nr : topo_nr;
   }
   return min_topo_nr;
@@ -1123,8 +1155,7 @@ auto Engine::compute_dependencies(
     uint64_t min_topo_nr) -> void {
   // Computes the number of dependencies for each function which requires grad
   std::vector<Node*> queue{root};
-  bool might_use_cuda = at::globalContext().hasCUDA();
-  bool will_use_cuda = false;
+  bool will_use_accelerator = false;
 
   // Queue contains all nodes that will start propagating gradients.
   // We no longer have to expand functions that don't require grad.
@@ -1135,8 +1166,8 @@ auto Engine::compute_dependencies(
     if (fn->topological_nr() < min_topo_nr) {
       continue;
     }
-    if (might_use_cuda && !will_use_cuda) {
-      will_use_cuda = fn->stream(c10::DeviceType::CUDA).has_value();
+    if (!will_use_accelerator) {
+      will_use_accelerator = fn->stream().has_value();
     }
     for (const auto& edge : fn->next_edges()) {
       if (auto next_ptr = edge.function.get()) {
@@ -1148,9 +1179,10 @@ auto Engine::compute_dependencies(
     }
   }
 
-  if (will_use_cuda) {
-    // Collects current streams for devices where this process has a context,
-    // so GraphTask::exec_post_processing can sync them with leaf_streams.
+  if (will_use_accelerator) {
+    // Collects current streams for devices where this process has a
+    // context, so GraphTask::exec_post_processing can sync them with
+    // leaf_streams.
     task.stash_current_streams();
   }
 }
@@ -1162,9 +1194,9 @@ auto Engine::execute(
     bool create_graph,
     bool accumulate_grad,
     const edge_list& outputs) -> variable_list {
-  // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
   validate_outputs(
       root_edges,
+      // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
       const_cast<variable_list&>(inputs),
       [](const std::string& msg) { return msg; });
   if (accumulate_grad && create_graph) {
@@ -1182,7 +1214,7 @@ auto Engine::execute(
   TORCH_INTERNAL_ASSERT(compiled_autograd != COMPILED_AUTOGRAD_POISON);
 
   // accumulate_grad is true if and only if the frontend call was to
-  // grad(), not backward(). grad() returns the sum of the gradients
+  // backward(), not grad(). grad() returns the sum of the gradients
   // w.r.t. the inputs and thus needs the inputs to be present.
   TORCH_CHECK_VALUE(
       accumulate_grad || !outputs.empty(), "grad requires non-empty inputs.");
@@ -1225,7 +1257,6 @@ auto Engine::execute(
 
   if (compiled_autograd != nullptr) {
     // see [Note: Compiled Autograd]
-    TORCH_CHECK(!keep_graph, "compiled_autograd does not support keep_graph");
     TORCH_CHECK(
         !create_graph, "compiled_autograd does not support create_graph");
     _thread_check.release();
@@ -1242,8 +1273,7 @@ auto Engine::execute(
     auto input = inputs.at(0);
 
     const auto input_stream = InputMetadata(input).stream();
-    const auto opt_next_stream =
-        root_edges.at(0).function->stream(c10::DeviceType::CUDA);
+    auto opt_next_stream = root_edges.at(0).function->stream();
     input_buffer.add(
         root_edges.at(0).input_nr,
         std::move(input),
@@ -1511,27 +1541,18 @@ void Engine::add_thread_pool_task(const std::weak_ptr<GraphTask>& graph_task) {
   thread_pool_shared_->work_.notify_one();
 }
 
-// Remembers current streams on all devices where a context has been created.
-// Only called if Engine::execute detects at least one node runs on a cuda
-// stream.
+// Remembers current streams on all devices where a context has been created for
+// This function assumes the accelerator device is available.
 void GraphTask::stash_current_streams() {
-  const auto guard = c10::impl::VirtualGuardImpl{c10::DeviceType::CUDA};
-  auto num_gpus = guard.deviceCount();
-  caller_current_streams_.resize(num_gpus);
-  if (num_gpus > 0) {
-    for (c10::DeviceIndex idx = 0; idx < num_gpus; idx++) {
-#if defined(USE_ROCM) && (ROCM_VERSION < 50000)
-      // If the build targets ROCM, stash streams for all visible devices
-      // unconditionally, to work around
-      // https://github.com/pytorch/pytorch/issues/59750.
-      // TODO: Remove ROCM-specific behavior when
-      // https://github.com/pytorch/pytorch/issues/59750 is fixed.
-      if (true) {
-#else
-      if (at::detail::getCUDAHooks().hasPrimaryContext(idx)) {
-#endif
-        caller_current_streams_[idx] =
-            guard.getStream({c10::DeviceType::CUDA, idx});
+  const auto accelerator = at::getAccelerator(true).value();
+  const auto guard = c10::impl::VirtualGuardImpl{accelerator};
+  auto num_devices = guard.deviceCount();
+  caller_current_streams_.resize(num_devices);
+  if (num_devices > 0) {
+    for (c10::DeviceIndex idx = 0; idx < num_devices; idx++) {
+      if (at::globalContext().getAcceleratorHooksInterface().hasPrimaryContext(
+              idx)) {
+        caller_current_streams_[idx] = guard.getStream({accelerator, idx});
       } else {
         caller_current_streams_[idx] = c10::nullopt;
       }
@@ -1586,7 +1607,7 @@ void GraphTask::init_to_execute(
       // In terms of populating the rest of exec_info though, you can basically
       // think of this as the same as setting `needed_` is true directly.
       if (!info.captures_) {
-        info.captures_ = make_unique<std::vector<ExecInfo::Capture>>();
+        info.captures_ = std::make_unique<std::vector<ExecInfo::Capture>>();
       }
       info.captures_->emplace_back(output_edge.input_nr, output_idx++);
     }
@@ -1651,5 +1672,4 @@ void GraphTask::init_to_execute(
   }
 }
 
-} // namespace autograd
-} // namespace torch
+} // namespace torch::autograd
