@@ -11,6 +11,7 @@
 #include <torch/csrc/utils/pybind.h>
 #include <torch/csrc/utils/python_strings.h>
 
+#include <iostream>
 #include <sstream>
 
 using torch::autograd::Variable;
@@ -92,11 +93,14 @@ bool _call_hooks(PyObject* dict, PyObject* args) {
 
 } // namespace
 
-PyFunctionTensorPreHook::PyFunctionTensorPreHook(PyObject* dict, int value_idx)
+PyFunctionTensorPreHook::PyFunctionTensorPreHook(
+    PyObject* dict,
+    size_t value_idx)
     : dict(dict), value_idx(value_idx) {
   Py_INCREF(dict);
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 PyFunctionTensorPreHook::~PyFunctionTensorPreHook() {
   // If python is already dead, leak the wrapped python objects
   if (Py_IsInitialized()) {
@@ -125,6 +129,7 @@ PyFunctionPreHook::PyFunctionPreHook(PyObject* dict) : dict(dict) {
   Py_INCREF(dict);
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 PyFunctionPreHook::~PyFunctionPreHook() {
   // If python is already dead, leak the wrapped python objects
   if (Py_IsInitialized()) {
@@ -147,6 +152,7 @@ PyFunctionPostHook::PyFunctionPostHook(PyObject* dict) : dict(dict) {
   Py_INCREF(dict);
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 PyFunctionPostHook::~PyFunctionPostHook() {
   // If python is already dead, leak the wrapped python objects
   if (Py_IsInitialized()) {
@@ -169,17 +175,18 @@ auto PyFunctionPostHook::operator()(
 }
 
 void PyFunctionTensorPreHook::compiled_args(CompiledNodeArgs& args) {
-  PyObject *key, *value;
+  PyObject *key = nullptr, *value = nullptr;
   Py_ssize_t pos = 0;
   while (PyDict_Next(dict, &pos, &key, &value)) {
     Py_INCREF(value);
     args.add_tensor_pre_hook(
-        c10::SafePyObject(value, getPyInterpreter()), value_idx);
+        c10::SafePyObject(value, getPyInterpreter()),
+        static_cast<int>(value_idx));
   }
 }
 
 void PyFunctionPreHook::compiled_args(CompiledNodeArgs& args) {
-  PyObject *key, *value;
+  PyObject *key = nullptr, *value = nullptr;
   Py_ssize_t pos = 0;
   while (PyDict_Next(dict, &pos, &key, &value)) {
     Py_INCREF(value);
@@ -188,7 +195,7 @@ void PyFunctionPreHook::compiled_args(CompiledNodeArgs& args) {
 }
 
 void PyFunctionPostHook::compiled_args(CompiledNodeArgs& args) {
-  PyObject *key, *value;
+  PyObject *key = nullptr, *value = nullptr;
   Py_ssize_t pos = 0;
   while (PyDict_Next(dict, &pos, &key, &value)) {
     Py_INCREF(value);
@@ -202,6 +209,7 @@ PyFunctionTensorPostAccGradHooks::PyFunctionTensorPostAccGradHooks(
   Py_INCREF(dict);
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 PyFunctionTensorPostAccGradHooks::~PyFunctionTensorPostAccGradHooks() {
   // If python is already dead, leak the wrapped python objects
   if (Py_IsInitialized()) {
@@ -220,12 +228,37 @@ auto PyFunctionTensorPostAccGradHooks::operator()(const Variable& tensor)
       returned_none, "Tensor post accumulate grad hooks should return None.");
 }
 
+void PyFunctionTensorPostAccGradHooks::compiled_args(
+    torch::dynamo::autograd::CompiledNodeArgs& args) {
+  PyObject *key = nullptr, *value = nullptr;
+  Py_ssize_t pos = 0;
+  while (PyDict_Next(dict, &pos, &key, &value)) {
+    Py_INCREF(value);
+    c10::SafePyObject hook_obj(value, getPyInterpreter());
+    args.add_post_acc_grad_hook(std::move(hook_obj));
+  }
+}
+
+void PyFunctionTensorPostAccGradHooks::apply_with_saved(
+    Variable& tensor,
+    torch::dynamo::autograd::SwapSavedVariables& saved) {
+  for (const auto hook : saved.get_curr_node_call().post_acc_grad_hooks) {
+    THPObjectPtr py_var(THPVariable_Wrap(tensor));
+    PyObject_CallMethod(
+        saved.get_py_compiler(),
+        "post_acc_grad_hook",
+        "Oi",
+        py_var.get(),
+        hook);
+  }
+}
+
 } // namespace autograd
 } // namespace torch
 
 static PyObject* wrap_variables(const variable_list& c_variables) {
   size_t num_vars = c_variables.size();
-  THPObjectPtr tuple(PyTuple_New(num_vars));
+  THPObjectPtr tuple(PyTuple_New(static_cast<Py_ssize_t>(num_vars)));
   if (!tuple)
     throw python_error();
   for (const auto i : c10::irange(num_vars)) {
