@@ -9,8 +9,6 @@
 
 #include <ATen/Tensor.h>
 
-#include <cstdint>
-#include <list>
 #include <memory>
 #include <sstream>
 
@@ -119,7 +117,7 @@ void SavedVariable::reset_data() {
 }
 
 SavedVariable::SavedVariable(
-    const c10::optional<Variable>& variable,
+    const std::optional<Variable>& variable,
     bool is_output,
     bool is_inplace_on_view)
     : SavedVariable(
