@@ -7,7 +7,9 @@
 #include <torch/csrc/autograd/function.h>
 #include <torch/csrc/autograd/functions/accumulate_grad.h>
 #include <torch/csrc/autograd/functions/tensor.h>
+#include <torch/csrc/autograd/functions/utils.h>
 #include <torch/csrc/autograd/generated/Functions.h>
+#include <torch/csrc/autograd/generated/ViewFuncs.h>
 #include <torch/csrc/autograd/utils/error_messages.h>
 
 #include <ATen/ATen.h>
@@ -15,23 +17,33 @@
 #include <ATen/MemoryOverlap.h>
 #include <c10/util/Exception.h>
 
-#include <iostream>
-#include <list>
 #include <memory>
 #include <mutex>
 #include <stdexcept>
 #include <string>
-#include <typeinfo>
 #include <utility>
 #include <vector>
 
 namespace torch {
 namespace autograd {
 
+// Returns a ViewFunc with a corresponding view that matches the shape,
+// stride, and storage offset of the given tensor.
+// NB: On mobile, the as_strided() op and thus the generated AsStridedViewFunc
+// may not be available.
+static std::unique_ptr<ViewFunc> create_view_func_matching(const Variable& t) {
+#ifdef AS_STRIDED_VIEW_FUNC_AVAILABLE
+  return std::make_unique<torch::autograd::generated::AsStridedViewFunc>(
+      t.sym_sizes(), t.sym_strides(), t.sym_storage_offset());
+#else
+  return std::make_unique<ErroringViewFunc>("as_strided() not available");
+#endif
+}
+
 DifferentiableViewMeta::DifferentiableViewMeta(
     at::TensorImpl* self_impl,
-    c10::optional<ViewInfo> backward_info,
-    c10::optional<ViewInfo> forward_info,
+    std::optional<ViewInfo> backward_info,
+    std::optional<ViewInfo> forward_info,
     bool shared_view_info,
     CreationMeta creation_meta)
     : AutogradMeta(self_impl),
@@ -61,7 +73,8 @@ DifferentiableViewMeta::DifferentiableViewMeta(
 ViewInfo ViewInfo::chain(
     const Variable& base,
     const Variable& tensor,
-    std::function<Variable(const Variable&)> view_func) const {
+    std::unique_ptr<ViewFunc> view_func,
+    std::function<Variable(const Variable&)> rev_view_func) const {
   // Set `view_func` using the root base as input.
   // `view_func` is used to recover views in backward when either as_strided is
   // not supported or the view function changes the metadata which is not
@@ -71,55 +84,66 @@ ViewInfo ViewInfo::chain(
   if (view_func) {
     // both current_view and it's parent have a view_func
     if (view_fn_) {
-      // Copy parent view function to gain ownership
-      auto prev_fn = view_fn_;
-      view_func = [=](const at::Tensor& root_base) {
-        auto temp = prev_fn(root_base);
-        return view_func(temp);
+      view_func = std::make_unique<ChainedViewFunc>(
+          view_fn_->clone_and_set(), std::move(view_func));
+
+      // assume view_fn_ / rev_view_fn_ always exist together or neither are set
+      auto prev_rev_fn = rev_view_fn_;
+      rev_view_func = [=](const at::Tensor& root_view) {
+        auto temp = rev_view_func(root_view);
+        return prev_rev_fn(temp);
       };
     } else {
       // current_view has a view_func and but it's parent doesn't have one
       if (base.unsafeGetTensorImpl()->support_as_strided()) {
-        auto size = base.sym_sizes().vec();
-        auto stride = base.sym_strides().vec();
-        auto storage_offset = base.sym_storage_offset();
-        view_func = [=](const at::Tensor& root_base) {
-          auto temp = root_base.as_strided_symint(size, stride, storage_offset);
-          return view_func(temp);
+        auto match_base_view_func = create_view_func_matching(base);
+        view_func = std::make_unique<ChainedViewFunc>(
+            std::move(match_base_view_func), std::move(view_func));
+
+        // assume view_fn_ / rev_view_fn_ always exist together or neither are
+        // set
+        const auto& root_base = base._base();
+        auto root_base_size = root_base.sym_sizes().vec();
+        auto root_base_stride = root_base.sym_strides().vec();
+        auto root_base_storage_offset = root_base.sym_storage_offset();
+        rev_view_func = [=](const at::Tensor& root_view) {
+          auto temp = rev_view_func(root_view);
+          return temp.as_strided_symint(
+              root_base_size, root_base_stride, root_base_storage_offset);
         };
       } else {
-        // When base is a view but doesn't carry a view_fn in
-        // DifferentiableViewMeta, it's a view that doesn't support inplace
-        // update, e.g. unbind. In this case we should throw an error when
-        // inplace update happens in **forward**. One would naturally think the
-        // following function will be first called in backward pass. But the
-        // first call site is indeed in **forward** pass when we refresh
-        // `grad_fn` triggered by inplace update. Search Note [View + Inplace
-        // update for view tensor] to for the call site.
-        view_func = [=](const at::Tensor& root_base) {
-          TORCH_CHECK(
-              false,
-              "This view is the output of a function that returns multiple views."
-              "Such functions do not allow the output views to be modified inplace."
-              "You should replace the inplace operation by an out-of-place one");
-          return root_base;
+        // This case should be relatively rare: parent view doesn't have a
+        // view_func() AND as_strided() isn't supported; there's no obvious way
+        // to chain the two views.
+        auto error_msg =
+            ("Attempted to chain views when the parent view has no view_func() and "
+             "does not support as_strided(). This is not supported.");
+        view_func = std::make_unique<ErroringViewFunc>(error_msg);
+        rev_view_func = [=](const at::Tensor& root_view) {
+          TORCH_CHECK(false, error_msg);
+          return root_view;
         };
       }
     }
   } else if (view_fn_) {
     // if current_view doesn't have a view_func but it's parent has one
-    // Copy parent view function to gain ownership
-    auto prev_view_fn = view_fn_;
-    auto size = tensor.sym_sizes().vec();
-    auto stride = tensor.sym_strides().vec();
-    auto storage_offset = tensor.sym_storage_offset();
-    view_func = [=](const at::Tensor& root_base) {
-      auto temp = prev_view_fn(root_base);
-      return temp.as_strided_symint(size, stride, storage_offset);
+    auto match_tensor_view_func = create_view_func_matching(tensor);
+    view_func = std::make_unique<ChainedViewFunc>(
+        view_fn_->clone_and_set(), std::move(match_tensor_view_func));
+
+    // assume view_fn_ / rev_view_fn_ always exist together or neither are set
+    auto prev_rev_view_fn = rev_view_fn_;
+    auto base_size = base.sym_sizes().vec();
+    auto base_stride = base.sym_strides().vec();
+    auto base_storage_offset = base.sym_storage_offset();
+    rev_view_func = [=](const at::Tensor& root_view) {
+      auto temp = root_view.as_strided_symint(
+          base_size, base_stride, base_storage_offset);
+      return prev_rev_view_fn(temp);
     };
   }
 
-  return ViewInfo(base_, std::move(view_func));
+  return ViewInfo(base_, std::move(view_func), std::move(rev_view_func));
 }
 
 namespace {
@@ -204,13 +228,18 @@ void rebase_history(const Variable& self, Edge gradient_edge) {
     TORCH_CHECK(
         gradient_edge.function->num_inputs() == 1,
         "Functions which modify views in-place must return a single Variable");
-    auto view_info = diff_view_meta->get_backward_view();
+    const auto& view_info = diff_view_meta->get_backward_view();
     diff_view_meta->output_nr_ = gradient_edge.input_nr;
     auto copy_slices = std::make_shared<CopySlices>(
         view_info.base_,
         at::TensorGeometry(self),
-        view_info.view_fn_,
+        view_info.has_view_fn() ? view_info.view_fn().clone_and_set() : nullptr,
         std::move(gradient_edge.function));
+    if (self.requires_grad()) {
+      // If self did not previously require grad, there are no hooks to move
+      torch::autograd::impl::update_tensor_hooks_on_new_gradfn(
+          view_info.base_, view_info.base_.grad_fn(), copy_slices);
+    }
     set_gradient_edge(view_info.base_, {std::move(copy_slices), 0});
     self.grad_fn(); // trigger an update to the view's grad_fn
     return;
@@ -226,10 +255,9 @@ void create_cpp_hook(const at::TensorBase& self, bool is_retains_grad_hook) {
   const auto& fn = self.grad_fn();
   std::shared_ptr<hooks_list>& list =
       materialize_autograd_meta(self)->cpp_hooks_list_;
-  // NOLINTNEXTLINE(modernize-make-shared)
   list.reset(new hooks_list());
-  std::unique_ptr<FunctionPreHook> hook_ptr{
-      new CppFunctionTensorPreHook(list, self.output_nr())};
+  auto hook_ptr =
+      std::make_unique<CppFunctionTensorPreHook>(list, self.output_nr());
   // NB: we could potentially only update hooks_ if !fn, but it shouldn't
   // matter
   //     and this was the way before, so we keep it like this for now.
@@ -536,9 +564,10 @@ void VariableHooks::retain_grad(const at::TensorBase& self) const {
   };
 
   const auto& fn = self.grad_fn();
-  std::unique_ptr<FunctionPreHook> hook_ptr{new CppFunctionSingleTensorPreHook(
-      std::move(retain_grad_hook), self.output_nr())};
-  fn->add_retains_grad_hook(std::move(hook_ptr), self.output_nr());
+  fn->add_retains_grad_hook(
+      std::make_unique<CppFunctionSingleTensorPreHook>(
+          std::move(retain_grad_hook), self.output_nr()),
+      self.output_nr());
   impl::get_autograd_meta(self)->retains_grad_ = true;
 }
 
@@ -553,10 +582,10 @@ bool VariableHooks::retains_grad(const at::TensorBase& self) const {
 void VariableHooks::_backward(
     const Tensor& self,
     at::TensorList inputs,
-    const c10::optional<Tensor>& gradient,
-    c10::optional<bool> keep_graph,
+    const std::optional<Tensor>& gradient,
+    std::optional<bool> keep_graph,
     bool create_graph) const {
-  // TODO torch::autograd::backward should take the c10::optional<Tensor>
+  // TODO torch::autograd::backward should take the std::optional<Tensor>
   // gradient directly instead of us having to unwrap it to Tensor _gradient
   // here.
   Tensor _gradient = gradient.has_value() ? *gradient : Tensor();
@@ -624,7 +653,7 @@ const std::shared_ptr<torch::autograd::Node>& VariableHooks::grad_fn(
   if (diff_view_meta && diff_view_meta->has_bw_view()) {
     // See NOTE [ View + Inplace detection ]
     std::lock_guard<std::mutex> lock(diff_view_meta->mutex_);
-    auto view_info = diff_view_meta->get_backward_view();
+    auto& view_info = diff_view_meta->get_backward_view();
     if (!diff_view_meta->grad_fn_ && !view_info.base_.requires_grad()) {
       return diff_view_meta->grad_fn_;
     }
@@ -664,7 +693,7 @@ const std::shared_ptr<torch::autograd::Node>& VariableHooks::grad_fn(
       // in VariableType_x.cpp
       //       that would provide a way to recreate the grad_fn chain.
       if (view_info.has_view_fn()) {
-        auto view_fn = view_info.view_fn();
+        auto& view_fn = view_info.view_fn();
         Tensor diff_view;
         {
           // We can reach this path with grad_mode disabled, e.g. engine
@@ -685,7 +714,8 @@ const std::shared_ptr<torch::autograd::Node>& VariableHooks::grad_fn(
             view_info.base_.options(),
             self.sym_sizes(), // Note: sizes(), not base_.sizes(), is
                               // intentional
-            self.unsafeGetTensorImpl()->is_python_dispatch());
+            self.unsafeGetTensorImpl()->is_python_dispatch(),
+            self.is_nested());
         diff_view_meta->grad_fn_ = std::move(fn);
       }
       diff_view_meta->set_attr_version(current_version);
@@ -724,7 +754,7 @@ unsigned VariableHooks::_register_hook(
   auto& list = torch::autograd::impl::get_autograd_meta(self)->cpp_hooks_list_;
   if (!list) {
     torch::autograd::impl::create_cpp_hook(
-        self, /*is_retains_grad_hook=*/false);
+        self, /*is_retains_grad_hooks=*/false);
   }
   unsigned idx = list->size();
   list->push_back(hook);
@@ -823,5 +853,59 @@ void handle_view_on_rebase(
   }
 }
 
+std::vector<c10::SymInt> ChainedViewFunc::get_symints() const {
+  auto symints = first->get_symints();
+  auto second_symints = second->get_symints();
+  symints.reserve(symints.size() + second_symints.size());
+  symints.insert(
+      symints.end(),
+      std::make_move_iterator(second_symints.begin()),
+      std::make_move_iterator(second_symints.end()));
+  return symints;
+}
+
+std::vector<at::Tensor> ChainedViewFunc::get_tensors() const {
+  auto tensors = first->get_tensors();
+  auto second_tensors = second->get_tensors();
+  tensors.reserve(tensors.size() + second_tensors.size());
+  tensors.insert(
+      tensors.end(),
+      std::make_move_iterator(second_tensors.begin()),
+      std::make_move_iterator(second_tensors.end()));
+  return tensors;
+}
+
+at::Tensor ChainedViewFunc::operator()(const at::Tensor& input_base) const {
+  return (*second)((*first)(input_base));
+}
+
+std::unique_ptr<ViewFunc> ChainedViewFunc::clone_and_set(
+    std::optional<std::vector<c10::SymInt>> symints,
+    std::optional<std::vector<at::Tensor>> tensors) const {
+  std::optional<std::vector<c10::SymInt>> first_symints;
+  std::optional<std::vector<c10::SymInt>> second_symints;
+  if (symints.has_value()) {
+    TORCH_INTERNAL_ASSERT(symints->size() == num_symints());
+    first_symints = std::vector<c10::SymInt>(
+        symints->begin(), symints->begin() + first->num_symints());
+    second_symints = std::vector<c10::SymInt>(
+        symints->begin() + first->num_symints(), symints->end());
+  }
+
+  std::optional<std::vector<at::Tensor>> first_tensors;
+  std::optional<std::vector<at::Tensor>> second_tensors;
+  if (tensors.has_value()) {
+    TORCH_INTERNAL_ASSERT(tensors->size() == num_tensors());
+    first_tensors = std::vector<at::Tensor>(
+        tensors->begin(), tensors->begin() + first->num_tensors());
+    second_tensors = std::vector<at::Tensor>(
+        tensors->begin() + first->num_tensors(), tensors->end());
+  }
+
+  return std::make_unique<ChainedViewFunc>(
+      first->clone_and_set(first_symints, first_tensors),
+      second->clone_and_set(second_symints, second_tensors));
+}
+
 } // namespace autograd
 } // namespace torch
