@@ -4,9 +4,7 @@
 #include <nvToolsExt.h>
 #include <torch/csrc/utils/pybind.h>
 
-namespace torch {
-namespace cuda {
-namespace shared {
+namespace torch::cuda::shared {
 
 void initNvtxBindings(PyObject* module) {
   auto m = py::handle(module).cast<py::module>();
@@ -19,6 +17,4 @@ void initNvtxBindings(PyObject* module) {
   nvtx.def("markA", nvtxMarkA);
 }
 
-} // namespace shared
-} // namespace cuda
-} // namespace torch
+} // namespace torch::cuda::shared
