@@ -1,4 +1,3 @@
-#include <torch/csrc/distributed/c10d/TCPStoreBackend.hpp>
 
 #include <c10/util/irange.h>
 #include <fcntl.h>
@@ -16,6 +15,8 @@
 #include <unistd.h>
 #endif
 
+#include <torch/csrc/distributed/c10d/TCPStoreBackend.hpp>
+
 #ifdef _WIN32
 #include <torch/csrc/distributed/c10d/WinSockUtils.hpp>
 #else
@@ -24,11 +25,10 @@
 
 #include <torch/csrc/distributed/c10d/socket.h>
 
-namespace c10d {
-namespace detail {
+namespace c10d::detail {
 
 // Background thread parent class methods
-BackgroundThread::BackgroundThread() {}
+BackgroundThread::BackgroundThread() = default;
 
 BackgroundThread::~BackgroundThread() = default;
 
@@ -55,7 +55,7 @@ class TCPStoreMasterDaemon : public BackgroundThread {
 
   ~TCPStoreMasterDaemon() override;
 
-  std::uint16_t port() const override;
+  uint16_t port() const override;
 
  protected:
   void run() override;
@@ -67,10 +67,12 @@ class TCPStoreMasterDaemon : public BackgroundThread {
 
   void queryFds(std::vector<struct pollfd>& fds);
   void query(int socket);
+
   void clearSocketWaitState(int socket);
 
   // The master runs on a single thread so only
   // one handler can be executed at a time
+  void validateHandler(int socket);
   void setHandler(int socket);
   void compareSetHandler(int socket);
   void addHandler(int socket);
@@ -83,6 +85,9 @@ class TCPStoreMasterDaemon : public BackgroundThread {
   void multiGetHandler(int socket);
   void multiSetHandler(int socket);
   void cancelWaitHandler(int socket);
+  void addMiscellaneousSocket(int socket);
+  void removeMiscellaneousSocket(int socket);
+  bool isMiscellaneousSocket(int socket);
 
   bool checkKeys(const std::vector<std::string>& keys) const;
   // Helper function to alerts waiting workers, used in setHandler, getHandler
@@ -94,6 +99,8 @@ class TCPStoreMasterDaemon : public BackgroundThread {
   std::unordered_map<std::string, std::vector<int>> waitingSockets_;
   // From socket -> number of keys awaited
   std::unordered_map<int, size_t> keysAwaited_;
+  // miscellaneous sockets
+  std::unordered_set<int> miscellaneousSockets_;
 
   Socket storeListenSocket_;
   std::vector<Socket> sockets_{};
@@ -163,7 +170,21 @@ void TCPStoreMasterDaemon::closeStopSignal() {
 
 void TCPStoreMasterDaemon::stop() {
   if (controlPipeFd_[1] != -1) {
-    ::write(controlPipeFd_[1], "\0", 1);
+    ssize_t written_bytes = -1;
+    while (true) {
+      written_bytes = ::write(controlPipeFd_[1], "\0", 1);
+      if (written_bytes < 0) {
+        if (errno == EAGAIN) {
+          continue;
+        }
+        TORCH_CHECK(false, "Failed to write the control pipe:", errno);
+      }
+      break;
+    }
+    if (written_bytes == 0) {
+      TORCH_CHECK(false, "Failed to write the control pipe");
+    }
+
     // close the write end of the pipe
     ::close(controlPipeFd_[1]);
     controlPipeFd_[1] = -1;
@@ -234,7 +255,17 @@ void TCPStoreMasterDaemon::clearSocketWaitState(int socket) {
 void TCPStoreMasterDaemon::query(int socket) {
   QueryType qt;
   tcputil::recvBytes<QueryType>(socket, &qt, 1);
-  if (qt == QueryType::SET) {
+
+  if (isMiscellaneousSocket(socket)) {
+    removeMiscellaneousSocket(socket);
+    if (qt == QueryType::VALIDATE) {
+      validateHandler(socket);
+    } else {
+      // real miscellaneous client: the first msg is not VALIDATE
+      TORCH_CHECK(
+          false, "Miscellaneous client without VALIDATE query is detected");
+    }
+  } else if (qt == QueryType::SET) {
     setHandler(socket);
 
   } else if (qt == QueryType::COMPARE_SET) {
@@ -291,6 +322,16 @@ void TCPStoreMasterDaemon::doSet(
   wakeupWaitingClients(key);
 }
 
+void TCPStoreMasterDaemon::validateHandler(int socket) {
+  uint32_t validateNumber = 0;
+  tcputil::recvBytes<uint32_t>(socket, &validateNumber, 1);
+  if (validateNumber != detail::validationMagicNumber) {
+    TORCH_CHECK(
+        false,
+        "Miscellaneous client with incorrect VALIDATE query is detected");
+  }
+}
+
 void TCPStoreMasterDaemon::setHandler(int socket) {
   std::string key = tcputil::recvString(socket);
   std::vector<uint8_t> newData = tcputil::recvVector<uint8_t>(socket);
@@ -443,6 +484,23 @@ bool TCPStoreMasterDaemon::checkKeys(
   });
 }
 
+void TCPStoreMasterDaemon::addMiscellaneousSocket(int socket) {
+  if (miscellaneousSockets_.find(socket) == miscellaneousSockets_.end()) {
+    miscellaneousSockets_.insert(socket);
+  }
+}
+
+void TCPStoreMasterDaemon::removeMiscellaneousSocket(int socket) {
+  auto it = miscellaneousSockets_.find(socket);
+  if (it != miscellaneousSockets_.end()) {
+    miscellaneousSockets_.erase(it);
+  }
+}
+
+bool TCPStoreMasterDaemon::isMiscellaneousSocket(int socket) {
+  return miscellaneousSockets_.find(socket) != miscellaneousSockets_.end();
+}
+
 #ifdef _WIN32
 void TCPStoreMasterDaemon::run() {
   std::vector<struct pollfd> fds;
@@ -471,9 +529,8 @@ void TCPStoreMasterDaemon::run() {
     // accept new connections.
     if (fds[0].revents != 0) {
       if (!(fds[0].revents & POLLIN)) {
-        throw std::system_error(
-            ECONNABORTED,
-            std::system_category(),
+        C10_THROW_ERROR(
+            DistStoreError,
             "Unexpected poll revent on the master's listening socket: " +
                 std::to_string(fds[0].revents));
       }
@@ -481,6 +538,7 @@ void TCPStoreMasterDaemon::run() {
       int rawSocket = socket.handle();
       sockets_.emplace_back(std::move(socket));
       tcputil::addPollfd(fds, rawSocket, POLLIN);
+      addMiscellaneousSocket(rawSocket);
     }
     queryFds(fds);
   }
@@ -513,9 +571,8 @@ void TCPStoreMasterDaemon::run() {
     // accept new connections.
     if (fds[0].revents != 0) {
       if (fds[0].revents ^ POLLIN) {
-        throw std::system_error(
-            ECONNABORTED,
-            std::system_category(),
+        C10_THROW_ERROR(
+            DistStoreError,
             "Unexpected poll revent on the master's listening socket: " +
                 std::to_string(fds[0].revents));
       }
@@ -523,6 +580,8 @@ void TCPStoreMasterDaemon::run() {
       int rawSocket = socket.handle();
       sockets_.emplace_back(std::move(socket));
       tcputil::addPollfd(fds, rawSocket, POLLIN);
+      // all clients are miscellaneous before getting its validation query
+      addMiscellaneousSocket(rawSocket);
     }
 
     // The pipe receives an event which tells us to shutdown the daemon
@@ -530,9 +589,8 @@ void TCPStoreMasterDaemon::run() {
       // The main thread will write a byte to the pipe then close it before
       // joining the background thread
       if (fds[1].revents & ~(POLLIN | POLLHUP)) {
-        throw std::system_error(
-            ECONNABORTED,
-            std::system_category(),
+        C10_THROW_ERROR(
+            DistStoreError,
             "Unexpected poll revent on the control pipe's reading fd: " +
                 std::to_string(fds[1].revents));
       }
@@ -553,5 +611,4 @@ std::unique_ptr<BackgroundThread> create_tcpstore_backend(
   return std::make_unique<TCPStoreMasterDaemon>(std::move(socket));
 }
 
-} // namespace detail
-} // namespace c10d
+} // namespace c10d::detail
