@@ -1,11 +1,14 @@
 #include <torch/csrc/dynamo/python_compiled_autograd.h>
 
 #include <torch/csrc/autograd/engine.h>
+#include <torch/csrc/autograd/functions/accumulate_grad.h>
 #include <torch/csrc/dynamo/compiled_autograd.h>
 #include <torch/csrc/jit/python/pybind_utils.h>
 #include <torch/csrc/python_headers.h>
 #include <torch/csrc/utils/pythoncapi_compat.h>
 #include <iostream>
+#include <sstream>
+#include <string>
 #include <vector>
 
 /*
@@ -49,7 +52,7 @@ namespace torch::dynamo::autograd {
 using c10::SymInt;
 
 static PyObject* wrap_int_list(const std::vector<int64_t>& inputs) {
-  PyObject* pyinput = PyTuple_New(inputs.size());
+  PyObject* pyinput = PyTuple_New(static_cast<Py_ssize_t>(inputs.size()));
   for (const auto i : c10::irange(inputs.size())) {
     PyTuple_SET_ITEM(pyinput, i, PyLong_FromSsize_t(inputs[i]));
   }
@@ -58,7 +61,7 @@ static PyObject* wrap_int_list(const std::vector<int64_t>& inputs) {
 
 static PyObject* convert_hook_list(std::vector<c10::SafePyObject>& inputs) {
   // inplace, consumes the input hooks
-  PyObject* pyinput = PyTuple_New(inputs.size());
+  PyObject* pyinput = PyTuple_New(static_cast<Py_ssize_t>(inputs.size()));
   for (const auto i : c10::irange(inputs.size())) {
     PyTuple_SET_ITEM(pyinput, i, inputs[i].release());
   }
@@ -70,6 +73,7 @@ static PyObject* check(PyObject* pyresult) {
     // see https://github.com/pytorch/pytorch/pull/34845
     python_error err;
     err.persist();
+    // NOLINTNEXTLINE(misc-throw-by-value-catch-by-reference)
     throw err;
   }
   return pyresult;
@@ -80,6 +84,82 @@ static void check(bool result) {
     check(nullptr);
 }
 
+// snapshot of python verbose logging toggle
+static PyObject* python_verbose_logger = nullptr;
+struct VerboseLogger {
+  static std::optional<VerboseLogger> maybe_create() {
+    if (python_verbose_logger == nullptr) {
+      return std::nullopt;
+    }
+    return VerboseLogger();
+  }
+
+  void verbose_log_fn(std::string_view msg) const {
+    TORCH_CHECK(python_verbose_logger != nullptr);
+    check(PyObject_CallFunction(python_verbose_logger, "s", msg.data()));
+  }
+
+  void log_node_check(
+      const Node& fn,
+      size_t size_inputs_num,
+      std::unordered_set<CacheKey> cached_keys,
+      const CacheKey& key,
+      size_t node_idx) {
+    std::string node_name =
+        fn.name() + " (NodeCall " + std::to_string(node_idx) + ")";
+
+    cumulative_sizes_per_node[size_inputs_num] = node_name;
+
+    if (!logged_node_miss && cached_keys.find(key) == cached_keys.end()) {
+      _log_node_miss(typeid(fn), cached_keys, key, node_name);
+      logged_node_miss = true;
+    }
+  }
+
+  void _log_node_miss(
+      const std::type_info& node_type,
+      std::unordered_set<CacheKey> cached_keys,
+      const CacheKey& key,
+      const std::string& node_name) const {
+    std::ostringstream oss;
+    oss << "Cache miss due to new autograd node: " << node_name
+        << " with key size " << std::to_string(key.key_size)
+        << ", previous key sizes=[";
+
+    for (auto it = cached_keys.begin(); it != cached_keys.end(); it++) {
+      if (it->node_type != node_type) {
+        continue;
+      }
+      oss << it->key_size;
+      if (std::next(it) != cached_keys.end()) {
+        oss << ",";
+      }
+    }
+    oss << "]";
+    verbose_log_fn(oss.str());
+  }
+
+  void log_dynamic_shapes_check(size_t size_idx) const {
+    if (cumulative_sizes_per_node.empty()) {
+      return;
+    }
+
+    auto it = cumulative_sizes_per_node.lower_bound(size_idx);
+    TORCH_CHECK(it != cumulative_sizes_per_node.end());
+    size_t start_idx =
+        it == cumulative_sizes_per_node.begin() ? 0 : std::prev(it)->first;
+    verbose_log_fn(
+        "Cache miss due to changed shapes: marking size idx " +
+        std::to_string(size_idx - start_idx) + " of " + it->second +
+        " as dynamic");
+  }
+
+  // track which size index belongs to which node
+  std::map<size_t, std::string> cumulative_sizes_per_node;
+  // only log cache miss due to node key once
+  bool logged_node_miss = false;
+};
+
 struct CacheNode {
   // A node in the shadow graph, we follow next edges until we reach the end of
   // the graph
@@ -88,9 +168,11 @@ struct CacheNode {
     return &_root;
   }
 
-  CacheNode* lookup(const CacheKey& key) {
+  CacheNode* lookup(const CacheKey& key, bool create = true) {
     auto it = next.find(key);
     if (it == next.end()) {
+      if (!create)
+        return nullptr;
       // caller's key is in temporary memory, must copy it
       CacheKeyBuffer buffer(key.key, key.key_size);
       CacheKey key_with_storage(key.node_type, buffer.get(), key.key_size);
@@ -104,18 +186,20 @@ struct CacheNode {
     next.clear();
     key_storage.clear();
     expected_sizes.clear();
-    output_grad_targets.clear();
+    runtime_wrapper = nullptr;
     compiled_fn = nullptr;
   }
 
   bool is_empty() const {
-    return next.size() == 0 && !compiled_fn;
+    return next.empty() && !compiled_fn;
   }
 
-  CacheNode() : compiled_fn(nullptr) {}
+  CacheNode() : runtime_wrapper(nullptr), compiled_fn(nullptr) {}
   ~CacheNode() {
     if (!Py_IsInitialized()) {
-      compiled_fn.release(); // leak on shutdown
+      // leak on shutdown
+      runtime_wrapper.release();
+      compiled_fn.release();
     }
   }
   CacheNode(CacheNode&&) = delete;
@@ -123,7 +207,9 @@ struct CacheNode {
   CacheNode& operator=(const CacheNode&) = delete;
   CacheNode& operator=(CacheNode&&) = delete;
 
-  bool check_dynamic_sizes(AutogradCompilerCall& call) {
+  bool check_dynamic_sizes(
+      AutogradCompilerCall& call,
+      const std::optional<VerboseLogger>& vlogger) {
     /*
     We start off by assuming everything is static, then we mark things
     as dynamic when we see them change.  This function:
@@ -144,12 +230,19 @@ struct CacheNode {
     TORCH_INTERNAL_ASSERT(expected_sizes.size() == call.all_size_inputs.size());
     for (const auto i : c10::irange(len)) {
       auto& expected = expected_sizes[i];
-      if (expected.dyn_type == SizeInput::DYNAMIC ||
-          expected.value != data[i].value) {
-        cache_hit = cache_hit && expected.dyn_type == SizeInput::DYNAMIC;
-        if (expected.value != data[i].value) {
-          expected = SizeInput(SizeInput::DYNAMIC, data[i].value);
+      bool was_dynamic = expected.dyn_type == SizeInput::DYNAMIC;
+      bool changed_value = expected.value != data[i].value;
+      if (changed_value) {
+        if (!was_dynamic) {
+          cache_hit = false;
+          if (vlogger.has_value()) {
+            vlogger->log_dynamic_shapes_check(i);
+          }
         }
+        expected = SizeInput(SizeInput::DYNAMIC, data[i].value);
+      }
+
+      if (changed_value || was_dynamic) {
         if (call.dyn_size_inputs.empty()) {
           call.dyn_size_inputs.reserve(len);
         }
@@ -160,12 +253,13 @@ struct CacheNode {
     if (!cache_hit) {
       // we missed cache because static size inputs didn't match; force
       // recompilation with the varying size input as dynamic
+      runtime_wrapper = nullptr;
       compiled_fn = nullptr;
     }
     return cache_hit;
   }
 
-  PyObject* wrap_dynamic_inputs() {
+  PyObject* wrap_dynamic_inputs() const {
     size_t dynamic_count = 0;
     size_t idx = 0;
     for (const auto& i : expected_sizes) {
@@ -173,7 +267,7 @@ struct CacheNode {
         ++dynamic_count;
       }
     }
-    PyObject* pyinput = PyTuple_New(dynamic_count);
+    PyObject* pyinput = PyTuple_New(static_cast<Py_ssize_t>(dynamic_count));
     for (const auto& i : expected_sizes) {
       if (i.dyn_type == SizeInput::DYNAMIC) {
         PyTuple_SET_ITEM(pyinput, idx++, PyLong_FromSsize_t(i.value));
@@ -183,11 +277,12 @@ struct CacheNode {
     return pyinput;
   }
 
-  std::vector<c10::optional<SymInt>> unwrap_dynamic_inputs(PyObject* pyresult) {
+  std::vector<std::optional<SymInt>> unwrap_dynamic_inputs(
+      PyObject* pyresult) const {
     TORCH_INTERNAL_ASSERT(PyList_CheckExact(pyresult));
     size_t idx = 0;
     size_t result_len = PyList_GET_SIZE(pyresult);
-    std::vector<c10::optional<SymInt>> result;
+    std::vector<std::optional<SymInt>> result;
     result.reserve(expected_sizes.size());
     for (const auto& i : expected_sizes) {
       if (i.dyn_type == SizeInput::DYNAMIC) {
@@ -207,18 +302,13 @@ struct CacheNode {
   std::vector<CacheKeyBuffer> key_storage;
   std::vector<SizeInput> expected_sizes;
 
+  THPObjectPtr runtime_wrapper;
   THPObjectPtr compiled_fn;
-  // Maps each return value of compiled_fn to an input index.  After the graph
-  // runs we do: `inputs[output_grad_targets[i]].mutable_grad() = outputs[i]`
-  std::vector<size_t> output_grad_targets;
 };
 
 struct InputBuffers : public std::unordered_map<Node*, InputBuffer> {
   InputBuffer& lookup(Node* function) {
-    auto it = find(function);
-    if (it == end()) {
-      it = emplace(function, InputBuffer(function->num_inputs())).first;
-    }
+    auto it = emplace(function, InputBuffer(function->num_inputs())).first;
     return it->second;
   }
 };
@@ -242,11 +332,29 @@ static PyObject* is_cache_empty(PyObject* dummy, PyObject* args) {
   END_HANDLE_TH_ERRORS;
 }
 
+static PyObject* set_verbose_logger(PyObject* dummy, PyObject* args) {
+  HANDLE_TH_ERRORS;
+  PyObject* logger = nullptr;
+  if (!PyArg_ParseTuple(args, "O", &logger)) {
+    Py_RETURN_FALSE;
+  }
+
+  if (logger == Py_None) {
+    python_verbose_logger = nullptr;
+  } else {
+    python_verbose_logger = logger;
+  }
+  Py_RETURN_TRUE;
+  END_HANDLE_TH_ERRORS;
+}
+
+// NOLINTNEXTLINE(*array*)
 static PyMethodDef _methods[] = {
-    {"set_autograd_compiler", set_autograd_compiler, METH_VARARGS, NULL},
-    {"clear_cache", clear_cache, METH_NOARGS, NULL},
-    {"is_cache_empty", is_cache_empty, METH_NOARGS, NULL},
-    {NULL, NULL, 0, NULL}};
+    {"set_autograd_compiler", set_autograd_compiler, METH_VARARGS, nullptr},
+    {"clear_cache", clear_cache, METH_NOARGS, nullptr},
+    {"is_cache_empty", is_cache_empty, METH_NOARGS, nullptr},
+    {"set_verbose_logger", set_verbose_logger, METH_VARARGS, nullptr},
+    {nullptr, nullptr, 0, nullptr}};
 
 static struct PyModuleDef _module = {
     PyModuleDef_HEAD_INIT,
@@ -264,9 +372,9 @@ static TraceState call_begin_capture(
   THPObjectPtr pyinput(THPVariable_WrapList(compiler_call.tensor_args.inputs));
   THPObjectPtr pysizeinput(cache.wrap_dynamic_inputs());
   THPObjectPtr pyresult(check(PyObject_CallMethodObjArgs(
-      self, method_name, pyinput.get(), pysizeinput.get(), NULL)));
+      self, method_name, pyinput.get(), pysizeinput.get(), nullptr)));
 
-  PyObject *fake_inputs, *fake_sizes;
+  PyObject *fake_inputs{nullptr}, *fake_sizes{nullptr};
   check(PyArg_ParseTuple(pyresult.get(), "OO", &fake_inputs, &fake_sizes));
 
   variable_list proxy_inputs = THPVariable_UnpackList(fake_inputs);
@@ -290,31 +398,35 @@ static PyObject* call_end_capture(PyObject* self, const variable_list& inputs) {
 struct ClosingTHPObjectPtr : public THPObjectPtr {
   ClosingTHPObjectPtr(PyObject* o) : THPObjectPtr(o) {}
   ~ClosingTHPObjectPtr() {
+    if (PyErr_Occurred()) {
+      // do nothing, do not attempt to close
+      return;
+    }
     static PyObject* method_name = PyUnicode_InternFromString("close");
-    check(PyObject_CallMethodNoArgs(get(), method_name));
+    if (PyObject_CallMethodNoArgs(get(), method_name) == nullptr) {
+      PyErr_WriteUnraisable(get());
+      PyErr_Clear();
+    }
   }
 };
 
-variable_list compiled_autograd(
+// Only call this function while holding GIL
+CacheNode* _compiled_autograd_impl(
     const std::shared_ptr<Node>& graph_root,
     GraphTask& graph_task,
     bool accumulate_grad,
-    const edge_list& output_edges) {
-  TORCH_CHECK(
-      output_edges.empty() || !accumulate_grad,
-      "specifying inputs= with .backward() not yet implemented for compiled autograd")
-  TORCH_CHECK(
-      c10::impl::TorchDispatchModeTLS::stack_len() == 0,
-      "TorchDispatchMode not yet implemented for compiled autograd")
-  static std::mutex lock;
-  std::lock_guard<std::mutex> lock_guard(lock);
-  pybind11::gil_scoped_acquire gil;
-  at::ThreadLocalStateGuard tls_guard(graph_task.thread_locals_);
+    const edge_list& output_edges,
+    THPObjectPtr* graph_arg_inputs,
+    THPObjectPtr* graph_arg_sizes,
+    THPObjectPtr* graph_arg_hooks) {
   std::unordered_map<Node*, int>& dependencies = graph_task.dependencies_;
   std::vector<std::shared_ptr<Node>> worklist{graph_root};
   AutogradCompilerCall compiler_call;
+
   for (const auto i : c10::irange(output_edges.size())) {
-    compiler_call.node_calls.lookup(output_edges[i].function)
+    compiler_call.node_calls
+        .lookup(output_edges[i].function)
+        // NOLINTNEXTLINE(*-narrowing-conversions)
         .mark_output(output_edges[i].input_nr, i);
   }
   const bool check_exec_info = !graph_task.exec_info_.empty();
@@ -323,6 +435,8 @@ variable_list compiled_autograd(
   calls.reserve(
       check_exec_info ? graph_task.exec_info_.size() : dependencies.size() + 1);
 
+  int i = 0;
+  std::optional<VerboseLogger> vlogger = VerboseLogger::maybe_create();
   while (!worklist.empty()) {
     std::shared_ptr<Node> fn = std::move(worklist.back());
     worklist.pop_back();
@@ -336,7 +450,20 @@ variable_list compiled_autograd(
         fn->compiled_args(node_args);
         node_args.collect(call.node->next_edges());
       }
-      cache = cache->lookup(node_args.key());
+      CacheKey key = node_args.key();
+      if (vlogger.has_value()) {
+        std::unordered_set<CacheKey> cached_keys;
+        for (const auto& [k, _] : cache->next) {
+          cached_keys.emplace(k);
+        }
+        vlogger->log_node_check(
+            *fn,
+            compiler_call.all_size_inputs.size(),
+            std::move(cached_keys),
+            key,
+            i);
+      }
+      cache = cache->lookup(key);
     }
 
     for (const auto& edge : fn->next_edges()) {
@@ -359,19 +486,21 @@ variable_list compiled_autograd(
         worklist.emplace_back(edge.function);
       }
     }
+    i++;
   }
 
   // TODO(jansel): some dynamic sizes seem to be ints not symints
-  if (!cache->check_dynamic_sizes(compiler_call)) {
+  if (!cache->check_dynamic_sizes(compiler_call, vlogger)) {
     // cache miss, need to capture FX graph
     ClosingTHPObjectPtr py_compiler(
         check(PyObject_CallNoArgs((the_autograd_compiler))));
+
     TraceState state = call_begin_capture(
         py_compiler, *cache, compiler_call, output_edges.size());
     InputBuffers input_buffers;
 
-    for (NodeCall* call_ptr : calls) {
-      NodeCall& call = *call_ptr;
+    for (size_t i = 0; i < calls.size(); i++) {
+      NodeCall& call = *calls[i];
       // TODO(jansel): consider adding some of this stuff:
       // guard(local_graph_task); NodeGuard ndguard(task.fn_); const auto
       // opt_parent_stream = (*func).stream(c10::DeviceType::CUDA);
@@ -380,9 +509,11 @@ variable_list compiled_autograd(
       // at::getStepCallbacksUnlessEmpty(at::RecordScope::BACKWARD_FUNCTION);
       // if (C10_UNLIKELY(step_callbacks.has_value())) { ... }
 
-      variable_list inputs = input_buffers.lookup(call.node.get()).buffer;
+      variable_list inputs =
+          std::move(input_buffers.lookup(call.node.get()).buffer);
+      input_buffers.erase(call.node.get());
 
-      if (call.tensor_pre_hooks.size() > 0) {
+      if (!call.tensor_pre_hooks.empty()) {
         THPObjectPtr pyinputs(THPVariable_WrapList(inputs));
         for (const auto& hook : call.tensor_pre_hooks) {
           pyinputs = check(PyObject_CallMethod(
@@ -406,7 +537,7 @@ variable_list compiled_autograd(
       if (!call.needed) {
         continue;
       }
-      if (call.pre_hooks.size() > 0) {
+      if (!call.pre_hooks.empty()) {
         THPObjectPtr pyinputs(THPVariable_WrapList(inputs));
         for (const auto hook : call.pre_hooks) {
           pyinputs = check(PyObject_CallMethod(
@@ -415,8 +546,19 @@ variable_list compiled_autograd(
         inputs = THPVariable_UnpackList(pyinputs);
       }
 
-      SwapSavedVariables saved(compiler_call, state);
+      if (python_verbose_logger != nullptr) {
+        std::string _node_name = call.node->name();
+        THPObjectPtr node_name(PyUnicode_FromString(_node_name.data()));
+        TORCH_INTERNAL_ASSERT(node_name != nullptr);
+        THPObjectPtr set_node_origin(
+            PyObject_GetAttrString(py_compiler.get(), "set_node_origin"));
+        check(PyObject_CallFunction(
+            set_node_origin, "OI", node_name.get(), i, nullptr));
+      }
+
+      SwapSavedVariables saved(compiler_call, state, py_compiler.get(), call);
       variable_list outputs = call.node->apply_with_saved(inputs, saved);
+
       saved.debug_asserts();
       saved.before(call.node->next_edges());
       validate_outputs(
@@ -429,7 +571,7 @@ variable_list compiled_autograd(
       saved.after(call.node->next_edges());
       saved.debug_asserts();
 
-      if (call.post_hooks.size() > 0) {
+      if (!call.post_hooks.empty()) {
         THPObjectPtr pyinputs(THPVariable_WrapList(inputs));
         THPObjectPtr pyoutputs(THPVariable_WrapList(outputs));
         for (const auto hook : call.post_hooks) {
@@ -454,44 +596,78 @@ variable_list compiled_autograd(
       }
     }
 
-    cache->compiled_fn = check(call_end_capture(py_compiler, state.outputs));
-    cache->output_grad_targets = std::move(state.output_grad_targets);
+    PyObject* res = check(call_end_capture(py_compiler, state.outputs));
+    TORCH_CHECK(PyTuple_Check(res), "Expected end_capture to return tuple");
+    TORCH_CHECK(
+        PyTuple_Size(res) == 2,
+        "Expected end_capture to return tuple of size 2");
+    cache->runtime_wrapper = Py_NewRef(PyTuple_GetItem(res, 0));
+    TORCH_CHECK(
+        PyCallable_Check(cache->runtime_wrapper),
+        "Expected end_capture to return runtime_wrapper");
+    cache->compiled_fn = Py_NewRef(PyTuple_GetItem(res, 1));
+    TORCH_CHECK(
+        PyCallable_Check(cache->compiled_fn),
+        "Expected end_capture to return compiled_fn");
     state.debug_asserts();
-  }
+  } // End cache miss region
 
-  // TODO(jansel): we should release all the variables and then use a
-  //               boxed calling convention so activation memory can be freed
   // TODO(jansel): clear grads we will overwrite below
-  for (auto& call : calls) {
-    call->node->release_variables();
+  if (!graph_task.keep_graph_) {
+    for (auto& call : calls) {
+      call->node->release_variables();
+    }
   }
 
-  THPObjectPtr inputs(THPVariable_WrapList(compiler_call.tensor_args.inputs));
-  THPObjectPtr sizes(wrap_int_list(compiler_call.dyn_size_inputs));
-  THPObjectPtr hooks(convert_hook_list(compiler_call.hooks));
+  *graph_arg_inputs = THPVariable_WrapList(compiler_call.tensor_args.inputs);
+  *graph_arg_sizes = wrap_int_list(compiler_call.dyn_size_inputs);
+  *graph_arg_hooks = convert_hook_list(compiler_call.hooks);
+  return cache;
+}
+
+variable_list compiled_autograd(
+    const std::shared_ptr<Node>& graph_root,
+    GraphTask& graph_task,
+    bool accumulate_grad,
+    const edge_list& output_edges) {
+  TORCH_CHECK(
+      output_edges.empty() || !accumulate_grad,
+      "specifying inputs= with .backward() not yet implemented for compiled autograd")
+  TORCH_CHECK(
+      c10::impl::TorchDispatchModeTLS::stack_len() == 0,
+      "TorchDispatchMode not yet implemented for compiled autograd")
+  static std::mutex lock;
+  std::lock_guard<std::mutex> lock_guard(lock);
+  pybind11::gil_scoped_acquire gil;
+  at::ThreadLocalStateGuard tls_guard(graph_task.thread_locals_);
+
+  THPObjectPtr inputs;
+  THPObjectPtr sizes;
+  THPObjectPtr hooks;
+  CacheNode* cache = _compiled_autograd_impl(
+      graph_root,
+      graph_task,
+      accumulate_grad,
+      output_edges,
+      &inputs,
+      &sizes,
+      &hooks);
+
   THPObjectPtr pyresult(check(PyObject_CallFunctionObjArgs(
-      cache->compiled_fn.get(), inputs.get(), sizes.get(), hooks.get(), NULL)));
+      cache->runtime_wrapper.get(),
+      cache->compiled_fn.get(),
+      inputs.get(),
+      sizes.get(),
+      hooks.get(),
+      NULL)));
   variable_list outputs = THPVariable_UnpackList(pyresult);
-  if (accumulate_grad) {
-    TORCH_INTERNAL_ASSERT(outputs.size() == cache->output_grad_targets.size());
-    for (const auto i : c10::irange(outputs.size())) {
-      // Here we set the `var.grad = ...` for each call to
-      // `saved.assign_mutable_grad(var, ...)`.  For the case on inplace grad
-      // accumuation there will be an `add_` op in the graph and no return
-      // value.
-      compiler_call.tensor_args.inputs[cache->output_grad_targets[i]]
-          .mutable_grad() = outputs[i];
-    }
-    return variable_list();
-  } else {
-    TORCH_INTERNAL_ASSERT(outputs.size() == output_edges.size());
-    return outputs;
-  }
+  TORCH_INTERNAL_ASSERT(outputs.size() == output_edges.size());
+  return outputs;
 }
 
 static PyObject* set_autograd_compiler(PyObject* dummy, PyObject* args) {
   HANDLE_TH_ERRORS;
-  PyObject* obj;
+  PyObject* obj = nullptr;
   if (!PyArg_ParseTuple(args, "O", &obj)) {
     return nullptr;
   }
