@@ -364,7 +364,7 @@ static void emitCheckFor(
 std::string generateKernel(
     const std::string& name,
     const Graph& graph,
-    const std::vector<std::pair<const Value*, const c10::optional<TensorDesc>>>&
+    const std::vector<std::pair<const Value*, const std::optional<TensorDesc>>>&
         inputs,
     const std::vector<std::pair<const Value*, const TensorDesc>>& outputs,
     const bool use_cuda) {
@@ -659,25 +659,6 @@ std::string generateKernel(
     env.s("RandInit", "");
   }
 
-  // HIP headers must be included until precompiled header feature is available
-  // clang-format off
-#if defined(USE_ROCM)
-#if ROCM_VERSION < 40200
-  if (use_cuda && has_half_tensor) {
-    env.s("RuntimeHeader", R"(
-#include <hip/hip_runtime.h>
-#include <hip/hip_fp16.h>
-)");
-  } else if (use_cuda) {
-    env.s("RuntimeHeader", R"(
-#include <hip/hip_runtime.h>
-)");
-  }
-#else
-  // Still need the key defined, but empty.
-  env.s("RuntimeHeader", R"()");
-#endif
-#endif
   // clang-format on
 
   // Instantiates the CUDA or CPU-specific templates
