@@ -118,7 +118,7 @@ void ImplicitCastForBinaryInplaceOps(Block* b) {
       if ((shape_node->kind() == prim::NumToTensor) &&
           (shape_node->inputs().at(0)->node()->kind() == aten::size)) {
         std::cerr
-            << "In-place op on output of tensor.shape. See https://pytorch.org/docs/master/onnx.html#"
+            << "In-place op on output of tensor.shape. See https://pytorch.org/docs/main/onnx.html#"
             << "avoid-inplace-operations-when-using-tensor-shape-in-tracing-mode"
             << std::endl;
       }
