@@ -23,7 +23,6 @@
 #include <torch/csrc/jit/passes/tensorexpr_fuser.h>
 #include <torch/csrc/jit/runtime/exception_message.h>
 #include <torch/csrc/jit/runtime/symbolic_shape_registry.h>
-#include <torch/csrc/utils/memory.h>
 #include <algorithm>
 #include <memory>
 #include <numeric>
@@ -66,7 +65,7 @@ namespace jit {
 
 struct ShapeArg
     : public std::
-          pair<c10::optional<c10::ShapeSymbol>, c10::optional<int64_t>> {
+          pair<std::optional<c10::ShapeSymbol>, std::optional<int64_t>> {
   using pair::pair;
 
   static ShapeArg unknownInteger() {
@@ -88,11 +87,11 @@ struct ShapeArg
     }
   }
 
-  c10::optional<int64_t> asConstantInt() const {
+  std::optional<int64_t> asConstantInt() const {
     return this->second;
   }
 
-  c10::optional<c10::ShapeSymbol> asShapeSymbol() const {
+  std::optional<c10::ShapeSymbol> asShapeSymbol() const {
     return this->first;
   }
 
@@ -174,13 +173,13 @@ bool symbolicShapeAnalysisTestModeEnabled() {
   return symbolic_shape_analysis_test_mode;
 }
 
-using SSArgument = c10::variant<ShapeArguments, IValue>;
+using SSArgument = std::variant<ShapeArguments, IValue>;
 
 static std::ostream& operator<<(std::ostream& out, const SSArgument& sa) {
-  if (const IValue* iv = c10::get_if<IValue>(&sa)) {
+  if (const IValue* iv = std::get_if<IValue>(&sa)) {
     out << *iv;
   } else {
-    out << c10::get<ShapeArguments>(sa);
+    out << std::get<ShapeArguments>(sa);
   }
   return out;
 }
@@ -209,7 +208,7 @@ bool isListOfTensors(const TypePtr& type) {
       type->cast<ListType>()->getElementType()->cast<TensorType>();
 }
 
-c10::optional<size_t> normIndex(int64_t index, size_t len) {
+std::optional<size_t> normIndex(int64_t index, size_t len) {
   if (index < 0) {
     index = index + len;
   }
@@ -256,7 +255,7 @@ c10::SymbolicShape extractListShape(
     return c10::SymbolicShape();
   }
   Node* list_construct = list->node();
-  std::vector<c10::optional<int64_t>> output_shape;
+  std::vector<std::optional<int64_t>> output_shape;
   for (Value* input : list_construct->inputs()) {
     if (symbolic_shape_values.count(input)) {
       output_shape.emplace_back(symbolic_shape_values[input]);
@@ -378,11 +377,11 @@ struct SymbolicShapeOpAnalyzer {
       SSArgument& argument = inputs_[op_in_index];
       Value* graph_in_var = shape_compute_graph_->inputs().at(op_in_index);
 
-      if (IValue* cur_val = c10::get_if<IValue>(&argument)) {
+      if (IValue* cur_val = std::get_if<IValue>(&argument)) {
         GRAPH_DEBUG("Substituting constant input ", *cur_val);
         replaceWithIValue(graph_in_var, *cur_val);
       } else {
-        auto cur_arg = c10::get<ShapeArguments>(argument);
+        auto cur_arg = std::get<ShapeArguments>(argument);
         if (cur_arg.has_dim()) {
           graph_in_var->setType(ListType::ofInts());
         }
@@ -424,7 +423,7 @@ struct SymbolicShapeOpAnalyzer {
         "Missing Arg for Shape Graph");
     for (const auto index :
          c10::irange(shape_compute_graph_->inputs().size())) {
-      auto shape_arguments = c10::get_if<ShapeArguments>(&inputs_[index]);
+      auto shape_arguments = std::get_if<ShapeArguments>(&inputs_[index]);
       if (!shape_arguments || !shape_arguments->has_dim()) {
         continue;
       }
@@ -606,7 +605,7 @@ struct SymbolicShapeOpAnalyzer {
     shape_compute_graph_ = graph->copy();
   }
 
-  c10::optional<std::vector<c10::SymbolicShape>> run(
+  std::optional<std::vector<c10::SymbolicShape>> run(
       std::vector<SSArgument>& inputs) {
     if (!shape_compute_graph_) {
       return c10::nullopt;
@@ -814,7 +813,7 @@ struct SymbolicShapeGraphAnalyzer {
         beg_->owningBlock() == end_->owningBlock() && end_->isAfter(beg_));
   }
 
-  c10::optional<ShapeComputeGraphMapping> run() {
+  std::optional<ShapeComputeGraphMapping> run() {
     AliasDb db(graph_);
     std::unordered_map<Node*, std::shared_ptr<Graph>> partial_evaluated_graphs =
         propagateShapesAndGatherPartialEvalShapeGraphs(db);
@@ -1121,7 +1120,7 @@ void PropagateShapesOnGraph(std::shared_ptr<Graph>& graph) {
   PropagateShapesOnBlock(graph->block(), db);
 }
 
-c10::optional<ShapeComputeGraphMapping>
+std::optional<ShapeComputeGraphMapping>
 PropagateShapesAndBuildLargeShapeComputeGraph(
     std::shared_ptr<Graph>& graph,
     Node* beg,
@@ -1129,7 +1128,7 @@ PropagateShapesAndBuildLargeShapeComputeGraph(
   return SymbolicShapeGraphAnalyzer(graph, beg, end).run();
 }
 
-TORCH_API c10::optional<std::vector<c10::SymbolicShape>>
+TORCH_API std::optional<std::vector<c10::SymbolicShape>>
 calculateSymbolicShapesOnOp(
     const FunctionSchema* schema,
     const std::vector<SSAInput>& inputs) {
@@ -1147,10 +1146,10 @@ calculateSymbolicShapesOnOp(
 
   std::vector<SSArgument> ssa_args;
   for (auto& arg : inputs) {
-    if (const IValue* ival = c10::get_if<IValue>(&arg)) {
+    if (const IValue* ival = std::get_if<IValue>(&arg)) {
       ssa_args.emplace_back(*ival);
     } else {
-      const c10::SymbolicShape* ss = c10::get_if<c10::SymbolicShape>(&arg);
+      const c10::SymbolicShape* ss = std::get_if<c10::SymbolicShape>(&arg);
       ssa_args.emplace_back(ShapeArguments(*ss));
     }
   }
