@@ -14,7 +14,6 @@
 #include <torch/csrc/jit/passes/constant_pooling.h>
 #include <torch/csrc/jit/passes/constant_propagation.h>
 #include <torch/csrc/jit/passes/create_autodiff_subgraphs.h>
-#include <torch/csrc/jit/passes/cuda_graph_fuser.h>
 #include <torch/csrc/jit/passes/dead_code_elimination.h>
 #include <torch/csrc/jit/passes/decompose_ops.h>
 #include <torch/csrc/jit/passes/graph_fuser.h>
@@ -36,6 +35,7 @@
 #include <torch/csrc/jit/passes/tensorexpr_fuser.h>
 #include <torch/csrc/jit/passes/update_differentiable_graph_requires_grad.h>
 #include <torch/csrc/jit/passes/utils/subgraph_utils.h>
+#include <chrono>
 #include <mutex>
 
 C10_DEFINE_bool(
@@ -58,6 +58,16 @@ C10_DEFINE_bool(
     false,
     "fuse on 12 dynamic compilations");
 
+C10_DEFINE_bool(
+    torch_jit_release_profiling_graph_after_optimization,
+    false,
+    "After getOptimizedPlanFor release the optimization record for reduction of memory in inference. This is aggressive memory saving, and please be cautious!");
+
+C10_DEFINE_int32(
+    torch_jit_release_profiling_graph_delay_in_seconds,
+    60,
+    "How long to wait before releasing the profiling graph after optimizaiton is done. Only used if torch_jit_release_profiling_graph_after_optimization is set to true.");
+
 constexpr size_t kDefaultNumProfiledRuns = 1;
 constexpr size_t kDefaultBailoutDepth = 20;
 
@@ -70,8 +80,16 @@ C10_DEFINE_int64(
     kDefaultBailoutDepth,
     "Number of re-specializations");
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
+
+namespace {
+int32_t getNowInSecs() {
+  auto currentTimePoint = std::chrono::system_clock::now();
+  auto durationSinceEpoch = std::chrono::duration_cast<std::chrono::seconds>(
+      currentTimePoint.time_since_epoch());
+  return static_cast<int32_t>(durationSinceEpoch.count());
+}
+} // namespace
 
 #if defined(C10_MOBILE)
 static std::atomic<bool> executor_mode{true};
@@ -100,7 +118,7 @@ static FusionStrategy getInitialStrategy() {
 }
 
 // defer initial value so that we can load in gflags
-static c10::optional<FusionStrategy> fusion_strategy = c10::nullopt;
+static std::optional<FusionStrategy> fusion_strategy = c10::nullopt;
 
 FusionStrategy getFusionStrategy() {
   std::lock_guard<std::mutex> guard(fusion_strategy_lock);
@@ -448,7 +466,6 @@ void ProfilingGraphExecutorImpl::runNoGradOptimizations(
     }
 
     // Run custom post-fusion passes
-    // e.g. NVFuser
     for (const auto& passPair : getCustomPostPasses()) {
       passPair.first(graph);
     }
@@ -596,7 +613,7 @@ size_t ProfilingGraphExecutorImpl::getInstantiatedBailoutDepth() {
 
 const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
     Stack& stack,
-    c10::optional<size_t> remaining_bailout_depth) {
+    std::optional<size_t> remaining_bailout_depth) {
   GRAPH_DEBUG("Running ProfilingGraphExecutorImpl ", this);
 
   // TODO: instantiate simple executor when getProfilingMode() is false
@@ -634,27 +651,23 @@ const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
     runProfilingInsensitiveOptimizations(copy);
     GRAPH_DUMP("Optimized SimpleExecutor Graph: ", copy);
     optimized_plan_ = ExecutionPlan(copy, function_name_);
+    time_optimized_plan_created_ = getNowInSecs();
     return *optimized_plan_;
   }
 
+  bool profiling_record_created_in_this_call = false;
   // if a profiling graph hasn't been created yet
   if (!pr_) {
     auto copy = graph->copy();
     runProfilingInsensitiveOptimizations(copy);
     pr_ = ProfilingRecord::instrumentGraph(copy);
+    profiling_record_created_in_this_call = true;
     // `InsertProfileNodesForSpecializeAutogradZero` profiles a definition vs a
     // use and it doesn't expect any profile nodes between a graph input and its
     // consumer, `aten::_grad_sum_to_size`. This means we need to run it first,
     // before any other pass that could insert `prim::iprofile_value` node on
     // `aten::_grad_sum_to_size` input.
     InsertProfileNodesForSpecializeAutogradZero(pr_.get());
-    // `InsertProfileNodesForCUDAFuser` inserts profile node for non-tensor
-    // value
-#ifndef C10_MOBILE
-    if (torch::jit::fuser::cuda::isEnabled()) {
-      torch::jit::fuser::cuda::InsertProfileNodesForCUDAFuser(pr_.get());
-    }
-#endif
     GRAPH_DUMP("Profiled Graph: ", pr_->graph());
     profiling_plan_ = ExecutionPlan(pr_->graph(), function_name_);
     // fall-through
@@ -675,17 +688,32 @@ const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
   CheckStrictFusion(copy);
   GRAPH_DUMP("Optimized Graph: ", copy);
   optimized_plan_ = ExecutionPlan(copy, function_name_);
+  time_optimized_plan_created_ = getNowInSecs();
+  // If the profiled graph was created in this call, then we can release it
+  // right.
+  if (FLAGS_torch_jit_release_profiling_graph_after_optimization &&
+      profiling_record_created_in_this_call) {
+    clearTheGraphCompilationIntermediateGraphs();
+  }
   return *optimized_plan_;
 }
 
 const ExecutionPlan& ProfilingGraphExecutorImpl::getPlanFor(
     Stack& stack,
-    c10::optional<size_t> remaining_bailout_depth) {
+    std::optional<size_t> remaining_bailout_depth) {
   std::lock_guard<std::mutex> lock(compile_mutex);
 
   // IMPORTANT: This is a hot path of calling a torchscript function. Try not to
   // add any code above this.
   if (optimized_plan_) {
+    if (FLAGS_torch_jit_release_profiling_graph_after_optimization &&
+        !is_graph_extra_memory_released_) {
+      int32_t now = getNowInSecs();
+      if ((now - time_optimized_plan_created_) >
+          FLAGS_torch_jit_release_profiling_graph_delay_in_seconds) {
+        clearTheGraphCompilationIntermediateGraphs();
+      }
+    }
     return *optimized_plan_;
   }
   // if depth is not set, use
@@ -776,5 +804,27 @@ void ProfilingGraphExecutorImpl::runFinalOptimizations(
   AddIfThenElseOp(graph);
 }
 
-} // namespace jit
-} // namespace torch
+void ProfilingGraphExecutorImpl::debugFlushCompilationCache() {
+  std::lock_guard<std::mutex> lock(compile_mutex);
+  pr_.reset();
+  fallback_plan_.reset();
+  profiling_plan_.reset();
+  optimized_plan_.reset();
+  // prevent memory leaks
+  fallback_functions_.clear();
+  remaining_bailout_depth_.reset();
+  // TODO - would be nice to have it initialized in subsequent use
+  fusion_strategy_ = getFusionStrategy();
+  time_optimized_plan_created_ = 0;
+  is_graph_extra_memory_released_ = false;
+}
+
+void ProfilingGraphExecutorImpl::clearTheGraphCompilationIntermediateGraphs() {
+  is_graph_extra_memory_released_ = true;
+  profiling_plan_.reset();
+  fallback_plan_.reset();
+  graph.reset();
+  pr_.reset();
+}
+
+} // namespace torch::jit
