@@ -8,7 +8,6 @@
 #include <c10/macros/Macros.h>
 #include <c10/util/MaybeOwned.h>
 #include <c10/util/irange.h>
-#include <caffe2/core/scope_guard.h>
 #include <caffe2/core/timer.h>
 #include <torch/csrc/jit/ir/alias_analysis.h>
 #include <torch/csrc/jit/jit_log.h>
@@ -53,8 +52,7 @@ C10_DEFINE_bool(
     false,
     "If true, disable the memory overlap check in debug mode in ProcessedNode::run()");
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 namespace {
 
@@ -287,7 +285,7 @@ void PrepareGraphForStaticModule(
   ForceNonEmptyOutputs(*graph);
 }
 
-std::pair<std::shared_ptr<Graph>, c10::optional<Module>> PrepareForStaticModule(
+std::pair<std::shared_ptr<Graph>, std::optional<Module>> PrepareForStaticModule(
     const torch::jit::Module& m,
     bool is_frozen,
     const StaticModuleOptions& opts,
@@ -317,7 +315,7 @@ std::pair<std::shared_ptr<Graph>, c10::optional<Module>> PrepareForStaticModule(
   return std::make_pair(graph, module);
 }
 
-std::pair<std::shared_ptr<Graph>, c10::optional<Module>> PrepareForStaticModule(
+std::pair<std::shared_ptr<Graph>, std::optional<Module>> PrepareForStaticModule(
     std::shared_ptr<torch::jit::Graph> graph,
     const StaticModuleOptions& opts,
     std::vector<IValue> sample_inputs) {
@@ -545,7 +543,7 @@ StaticModule::StaticModule(
           opts) {}
 
 StaticModule::StaticModule(
-    std::pair<std::shared_ptr<torch::jit::Graph>, c10::optional<Module>>
+    std::pair<std::shared_ptr<torch::jit::Graph>, std::optional<Module>>
         graph_and_module,
     const StaticModuleOptions& opts)
     : opts_(opts),
@@ -761,6 +759,31 @@ size_t StaticModule::prepareStaticNodeInfos(
   return node_idx - node_start;
 }
 
+#ifdef FBCODE_CAFFE2
+thread_local SROperatorObserver* tlsOpObserver = nullptr;
+
+void SROperatorObserver::setCurrentThreadObserver(
+    SROperatorObserver* observer) {
+  tlsOpObserver = observer;
+}
+
+SROperatorObserver* SROperatorObserver::getCurrentThreadObserver() {
+  return tlsOpObserver;
+}
+
+void SROperatorObserver::onStart(const Node* node) {
+  if (tlsOpObserver != nullptr && tlsOpObserver->startCb != nullptr) {
+    tlsOpObserver->startCb(node);
+  }
+}
+
+void SROperatorObserver::onEnd(const Node* node) {
+  if (tlsOpObserver != nullptr && tlsOpObserver->endCb != nullptr) {
+    tlsOpObserver->endCb(node);
+  }
+}
+#endif // FBCODE_CAFFE2
+
 BlockInfo::BlockInfo(uint32_t input_idx, Block& block)
     : input_idx_(input_idx), block_(block) {}
 
@@ -2053,6 +2076,9 @@ std::vector<IValue> ProcessedNode::inputs_ivalue_vec() const {
 }
 
 void ProcessedNode::run() {
+#ifdef FBCODE_CAFFE2
+  SROperatorObserver::onStart(node());
+#endif
 #ifndef PYTORCH_DISABLE_PER_OP_PROFILING
   auto step_callbacks =
       at::getStepCallbacksUnlessEmpty(at::RecordScope::STATIC_RUNTIME_OP);
@@ -2086,6 +2112,9 @@ void ProcessedNode::run() {
     DCHECK(verify_no_memory_overlap());
   }
 #endif
+#ifdef FBCODE_CAFFE2
+  SROperatorObserver::onEnd(node());
+#endif
 }
 
 static bool checkNoMemoryOverlap(const at::Tensor& a, const at::Tensor& b) {
@@ -2292,5 +2321,4 @@ const MemoryPlanner* StaticRuntime::get_memory_planner() const {
   return block_->get_memory_planner();
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
