@@ -1,15 +1,11 @@
 #include <ATen/core/interned_strings.h>
-#include <c10/core/CPUAllocator.h>
-#include <c10/core/impl/alloc_cpu.h>
 #include <caffe2/serialize/file_adapter.h>
 #include <caffe2/serialize/in_memory_adapter.h>
 #include <caffe2/serialize/inline_container.h>
 #include <caffe2/serialize/istream_adapter.h>
 #include <caffe2/serialize/read_adapter_interface.h>
-#include <caffe2/serialize/versions.h>
 
 #include <torch/csrc/jit/api/compilation_unit.h>
-#include <torch/csrc/jit/mobile/file_format.h>
 #include <torch/csrc/jit/serialization/import.h>
 #include <torch/csrc/jit/serialization/source_range_serialization.h>
 
@@ -17,10 +13,6 @@
 #include <ATen/core/ivalue_inl.h>
 #include <c10/util/Exception.h>
 #include <c10/util/irange.h>
-#include <torch/csrc/jit/serialization/import_export_helpers.h>
-#if !defined(C10_MOBILE) && !defined(C10_DISABLE_LEGACY_IMPORT)
-#include <torch/csrc/jit/serialization/import_legacy.h>
-#endif
 #include <torch/csrc/jit/frontend/script_type_parser.h>
 #include <torch/csrc/jit/ir/graph_utils.h>
 #include <torch/csrc/jit/ir/ir.h>
@@ -29,24 +21,21 @@
 #include <torch/csrc/jit/operator_upgraders/upgraders_entry.h>
 #include <torch/csrc/jit/passes/shape_analysis.h>
 #include <torch/csrc/jit/passes/subgraph_rewrite.h>
+#include <torch/csrc/jit/serialization/import_export_helpers.h>
 #include <torch/csrc/jit/serialization/import_read.h>
 #include <torch/csrc/jit/serialization/import_source.h>
-#include <torch/csrc/jit/serialization/pickle.h>
 #include <torch/csrc/jit/serialization/source_range_serialization.h>
 #include <torch/csrc/jit/serialization/unpickler.h>
 
 #include <ATen/ATen.h>
 #include <fmt/format.h>
 
-#include <fstream>
 #include <string>
 #include <unordered_map>
 #include <vector>
 
 namespace torch::jit {
 
-using caffe2::serialize::FileAdapter;
-using caffe2::serialize::IStreamAdapter;
 using caffe2::serialize::MemoryReadAdapter;
 using caffe2::serialize::PyTorchStreamReader;
 using caffe2::serialize::ReadAdapterInterface;
@@ -75,6 +64,43 @@ static void postSetStateValidate(const IValue& v) {
   }
 }
 
+// Decouple how to get obj from type. In this file it's dependent on
+// Method.run() and graph executor, etc.
+// For bytecode import we need to decouple these dependencies.
+c10::intrusive_ptr<c10::ivalue::Object> ObjLoaderFunc(
+    const at::StrongTypePtr& type,
+    IValue input) {
+  auto cls = type.type_->expect<at::ClassType>();
+  auto qn = cls->name();
+  size_t n = cls->numAttributes();
+  if (checkHasValidSetGetState(cls)) {
+    auto obj = c10::ivalue::Object::create(type, n);
+    // XXX: Do not optimize __setstate__, so that we don't try to
+    // specialize the class before it is initialized.
+    GraphOptimizerEnabledGuard guard(false);
+    Function& set_state = cls->getMethod("__setstate__");
+    // since we are in the middle of unpickling we might still have lists and
+    // dicts that do not have accurate tags (e.g. they report they are
+    // List[Any]). But we need to run __setstate__ which will check the input
+    // type and may access the tags. Since setstate has a known input type, we
+    // can correctly restore the tags now by apply the input type of set_state
+    // to the state object being passed.
+    // TODO: Remove once [serialization type tags] is landed
+    restoreAccurateTypeTags(
+        input, set_state.getSchema().arguments().at(1).type());
+    set_state({obj, input});
+    postSetStateValidate(obj);
+    return obj;
+  } else {
+    auto dict = std::move(input).toGenericDict();
+    auto obj = c10::ivalue::Object::create(type, n);
+    for (const auto i : c10::irange(n)) {
+      obj->setSlot(i, dict.at(cls->getAttributeName(i)));
+    }
+    return obj;
+  }
+}
+
 namespace {
 
 // This is a deserializer class which loads script modules from pt files.
@@ -123,7 +149,7 @@ class ScriptModuleDeserializer final {
             reader_->version()) {}
 
   Module deserialize(
-      c10::optional<at::Device> device,
+      std::optional<at::Device> device,
       ExtraFilesMap& extra_files,
       bool restore_shapes = false);
 
@@ -133,7 +159,7 @@ class ScriptModuleDeserializer final {
   std::shared_ptr<CompilationUnit> compilation_unit_;
   std::shared_ptr<PyTorchStreamReader> reader_;
   std::shared_ptr<DeserializationStorageContext> storage_context_;
-  c10::optional<at::Device> device_;
+  std::optional<at::Device> device_;
   std::vector<at::IValue> constants_table_;
   std::string code_prefix_;
   std::string pickle_dir_prefix_;
@@ -147,46 +173,12 @@ IValue ScriptModuleDeserializer::readArchive(const std::string& archive_name) {
     return c10::StrongTypePtr(compilation_unit_, std::move(cls));
   };
 
-  // Decouple how to get obj from type. In this file it's dependent on
-  // Method.run() and graph executor, etc.
-  // For bytecode import we need to decouple these dependencies.
-  auto obj_loader = [&](const at::StrongTypePtr& type, IValue input) {
-    auto cls = type.type_->expect<at::ClassType>();
-    auto qn = cls->name();
-    size_t n = cls->numAttributes();
-    if (checkHasValidSetGetState(cls)) {
-      auto obj = c10::ivalue::Object::create(type, n);
-      // XXX: Do not optimize __setstate__, so that we don't try to
-      // specialize the class before it is initialized.
-      GraphOptimizerEnabledGuard guard(false);
-      Function& set_state = cls->getMethod("__setstate__");
-      // since we are in the middle of unpickling we might still have lists and
-      // dicts that do not have accurate tags (e.g. they report they are
-      // List[Any]). But we need to run __setstate__ which will check the input
-      // type and may access the tags. Since setstate has a known input type, we
-      // can correctly restore the tags now by apply the input type of set_state
-      // to the state object being passed.
-      // TODO: Remove once [serialization type tags] is landed
-      restoreAccurateTypeTags(
-          input, set_state.getSchema().arguments().at(1).type());
-      set_state({obj, input});
-      postSetStateValidate(obj);
-      return obj;
-    } else {
-      auto dict = std::move(input).toGenericDict();
-      auto obj = c10::ivalue::Object::create(type, n);
-      for (const auto i : c10::irange(n)) {
-        obj->setSlot(i, dict.at(cls->getAttributeName(i)));
-      }
-      return obj;
-    }
-  };
   return readArchiveAndTensors(
       /*archive_name=*/archive_name,
       /*pickle_prefix=*/pickle_dir_prefix_,
       /*tensor_prefix=*/tensor_dir_prefix_,
       type_resolver,
-      obj_loader,
+      ObjLoaderFunc,
       device_,
       *reader_.get(),
       nullptr,
@@ -253,32 +245,25 @@ graph(%x, %packed_params, %stride, %padding, %dilation, %groups, %r_scale, %r_ze
 }
 
 Module ScriptModuleDeserializer::deserialize(
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool restore_shapes) {
   // we populate the upgraders map before any load starts
   populate_upgraders_graph_map();
 
-  C10_LOG_API_USAGE_ONCE("torch.script.load");
+  C10_LOG_API_USAGE_ONCE("torch.jit.load");
   device_ = device;
   // Load extra files.
   for (const auto& kv : extra_files) {
     const std::string& key = "extra/" + kv.first;
     if (reader_->hasRecord(key)) {
-      at::DataPtr meta_ptr;
-      // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-      size_t meta_size;
-      std::tie(meta_ptr, meta_size) = reader_->getRecord(key);
+      auto [meta_ptr, meta_size] = reader_->getRecord(key);
       extra_files[kv.first] =
           std::string(static_cast<char*>(meta_ptr.get()), meta_size);
     }
   }
   if (reader_->hasRecord("model.json") && code_prefix_ == "code/") {
-#if !defined(C10_MOBILE) && !defined(C10_DISABLE_LEGACY_IMPORT)
-    return torch::jit::LEGACY_deserialize(compilation_unit_, reader_, device_);
-#else
     AT_ERROR("Legacy model format is not supported on mobile.");
-#endif
   }
   auto tuple = readArchive("constants").toTuple();
   for (auto constant : tuple->elements()) {
@@ -319,7 +304,7 @@ Module ScriptModuleDeserializer::deserialize(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::istream& in,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     bool load_debug_files) {
   ExtraFilesMap extra_files;
   return import_ir_module(
@@ -330,7 +315,7 @@ static Module _load_jit_module_from_bytes(
     std::shared_ptr<char> data,
     size_t size,
     std::shared_ptr<CompilationUnit> cu,
-    c10::optional<c10::Device> device,
+    std::optional<c10::Device> device,
     ExtraFilesMap& extra_files,
     bool restore_shapes);
 
@@ -338,7 +323,7 @@ Module parse_and_initialize_jit_module(
     std::shared_ptr<char> data,
     size_t size,
     ExtraFilesMap& extra_files,
-    c10::optional<at::Device> device) {
+    std::optional<at::Device> device) {
   populate_upgraders_graph_map();
   ExtraFilesMap jit_files;
   std::vector<IValue> jit_constants;
@@ -357,7 +342,7 @@ Module parse_and_initialize_jit_module(
 Module load_jit_module_from_file(
     const std::string& filename,
     ExtraFilesMap& extra_files,
-    c10::optional<at::Device> device) {
+    std::optional<at::Device> device) {
   auto data = get_file_content(filename.c_str());
   return parse_and_initialize_jit_module(
       std::move(std::get<0>(data)), std::get<1>(data), extra_files, device);
@@ -366,7 +351,7 @@ Module load_jit_module_from_file(
 Module load_jit_module_from_stream(
     std::istream& in,
     ExtraFilesMap& extra_files,
-    c10::optional<at::Device> device) {
+    std::optional<at::Device> device) {
   auto data = get_stream_content(in);
   return parse_and_initialize_jit_module(
       std::move(std::get<0>(data)), std::get<1>(data), extra_files, device);
@@ -375,7 +360,7 @@ Module load_jit_module_from_stream(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::istream& in,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files,
     bool restore_shapes) {
@@ -383,14 +368,12 @@ Module import_ir_module(
   // NOTE: Zipformat can be large files. So using stream version directly
   // instead of reading the file all at once.
   if (getFileFormat(in) != FileFormat::FlatbufferFileFormat) {
-    auto reader = torch::make_unique<PyTorchStreamReader>(&in);
+    auto reader = std::make_unique<PyTorchStreamReader>(&in);
     reader->setShouldLoadDebugSymbol(load_debug_files);
     ScriptModuleDeserializer deserializer(std::move(cu), std::move(reader));
     return deserializer.deserialize(device, extra_files, restore_shapes);
   }
-  std::shared_ptr<char> data;
-  size_t size = 0;
-  std::tie(data, size) = get_stream_content(in);
+  auto [data, size] = get_stream_content(in);
   return _load_jit_module_from_bytes(
       data, size, cu, device, extra_files, restore_shapes);
 }
@@ -400,7 +383,7 @@ Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::shared_ptr<PyTorchStreamReader> reader,
     std::shared_ptr<DeserializationStorageContext> storage_context,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     std::string ts_id) {
   ScriptModuleDeserializer deserializer(
       std::move(cu),
@@ -415,7 +398,7 @@ Module import_ir_module(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     const std::string& filename,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     bool load_debug_files) {
   ExtraFilesMap extra_files;
   return import_ir_module(
@@ -425,21 +408,19 @@ Module import_ir_module(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     const std::string& filename,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files,
     bool restore_shapes) {
   // NOTE: Zipformat can be large files. So using stream version directly
   // instead of reading the file all at once.
   if (getFileFormat(filename) != FileFormat::FlatbufferFileFormat) {
-    auto reader = torch::make_unique<PyTorchStreamReader>(filename);
+    auto reader = std::make_unique<PyTorchStreamReader>(filename);
     reader->setShouldLoadDebugSymbol(load_debug_files);
     ScriptModuleDeserializer deserializer(std::move(cu), std::move(reader));
     return deserializer.deserialize(device, extra_files, restore_shapes);
   }
-  std::shared_ptr<char> data;
-  size_t size = 0;
-  std::tie(data, size) = get_file_content(filename.c_str());
+  auto [data, size] = get_file_content(filename.c_str());
   return _load_jit_module_from_bytes(
       data, size, cu, device, extra_files, restore_shapes);
 }
@@ -447,7 +428,7 @@ Module import_ir_module(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     bool load_debug_files) {
   ExtraFilesMap extra_files;
   return import_ir_module(
@@ -457,7 +438,7 @@ Module import_ir_module(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files) {
   std::shared_ptr<ReadAdapterInterface> rai_shared = std::move(rai);
@@ -468,7 +449,7 @@ Module import_ir_module(
 Module import_ir_module(
     std::shared_ptr<CompilationUnit> cu,
     std::shared_ptr<ReadAdapterInterface> rai,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files) {
   auto reader = std::make_shared<PyTorchStreamReader>(std::move(rai));
@@ -479,7 +460,7 @@ Module import_ir_module(
 
 Module load(
     std::istream& in,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
   return import_ir_module(std::move(cu), in, device, load_debug_files);
@@ -487,7 +468,7 @@ Module load(
 
 Module load(
     std::istream& in,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
@@ -497,7 +478,7 @@ Module load(
 
 Module load(
     const std::string& filename,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
   return import_ir_module(std::move(cu), filename, device, load_debug_files);
@@ -505,7 +486,7 @@ Module load(
 
 Module load(
     const std::string& filename,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
@@ -515,7 +496,7 @@ Module load(
 
 Module load(
     std::shared_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
+    std::optional<c10::Device> device,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
   ExtraFilesMap extra_files;
@@ -525,7 +506,7 @@ Module load(
 
 Module load(
     std::shared_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
+    std::optional<c10::Device> device,
     ExtraFilesMap& extra_files,
     bool load_debug_files) {
   auto cu = std::make_shared<CompilationUnit>();
@@ -537,7 +518,7 @@ Module _load_jit_module_from_bytes(
     std::shared_ptr<char> data,
     size_t size,
     std::shared_ptr<CompilationUnit> cu,
-    c10::optional<c10::Device> device,
+    std::optional<c10::Device> device,
     ExtraFilesMap& extra_files,
     bool restore_shapes) {
   TORCH_CHECK(size >= kFileFormatHeaderSize, "Unrecognized data format");
@@ -548,7 +529,7 @@ Module _load_jit_module_from_bytes(
     }
     case FileFormat::ZipFileFormat: {
       auto rai = std::make_unique<MemoryReadAdapter>(data.get(), size);
-      auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
+      auto reader = std::make_unique<PyTorchStreamReader>(std::move(rai));
       ScriptModuleDeserializer deserializer(std::move(cu), std::move(reader));
       return deserializer.deserialize(device, extra_files, restore_shapes);
     }
