@@ -10,6 +10,7 @@
 #include <torch/csrc/jit/serialization/unpickler.h>
 #include <torch/csrc/utils/byte_order.h>
 #include <string>
+#include <utility>
 
 namespace torch::jit {
 
@@ -406,7 +407,7 @@ PickleOpCode Unpickler::readInstruction() {
     } break;
     case PickleOpCode::TUPLE1: {
       TORCH_CHECK(
-          stack_.size() > 0,
+          !stack_.empty(),
           "Parsing error: stack_ contains ",
           stack_.size(),
           " elements, at least 1 expected");
@@ -451,6 +452,12 @@ PickleOpCode Unpickler::readInstruction() {
       auto list_ivalue = stack_.at(start - 1);
       readList(list_ivalue);
     } break;
+    case PickleOpCode::APPEND: {
+      TORCH_CHECK(
+          stack_.size() >= 2, "Parsing error: missing elements in stack_.");
+      auto list_ivalue = stack_.at(stack_.size() - 2);
+      readListElements(list_ivalue, stack_.size() - 1);
+    } break;
     case PickleOpCode::LIST: {
       IValue list_ivalue = c10::impl::GenericList(AnyType::get());
       readList(list_ivalue);
@@ -468,12 +475,12 @@ PickleOpCode Unpickler::readInstruction() {
           stack_.size());
       auto dict = c10::impl::GenericDict(AnyType::get(), AnyType::get());
       TORCH_CHECK(
-          stack_.size() % 2 == 0 && start % 2 == 0,
+          (stack_.size() - start) % 2 == 0,
           "Parsing error: stack_ is of size ",
           stack_.size(),
           " and start index is ",
           start,
-          ", but stack_ expected to contain even number of elements");
+          ", but stack_ is iterated by two elements at a time");
       for (size_t i = start; i < stack_.size(); i += 2) {
         dict.insert_or_assign(stack_[i], stack_[i + 1]);
       }
@@ -488,6 +495,13 @@ PickleOpCode Unpickler::readInstruction() {
           start > 0 && start <= stack_.size(),
           "Parsing error: wrong start index for stack_");
       auto dict = stack_.at(start - 1).toGenericDict();
+      TORCH_CHECK(
+          (stack_.size() - start) % 2 == 0,
+          "Parsing error: stack_ is of size ",
+          stack_.size(),
+          " and start index is ",
+          start,
+          ", but stack_ is iterated by two elemenst at a time");
       for (size_t i = start; i < stack_.size(); i += 2) {
         dict.insert_or_assign(stack_[i], stack_[i + 1]);
       }
@@ -808,7 +822,7 @@ void Unpickler::readGlobal(
     // like the other branches here because no REDUCE or BUILD will
     // be called on this value. Instead, we just put it on the stack
     // and return early
-    c10::optional<c10::ScalarType> scalar_type;
+    std::optional<c10::ScalarType> scalar_type;
 #define CHECK_SCALAR(_, name)          \
   if (class_name == #name "Storage") { \
     scalar_type = c10::k##name;        \
@@ -820,7 +834,7 @@ void Unpickler::readGlobal(
       return;
     }
 
-    c10::optional<at::QScheme> qscheme;
+    std::optional<at::QScheme> qscheme;
     for (int i = 0; i < at::COMPILE_TIME_NUM_QSCHEMES; ++i) {
       if (class_name == toString(static_cast<at::QScheme>(i))) {
         qscheme = static_cast<at::QScheme>(i);
@@ -1118,12 +1132,7 @@ std::string Unpickler::readBytes(size_t length) {
   return data;
 }
 
-// Pop all the list items off of the stack and append them to the list at
-// the corresponding MARK
-void Unpickler::readList(IValue list_ivalue) {
-  TORCH_CHECK(!marks_.empty(), "Parsing error: marks_ is empty");
-  size_t start = marks_.back();
-  marks_.pop_back();
+void Unpickler::readListElements(IValue list_ivalue, size_t start) {
   auto num_elements = stack_.size() - start;
   auto elements = c10::ArrayRef<IValue>(stack_).slice(start);
   if (list_ivalue.isIntList()) {
@@ -1159,10 +1168,18 @@ void Unpickler::readList(IValue list_ivalue) {
   } else {
     AT_ERROR("Unknown IValue list kind: ", list_ivalue.tagKind());
   }
-
   stack_.erase(stack_.begin() + start, stack_.end());
 }
 
+// Pop all the list items off of the stack and append them to the list at
+// the corresponding MARK
+void Unpickler::readList(IValue list_ivalue) {
+  TORCH_CHECK(!marks_.empty(), "Parsing error: marks_ is empty");
+  size_t start = marks_.back();
+  marks_.pop_back();
+  readListElements(std::move(list_ivalue), start);
+}
+
 inline bool is_valid_python_id_char(char c) {
   return c == '_' || c == '.' || (c >= '0' && c <= '9') ||
       (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z');
