@@ -9,7 +9,10 @@
 namespace torch::jit::tensorexpr {
 
 // Move the given user of `aten::cat` op to its inputs.
-Node* moveCatAfterUse(Node* cat, Node* user, std::shared_ptr<Graph> subgraph) {
+static Node* moveCatAfterUse(
+    Node* cat,
+    Node* user,
+    std::shared_ptr<Graph> subgraph) {
   // Example IR:
   //   %1 = ...
   //   %2 = ...
@@ -81,7 +84,7 @@ Node* moveCatAfterUse(Node* cat, Node* user, std::shared_ptr<Graph> subgraph) {
   return new_cat;
 }
 
-int numTensorInputs(Node* node) {
+static int numTensorInputs(Node* node) {
   int count = 0;
   for (auto v : node->inputs()) {
     if (v->type()->cast<c10::TensorType>()) {
@@ -94,7 +97,7 @@ int numTensorInputs(Node* node) {
 // Returns true if the given `cat` node promotes types.
 // If the inputs to `cat` are of different types, then the implementation
 // of `cat` is expected to promote type.
-bool doesCatPromoteTypes(Node* node) {
+static bool doesCatPromoteTypes(Node* node) {
   TORCH_INTERNAL_ASSERT(
       node->kind() == aten::cat,
       buildErrorMessage("Graph node is not aten::cat."));
@@ -137,7 +140,7 @@ bool doesCatPromoteTypes(Node* node) {
 //      - When the cat op promote types, the type of inputs to cat after moving
 //        it user needs to reflect the original type. This is currently not
 //        handled. TODO
-void moveCatOpToEnd(Node* cat, std::shared_ptr<Graph> subgraph) {
+static void moveCatOpToEnd(Node* cat, std::shared_ptr<Graph> subgraph) {
   TORCH_INTERNAL_ASSERT(
       cat->kind() == aten::cat,
       buildErrorMessage("Graph node is not aten::cat."));
@@ -159,7 +162,7 @@ void moveCatOpToEnd(Node* cat, std::shared_ptr<Graph> subgraph) {
 
 // Moves the users of `aten::cat` ops to its inputs whenever possible
 // in the given subgraph.
-void moveCatOpsToEnd(std::shared_ptr<Graph> subgraph) {
+static void moveCatOpsToEnd(std::shared_ptr<Graph> subgraph) {
   std::vector<Node*> cat_nodes;
   for (Node* n : subgraph->nodes()) {
     if (n->kind() == aten::cat) {
@@ -181,7 +184,7 @@ bool OptimizeCat(const std::shared_ptr<Graph>& graph) {
 
 void annotateInputShapes(
     const std::shared_ptr<Graph>& graph,
-    const std::vector<c10::optional<at::Tensor>>& example_inputs) {
+    const std::vector<std::optional<at::Tensor>>& example_inputs) {
   TORCH_INTERNAL_ASSERT(
       graph->inputs().size() == example_inputs.size(),
       buildErrorMessage("Given inputs do not match the fuser graph inputs."));
@@ -299,10 +302,10 @@ bool isGraphCompilable(const std::shared_ptr<Graph>& graph) {
   return true;
 }
 
-void fixupTypeInfoForValue(
+static void fixupTypeInfoForValue(
     Value* v,
-    c10::optional<at::ScalarType> scalar_type,
-    c10::optional<at::Device> device) {
+    std::optional<at::ScalarType> scalar_type,
+    std::optional<at::Device> device) {
   Node* n = v->node();
   auto const& t = v->type();
   if (t->kind() != TypeKind::TensorType) {
@@ -336,8 +339,8 @@ void fixupTypeInfoForValue(
   v->setType(new_tt);
 }
 
-c10::optional<at::ScalarType> inferScalarType(Node* n) {
-  c10::optional<at::ScalarType> scalar_type;
+static std::optional<at::ScalarType> inferScalarType(Node* n) {
+  std::optional<at::ScalarType> scalar_type;
   for (auto v : n->inputs()) {
     auto const& t = v->type();
     if (t->kind() == TypeKind::TensorType) {
@@ -355,8 +358,8 @@ c10::optional<at::ScalarType> inferScalarType(Node* n) {
   return scalar_type;
 }
 
-c10::optional<at::Device> inferDevice(Node* n) {
-  c10::optional<at::Device> device;
+static std::optional<at::Device> inferDevice(Node* n) {
+  std::optional<at::Device> device;
   for (auto v : n->inputs()) {
     auto const& t = v->type();
     if (t->kind() == TypeKind::TensorType) {
@@ -391,8 +394,8 @@ void fixupMissingShapeInfo(const std::shared_ptr<Graph>& graph) {
   }
 
   for (auto n : graph->nodes()) {
-    c10::optional<at::ScalarType> scalar_type = inferScalarType(n);
-    c10::optional<at::Device> device = inferDevice(n);
+    std::optional<at::ScalarType> scalar_type = inferScalarType(n);
+    std::optional<at::Device> device = inferDevice(n);
 
     for (auto v : n->outputs()) {
       fixupTypeInfoForValue(v, scalar_type, device);
@@ -420,14 +423,14 @@ std::shared_ptr<Graph> replaceListOutputWithTuple(
   return graph;
 }
 
-bool trimGraphOnce(const std::shared_ptr<Graph>& graph) {
+static bool trimGraphOnce(const std::shared_ptr<Graph>& graph) {
   Node* ret = graph->return_node();
   std::unordered_set<Value*> graph_inputs(
       graph->inputs().begin(), graph->inputs().end());
   std::unordered_set<Value*> outputs(
       graph->outputs().begin(), graph->outputs().end());
   bool changed = false;
-  for (int idx = 0; idx < ret->inputs().size(); idx++) {
+  for (size_t idx = 0; idx < ret->inputs().size(); idx++) {
     auto v = ret->inputs()[idx];
     if (graph_inputs.count(v)) {
       continue;
@@ -451,7 +454,8 @@ bool trimGraphOnce(const std::shared_ptr<Graph>& graph) {
   return changed;
 }
 
-std::shared_ptr<Graph> dequantizeResults(const std::shared_ptr<Graph>& graph) {
+static std::shared_ptr<Graph> dequantizeResults(
+    const std::shared_ptr<Graph>& graph) {
   for (auto v : graph->outputs()) {
     auto& t = v->type();
     if (t->kind() == TypeKind::TensorType) {
