@@ -1,6 +1,7 @@
 #pragma once
 
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <c10/util/string_utils.h>
@@ -72,14 +73,14 @@ class TORCH_API Cast : public ExprNode<Cast> {
   }
 
   void set_src_value(ExprPtr src_value) {
-    src_value_ = src_value;
+    src_value_ = std::move(src_value);
   }
 
   static ExprHandle make(Dtype dtype, const ExprHandle& src_value) {
     return ExprHandle(alloc<Cast>(dtype, src_value.node()));
   }
   Cast(Dtype dtype, ExprPtr src_value)
-      : ExprNodeBase(dtype, kCast), src_value_(src_value) {}
+      : ExprNodeBase(dtype, kCast), src_value_(std::move(src_value)) {}
 
   bool isConstant() const override {
     return src_value_->isConstant();
@@ -102,14 +103,14 @@ class TORCH_API BitCast : public ExprNode<BitCast> {
   }
 
   void set_src_value(ExprPtr src_value) {
-    src_value_ = src_value;
+    src_value_ = std::move(src_value);
   }
 
   static ExprHandle make(Dtype dtype, const ExprHandle& src_value) {
     return ExprHandle(alloc<BitCast>(dtype, src_value.node()));
   }
   BitCast(Dtype dtype, ExprPtr src_value)
-      : ExprNodeBase(dtype, kBitCast), src_value_(src_value) {
+      : ExprNodeBase(dtype, kBitCast), src_value_(std::move(src_value)) {
     TORCH_CHECK(src_value_->dtype().byte_size() == dtype.byte_size());
   }
 
@@ -140,11 +141,11 @@ class BinaryOpNode : public ExprNode<Op> {
   }
 
   void set_lhs(ExprPtr lhs) {
-    lhs_ = lhs;
+    lhs_ = std::move(lhs);
   }
 
   void set_rhs(ExprPtr rhs) {
-    rhs_ = rhs;
+    rhs_ = std::move(rhs);
   }
 
   static ExprHandle make(const ExprHandle& lhs, const ExprHandle& rhs) {
@@ -161,15 +162,15 @@ class BinaryOpNode : public ExprNode<Op> {
             // NOLINTNEXTLINE(clang-analyzer-core.CallAndMessage)
             BinaryOpDtype(lhs_v->dtype(), rhs_v->dtype(), ret_type),
             expr_type),
-        lhs_(CastIfNeeded(lhs_v, ExprNode<Op>::dtype())),
-        rhs_(CastIfNeeded(rhs_v, ExprNode<Op>::dtype())) {}
+        lhs_(CastIfNeeded(std::move(lhs_v), ExprNode<Op>::dtype())),
+        rhs_(CastIfNeeded(std::move(rhs_v), ExprNode<Op>::dtype())) {}
 
  private:
   static ExprPtr CastIfNeeded(ExprPtr expr, Dtype dst_dtype) {
     if (expr->dtype() == dst_dtype) {
       return expr;
     }
-    return Cast::make(dst_dtype, ExprHandle(expr)).node();
+    return Cast::make(dst_dtype, ExprHandle(std::move(expr))).node();
   }
 
   ExprPtr lhs_;
@@ -184,34 +185,39 @@ bool bin_op_deducer(...);
 
 class TORCH_API Add : public BinaryOpNode<Add> {
  public:
-  Add(ExprPtr lhs, ExprPtr rhs) : BinaryOpNode(lhs, rhs, IRNodeType::kAdd) {}
+  Add(ExprPtr lhs, ExprPtr rhs)
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kAdd) {}
 };
 
 class TORCH_API Sub : public BinaryOpNode<Sub> {
  public:
-  Sub(ExprPtr lhs, ExprPtr rhs) : BinaryOpNode(lhs, rhs, IRNodeType::kSub) {}
+  Sub(ExprPtr lhs, ExprPtr rhs)
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kSub) {}
 };
 
 class TORCH_API Mul : public BinaryOpNode<Mul> {
  public:
-  Mul(ExprPtr lhs, ExprPtr rhs) : BinaryOpNode(lhs, rhs, IRNodeType::kMul) {}
+  Mul(ExprPtr lhs, ExprPtr rhs)
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kMul) {}
 };
 
 class TORCH_API Div : public BinaryOpNode<Div> {
  public:
-  Div(ExprPtr lhs, ExprPtr rhs) : BinaryOpNode(lhs, rhs, IRNodeType::kDiv) {}
+  Div(ExprPtr lhs, ExprPtr rhs)
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kDiv) {}
 };
 
 class TORCH_API Mod : public BinaryOpNode<Mod> {
  public:
-  Mod(ExprPtr lhs, ExprPtr rhs) : BinaryOpNode(lhs, rhs, IRNodeType::kMod) {}
+  Mod(ExprPtr lhs, ExprPtr rhs)
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kMod) {}
 };
 
 template <typename Op>
 class BitwiseOpNode : public BinaryOpNode<Op> {
  public:
   BitwiseOpNode(ExprPtr lhs, ExprPtr rhs, IRNodeType type)
-      : BinaryOpNode<Op>(lhs, rhs, type) {}
+      : BinaryOpNode<Op>(std::move(lhs), std::move(rhs), type) {}
 
   static ExprHandle make(const ExprHandle& lhs, const ExprHandle& rhs) {
     if (!lhs.dtype().is_integral()) {
@@ -226,29 +232,32 @@ class BitwiseOpNode : public BinaryOpNode<Op> {
 
 class TORCH_API And : public BitwiseOpNode<And> {
  public:
-  And(ExprPtr lhs, ExprPtr rhs) : BitwiseOpNode(lhs, rhs, IRNodeType::kAnd) {}
+  And(ExprPtr lhs, ExprPtr rhs)
+      : BitwiseOpNode(std::move(lhs), std::move(rhs), IRNodeType::kAnd) {}
 };
 
 class TORCH_API Or : public BitwiseOpNode<Or> {
  public:
-  Or(ExprPtr lhs, ExprPtr rhs) : BitwiseOpNode(lhs, rhs, IRNodeType::kOr) {}
+  Or(ExprPtr lhs, ExprPtr rhs)
+      : BitwiseOpNode(std::move(lhs), std::move(rhs), IRNodeType::kOr) {}
 };
 
 class TORCH_API Xor : public BitwiseOpNode<Xor> {
  public:
-  Xor(ExprPtr lhs, ExprPtr rhs) : BitwiseOpNode(lhs, rhs, IRNodeType::kXor) {}
+  Xor(ExprPtr lhs, ExprPtr rhs)
+      : BitwiseOpNode(std::move(lhs), std::move(rhs), IRNodeType::kXor) {}
 };
 
 class TORCH_API Lshift : public BitwiseOpNode<Lshift> {
  public:
   Lshift(ExprPtr lhs, ExprPtr rhs)
-      : BitwiseOpNode(lhs, rhs, IRNodeType::kLshift) {}
+      : BitwiseOpNode(std::move(lhs), std::move(rhs), IRNodeType::kLshift) {}
 };
 
 class TORCH_API Rshift : public BitwiseOpNode<Rshift> {
  public:
   Rshift(ExprPtr lhs, ExprPtr rhs)
-      : BitwiseOpNode(lhs, rhs, IRNodeType::kRshift) {}
+      : BitwiseOpNode(std::move(lhs), std::move(rhs), IRNodeType::kRshift) {}
 };
 
 // TODO: add TORCH_API
@@ -259,7 +268,7 @@ class Max : public BinaryOpNode<Max> {
 
  public:
   Max(ExprPtr lhs, ExprPtr rhs, bool propagate_nans)
-      : BinaryOpNode(lhs, rhs, IRNodeType::kMax),
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kMax),
         propagate_nans_(propagate_nans) {}
 
   bool propagate_nans() const {
@@ -283,7 +292,7 @@ class Min : public BinaryOpNode<Min> {
 
  public:
   Min(ExprPtr lhs, ExprPtr rhs, bool propagate_nans)
-      : BinaryOpNode(lhs, rhs, IRNodeType::kMin),
+      : BinaryOpNode(std::move(lhs), std::move(rhs), IRNodeType::kMin),
         propagate_nans_(propagate_nans) {}
 
   bool propagate_nans() const {
@@ -343,16 +352,16 @@ ExprPtr getImmediateByType(Dtype dtype, T initialVal) {
 }
 
 template <typename T>
-ExprPtr immLike(ExprPtr e, T v) {
+ExprPtr immLike(const ExprPtr& e, T v) {
   return getImmediateByType<T>(e->dtype(), v);
 }
 
 template <typename T>
-ExprPtr immLike(ExprHandle e, T v) {
+ExprPtr immLike(const ExprHandle& e, T v) {
   return immLike(e.node(), v);
 }
 
-inline c10::optional<int64_t> intValue(ExprPtr e) {
+inline std::optional<int64_t> intValue(const ExprPtr& e) {
 #define TYPE_CASE(Type, Name)      \
   if (auto v = to<Name##Imm>(e)) { \
     return v->value();             \
@@ -362,12 +371,12 @@ inline c10::optional<int64_t> intValue(ExprPtr e) {
   return c10::nullopt;
 }
 
-inline c10::optional<int64_t> intValue(ExprHandle e) {
+inline std::optional<int64_t> intValue(const ExprHandle& e) {
   return intValue(e.node());
 }
 
 template <typename T>
-T immediateAs(ExprPtr e) {
+T immediateAs(const ExprPtr& e) {
 #define TYPE_CASE(Type, Name)                \
   if (Name##ImmPtr imm = to<Name##Imm>(e)) { \
     return imm->value();                     \
@@ -379,12 +388,12 @@ T immediateAs(ExprPtr e) {
 }
 
 template <typename T>
-T immediateAs(ExprHandle e) {
+T immediateAs(const ExprHandle& e) {
   return immediateAs<T>(e.node());
 }
 
 template <typename T>
-bool immediateEquals(ExprPtr e, T val) {
+bool immediateEquals(const ExprPtr& e, T val) {
 #define TYPE_CASE(Type, Name)                \
   if (Name##ImmPtr imm = to<Name##Imm>(e)) { \
     return imm->value() == val;              \
@@ -395,11 +404,11 @@ bool immediateEquals(ExprPtr e, T val) {
   return false;
 }
 
-TORCH_API bool immediateIsNegative(ExprPtr e);
+TORCH_API bool immediateIsNegative(const ExprPtr& e);
 
-TORCH_API bool immediateIsPositive(ExprPtr e);
+TORCH_API bool immediateIsPositive(const ExprPtr& e);
 
-TORCH_API bool immediateIsZero(ExprPtr e);
+TORCH_API bool immediateIsZero(const ExprPtr& e);
 
 // Represents a ramp vector node:
 //     [base, base + 1 * stride, ... , base + (lanes - 1) * stride]
@@ -413,11 +422,11 @@ class TORCH_API Ramp : public ExprNode<Ramp> {
   }
 
   void set_base(ExprPtr base) {
-    base_ = base;
+    base_ = std::move(base);
   }
 
   void set_stride(ExprPtr stride) {
-    stride_ = stride;
+    stride_ = std::move(stride);
   }
 
   static ExprHandle make(
@@ -435,8 +444,8 @@ class TORCH_API Ramp : public ExprNode<Ramp> {
 
   Ramp(ExprPtr base, ExprPtr stride, int lanes)
       : ExprNodeBase(Dtype(base->dtype(), lanes)),
-        base_(base),
-        stride_(stride),
+        base_(std::move(base)),
+        stride_(std::move(stride)),
         lanes_(lanes) {}
 
  private:
@@ -462,11 +471,11 @@ class TORCH_API Load : public ExprNode<Load> {
   }
 
   void set_buf(BufPtr buf) {
-    buf_ = buf;
+    buf_ = std::move(buf);
   }
 
   void set_indices(std::vector<ExprPtr> indices) {
-    indices_ = indices;
+    indices_ = std::move(indices);
   }
 
   static ExprHandle make(
@@ -492,7 +501,7 @@ class TORCH_API Broadcast : public ExprNode<Broadcast> {
   }
 
   void set_value(ExprPtr value) {
-    value_ = value;
+    value_ = std::move(value);
   }
 
   int lanes() const {
@@ -503,7 +512,7 @@ class TORCH_API Broadcast : public ExprNode<Broadcast> {
   }
   Broadcast(ExprPtr value, int lanes)
       : ExprNodeBase(Dtype(value->dtype(), lanes)),
-        value_(value),
+        value_(std::move(value)),
         lanes_(lanes) {}
 
  private:
@@ -528,15 +537,15 @@ class TORCH_API IfThenElse : public ExprNode<IfThenElse> {
   }
 
   void set_condition(ExprPtr condition) {
-    condition_ = condition;
+    condition_ = std::move(condition);
   }
 
   void set_true_value(ExprPtr true_value) {
-    true_ = true_value;
+    true_ = std::move(true_value);
   }
 
   void set_false_value(ExprPtr false_value) {
-    false_ = false_value;
+    false_ = std::move(false_value);
   }
 
   static ExprHandle make(
@@ -556,7 +565,10 @@ class TORCH_API IfThenElse : public ExprNode<IfThenElse> {
   }
 
   IfThenElse(ExprPtr c, ExprPtr t, ExprPtr f)
-      : ExprNodeBase(t->dtype()), condition_(c), true_(t), false_(f) {}
+      : ExprNodeBase(t->dtype()),
+        condition_(std::move(c)),
+        true_(std::move(t)),
+        false_(std::move(f)) {}
 
  private:
   ExprPtr condition_;
@@ -583,19 +595,19 @@ class TORCH_API CompareSelect : public ExprNode<CompareSelect> {
   }
 
   void set_lhs(ExprPtr lhs) {
-    lhs_ = lhs;
+    lhs_ = std::move(lhs);
   }
 
   void set_rhs(ExprPtr rhs) {
-    rhs_ = rhs;
+    rhs_ = std::move(rhs);
   }
 
   void set_ret_val1(ExprPtr ret_val1) {
-    ret_val1_ = ret_val1;
+    ret_val1_ = std::move(ret_val1);
   }
 
   void set_ret_val2(ExprPtr ret_val2) {
-    ret_val2_ = ret_val2;
+    ret_val2_ = std::move(ret_val2);
   }
 
   CompareSelectBias bias() const {
@@ -646,10 +658,10 @@ class TORCH_API CompareSelect : public ExprNode<CompareSelect> {
       CompareSelectOperation cmp_op,
       CompareSelectBias bias = kUnbiased)
       : ExprNodeBase(ret_val1->dtype()),
-        lhs_(lhs),
-        rhs_(rhs),
-        ret_val1_(ret_val1),
-        ret_val2_(ret_val2),
+        lhs_(std::move(lhs)),
+        rhs_(std::move(rhs)),
+        ret_val1_(std::move(ret_val1)),
+        ret_val2_(std::move(ret_val2)),
         compare_op_(cmp_op),
         bias_(bias) {}
 
@@ -660,8 +672,8 @@ class TORCH_API CompareSelect : public ExprNode<CompareSelect> {
       CompareSelectOperation cmp_op,
       CompareSelectBias bias = kUnbiased)
       : ExprNodeBase(kInt),
-        lhs_(lhs),
-        rhs_(rhs),
+        lhs_(std::move(lhs)),
+        rhs_(std::move(rhs)),
         ret_val1_(alloc<IntImm>(1)),
         ret_val2_(alloc<IntImm>(0)),
         compare_op_(cmp_op),
@@ -832,7 +844,7 @@ class TORCH_API Intrinsics : public ExprNode<Intrinsics> {
   // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
   Intrinsics(IntrinsicsOp op_type, ExprPtr v1)
       : ExprNodeBase(IntrinsicsDtype(op_type, v1->dtype())),
-        params_({v1}),
+        params_({std::move(v1)}),
         op_type_(op_type) {
     if (OpArgCount(op_type) != 1) {
       throw malformed_input("bad arg count in Intrinsics");
@@ -842,7 +854,7 @@ class TORCH_API Intrinsics : public ExprNode<Intrinsics> {
   // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
   Intrinsics(IntrinsicsOp op_type, ExprPtr v1, ExprPtr v2)
       : ExprNodeBase(IntrinsicsDtype(op_type, v1->dtype(), v2->dtype())),
-        params_({v1, v2}),
+        params_({std::move(v1), std::move(v2)}),
         op_type_(op_type) {
     if (OpArgCount(op_type) != 2) {
       throw malformed_input("bad arg count in Intrinsics");
