@@ -55,7 +55,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         auto sub_lambda = [](const ExprHandle& lhs, const ExprHandle& rhs) {
           // NB: sub isn't supported on boolean, no need to promote to integer.
@@ -86,7 +86,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_mul",
@@ -108,7 +108,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,                             \
          const std::vector<ExprHandle>& outputShape,                      \
          const std::vector<ExprHandle>& outputStrides,                    \
-         const c10::optional<ScalarType>& outputType,                     \
+         const std::optional<ScalarType>& outputType,                     \
          at::Device device) {                                             \
         return computeScalar(                                             \
             "aten_#op_name",                                              \
@@ -131,7 +131,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeScalar(
             "aten_div",
@@ -155,7 +155,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,                             \
          const std::vector<ExprHandle>& outputShape,                      \
          const std::vector<ExprHandle>& outputStrides,                    \
-         const c10::optional<ScalarType>& outputType,                     \
+         const std::optional<ScalarType>& outputType,                     \
          at::Device device) {                                             \
         return computeScalar(                                             \
             "aten_#op_name",                                              \
@@ -179,7 +179,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,                             \
          const std::vector<ExprHandle>& outputShape,                      \
          const std::vector<ExprHandle>& outputStrides,                    \
-         const c10::optional<ScalarType>& outputType,                     \
+         const std::optional<ScalarType>& outputType,                     \
          at::Device device) {                                             \
         return computeScalar(                                             \
             "aten_#op_name",                                              \
@@ -204,7 +204,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,                             \
          const std::vector<ExprHandle>& outputShape,                      \
          const std::vector<ExprHandle>& outputStrides,                    \
-         const c10::optional<ScalarType>& outputType,                     \
+         const std::optional<ScalarType>& outputType,                     \
          at::Device device) {                                             \
         return computeScalar(                                             \
             "aten_#op_name",                                              \
@@ -225,7 +225,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_div",
@@ -245,7 +245,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_and",
@@ -264,7 +264,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_or",
@@ -283,7 +283,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_xor",
@@ -302,7 +302,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_lshift",
@@ -321,7 +321,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_rshift",
@@ -340,7 +340,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_eq",
@@ -359,7 +359,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_ne",
@@ -378,7 +378,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_ge",
@@ -397,7 +397,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_gt",
@@ -416,7 +416,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_le",
@@ -435,7 +435,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_lt",
@@ -453,7 +453,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_min",
@@ -471,7 +471,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_max",
@@ -490,7 +490,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeThreeOperand(
             "aten_masked_fill",
@@ -513,15 +513,15 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         bool noMin = false;
         bool noMax = false;
-        if (c10::get_if<ArgNone>(&inputs[1])) {
+        if (std::get_if<ArgNone>(&inputs[1])) {
           noMin = true;
         }
 
-        if (c10::get_if<ArgNone>(&inputs[2])) {
+        if (std::get_if<ArgNone>(&inputs[2])) {
           noMax = true;
         }
 
@@ -561,7 +561,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeFourOperand(
             "aten_addcmul",
@@ -580,10 +580,10 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         // check if the activation is quantized
-        const BufHandle& x = c10::get<BufHandle>(inputs[0]);
+        const BufHandle& x = std::get<BufHandle>(inputs[0]);
         if (x.node()->qscale()) {
           return computeQuantizedSigmoidExternalCall(
               inputs, outputShape, outputStrides, outputType, device);
@@ -604,7 +604,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_silu",
@@ -620,7 +620,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_reciprocal",
@@ -636,7 +636,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_neg",
@@ -652,7 +652,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_isnan",
@@ -673,9 +673,9 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
-        auto A = c10::get<BufHandle>(inputs[0]);
+        auto A = std::get<BufHandle>(inputs[0]);
         if (A.node()->qscale()) {
           return computeQuantizedRelu(
               inputs, outputShape, outputStrides, outputType, device);
@@ -697,7 +697,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_leaky_relu",
@@ -719,7 +719,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_relu6",
@@ -739,9 +739,9 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
-        const auto& kApproximate = c10::get<std::string>(inputs[1]);
+        const auto& kApproximate = std::get<std::string>(inputs[1]);
         std::vector<ArgValue> operands = {inputs.front()};
         if (at::native::get_gelutype_enum(kApproximate) ==
             at::native::GeluType::Tanh) {
@@ -787,7 +787,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_log",
@@ -805,7 +805,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_log10",
@@ -823,7 +823,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_log1p",
@@ -841,7 +841,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_log2",
@@ -859,7 +859,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_exp",
@@ -877,7 +877,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_expm1",
@@ -895,7 +895,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_erf",
@@ -913,7 +913,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_erfc",
@@ -931,7 +931,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_cos",
@@ -949,7 +949,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_sin",
@@ -967,7 +967,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_tan",
@@ -985,9 +985,9 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
-        const BufHandle& rhs = c10::get<BufHandle>(inputs[1]);
+        const BufHandle& rhs = std::get<BufHandle>(inputs[1]);
         auto dtype = rhs.dtype();
         return computeOneOperand(
             "aten_type_as",
@@ -1005,7 +1005,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_pow",
@@ -1050,7 +1050,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_fmod",
@@ -1069,7 +1069,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeThreeOperand(
             "aten_lerp",
@@ -1089,7 +1089,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         auto imodImpl = [](const ExprHandle& lhs, const ExprHandle& rhs) {
           return Mod::make(lhs, rhs);
@@ -1137,7 +1137,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_acos",
@@ -1155,7 +1155,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_asin",
@@ -1173,7 +1173,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_cosh",
@@ -1191,7 +1191,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_sinh",
@@ -1209,7 +1209,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_atan",
@@ -1227,7 +1227,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_atan2",
@@ -1247,7 +1247,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_tanh",
@@ -1265,7 +1265,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeThreeOperand(
             "aten_hardtanh",
@@ -1286,7 +1286,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeThreeOperand(
             "aten_softplus",
@@ -1314,7 +1314,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_mish",
@@ -1333,7 +1333,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeFourOperand(
             "aten_elu",
@@ -1366,7 +1366,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_hardsigmoid",
@@ -1387,7 +1387,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_hardswish",
@@ -1410,7 +1410,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTwoOperand(
             "aten_hardshrink",
@@ -1433,7 +1433,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_sqrt",
@@ -1451,7 +1451,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_rsqrt",
@@ -1469,7 +1469,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_abs",
@@ -1488,7 +1488,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) { return computeSign(inputs, outputShape); });
 
   RegisterNNCLoweringsFunction aten_ceil(
@@ -1496,7 +1496,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_ceil",
@@ -1512,7 +1512,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_floor",
@@ -1528,7 +1528,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_round",
@@ -1544,7 +1544,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_trunc",
@@ -1560,7 +1560,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_cast_float",
@@ -1582,7 +1582,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         // see handling of aten::to in tensorexpr_fuser.cpp for why we only
         // need to handle the first input
@@ -1604,7 +1604,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeThreeOperand(
             "aten_threshold",
@@ -1628,7 +1628,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeConditionWithTwoOperand(
             "aten_where",
@@ -1646,7 +1646,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_frac",
@@ -1666,7 +1666,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeOneOperand(
             "aten_lgamma",
@@ -1684,7 +1684,7 @@ int nnc_lowerings_lazy_registration() {
   //     {"aten::rand_like"},
   //     [](const std::vector<ArgValue>& inputs,
   //        const std::vector<ExprHandle>& outputShape,
-  //        const c10::optional<ScalarType>& outputType,
+  //        const std::optional<ScalarType>& outputType,
   //        at::Device device) {
   //       return computeOneOperand(
   //           "aten_rand_like",
@@ -1701,14 +1701,14 @@ int nnc_lowerings_lazy_registration() {
   //     {"aten::slice"},
   //     [](const std::vector<ArgValue>& inputs,
   //        const std::vector<ExprHandle>& outputShape,
-  //        const c10::optional<ScalarType>& outputType,
+  //        const std::optional<ScalarType>& outputType,
   //        at::Device device) {
   //       return Compute(
   //           "aten_slice",
   //           outputShape,
   //           [&](const std::vector<VarHandle>& axes) {
   //             int64_t dim =
-  //                 at::maybe_wrap_dim(c10::get<int64_t>(inputs[1]),
+  //                 at::maybe_wrap_dim(std::get<int64_t>(inputs[1]),
   //                 axes.size());
   //             ExprHandle start = constant(inputs[2]);
   //             ExprHandle stride = constant(inputs[4]);
@@ -1723,14 +1723,14 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return Compute(
             "aten_unsqueeze",
             outputShape,
             outputStrides,
             [&](const std::vector<VarHandle>& axes) {
-              int64_t dim = c10::get<int64_t>(inputs[1]);
+              int64_t dim = std::get<int64_t>(inputs[1]);
               if (dim < 0) {
                 if (axes.empty()) {
                   throw malformed_input("axes are zero handling unsqueeze");
@@ -1749,7 +1749,7 @@ int nnc_lowerings_lazy_registration() {
                 }
               }
 
-              return broadcast(c10::get<BufHandle>(inputs[0]), indices);
+              return broadcast(std::get<BufHandle>(inputs[0]), indices);
             });
       });
   RegisterNNCLoweringsFunction aten_t(
@@ -1757,7 +1757,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeTranspose(
             {inputs[0], (int64_t)1, (int64_t)0},
@@ -1774,9 +1774,9 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
-        auto A = c10::get<BufHandle>(inputs[0]);
+        auto A = std::get<BufHandle>(inputs[0]);
         // Trivial case of 0-dim tensors: just a copy of the input
         if (A.ndim() == 0) {
           auto tensor = Compute(
@@ -1793,7 +1793,7 @@ int nnc_lowerings_lazy_registration() {
           }
           return tensor;
         }
-        auto permute_dims = c10::get<IntList>(inputs[1]);
+        auto permute_dims = std::get<IntList>(inputs[1]);
         auto tensor = Compute(
             "aten_permute",
             outputShape,
@@ -1848,7 +1848,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeSoftmax(inputs, outputShape, outputStrides, false);
       });
@@ -1858,7 +1858,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         return computeSoftmax(inputs, outputShape, outputStrides, true);
       });
@@ -1892,7 +1892,7 @@ int nnc_lowerings_lazy_registration() {
       [](const std::vector<ArgValue>& inputs,
          const std::vector<ExprHandle>& outputShape,
          const std::vector<ExprHandle>& outputStrides,
-         const c10::optional<ScalarType>& outputType,
+         const std::optional<ScalarType>& outputType,
          at::Device device) {
         auto add_lambda = [](const ExprHandle& lhs, const ExprHandle& rhs) {
           return boolToInteger(lhs) + boolToInteger(rhs);
