@@ -1,8 +1,7 @@
 #include <torch/csrc/jit/tensorexpr/registerizer.h>
+#include <iostream>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 namespace registerizer {
 
 // AccessInfo
@@ -53,10 +52,10 @@ void AccessInfo::merge(const std::shared_ptr<AccessInfo>& other) {
           "Expected ranks to match in registerizer in the fuser."));
 
   last_usage_ = other->last_usage();
-  for (auto s : other->stores()) {
+  for (const auto& s : other->stores()) {
     stores_.push_back(s);
   }
-  for (auto l : other->loads()) {
+  for (const auto& l : other->loads()) {
     loads_.push_back(l);
   }
 
@@ -99,7 +98,7 @@ bool AccessInfo::overlaps(const std::shared_ptr<AccessInfo>& other) {
 
 bool AccessInfo::dependsOnVar(VarPtr v) {
   VarFinder vf;
-  for (auto i : indices_) {
+  for (const auto& i : indices_) {
     i->accept(&vf);
   }
 
@@ -117,10 +116,10 @@ std::shared_ptr<AccessInfo> AccessInfo::cloneWithHiddenInfo(
   newInfo->firstUsageOverlapped_ = orig->firstUsageOverlapped_;
   newInfo->store_cost_ = orig->store_cost_;
   newInfo->load_cost_ = orig->load_cost_;
-  for (auto s : orig->stores_) {
+  for (const auto& s : orig->stores_) {
     newInfo->stores_.push_back(s);
   }
-  for (auto s : orig->loads_) {
+  for (const auto& s : orig->loads_) {
     newInfo->loads_.push_back(s);
   }
 
@@ -131,7 +130,7 @@ std::shared_ptr<AccessInfo> AccessInfo::cloneWithHiddenInfo(
 
 void AccessInfo::print() const {
   std::cout << "Access: " << *buf_ << "{";
-  for (auto i : indices_) {
+  for (const auto& i : indices_) {
     std::cout << *i << " ";
   }
   std::cout << "} stores: " << stores_.size() << " (" << *store_cost_ << ") -";
@@ -237,7 +236,7 @@ void RegisterizerAnalysis::visit(ForPtr v) {
       bool closed = false;
       // If this access depends on a locally scoped variable, it cannot be
       // hosted out of the loop.
-      for (auto v : currentScope_->localVars()) {
+      for (const auto& v : currentScope_->localVars()) {
         if (candidate->dependsOnVar(v)) {
           closeAccessIntoScope(candidate, currentScope_);
           closed = true;
@@ -364,7 +363,7 @@ void RegisterizerAnalysis::visit(BlockPtr v) {
 
   stmtStack_.push_front(v);
 
-  for (auto s : *v) {
+  for (const auto& s : *v) {
     s->accept(this);
     if (currentScope_->block() != v) {
       // merge the inner block's accesses into this Block's accesses.
@@ -396,7 +395,7 @@ void RegisterizerAnalysis::visit(StorePtr v) {
 
   // hash the Store:
   SimplifierHashType accessHash = hasher_.hash(v->buf());
-  for (auto i : v->indices()) {
+  for (const auto& i : v->indices()) {
     accessHash = hasher_.hash_combine(accessHash, i);
   }
 
@@ -445,7 +444,7 @@ void RegisterizerAnalysis::visit(LoadPtr v) {
   }
   // hash the Load:
   SimplifierHashType accessHash = hasher_.hash(v->buf());
-  for (auto i : v->indices()) {
+  for (const auto& i : v->indices()) {
     accessHash = hasher_.hash_combine(accessHash, i);
   }
 
@@ -487,7 +486,7 @@ void RegisterizerAnalysis::visit(LoadPtr v) {
 }
 
 // Loop and Conditional scopes are different in that it may or may not be
-// possible to hoist the intializer of a scalar variable outside the block
+// possible to hoist the initializer of a scalar variable outside the block
 // depending on if we can tell that the Buffer access is valid outside. This is
 // tricky because the access that demonstrates this may be later in the tree and
 // we haven't encountered it yet.
@@ -525,7 +524,7 @@ void RegisterizerAnalysis::mergeHiddenScope(bool allowClosed) {
 void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
   auto parent = currentScope_->parent();
 
-  // copy across current closed accceses, merging / closing as necessary
+  // copy across current closed accesses, merging / closing as necessary
   for (auto& candidate : currentScope_->closedAccesses()) {
     auto& parentAccesses = parent->getAccessMapByBuf(candidate->buf());
 
@@ -617,7 +616,7 @@ void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
 
       // If this access depends on a locally scoped variable, it cannot be
       // lifted out of the loop.
-      for (auto v : currentScope_->localVars()) {
+      for (const auto& v : currentScope_->localVars()) {
         if (candidate->dependsOnVar(v)) {
           closeAccessIntoScope(candidate, parent);
           handled = true;
@@ -664,7 +663,7 @@ ExprPtr RegisterizerReplacer::mutate(LoadPtr v) {
 
 StmtPtr RegisterizerReplacer::mutate(StorePtr v) {
   if (eliminatedIntializers_.count(v) != 0) {
-    // This store is the intializer for a scalar var that is already inserted.
+    // This store is the initializer for a scalar var that is already inserted.
     return nullptr;
   }
 
@@ -688,7 +687,7 @@ StmtPtr RegisterizerReplacer::mutate(BlockPtr v) {
   auto& scope = parentToAccesses_[v];
 
   std::vector<StmtPtr> stmts;
-  for (StmtPtr stmt : v->stmts()) {
+  for (const StmtPtr& stmt : v->stmts()) {
     {
       // Insert the initializer for any Scalars scoped to this block.
       auto it = scope.initializerPoints_.find(stmt);
@@ -743,7 +742,7 @@ void RegisterizerReplacer::buildReplacements() {
         alloc<Buf>(v, std::vector<ExprPtr>({}), info->buf()->dtype());
 
     bool first = true;
-    for (auto s : info->stores()) {
+    for (const auto& s : info->stores()) {
       if (first && info->first_usage() == s && !info->firstUsageOverlapped()) {
         info->replacement().initializer = alloc<Let>(v, s->value());
         eliminatedIntializers_.insert(s);
@@ -754,7 +753,7 @@ void RegisterizerReplacer::buildReplacements() {
       first = false;
     }
 
-    for (auto s : info->loads()) {
+    for (const auto& s : info->loads()) {
       loadToAccess_[s] = info;
     }
 
@@ -795,6 +794,4 @@ StmtPtr registerize(StmtPtr s) {
   return s;
 }
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
