@@ -30,7 +30,7 @@ static std::unordered_map<std::string, ParameterType> type_map = {
     {"double", ParameterType::DOUBLE},
     {"complex", ParameterType::COMPLEX},
     {"TensorList", ParameterType::TENSOR_LIST},
-    {"c10::List<c10::optional<Tensor>>", ParameterType::TENSOR_LIST},
+    {"c10::List<::std::optional<Tensor>>", ParameterType::TENSOR_LIST},
     {"IntArrayRef", ParameterType::INT_LIST},
     {"SymIntArrayRef", ParameterType::SYM_INT_LIST},
     {"ArrayRef<double>", ParameterType::FLOAT_LIST},
@@ -43,12 +43,14 @@ static std::unordered_map<std::string, ParameterType> type_map = {
     {"MemoryFormat", ParameterType::MEMORY_FORMAT},
     {"QScheme", ParameterType::QSCHEME},
     {"Device", ParameterType::DEVICE},
+    {"DeviceIndex", ParameterType::INT64},
     {"Stream", ParameterType::STREAM},
     {"std::string", ParameterType::STRING},
     {"c10::string_view", ParameterType::STRING},
     {"Dimname", ParameterType::DIMNAME},
     {"DimnameList", ParameterType::DIMNAME_LIST},
     {"ScalarList", ParameterType::SCALAR_LIST},
+    {"DispatchKeySet", ParameterType::DISPATCH_KEY_SET},
 };
 
 // Default arg name translations for compatibility with NumPy.
@@ -195,7 +197,7 @@ auto handle_torch_function_setter(
 }
 
 // Combines self and args into one tuple.
-auto combine_self_args(PyObject* self, PyObject* args) -> py::tuple {
+static auto combine_self_args(PyObject* self, PyObject* args) -> py::tuple {
   if (args == nullptr) {
     return py::make_tuple(py::handle(self));
   } else if (self == nullptr) {
@@ -233,7 +235,7 @@ auto handle_torch_function(
       torch_api_function.ptr() != nullptr, "torch API function must exist");
   py::tuple args_ = combine_self_args(self, args);
   return handle_torch_function_no_python_arg_parser(
-      {py::handle(self)},
+      {self},
       args_.ptr(),
       kwargs,
       func_name.c_str(),
@@ -257,9 +259,132 @@ static PyObject* get_type_of_overloaded_arg(PyObject* obj_or_type) {
   return (PyObject*)Py_TYPE(obj_or_type);
 }
 
+static py::object dispatch_on_subclass(
+    PyObject* args,
+    PyObject* kwargs,
+    at::ArrayRef<PyObject*> overloaded_args,
+    py::tuple py_types,
+    PyObject* torch_api_function,
+    bool is_torch_function,
+    const char* torch_function_name_str,
+    std::optional<c10::impl::TorchDispatchModeKey> maybe_mode_key =
+        c10::nullopt) {
+  py::object ret;
+  for (auto& arg : overloaded_args) {
+    py::object torch_function =
+        PyObject_FastGetAttrString(arg, torch_function_name_str);
+    if (!torch_function) {
+      TORCH_INTERNAL_ASSERT(0);
+    }
+    if (torch_function.ptr() == torch::disabled_torch_dispatch_impl()) {
+      // During __torch_dispatch__, don't dispatch on args with a disabled
+      // torch_dispatch. This code runs before infra modes, so we need to make
+      // sure that infra modes can run first. (In theory, maybe we can rearrange
+      // things so that infra modes are *always* attempted first, and just
+      // return NotImplemented when there are any user subclasses. Maybe that
+      // would fix this problem?)
+      continue;
+    }
+
+    // See https://github.com/pytorch/pytorch/issues/63767
+    if (is_torch_function &&
+        PyObject_FastGetAttrString(torch_function.ptr(), "__self__")
+            .is(py::handle(arg)) &&
+        torch_function.ptr() != torch::disabled_torch_function_impl()) {
+      TORCH_WARN(
+          "Defining your `",
+          torch_function_name_str,
+          "` as a plain method is deprecated ",
+          "and will be an error in future, please define it as a classmethod.");
+    }
+
+    ret = py::reinterpret_steal<py::object>(PyObject_CallFunctionObjArgs(
+        torch_function.ptr(),
+        torch_api_function,
+        py_types.ptr(),
+        args,
+        kwargs,
+        NULL));
+    if (ret.ptr() == nullptr) {
+      throw python_error();
+    }
+    if (ret.ptr() != Py_NotImplemented) {
+      // Return the reference to the result. This also covers the case where
+      // ret is NULL and __torch_function__/__torch_dispatch raised an
+      // exception, which we throw below
+      break;
+    }
+  }
+  return ret;
+}
+
+static std::tuple<py::object, py::object> dispatch_on_mode(
+    PyObject* args,
+    PyObject* kwargs,
+    py::tuple py_types,
+    PyObject* torch_api_function,
+    bool is_torch_function,
+    const char* torch_function_name_str) {
+  // Disable mode on the inside; this makes for a more user-friendly
+  // experience if you try to, e.g., print your tensors.
+  at::optional<torch::overrides::StashTorchFunctionModeGuard> tf_g;
+  at::optional<torch_dispatch_mode::StashTorchDispatchModeGuard> td_g;
+  py::object mode_obj;
+  // NB: We only really need keep the mode_obj live if the function call
+  // fails for error reporting, but whatever, Python refcounts are cheap
+  if (is_torch_function) {
+    tf_g.emplace();
+    mode_obj = py::reinterpret_borrow<py::object>(
+        tf_g->get_cur_mode()->ptr(getPyInterpreter()));
+  } else {
+    td_g.emplace();
+    mode_obj = py::reinterpret_borrow<py::object>(
+        td_g->get_cur_mode()->ptr(getPyInterpreter()));
+  }
+  py::object torch_function =
+      PyObject_FastGetAttrString(mode_obj.ptr(), torch_function_name_str);
+  if (!torch_function) {
+    TORCH_INTERNAL_ASSERT(0);
+  }
+  TORCH_INTERNAL_ASSERT(py_types.ptr() != nullptr);
+  TORCH_INTERNAL_ASSERT(args != nullptr);
+
+  TORCH_CHECK(
+      PyObject_FastGetAttrString(torch_function.ptr(), "__self__").is(mode_obj),
+      "Defining your mode's `",
+      torch_function_name_str,
+      "` as a classmethod is not supported, please make it a plain method");
+
+  // Blegh.  This accidentally works in PyObject_CallFunctionObjArgs below
+  // because the nullptr terminates the argument list ick ick ick.
+  py::object ret;
+  if (kwargs == nullptr) {
+    ret = py::reinterpret_steal<py::object>(PyObject_CallMethod(
+        mode_obj.ptr(),
+        torch_function_name_str,
+        "OOO",
+        torch_api_function,
+        py_types.ptr(),
+        args));
+  } else {
+    ret = py::reinterpret_steal<py::object>(PyObject_CallMethod(
+        mode_obj.ptr(),
+        torch_function_name_str,
+        "OOOO",
+        torch_api_function,
+        py_types.ptr(),
+        args,
+        kwargs));
+  }
+  if (ret.ptr() == nullptr) {
+    throw python_error();
+  }
+  return std::make_tuple(ret, mode_obj);
+}
+
 // See Note: [Overloaded args] for what they hold
 auto handle_torch_function_no_python_arg_parser(
-    at::ArrayRef<py::handle> overloaded_args,
+    at::ArrayRef<PyObject*> overloaded_args,
     PyObject* args,
     PyObject* kwargs,
     const char* func_name,
@@ -283,105 +408,96 @@ auto handle_torch_function_no_python_arg_parser(
   std::vector<py::object> overloaded_types;
   overloaded_types.reserve(overloaded_args.size());
   for (auto& arg : overloaded_args) {
-    overloaded_types.push_back(py::reinterpret_borrow<py::object>(
-        get_type_of_overloaded_arg(arg.ptr())));
+    overloaded_types.push_back(
+        py::reinterpret_borrow<py::object>(get_type_of_overloaded_arg(arg)));
   }
   py::tuple py_types = py::cast(overloaded_types);
   py::object ret;
-  PyObject* mode_obj = nullptr;
+  py::object mode_obj;
 
+  // Step 1: Try to dispatch based on the mode stack, *ignoring* infra
+  // torch_dispatch modes.
   const bool is_torch_function =
       torch_function_name == TorchFunctionName::TorchFunction;
   const auto is_mode_active = [&]() {
-    return is_torch_function ? at::impl::torch_function_mode_enabled()
-                             : c10::impl::dispatch_mode_enabled();
+    return is_torch_function
+        ? at::impl::torch_function_mode_enabled()
+        // Check if any *user* torch_dispatch modes are active (not including
+        // fake and proxy modes, which are special)
+        : c10::impl::dispatch_mode_enabled();
   };
+  // Note [__torch_dispatch__ dispatching order]
+  // The high-level idea motivating the dispatching
+  // order below is that: (1) modes get higher dispatch precedence over
+  // subclasses (2) "user" modes/subclasses get higher dispatch precedence over
+  // "infra" modes/subclasses.
+  //
+  // To give a complete example: let's say we are running torch.compile, with
+  // the following "user" modes and subclasses:
+  //   mode_stack: [ModeA]
+  //   user_args: [MyWrapperSubclassB(torchTensor)]
+
+  // During tracing in AOTAutograd tracing, we use some additional infra modes
+  // and subclasses to perform tracing:
+  //   FunctionalTensorMode, ProxyTorchDispatchMode, FakeTensorMode,
+  //   FunctionalTensor, FakeTensor
+  // The modified mode stack and tracing arguments will look like this:
+  //   mode_stack (user modes): [ModeA]
+  //   mode_stack (infra modes): [
+  //     FunctionalTensorMode, ProxyTorchDispatchMode, FakeTensorMode
+  //   ]
+  //   tracing_args: [
+  //     MyWrapperSubclassB(FunctionalTensor(_to_functional_tensor(FakeTensor)))
+  //   ]
+
+  // And the dispatching order that we want is as follows:
+  // (1) ModeA.__torch_dispatch__ (user modes highest)
+  // (2) MyWrapperSubclassB.__torch_dispatch__ (user subclasses next highest)
+  // (3) FunctionalTensorMode.__torch_dispatch__ (infra modes next highest)
+  // (4) ProxyTorchDispatchMode.__torch_dispatch__ (infra modes next highest)
+  // (5) FakeTensorMode.__torch_dispatch__ (infra modes next highest)
+  // (6) FakeTensor.__torch_fake_dispatch__ (infra subclasses next highest)
+
+  // Why does do FunctionalTensor and FakeTensor even need to be special-cased
+  // in the ordering?
+  // In theory we could remove their __torch_dispatch__, but both of these
+  // subclasses override sizes/strides metadata calls with __torch_dispatch__,
+  // which would mean a mode would be **required** to access their metadata.
 
   if (is_mode_active()) {
-    // Disable mode on the inside; this makes for a more user-friendly
-    // experience if you try to, e.g., print your tensors.
-    at::optional<torch::overrides::StashTorchFunctionModeGuard> tf_g;
-    at::optional<torch_dispatch_mode::StashTorchDispatchModeGuard> td_g;
-    if (is_torch_function) {
-      tf_g.emplace();
-      mode_obj = tf_g->get_cur_mode()->ptr(getPyInterpreter());
-    } else {
-      td_g.emplace();
-      mode_obj = td_g->get_cur_mode()->ptr(getPyInterpreter());
-    }
-    py::object torch_function =
-        PyObject_FastGetAttrString(mode_obj, torch_function_name_str);
-    if (!torch_function) {
-      TORCH_INTERNAL_ASSERT(0);
-    }
-    TORCH_INTERNAL_ASSERT(py_types.ptr() != nullptr);
-    TORCH_INTERNAL_ASSERT(args != nullptr);
-
-    TORCH_CHECK(
-        PyObject_FastGetAttrString(torch_function.ptr(), "__self__")
-            .is(py::reinterpret_borrow<py::object>(mode_obj)),
-        "Defining your mode's `",
-        torch_function_name_str,
-        "` as a classmethod is not supported, please make it a plain method");
-
-    // Blegh.  This accidentally works in PyObject_CallFunctionObjArgs below
-    // because the nullptr terminates the argument list ick ick ick.
-    if (kwargs == nullptr) {
-      ret = py::reinterpret_steal<py::object>(PyObject_CallMethod(
-          mode_obj,
-          torch_function_name_str,
-          "OOO",
-          torch_api_function,
-          py_types.ptr(),
-          args));
-    } else {
-      ret = py::reinterpret_steal<py::object>(PyObject_CallMethod(
-          mode_obj,
-          torch_function_name_str,
-          "OOOO",
-          torch_api_function,
-          py_types.ptr(),
-          args,
-          kwargs));
-    }
-    if (ret.ptr() == nullptr) {
-      throw python_error();
-    }
+    // Step 1: Try to dispatch on any user TorchDispatchModes (including infra
+    // modes, which will always be at the bottom of the mode stack).
+    auto ret_ = dispatch_on_mode(
+        args,
+        kwargs,
+        py_types,
+        torch_api_function,
+        is_torch_function,
+        torch_function_name_str);
+    ret = std::get<0>(ret_);
+    mode_obj = std::get<1>(ret_);
   }
+
+  // Step 2: Try to dispatch based on any user subclasses,
+  // ignoring any subclasses that have a _mode_key field
+  // (corresponding to infra subclasses)
+  // Note: user subclasses should always run *before* infra modes like
+  // proxy/fake. This is handles by having proxy/fake modes return
+  // NotImplemented when they see a user subclass that they don't understand.
   if (ret.ptr() == nullptr || ret.ptr() == Py_NotImplemented) {
-    for (auto& arg : overloaded_args) {
-      py::object torch_function =
-          PyObject_FastGetAttrString(arg.ptr(), torch_function_name_str);
-      if (!torch_function) {
-        TORCH_INTERNAL_ASSERT(0);
-      }
-
-      // See https://github.com/pytorch/pytorch/issues/63767
-      if (PyObject_FastGetAttrString(torch_function.ptr(), "__self__")
-              .is(arg) &&
-          torch_function.ptr() != torch::disabled_torch_function_impl()) {
-        TORCH_WARN(
-            "Defining your `",
-            torch_function_name_str,
-            "` as a plain method is deprecated ",
-            "and will be an error in future, please define it as a classmethod.");
-      }
-
-      ret = py::reinterpret_steal<py::object>(PyObject_CallFunctionObjArgs(
-          torch_function.ptr(),
-          torch_api_function,
-          py_types.ptr(),
-          args,
-          kwargs,
-          NULL));
-      if (ret.ptr() != Py_NotImplemented) {
-        // Return the reference to the result. This also covers the case where
-        // ret is NULL and __torch_function__/__torch_dispatch raised an
-        // exception, which we throw below
-        break;
-      }
+    auto curr_ret = dispatch_on_subclass(
+        args,
+        kwargs,
+        overloaded_args,
+        py_types,
+        torch_api_function,
+        is_torch_function,
+        torch_function_name_str);
+    if (curr_ret.ptr() != nullptr) {
+      ret = curr_ret;
     }
   }
+
   if (ret.ptr() == nullptr) {
     // if an exception occurred in a user's implementation of
     // __torch_function__, throw it
@@ -390,7 +506,7 @@ auto handle_torch_function_no_python_arg_parser(
     // all __torch_function__ implementations in overloaded_args
     // returned NotImplemented, so we raise a TypeError.
     std::stringstream ss;
-    ss << "no implementation found for '";
+    ss << "Multiple dispatch failed for '";
     if (module_name && func_name) {
       ss << module_name << "." << func_name;
     } else {
@@ -398,14 +514,16 @@ auto handle_torch_function_no_python_arg_parser(
       ss << py::str(fn.attr("__module__")) << "."
          << py::str(fn.attr("__name__"));
     }
-    ss << "' on types that implement " << torch_function_name_str << ": [";
-    for (auto& arg : overloaded_args) {
-      ss << py::repr(get_type_of_overloaded_arg(arg.ptr()));
-      if (!arg.is(overloaded_args.back())) {
-        ss << ", ";
-      }
+    ss << "'; all " << torch_function_name_str
+       << " handlers returned NotImplemented:\n\n";
+    if (mode_obj) {
+      ss << "  - mode object " << py::repr(mode_obj) << "\n";
     }
-    ss << "]";
+    for (auto& arg : overloaded_args) {
+      ss << "  - tensor subclass " << py::repr(get_type_of_overloaded_arg(arg))
+         << "\n";
+    }
+    ss << "\nFor more information, try re-running with TORCH_LOGS=not_implemented";
     const std::string& tmp = ss.str();
     PyErr_SetString(PyExc_TypeError, tmp.c_str());
     throw python_error();
@@ -423,21 +541,13 @@ auto handle_torch_function(
     const char* func_name_override) -> PyObject* {
   py::object torch_api_function = PyObject_FastGetAttrString(
       torch_api,
-      (char*)(func_name_override ? func_name_override : r.get_func_name().c_str()));
+      (char*)(func_name_override ? func_name_override
+                                 : r.get_func_name().c_str()));
   TORCH_INTERNAL_ASSERT(
       torch_api_function.ptr() != nullptr, "torch API function must exist");
-  py::object ret;
   py::tuple args_ = combine_self_args(self, args);
-  // overloaded_args already all have unique types
-  std::vector<py::object> overloaded_types;
-  overloaded_types.reserve(r.signature.overloaded_args.size());
-  for (auto& arg : r.signature.overloaded_args) {
-    overloaded_types.push_back(
-        py::reinterpret_borrow<py::object>((PyObject*)Py_TYPE(arg.ptr())));
-  }
-  py::tuple py_types = py::cast(overloaded_types);
   return handle_torch_function_no_python_arg_parser(
-      r.signature.overloaded_args,
+      r.overloaded_args,
       args_.ptr(),
       kwargs,
       r.get_func_name().c_str(),
@@ -467,7 +577,7 @@ auto handle_torch_function_indexing(
   } else {
     index_tup = py::make_tuple(py::handle(index));
   }
-  std::vector<py::handle> overridable_args;
+  std::vector<PyObject*> overridable_args;
   is_tensor_and_append_overloaded(self, &overridable_args);
   auto size = PyTuple_GET_SIZE(index_tup.ptr());
   for (auto i : c10::irange(size)) {
@@ -512,7 +622,7 @@ auto handle_torch_function_indexing(
  *  entry in overloaded_args for this type with higher precedence than
  *  the superclass.
  *
- *  See torch._overrides._get_overloaded_types_and_args for the equivalent
+ *  See torch._overrides._get_overloaded_args for the equivalent
  *  function in the Python __torch_function__ implementation.
  *
  *  The precedence-determining algorithm implemented in this function is
@@ -532,13 +642,13 @@ auto handle_torch_function_indexing(
  */
 
 static void append_overloaded_arg(
-    std::vector<py::handle>* overloaded_args,
+    std::vector<PyObject*>* overloaded_args,
     PyObject* obj,
     bool obj_is_type) {
   bool class_not_seen_yet = true;
   PyObject* obj_type = obj_is_type ? obj : (PyObject*)Py_TYPE(obj);
   for (auto& arg : *overloaded_args) {
-    if (obj_type == get_type_of_overloaded_arg(arg.ptr())) {
+    if (obj_type == get_type_of_overloaded_arg(arg)) {
       // obj is the same type as another parameter we've seen in a prior
       // iteration of the loop over parameters so we already have an entry
       // with the proper __torch_function__ implementation to call, so skip
@@ -551,9 +661,7 @@ static void append_overloaded_arg(
     auto arg_index = overloaded_args->size();
     for (const auto j : c10::irange(arg_index)) {
       if (PyObject_IsSubclass(
-              obj_type,
-              (PyObject*)(get_type_of_overloaded_arg(
-                  (*overloaded_args)[j].ptr())))) {
+              obj_type, get_type_of_overloaded_arg((*overloaded_args)[j]))) {
         // obj is a subclass of another object we've seen already so its
         // __torch_function__ should be called first, therefore we
         // insert it into overloaded_args before the superclass
@@ -570,20 +678,20 @@ static void append_overloaded_arg(
 }
 
 void append_overloaded_tensor(
-    std::vector<py::handle>* overloaded_args,
+    std::vector<PyObject*>* overloaded_args,
     PyObject* obj) {
   append_overloaded_arg(overloaded_args, obj, /*obj_is_type*/ false);
 }
 
 void append_overloaded_type(
-    std::vector<py::handle>* overloaded_args,
+    std::vector<PyObject*>* overloaded_args,
     PyObject* obj) {
   append_overloaded_arg(overloaded_args, obj, /*obj_is_type*/ true);
 }
 
 bool is_tensor_and_append_overloaded(
     PyObject* obj,
-    std::vector<py::handle>* overloaded_args) {
+    std::vector<PyObject*>* overloaded_args) {
   if (THPVariable_CheckExact(obj)) {
     // torch.Tensor instances (not subclasses, except for Parameter)
     return true;
@@ -601,7 +709,7 @@ bool is_tensor_and_append_overloaded(
   return false;
 }
 
-bool is_scalar_list(PyObject* obj) {
+static bool is_scalar_list(PyObject* obj) {
   auto tuple = six::isTuple(obj);
   if (!(tuple || PyList_Check(obj))) {
     return false;
@@ -620,7 +728,7 @@ bool is_scalar_list(PyObject* obj) {
 
 bool is_tensor_list_and_append_overloaded(
     PyObject* obj,
-    std::vector<py::handle>* overloaded_args,
+    std::vector<PyObject*>* overloaded_args,
     int argnum,
     bool throw_error) {
   auto tuple = six::isTuple(obj);
@@ -634,10 +742,13 @@ bool is_tensor_list_and_append_overloaded(
         tuple ? PyTuple_GET_ITEM(obj, idx) : PyList_GET_ITEM(obj, idx);
     if (!is_tensor_and_append_overloaded(iobj, overloaded_args)) {
       if (throw_error) {
-        throw TypeError(
-            "expected Tensor as element %d in argument %d, but got %s",
-            static_cast<int>(idx),
+        TORCH_CHECK_TYPE(
+            false,
+            "expected Tensor as element ",
+            idx,
+            " in argument ",
             argnum,
+            ", but got ",
             Py_TYPE(iobj)->tp_name);
       }
       return false;
@@ -646,7 +757,7 @@ bool is_tensor_list_and_append_overloaded(
   return true;
 }
 
-bool is_float_or_complex_list(PyObject* obj) {
+static bool is_float_or_complex_list(PyObject* obj) {
   auto tuple = six::isTuple(obj);
   if (!(tuple || PyList_Check(obj))) {
     return false;
@@ -664,61 +775,35 @@ bool is_float_or_complex_list(PyObject* obj) {
   return true;
 }
 
-static bool is_int_list(
-    PyObject* obj,
-    int broadcast_size,
-    int64_t* failed_idx = nullptr) {
-  if (PyTuple_Check(obj) || PyList_Check(obj)) {
-    auto len = PySequence_Size(obj);
-    if (len == 0) {
-      return true;
-    }
-
-    auto item = py::reinterpret_steal<py::object>(PySequence_GetItem(obj, 0));
-    bool int_first = false;
-    if (THPUtils_checkIndex(item.ptr())) {
-      // we still have to check that the rest of items are NOT symint nodes
-      int_first = true;
-    }
-
-    // Make sure none of the later arguments are SymInt
-    // NB: do NOT check that the later arguments are ints, as this is
-    // BC-breaking for FX
-    for (int i = 1; i < len; i++) {
-      if (torch::is_symint(
-              py::reinterpret_steal<py::object>(PySequence_GetItem(obj, i)))) {
-        if (failed_idx != nullptr) {
-          *failed_idx = i;
-        }
-        return false;
-      }
-    }
-
-    if (int_first) {
-      return true;
-    }
-
-    // NOTE: JIT tracer allows arbitrary scalar tensors to act as ints
-    // in an intlist argument. Even float or complex scalar tensors.
-    bool r =
-        (jit::tracer::isTracing() && THPVariable_Check(item.ptr()) &&
-         THPVariable_Unpack(item.ptr()).sizes().empty());
-    if (!r && failed_idx != nullptr) {
-      *failed_idx = 0;
-    }
-    return r;
-  }
-  // if a size is specified (e.g. IntArrayRef[2]) we also allow passing a single
-  // int
-  return broadcast_size > 0 && THPUtils_checkLong(obj);
-}
-
 static bool is_int_or_symint(PyObject* obj) {
   // THPUtils_checkIndex may call __index__ or __int__
   // which may have side effects if obj is a symint node
   // so we do `is_symint` check first
   // TODO: maybe we should be using checkLong here?
-  return torch::is_symint(py::handle(obj)) || THPUtils_checkIndex(obj);
+  if (torch::is_symint(py::handle(obj))) {
+    return true;
+  }
+
+  // FakeTensor(..., size=()) is qualified for SymInt param,
+  // but we can't go via __index__ (below) as we would normally
+  // do for regular tensors, because __index__ first forces a
+  // conversion into an int, which in general you cannot do
+  // if you have an unbacked SymInt.  So this fastpath ensures
+  // that we still allow for fake tensors in this case, but
+  // for regular tensors it's redundant with the test below.
+  if (THPVariable_Check(obj)) {
+    auto& var = THPVariable_Unpack(obj);
+    if (TORCH_GUARD_SIZE_OBLIVIOUS(var.sym_numel().sym_eq(1)) &&
+        at::isIntegralType(var.dtype().toScalarType(), /*include_bool*/ true)) {
+      return true;
+    }
+  }
+
+  if (THPUtils_checkIndex(obj)) {
+    return true;
+  }
+
+  return false;
 }
 
 static bool is_int_or_symint_list(
@@ -734,6 +819,7 @@ static bool is_int_or_symint_list(
     if (is_int_or_symint(item.ptr())) {
       return true;
     }
+
     // NOTE: JIT tracer allows arbitrary scalar tensors to act as ints
     // in an intlist argument. Even float or complex scalar tensors.
     bool r =
@@ -744,15 +830,16 @@ static bool is_int_or_symint_list(
     }
     return r;
   }
+
   // if a size is specified (e.g. IntArrayRef[2]) we also allow passing a single
   // int
-  return broadcast_size > 0 && THPUtils_checkLong(obj);
+  return broadcast_size > 0 && is_int_or_symint(obj);
 }
 
 // argnum is needed for raising the TypeError, it's used in the error message.
 auto FunctionParameter::check(
     PyObject* obj,
-    std::vector<py::handle>& overloaded_args,
+    std::vector<PyObject*>& overloaded_args,
     int argnum,
     int64_t* failed_idx) -> bool {
   switch (type_) {
@@ -769,12 +856,12 @@ auto FunctionParameter::check(
       if (THPUtils_checkScalar(obj)) {
         return true;
       }
-      // fallthrough
+      [[fallthrough]];
     case ParameterType::COMPLEX:
       if (PyComplex_Check(obj)) {
         return true;
       }
-      // fallthrough
+      [[fallthrough]];
     case ParameterType::DOUBLE: {
       if (THPUtils_checkDouble(obj)) {
         return true;
@@ -783,6 +870,11 @@ auto FunctionParameter::check(
         const auto& var = THPVariable_Unpack(obj);
         return !var.requires_grad() && var.dim() == 0;
       }
+      if (torch::is_symfloat(py::handle(obj)) ||
+          torch::is_symint(py::handle(obj))) {
+        // This will induce a guard
+        return true;
+      }
       return false;
     }
     case ParameterType::INT64: {
@@ -794,6 +886,10 @@ auto FunctionParameter::check(
         return at::isIntegralType(var.scalar_type(), /*includeBool=*/false) &&
             !var.requires_grad() && var.dim() == 0;
       }
+      if (torch::is_symint(py::handle(obj))) {
+        // This will induce a guard
+        return true;
+      }
       return false;
     }
     case ParameterType::DIMNAME:
@@ -810,8 +906,6 @@ auto FunctionParameter::check(
       return is_tensor_list_and_append_overloaded(
           obj, &overloaded_args, argnum, true /* throw_error */);
     }
-    case ParameterType::INT_LIST:
-      return is_int_list(obj, size, failed_idx);
     case ParameterType::FLOAT_LIST:
       return is_float_or_complex_list(obj);
     case ParameterType::GENERATOR:
@@ -841,8 +935,12 @@ auto FunctionParameter::check(
       return is_scalar_list(obj);
     case ParameterType::SYM_INT:
       return is_int_or_symint(obj);
+    // Allow SymInt where int is expected; we'll guard in this case
+    case ParameterType::INT_LIST:
     case ParameterType::SYM_INT_LIST:
       return is_int_or_symint_list(obj, size, failed_idx);
+    case ParameterType::DISPATCH_KEY_SET:
+      return py::isinstance<c10::DispatchKeySet>(py::handle(obj));
     default:
       throw std::runtime_error("unknown parameter type");
   }
@@ -898,19 +996,20 @@ std::string FunctionParameter::type_name() const {
       return "tuple of Scalars";
     case ParameterType::SYM_INT_LIST:
       return "tuple of ints";
+    case ParameterType::DISPATCH_KEY_SET:
+      return "DispatchKeySet";
     default:
       throw std::runtime_error("unknown parameter type");
   }
 }
 
-static inline c10::optional<int64_t> parse_as_integer(const std::string& s) {
+static inline std::optional<int64_t> parse_as_integer(const std::string& s) {
   if (s.empty())
     return c10::nullopt;
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  char* str_end;
+  char* str_end = nullptr;
   long ans = strtol(s.c_str(), &str_end, 0);
   // *str_end == 0 if the entire string was parsed as an integer.
-  return (*str_end == 0) ? c10::optional<int64_t>(ans) : c10::nullopt;
+  return (*str_end == 0) ? std::optional<int64_t>(ans) : c10::nullopt;
 }
 
 /*
@@ -1020,7 +1119,8 @@ void FunctionParameter::set_default_str(const std::string& str) {
   if (str == "None") {
     allow_none = true;
   }
-  if (type_ == ParameterType::TENSOR) {
+  if (type_ == ParameterType::TENSOR ||
+      type_ == ParameterType::DISPATCH_KEY_SET) {
     if (str != "None") {
       throw std::runtime_error(
           "default value for Tensor must be none, got: " + str);
@@ -1107,6 +1207,7 @@ void FunctionParameter::set_default_str(const std::string& str) {
   } else {
     throw std::runtime_error("unknown parameter type");
   }
+  default_value = str;
 }
 
 // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
@@ -1180,7 +1281,6 @@ FunctionSignature::FunctionSignature(const std::string& fmt, int index)
 }
 
 std::string FunctionSignature::toString() const {
-  // TODO: consider printing more proper schema strings with defaults,
   // optionals, etc.
   std::ostringstream ss;
   bool keyword_already = false;
@@ -1195,6 +1295,9 @@ std::string FunctionSignature::toString() const {
       keyword_already = true;
     }
     ss << param.type_name() << " " << param.name;
+    if (param.optional) {
+      ss << " = " << param.default_value;
+    }
     i++;
   }
   ss << ")";
@@ -1301,6 +1404,7 @@ bool FunctionSignature::parse(
     PyObject* args,
     PyObject* kwargs,
     PyObject* dst[], // NOLINT
+    std::vector<PyObject*>& overloaded_args,
     bool raise_exception) {
   Py_ssize_t nargs = args ? PyTuple_GET_SIZE(args) : 0;
   auto remaining_kwargs = kwargs ? PyDict_Size(kwargs) : 0;
@@ -1310,14 +1414,10 @@ bool FunctionSignature::parse(
   // if there is a single positional IntArrayRef argument, i.e. expand(..),
   // view(...), allow a var-args style IntArrayRef, so expand(5,3) behaves as
   // expand((5,3))
-  int int_list_overload = false;
   if (max_pos_args == 1 &&
       (params[0].type_ == ParameterType::INT_LIST ||
        params[0].type_ == ParameterType::SYM_INT_LIST)) {
     allow_varargs_intlist = true;
-    if (params[0].type_ == ParameterType::INT_LIST) {
-      int_list_overload = true;
-    }
   }
 
   if (static_cast<size_t>(nargs) > max_pos_args && !allow_varargs_intlist) {
@@ -1328,13 +1428,9 @@ bool FunctionSignature::parse(
     return false;
   }
 
-  if (!overloaded_args.empty()) {
-    overloaded_args.clear();
-  }
-
   int i = 0;
   if (self != nullptr && check_has_torch_function(self, /*ignore_mode*/ true)) {
-    append_overloaded_tensor(&this->overloaded_args, self);
+    append_overloaded_tensor(&overloaded_args, self);
   }
   for (auto& param : params) {
     PyObject* obj = nullptr;
@@ -1369,16 +1465,14 @@ bool FunctionSignature::parse(
         missing_args(*this, i);
       }
       return false;
-    } else if (param.check(obj, this->overloaded_args, i, &failed_idx)) {
+    } else if (param.check(obj, overloaded_args, i, &failed_idx)) {
       dst[i++] = obj;
       // XXX: the Variable check is necessary because sizes become tensors when
       // tracer is enabled. This behavior easily leads to ambiguities, and we
       // should avoid having complex signatures that make use of it...
     } else if (
         varargs_eligible &&
-        ((int_list_overload
-              ? is_int_list(args, param.size, &failed_idx)
-              : is_int_or_symint_list(args, param.size, &failed_idx)))) {
+        (is_int_or_symint_list(args, param.size, &failed_idx))) {
       // take all positional arguments as this parameter
       // e.g. permute(1, 2, 3) -> permute((1, 2, 3))
       dst[i++] = args;
@@ -1442,7 +1536,9 @@ bool FunctionSignature::parse(
   return true;
 }
 
-PythonArgParser::PythonArgParser(std::vector<std::string> fmts, bool traceable)
+PythonArgParser::PythonArgParser(
+    const std::vector<std::string>& fmts,
+    bool traceable)
     : max_args(0), traceable(traceable) {
   int index = 0;
   for (auto& fmt : fmts) {
@@ -1493,15 +1589,20 @@ PythonArgs PythonArgParser::raw_parse(
     PyObject* parsed_args[]) { // NOLINT
   if (signatures_.size() == 1) {
     auto& signature = signatures_[0];
-    signature.parse(self, args, kwargs, parsed_args, true);
+    std::vector<PyObject*> overloaded_args;
+    signature.parse(self, args, kwargs, parsed_args, overloaded_args, true);
     check_deprecated(signature);
-    return PythonArgs(traceable, signature, parsed_args);
+    return PythonArgs(
+        traceable, signature, parsed_args, std::move(overloaded_args));
   }
 
   for (auto& signature : signatures_) {
-    if (signature.parse(self, args, kwargs, parsed_args, false)) {
+    std::vector<PyObject*> overloaded_args;
+    if (signature.parse(
+            self, args, kwargs, parsed_args, overloaded_args, false)) {
       check_deprecated(signature);
-      return PythonArgs(traceable, signature, parsed_args);
+      return PythonArgs(
+          traceable, signature, parsed_args, std::move(overloaded_args));
     }
   }
 
@@ -1513,8 +1614,8 @@ void PythonArgParser::print_error(
     PyObject* args,
     PyObject* kwargs,
     PyObject* parsed_args[]) { // NOLINT
-  // NOLINTNEXTLINE(clang-analyzer-core.NullDereference)
-  size_t num_args = PyTuple_GET_SIZE(args) + (kwargs ? PyDict_Size(kwargs) : 0);
+  size_t num_args =
+      (args ? PyTuple_GET_SIZE(args) : 0) + (kwargs ? PyDict_Size(kwargs) : 0);
   std::vector<unsigned> plausible_idxs;
   unsigned i = 0;
   for (auto& signature : signatures_) {
@@ -1527,7 +1628,8 @@ void PythonArgParser::print_error(
 
   if (plausible_idxs.size() == 1) {
     auto& signature = signatures_[plausible_idxs[0]];
-    signature.parse(self, args, kwargs, parsed_args, true);
+    std::vector<PyObject*> overloaded_args;
+    signature.parse(self, args, kwargs, parsed_args, overloaded_args, true);
   }
 
   auto options = get_signatures();
@@ -1560,7 +1662,21 @@ at::Tensor PythonArgs::tensor_slow(int i) {
   if (PyBool_Check(obj)) {
     scalar = at::Scalar(THPUtils_unpackBool(obj));
   } else if (THPUtils_checkLong(obj)) {
-    scalar = at::Scalar(THPUtils_unpackLong(obj));
+    int overflow = -1;
+    long long value = PyLong_AsLongLongAndOverflow(obj, &overflow);
+    if (value == -1 && PyErr_Occurred()) {
+      throw python_error();
+    }
+    if (overflow != 0) {
+      // try unsigned
+      unsigned long long value = PyLong_AsUnsignedLongLong(obj);
+      if (value == static_cast<unsigned long long>(-1) && PyErr_Occurred()) {
+        throw python_error();
+      }
+      scalar = at::Scalar(static_cast<uint64_t>(value));
+    } else {
+      scalar = at::Scalar(static_cast<int64_t>(value));
+    }
   } else if (PyComplex_Check(obj)) {
     scalar = at::Scalar(THPUtils_unpackComplexDouble(obj));
   } else if (THPUtils_checkDouble(obj)) {
@@ -1577,6 +1693,9 @@ at::Tensor PythonArgs::tensor_slow(int i) {
   } else if (torch::is_symfloat(py::handle(obj))) {
     save_symint = true;
     scalar = at::Scalar(std::numeric_limits<double>::quiet_NaN());
+  } else if (torch::is_symbool(py::handle(obj))) {
+    save_symint = true;
+    scalar = at::Scalar(true);
   } else {
     // NB: Are you here because you passed None to a Variable method,
     // and you expected an undefined tensor to be returned?   Don't add
@@ -1620,7 +1739,21 @@ at::Scalar PythonArgs::scalar_slow(PyObject* arg) {
   }
 
   if (THPUtils_checkLong(arg)) {
-    return at::Scalar(static_cast<int64_t>(THPUtils_unpackLong(arg)));
+    int overflow = -1;
+    long long value = PyLong_AsLongLongAndOverflow(arg, &overflow);
+    if (value == -1 && PyErr_Occurred()) {
+      throw python_error();
+    }
+    if (overflow != 0) {
+      // try unsigned
+      unsigned long long value = PyLong_AsUnsignedLongLong(arg);
+      if (value == static_cast<unsigned long long>(-1) && PyErr_Occurred()) {
+        throw python_error();
+      }
+      return at::Scalar(static_cast<uint64_t>(value));
+    } else {
+      return at::Scalar(static_cast<int64_t>(value));
+    }
   }
 
   if (PyBool_Check(arg)) {
@@ -1639,6 +1772,13 @@ at::Scalar PythonArgs::scalar_slow(PyObject* arg) {
     return at::Scalar(py::cast<c10::SymFloat>(arg));
   }
 
+  if (torch::is_symbool(arg)) {
+    // Windows build fails with C2440: '<function-style-cast>'
+    // when at:Scalar(py::cast<c10::SymBool>(arg))
+    auto sym_bool = py::handle(arg).cast<c10::SymBool>();
+    return at::Scalar(sym_bool);
+  }
+
   return at::Scalar(THPUtils_unpackDouble(arg));
 }
 
