@@ -1,12 +1,13 @@
-# coding=utf-8
-
-import copy
+# mypy: allow-untyped-defs
 import functools
-from typing import List
+from typing import List, TYPE_CHECKING
 
 import torch
-import torch.distributed._shard.sharding_spec as shard_spec
-from torch.distributed._shard.partial_tensor import _PartialTensor
+
+if TYPE_CHECKING:
+    from torch.distributed._shard.sharding_spec import ShardingSpec
+else:
+    ShardingSpec = "ShardingSpec"
 
 from .api import (
     _CUSTOM_SHARDED_OPS,
@@ -21,7 +22,7 @@ from .metadata import ShardMetadata  # noqa: F401
 from torch.distributed._shard.op_registry_utils import _decorator_func
 
 
-def empty(sharding_spec: shard_spec.ShardingSpec,
+def empty(sharding_spec: ShardingSpec,
           *size,
           dtype=None,
           layout=torch.strided,
@@ -42,7 +43,7 @@ def empty(sharding_spec: shard_spec.ShardingSpec,
 
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -73,7 +74,7 @@ def empty(sharding_spec: shard_spec.ShardingSpec,
         init_rrefs=init_rrefs,
     )
 
-def ones(sharding_spec: shard_spec.ShardingSpec,
+def ones(sharding_spec: ShardingSpec,
          *size,
          dtype=None,
          layout=torch.strided,
@@ -94,7 +95,7 @@ def ones(sharding_spec: shard_spec.ShardingSpec,
 
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -124,7 +125,7 @@ def ones(sharding_spec: shard_spec.ShardingSpec,
         init_rrefs=init_rrefs
     )
 
-def zeros(sharding_spec: shard_spec.ShardingSpec,
+def zeros(sharding_spec: ShardingSpec,
           *size,
           dtype=None,
           layout=torch.strided,
@@ -145,7 +146,7 @@ def zeros(sharding_spec: shard_spec.ShardingSpec,
 
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -175,7 +176,7 @@ def zeros(sharding_spec: shard_spec.ShardingSpec,
         init_rrefs=init_rrefs
     )
 
-def full(sharding_spec: shard_spec.ShardingSpec,
+def full(sharding_spec: ShardingSpec,
          size,
          fill_value,
          *,
@@ -187,7 +188,7 @@ def full(sharding_spec: shard_spec.ShardingSpec,
          process_group=None,
          init_rrefs=False) -> ShardedTensor:
     """
-    Creates a :class:`ShardedTensor` filled with fill_value. The tensor’s dtype
+    Creates a :class:`ShardedTensor` filled with fill_value. The tensor's dtype
         is inferred from fill_value. If dtype is specified, it will override the
         inferred type from fill_value. Needs to be called on all ranks in an SPMD fashion.
     Args:
@@ -195,10 +196,10 @@ def full(sharding_spec: shard_spec.ShardingSpec,
             describing how to shard the Tensor.
         size (int...):  a list, tuple, or `torch.Size` of integers defining the shape of the
             output tensor.
-        fill_value (Scalar) – the value to fill the output tensor with.
+        fill_value (Scalar) - the value to fill the output tensor with.
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -228,7 +229,7 @@ def full(sharding_spec: shard_spec.ShardingSpec,
     torch.nn.init.constant_(sharded_tensor, fill_value)  # type: ignore[arg-type]
     return sharded_tensor
 
-def rand(sharding_spec: shard_spec.ShardingSpec,
+def rand(sharding_spec: ShardingSpec,
          *size,
          dtype=None,
          layout=torch.strided,
@@ -250,7 +251,7 @@ def rand(sharding_spec: shard_spec.ShardingSpec,
 
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -281,7 +282,7 @@ def rand(sharding_spec: shard_spec.ShardingSpec,
     torch.nn.init.uniform_(sharded_tensor, 0, 1)  # type: ignore[arg-type]
     return sharded_tensor
 
-def randn(sharding_spec: shard_spec.ShardingSpec,
+def randn(sharding_spec: ShardingSpec,
           *size,
           dtype=None,
           layout=torch.strided,
@@ -304,7 +305,7 @@ def randn(sharding_spec: shard_spec.ShardingSpec,
 
     Keyword args:
         dtype (:class:`torch.dtype`, optional): the desired data type of returned tensor.
-            Default: if ``None``, uses a global default (see :func:`torch.set_default_tensor_type`).
+            Default: if ``None``, uses a global default (see :func:`torch.set_default_dtype`).
         layout (:class:`torch.layout`, optional): the desired layout of returned Tensor.
             Default: ``torch.strided``.
         requires_grad (bool, optional): If autograd should record operations on the
@@ -411,7 +412,7 @@ def pre_load_state_dict_hook(module, state_dict, prefix, local_metadata, strict,
     Pre-load state dict hook to add ShardedTensor to the module.
     """
     for submodule_name, submodule in module.named_modules():
-        for attr_name, attr in submodule.__dict__.items():
+        for attr_name in submodule.__dict__.keys():
             mod_prefix = prefix + submodule_name
             key = mod_prefix + ('.' if mod_prefix else '') + attr_name
             if key in state_dict:
@@ -427,6 +428,7 @@ def custom_sharded_op_impl(func):
     parameters, the function provided will be invoked for that operator.
 
     Example::
+        >>> # xdoctest: +SKIP
         >>> @custom_sharded_op_impl(torch.nn.functional.linear)
         >>> def my_custom_sharded_linear(types, args, kwargs, process_group):
         >>>     ...
