@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 import operator
 from contextlib import contextmanager
 from enum import Enum
@@ -6,7 +7,6 @@ from typing import Any, cast, Dict, List, Optional, Tuple
 
 import torch
 
-import torch.distributed.distributed_c10d as c10d
 import torch.fx as fx
 import torch.library
 import torch.nn as nn
@@ -15,16 +15,16 @@ import torch.utils._pytree as pytree
 
 from torch.distributed._spmd.batch_dim_utils import BatchDimAnalyzer
 from torch.distributed._tensor import DeviceMesh, distribute_tensor, Replicate, Shard
-
-from torch.distributed._tensor._utils import compute_local_shape
-from torch.distributed._tensor.op_schema import (
+from torch.distributed._tensor._op_schema import (
     OpStrategy,
     PlacementStrategy,
     StrategyType,
     TupleStrategy,
 )
+from torch.distributed._tensor._redistribute import redistribute_local_tensor
+
+from torch.distributed._tensor._utils import compute_local_shape
 from torch.distributed._tensor.placement_types import _Partial, DTensorSpec, Placement
-from torch.distributed._tensor.redistribute import redistribute_local_tensor
 from torch.fx import GraphModule
 from torch.fx.experimental.proxy_tensor import make_fx
 from torch.fx.passes.shape_prop import _extract_tensor_metadata
@@ -41,7 +41,8 @@ _spmd_lib_impl.impl("tag_grad", lambda x: x, "CompositeExplicitAutograd")
 
 
 class DataParallelStyle(Enum):
-    """
+    """This enum represents the style of the data-parallel operation.
+
     We have three types of Data Parallel style:
     1. DEFAULT: the default data parallel style, which is to represent a mixed
                 replicate and fully shard behavior. For each parameter that is able
@@ -64,8 +65,8 @@ class DataParallelStyle(Enum):
 
 
 class NodeType(Enum):
-    """
-    NodeType is a enum that records the type of the tensors in the graph.
+    """NodeType is an enum that records the type of the tensors in the graph.
+
     This is used to determine the data parallel strategy.
     """
 
@@ -77,9 +78,8 @@ class NodeType(Enum):
 
 
 class DataParallelStrategy(OpStrategy):
-    """
-    DataParallelStrategy is a special case of OpStrategy that only records
-    the "data parallel style" placement strategy for each fx Node.
+    """DataParallelStrategy is a special case of OpStrategy that only records the "data parallel style" placement
+    strategy for each fx Node.
 
     It takes a list of PlacementStrategy, where each PlacementStrategy describes
     one way to distribute the tensor and computation. In the DataParallel case,
@@ -113,17 +113,14 @@ class DataParallelStrategy(OpStrategy):
 
 @contextmanager
 def gradients_tagging(params: Dict[str, torch.Tensor]):
-    """
-    This is a helper function that tags the gradient of the parameters
-    with a special tag, so that we can identify them during SPMD expansion.
+    """Tag the gradient of the parameters with a special tag, so that we can identify them during SPMD expansion.
 
     It's safe to trace those hooks and we would remove those nodes later.
     """
-
     tagging_hooks = []
     try:
         for p in params.values():
-            h = p.register_hook(lambda grad: torch.ops._spmd.tag_grad(grad))
+            h = p.register_hook(torch.ops._spmd.tag_grad)
             tagging_hooks.append(h)
         yield
     finally:
@@ -135,11 +132,9 @@ def gradients_tagging(params: Dict[str, torch.Tensor]):
 def _gen_shard_strategy(
     mesh: DeviceMesh, shard_dim: int, input_specs: Optional[List[DTensorSpec]] = None
 ) -> PlacementStrategy:
-    """
-    util function to generate a shard strategy on shard_dim
-    """
+    """Util function to generate a shard strategy on shard_dim."""
     return PlacementStrategy(
-        output_spec=DTensorSpec(mesh=mesh, placements=(Shard(shard_dim),)),
+        output_specs=DTensorSpec(mesh=mesh, placements=(Shard(shard_dim),)),
         input_specs=input_specs,
     )
 
@@ -147,19 +142,15 @@ def _gen_shard_strategy(
 def _gen_replicate_strategy(
     mesh: DeviceMesh, input_specs: Optional[List[DTensorSpec]] = None
 ) -> PlacementStrategy:
-    """
-    util function to generate a replicate strategy
-    """
+    """Util function to generate a replicate strategy."""
     return PlacementStrategy(
-        output_spec=DTensorSpec(mesh=mesh, placements=(Replicate(),)),
+        output_specs=DTensorSpec(mesh=mesh, placements=(Replicate(),)),
         input_specs=input_specs,
     )
 
 
 def _gen_partial_strategy(mesh: DeviceMesh) -> PlacementStrategy:
-    """
-    util function to generate a partial strategy
-    """
+    """Util function to generate a partial strategy."""
     # NOTE: we use AVG by default, avg reduction is needed depending on
     # the loss function, for most loss function it should do
     # gradient averaging. There might be certain cases it should
@@ -167,9 +158,8 @@ def _gen_partial_strategy(mesh: DeviceMesh) -> PlacementStrategy:
     # TODO: Only NCCL supports AVG so using backend like Gloo would
     # crash, we should figure out a way to support avg reduction
     # for non-NCCL backend
-    reduce_op = c10d.ReduceOp.AVG  # type: ignore[attr-defined]
     return PlacementStrategy(
-        output_spec=DTensorSpec(mesh=mesh, placements=(_Partial(reduce_op),)),
+        output_specs=DTensorSpec(mesh=mesh, placements=(_Partial("avg"),)),
     )
 
 
@@ -180,10 +170,7 @@ def build_data_parallel_strategies(
     mesh: DeviceMesh,
     batch_dim: int = 0,
 ) -> Dict[fx.Node, StrategyType]:
-    """
-    This function loop through the train step graph and build the
-    data parallel strategy for each fx Node
-    """
+    """Loop through the train step graph and build the data parallel strategy for each fx Node."""
     activation_idx = num_params + num_states
     non_compute_ops = [
         aten.clone.default,
@@ -324,7 +311,7 @@ def build_data_parallel_strategies(
                         output_spec = batch_dim_analyzer.compute_act_spec(node, mesh)
 
                         shard_strategy = PlacementStrategy(
-                            output_spec=output_spec, input_specs=[arg_node_spec]
+                            output_specs=output_spec, input_specs=[arg_node_spec]
                         )
                         dp_strategy_map[node] = DataParallelStrategy(
                             NodeType.ACT, [shard_strategy]
@@ -469,7 +456,7 @@ def build_data_parallel_strategies(
                         output_spec = batch_dim_analyzer.compute_act_spec(node, mesh)
 
                         act_strategy = PlacementStrategy(
-                            output_spec=output_spec, input_specs=input_specs
+                            output_specs=output_spec, input_specs=input_specs
                         )
 
                         dp_strategy_map[node] = DataParallelStrategy(
@@ -497,7 +484,7 @@ def build_data_parallel_strategies(
 
                     act_spec = batch_dim_analyzer.compute_act_spec(node, mesh)
                     op_strategy = PlacementStrategy(
-                        output_spec=act_spec, input_specs=input_specs
+                        output_specs=act_spec, input_specs=input_specs
                     )
                     dp_strategy_map[node] = DataParallelStrategy(
                         NodeType.ACT, [op_strategy]
@@ -518,9 +505,7 @@ def mark_data_parallel_shardings(
     dp_strategy_map: Dict[fx.Node, StrategyType],
     parallel_mode: DataParallelStyle = DataParallelStyle.FULLY_SHARD,
 ) -> None:
-    """
-    This function marks the sharding for the nodes in the train_step_graph
-    """
+    """Mark the sharding for the nodes in the train_step_graph."""
     activation_idx = num_parameters + num_states
     placeholder_idx = 0
     for node in train_step_graph.graph.nodes:
@@ -555,7 +540,7 @@ def mark_data_parallel_shardings(
                 # mark activation as sharded on batch dim
                 node_sharding = node_strategies[0]
 
-            node.meta["sharding"] = node_sharding
+            node.meta["sharding"] = node_sharding  # type: ignore[possibly-undefined]
 
             placeholder_idx += 1
         elif node.op == "call_function":
@@ -601,9 +586,7 @@ def mark_data_parallel_shardings(
 
 
 def _partition_val(val: Any, spec: DTensorSpec) -> Any:
-    """
-    util function to convert a full tensor val to its local component
-    """
+    """Util function to convert a full tensor val to its local component."""
     if isinstance(val, torch.Tensor):
         local_shard = val
         if val.ndim == 0:
@@ -614,7 +597,7 @@ def _partition_val(val: Any, spec: DTensorSpec) -> Any:
         for idx, placement in enumerate(spec.placements):
             if placement.is_shard():
                 placement = cast(Shard, placement)
-                num_chunks = spec.mesh.size(dim=idx)
+                num_chunks = spec.mesh.size(mesh_dim=idx)
                 my_coord = spec.mesh.get_coordinate()
                 assert my_coord is not None, "current rank not in mesh!"
                 my_coord_on_mesh_dim = my_coord[idx]
@@ -629,10 +612,7 @@ def _partition_val(val: Any, spec: DTensorSpec) -> Any:
 
 
 def partitioner(graph: GraphModule) -> GraphModule:
-    """
-    Graph partitioner that partitions the single device graph
-    to distributed graph
-    """
+    """Graph partitioner that partitions the single device graph to distributed graph."""
     shape_adjustment_ops = {
         aten._unsafe_view.default: 1,
         aten.expand.default: 1,
@@ -761,13 +741,12 @@ def partition_data_parallel(
     parallel_style: DataParallelStyle,
     input_batch_dim: int,
 ) -> GraphModule:
-    """
-    The entry point function to partition the graph to data parallel
-    graph, it also shard/replicate the model parameters and optimizer
-    states to DTensors.
+    """Partition the graph to into a data parallel graph.
+
+    This function also shards/replicates the model parameters and optimizer states to DTensors.
     """
     num_params_buffers = len(params_buffers)
-    flattened_states = pytree.tree_flatten(named_states)[0]
+    flattened_states = pytree.tree_leaves(named_states)
     num_states = len(flattened_states)
 
     changed = graph.graph.eliminate_dead_code()
