@@ -6,3 +6,5 @@ from .tensor_ops import *  # noqa: F403
 from .pointwise_ops import *  # noqa: F403
 from .random_ops import *  # noqa: F403
 from .view_ops import *  # noqa: F403
+from .conv_ops import *  # noqa: F403
+from .experimental_ops import *  # noqa: F403
