@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 __all__ = ["shutdown", "get_worker_info", "remote", "rpc_sync",
            "rpc_async", "RRef", "AllGatherStates", "method_factory", "new_method"]
 
@@ -612,8 +613,8 @@ def remote(to, func, args=None, kwargs=None, timeout=UNSET_RPC_TIMEOUT):
 
         >>> # On both workers:
         >>> @torch.jit.script
-        >>> def my_script_add(t1, t2):
-        >>>    return torch.add(t1, t2)
+        >>> def my_script_add(tensor: torch.Tensor, scalar: int):
+        >>>    return torch.add(tensor, scalar)
 
         >>> # On worker 0:
         >>> import torch.distributed.rpc as rpc
@@ -793,8 +794,8 @@ def rpc_sync(to, func, args=None, kwargs=None, timeout: float = UNSET_RPC_TIMEOU
 
         >>> # On both workers:
         >>> @torch.jit.script
-        >>> def my_script_add(t1, t2):
-        >>>    return torch.add(t1, t2)
+        >>> def my_script_add(tensor: torch.Tensor, scalar: int):
+        >>>    return torch.add(tensor, scalar)
 
         >>> # On worker 0:
         >>> import torch.distributed.rpc as rpc
@@ -887,8 +888,8 @@ def rpc_async(to, func, args=None, kwargs=None, timeout=UNSET_RPC_TIMEOUT):
 
         >>> # On both workers:
         >>> @torch.jit.script
-        >>> def my_script_add(t1, t2):
-        >>>    return torch.add(t1, t2)
+        >>> def my_script_add(tensor: torch.Tensor, scalar: int):
+        >>>    return torch.add(tensor, scalar)
 
         >>> # On worker 0:
         >>> import torch.distributed.rpc as rpc
