@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 from functools import reduce
 import torch
 import operator
@@ -11,12 +12,7 @@ import itertools
 
 from torch.fx.experimental.unification import Var  # type: ignore[attr-defined]
 
-
-try:
-    import sympy  # type: ignore[import]
-    HAS_SYMPY = True
-except ImportError:
-    HAS_SYMPY = False
+import sympy
 
 _INFERENCE_RULES: Dict[Target, Callable] = {}
 _REFINEMENT_RULES: Dict[Target, Callable] = {}
@@ -184,7 +180,7 @@ def get_attr_inference_rule(n: Node, traced):
     if attr_name == "shape":
         n.type = Dyn
     else:
-        raise TypeError("Not yet implelemted")
+        raise TypeError("Not yet implemented")
 
     # TODO. We leave it like this till we add a type to represent tensor sizes
     return n.type
@@ -192,7 +188,7 @@ def get_attr_inference_rule(n: Node, traced):
 @register_inference_rule(torch.transpose)
 def transpose_inference_rule(n: Node):
     """
-    We check that dimentions for the transpose operations
+    We check that dimensions for the transpose operations
     are within range of the tensor type of the node
     """
     if n.target == torch.transpose:
@@ -249,8 +245,8 @@ def reshape_inference_rule(n: Node):
     elif isinstance(t1, TensorType):
         assert isinstance(t1, TensorType)
         a = [e if e != Dyn else 1 for e in t1.__args__]
-        p1 = reduce(lambda x, y: x * y, a)
-        p2 = reduce(lambda x, y: x * y, t2)
+        p1 = reduce(operator.mul, a)
+        p2 = reduce(operator.mul, t2)
         if p1 % p2 == 0 or p2 % p1 == 0:
             n.type = t2_type
             return t2_type
@@ -305,7 +301,7 @@ def calculate_out_dimension(d_in, module_instance, index):
     dilation = (module_instance.dilation, module_instance.dilation) \
         if isinstance(module_instance.dilation, int) else module_instance.dilation
 
-    DIMENSION_TYPES = (int, sympy.Symbol) if HAS_SYMPY else (int,)
+    DIMENSION_TYPES = (int, sympy.Symbol)
 
     if d_in == Dyn:
         return Dyn
@@ -503,11 +499,11 @@ def flatten_check(tensor_type, start_dim, end_dim):
         if Dyn in mid:
             mid = [Dyn]
         else:
-            mid = [reduce(lambda x, y: x * y, my_args[start_dim:end_dim])]
+            mid = [reduce(operator.mul, my_args[start_dim:end_dim])]
         new_type_list = lhs + mid + rhs
         return TensorType(tuple(new_type_list))
     else:
-        raise TypeError(f'Incompatable dimentions {start_dim}, {end_dim - 1} in type {tensor_type}')
+        raise TypeError(f'Incompatible dimensions {start_dim}, {end_dim - 1} in type {tensor_type}')
 
 @register_inference_rule(torch.flatten)
 def flatten_inference_rule(n: Node):
@@ -814,18 +810,15 @@ class Refine:
         """
         Replace all unknown types with fresh type variables.
         """
-        if HAS_SYMPY:
-            if isinstance(typ, Var):
-                return sympy.symbols(str(typ))
-            elif isinstance(typ, TensorType):
-                new_args = [self.convert_to_sympy_symbols(a) for a in typ.__args__]
-                return TensorType(tuple(new_args))
-            elif isinstance(typ, list):
-                return [self.convert_to_sympy_symbols(t) for t in typ]
-            elif isinstance(typ, tuple):
-                return (self.convert_to_sympy_symbols(t) for t in typ)
-            else:
-                return typ
+        if isinstance(typ, Var):
+            return sympy.symbols(str(typ))
+        elif isinstance(typ, TensorType):
+            new_args = [self.convert_to_sympy_symbols(a) for a in typ.__args__]
+            return TensorType(tuple(new_args))
+        elif isinstance(typ, list):
+            return [self.convert_to_sympy_symbols(t) for t in typ]
+        elif isinstance(typ, tuple):
+            return (self.convert_to_sympy_symbols(t) for t in typ)
         else:
             return typ
 
@@ -865,29 +858,26 @@ class Refine:
             pass
 
     def infer_symbolic_relations(self, n: Node):
-        if HAS_SYMPY:
-            n.type = self.convert_to_sympy_symbols(n.type)
-            if n.op == 'call_function':
-                if n.target in _RULES:
-                    return _RULES[n.target](n)
-                else:
-                    pass
-
-            if n.op == 'call_module':
-                module_instance = self.traced.get_submodule(n.target)
-                if type(module_instance) in _RULES:
-                    return _RULES[type(module_instance)](n, module_instance)
-                else:
-                    pass
-
-            if n.op == 'output':
-                def get_node_type(a):
-                    return a.type
-                n.type = torch.fx.node.map_arg(n.args[0], get_node_type)
-                return n.type
-
+        n.type = self.convert_to_sympy_symbols(n.type)
+        if n.op == 'call_function':
+            if n.target in _RULES:
+                return _RULES[n.target](n)
             else:
                 pass
+
+        if n.op == 'call_module':
+            module_instance = self.traced.get_submodule(n.target)
+            if type(module_instance) in _RULES:
+                return _RULES[type(module_instance)](n, module_instance)
+            else:
+                pass
+
+        if n.op == 'output':
+            def get_node_type(a):
+                return a.type
+            n.type = torch.fx.node.map_arg(n.args[0], get_node_type)
+            return n.type
+
         else:
             pass
 
