@@ -1,4 +1,4 @@
-import warnings
+# mypy: allow-untyped-defs
 from collections import OrderedDict, abc as container_abcs
 from itertools import chain, islice
 import operator
@@ -9,25 +9,42 @@ from ..parameter import Parameter
 from torch._jit_internal import _copy_to_script_wrapper
 
 from typing import Any, Dict, Iterable, Iterator, Mapping, Optional, overload, Tuple, TypeVar, Union
+from typing_extensions import Self
+from typing_extensions import deprecated
 
 __all__ = ['Container', 'Sequential', 'ModuleList', 'ModuleDict', 'ParameterList', 'ParameterDict']
 
 T = TypeVar('T', bound=Module)
 
 
-class Container(Module):
+# Copied from torch.nn.modules.module, required for a custom __repr__ for ModuleList
+def _addindent(s_, numSpaces):
+    s = s_.split('\n')
+    # don't do anything for single-line stuff
+    if len(s) == 1:
+        return s_
+    first = s.pop(0)
+    s = [(numSpaces * ' ') + line for line in s]
+    s = '\n'.join(s)
+    s = first + '\n' + s
+    return s
 
+
+@deprecated(
+    "`nn.Container` is deprecated. "
+    "All of it's functionality is now implemented in `nn.Module`. Subclass that instead.",
+    category=FutureWarning,
+)
+class Container(Module):
     def __init__(self, **kwargs: Any) -> None:
-        super(Container, self).__init__()
-        # DeprecationWarning is ignored by default <sigh>
-        warnings.warn("nn.Container is deprecated. All of it's functionality "
-                      "is now implemented in nn.Module. Subclass that instead.")
+        super().__init__()
         for key, value in kwargs.items():
             self.add_module(key, value)
 
 
 class Sequential(Module):
     r"""A sequential container.
+
     Modules will be added to it in the order they are passed in the
     constructor. Alternatively, an ``OrderedDict`` of modules can be
     passed in. The ``forward()`` method of ``Sequential`` accepts any
@@ -82,7 +99,7 @@ class Sequential(Module):
         ...
 
     def __init__(self, *args):
-        super(Sequential, self).__init__()
+        super().__init__()
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
                 self.add_module(key, module)
@@ -90,12 +107,12 @@ class Sequential(Module):
             for idx, module in enumerate(args):
                 self.add_module(str(idx), module)
 
-    def _get_item_by_idx(self, iterator, idx) -> T:
-        """Get the idx-th item of the iterator"""
+    def _get_item_by_idx(self, iterator, idx) -> T:  # type: ignore[misc, type-var]
+        """Get the idx-th item of the iterator."""
         size = len(self)
         idx = operator.index(idx)
         if not -size <= idx < size:
-            raise IndexError('index {} is out of range'.format(idx))
+            raise IndexError(f'index {idx} is out of range')
         idx %= size
         return next(islice(iterator, idx, None))
 
@@ -135,15 +152,14 @@ class Sequential(Module):
             return ret
         else:
             raise ValueError('add operator supports only objects '
-                             'of Sequential class, but {} is given.'.format(
-                                 str(type(other))))
+                             f'of Sequential class, but {str(type(other))} is given.')
 
     def pop(self, key: Union[int, slice]) -> Module:
         v = self[key]
         del self[key]
         return v
 
-    def __iadd__(self, other) -> 'Sequential':
+    def __iadd__(self, other) -> Self:
         if isinstance(other, Sequential):
             offset = len(self)
             for i, module in enumerate(other):
@@ -151,8 +167,7 @@ class Sequential(Module):
             return self
         else:
             raise ValueError('add operator supports only objects '
-                             'of Sequential class, but {} is given.'.format(
-                                 str(type(other))))
+                             f'of Sequential class, but {str(type(other))} is given.')
 
     def __mul__(self, other: int) -> 'Sequential':
         if not isinstance(other, int):
@@ -171,7 +186,7 @@ class Sequential(Module):
     def __rmul__(self, other: int) -> 'Sequential':
         return self.__mul__(other)
 
-    def __imul__(self, other: int) -> 'Sequential':
+    def __imul__(self, other: int) -> Self:
         if not isinstance(other, int):
             raise TypeError(f"unsupported operand type(s) for *: {type(self)} and {type(other)}")
         elif (other <= 0):
@@ -187,7 +202,7 @@ class Sequential(Module):
 
     @_copy_to_script_wrapper
     def __dir__(self):
-        keys = super(Sequential, self).__dir__()
+        keys = super().__dir__()
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
@@ -205,7 +220,7 @@ class Sequential(Module):
         return input
 
     def append(self, module: Module) -> 'Sequential':
-        r"""Appends a given module to the end.
+        r"""Append a given module to the end.
 
         Args:
             module (nn.Module): module to append
@@ -216,11 +231,11 @@ class Sequential(Module):
     def insert(self, index: int, module: Module) -> 'Sequential':
         if not isinstance(module, Module):
             raise AssertionError(
-                'module should be of type: {}'.format(Module))
+                f'module should be of type: {Module}')
         n = len(self._modules)
         if not (-n <= index <= n):
             raise IndexError(
-                'Index out of range: {}'.format(index))
+                f'Index out of range: {index}')
         if index < 0:
             index += n
         for i in range(n, index, -1):
@@ -248,7 +263,7 @@ class ModuleList(Module):
 
         class MyModule(nn.Module):
             def __init__(self):
-                super(MyModule, self).__init__()
+                super().__init__()
                 self.linears = nn.ModuleList([nn.Linear(10, 10) for i in range(10)])
 
             def forward(self, x):
@@ -261,15 +276,15 @@ class ModuleList(Module):
     _modules: Dict[str, Module]  # type: ignore[assignment]
 
     def __init__(self, modules: Optional[Iterable[Module]] = None) -> None:
-        super(ModuleList, self).__init__()
+        super().__init__()
         if modules is not None:
             self += modules
 
     def _get_abs_string_index(self, idx):
-        """Get the absolute index for the list of modules"""
+        """Get the absolute index for the list of modules."""
         idx = operator.index(idx)
         if not (-len(self) <= idx < len(self)):
-            raise IndexError('index {} is out of range'.format(idx))
+            raise IndexError(f'index {idx} is out of range')
         if idx < 0:
             idx += len(self)
         return str(idx)
@@ -303,7 +318,7 @@ class ModuleList(Module):
     def __iter__(self) -> Iterator[Module]:
         return iter(self._modules.values())
 
-    def __iadd__(self, modules: Iterable[Module]) -> 'ModuleList':
+    def __iadd__(self, modules: Iterable[Module]) -> Self:
         return self.extend(modules)
 
     def __add__(self, other: Iterable[Module]) -> 'ModuleList':
@@ -312,9 +327,41 @@ class ModuleList(Module):
             combined.add_module(str(i), module)
         return combined
 
+    def __repr__(self):
+        """Return a custom repr for ModuleList that compresses repeated module representations."""
+        list_of_reprs = [repr(item) for item in self]
+        if len(list_of_reprs) == 0:
+            return self._get_name() + '()'
+
+        start_end_indices = [[0, 0]]
+        repeated_blocks = [list_of_reprs[0]]
+        for i, r in enumerate(list_of_reprs[1:], 1):
+            if r == repeated_blocks[-1]:
+                start_end_indices[-1][1] += 1
+                continue
+
+            start_end_indices.append([i, i])
+            repeated_blocks.append(r)
+
+        lines = []
+        main_str = self._get_name() + '('
+        for (start_id, end_id), b in zip(start_end_indices, repeated_blocks):
+            local_repr = f"({start_id}): {b}"  # default repr
+
+            if start_id != end_id:
+                n = end_id - start_id + 1
+                local_repr = f"({start_id}-{end_id}): {n} x {b}"
+
+            local_repr = _addindent(local_repr, 2)
+            lines.append(local_repr)
+
+        main_str += '\n  ' + '\n  '.join(lines) + '\n'
+        main_str += ')'
+        return main_str
+
     @_copy_to_script_wrapper
     def __dir__(self):
-        keys = super(ModuleList, self).__dir__()
+        keys = super().__dir__()
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
@@ -330,7 +377,7 @@ class ModuleList(Module):
         self._modules[str(index)] = module
 
     def append(self, module: Module) -> 'ModuleList':
-        r"""Appends a given module to the end of the list.
+        r"""Append a given module to the end of the list.
 
         Args:
             module (nn.Module): module to append
@@ -343,8 +390,8 @@ class ModuleList(Module):
         del self[key]
         return v
 
-    def extend(self, modules: Iterable[Module]) -> 'ModuleList':
-        r"""Appends modules from a Python iterable to the end of the list.
+    def extend(self, modules: Iterable[Module]) -> Self:
+        r"""Append modules from a Python iterable to the end of the list.
 
         Args:
             modules (iterable): iterable of modules to append
@@ -388,7 +435,7 @@ class ModuleDict(Module):
 
         class MyModule(nn.Module):
             def __init__(self):
-                super(MyModule, self).__init__()
+                super().__init__()
                 self.choices = nn.ModuleDict({
                         'conv': nn.Conv2d(10, 10, 3),
                         'pool': nn.MaxPool2d(3)
@@ -407,7 +454,7 @@ class ModuleDict(Module):
     _modules: Dict[str, Module]  # type: ignore[assignment]
 
     def __init__(self, modules: Optional[Mapping[str, Module]] = None) -> None:
-        super(ModuleDict, self).__init__()
+        super().__init__()
         if modules is not None:
             self.update(modules)
 
@@ -434,8 +481,7 @@ class ModuleDict(Module):
         return key in self._modules
 
     def clear(self) -> None:
-        """Remove all items from the ModuleDict.
-        """
+        """Remove all items from the ModuleDict."""
         self._modules.clear()
 
     def pop(self, key: str) -> Module:
@@ -450,25 +496,21 @@ class ModuleDict(Module):
 
     @_copy_to_script_wrapper
     def keys(self) -> Iterable[str]:
-        r"""Return an iterable of the ModuleDict keys.
-        """
+        r"""Return an iterable of the ModuleDict keys."""
         return self._modules.keys()
 
     @_copy_to_script_wrapper
     def items(self) -> Iterable[Tuple[str, Module]]:
-        r"""Return an iterable of the ModuleDict key/value pairs.
-        """
+        r"""Return an iterable of the ModuleDict key/value pairs."""
         return self._modules.items()
 
     @_copy_to_script_wrapper
     def values(self) -> Iterable[Module]:
-        r"""Return an iterable of the ModuleDict values.
-        """
+        r"""Return an iterable of the ModuleDict values."""
         return self._modules.values()
 
     def update(self, modules: Mapping[str, Module]) -> None:
-        r"""Update the :class:`~torch.nn.ModuleDict` with the key-value pairs from a
-        mapping or an iterable, overwriting existing keys.
+        r"""Update the :class:`~torch.nn.ModuleDict` with key-value pairs from a mapping, overwriting existing keys.
 
         .. note::
             If :attr:`modules` is an ``OrderedDict``, a :class:`~torch.nn.ModuleDict`, or
@@ -522,7 +564,7 @@ class ParameterList(Module):
 
         class MyModule(nn.Module):
             def __init__(self):
-                super(MyModule, self).__init__()
+                super().__init__()
                 self.params = nn.ParameterList([nn.Parameter(torch.randn(10, 10)) for i in range(10)])
 
             def forward(self, x):
@@ -533,16 +575,16 @@ class ParameterList(Module):
     """
 
     def __init__(self, values: Optional[Iterable[Any]] = None) -> None:
-        super(ParameterList, self).__init__()
+        super().__init__()
         self._size = 0
         if values is not None:
             self += values
 
     def _get_abs_string_index(self, idx):
-        """Get the absolute index for the list of modules"""
+        """Get the absolute index for the list of modules."""
         idx = operator.index(idx)
         if not (-len(self) <= idx < len(self)):
-            raise IndexError('index {} is out of range'.format(idx))
+            raise IndexError(f'index {idx} is out of range')
         if idx < 0:
             idx += len(self)
         return str(idx)
@@ -583,16 +625,16 @@ class ParameterList(Module):
     def __iter__(self) -> Iterator[Any]:
         return iter(self[i] for i in range(len(self)))
 
-    def __iadd__(self, parameters: Iterable[Any]) -> 'ParameterList':
+    def __iadd__(self, parameters: Iterable[Any]) -> Self:
         return self.extend(parameters)
 
     def __dir__(self):
-        keys = super(ParameterList, self).__dir__()
+        keys = super().__dir__()
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
     def append(self, value: Any) -> 'ParameterList':
-        """Appends a given value at the end of the list.
+        """Append a given value at the end of the list.
 
         Args:
             value (Any): value to append
@@ -602,8 +644,8 @@ class ParameterList(Module):
         self[new_idx] = value
         return self
 
-    def extend(self, values: Iterable[Any]) -> 'ParameterList':
-        """Appends values from a Python iterable to the end of the list.
+    def extend(self, values: Iterable[Any]) -> Self:
+        """Append values from a Python iterable to the end of the list.
 
         Args:
             values (iterable): iterable of values to append
@@ -621,7 +663,10 @@ class ParameterList(Module):
         for k, p in enumerate(self):
             if isinstance(p, torch.Tensor):
                 size_str = 'x'.join(str(size) for size in p.size())
-                device_str = '' if not p.is_cuda else ' (GPU {})'.format(p.get_device())
+                if p.device.type in ["cuda", torch._C._get_privateuse1_backend_name()]:
+                    device_str = f' ({p.device})'
+                else:
+                    device_str = ''
                 parastr = '{} containing: [{} of size {}{}]'.format(
                     "Parameter" if isinstance(p, Parameter) else "Tensor",
                     p.dtype, size_str, device_str)
@@ -662,7 +707,7 @@ class ParameterDict(Module):
 
         class MyModule(nn.Module):
             def __init__(self):
-                super(MyModule, self).__init__()
+                super().__init__()
                 self.params = nn.ParameterDict({
                         'left': nn.Parameter(torch.randn(5, 10)),
                         'right': nn.Parameter(torch.randn(5, 10))
@@ -674,7 +719,7 @@ class ParameterDict(Module):
     """
 
     def __init__(self, parameters: Any = None) -> None:
-        super(ParameterDict, self).__init__()
+        super().__init__()
         self._keys: Dict[str, None] = {}
         if parameters is not None:
             self.update(parameters)
@@ -719,8 +764,7 @@ class ParameterDict(Module):
         return reversed(list(self._keys))
 
     def copy(self) -> 'ParameterDict':
-        """Returns a copy of this :class:`~torch.nn.ParameterDict` instance.
-        """
+        """Return a copy of this :class:`~torch.nn.ParameterDict` instance."""
         # We have to use an OrderedDict because the ParameterDict constructor
         # behaves differently on plain dict vs OrderedDict
         return ParameterDict(OrderedDict((k, self[k]) for k in self._keys))
@@ -729,7 +773,9 @@ class ParameterDict(Module):
         return key in self._keys
 
     def setdefault(self, key: str, default: Optional[Any] = None) -> Any:
-        """If key is in the ParameterDict, return its value.
+        """Set the default for a key in the Parameterdict.
+
+        If key is in the ParameterDict, return its value.
         If not, insert `key` with a parameter `default` and return `default`.
         `default` defaults to `None`.
 
@@ -737,14 +783,12 @@ class ParameterDict(Module):
             key (str): key to set default for
             default (Any): the parameter set to the key
         """
-
         if key not in self:
             self[key] = default
         return self[key]
 
     def clear(self) -> None:
-        """Remove all items from the ParameterDict.
-        """
+        """Remove all items from the ParameterDict."""
         for k in self._keys.copy():
             del self[k]
 
@@ -759,9 +803,7 @@ class ParameterDict(Module):
         return v
 
     def popitem(self) -> Tuple[str, Any]:
-        """Remove and return the last inserted `(key, parameter)` pair
-        from the ParameterDict
-        """
+        """Remove and return the last inserted `(key, parameter)` pair from the ParameterDict."""
         k, _ = self._keys.popitem()
         # We need the key in the _keys to be able to access/del
         self._keys[k] = None
@@ -770,8 +812,7 @@ class ParameterDict(Module):
         return k, val
 
     def get(self, key: str, default: Optional[Any] = None) -> Any:
-        r"""Return the parameter associated with key if present.
-        Otherwise return default if provided, None if not.
+        r"""Return the parameter associated with key if present. Otherwise return default if provided, None if not.
 
         Args:
             key (str): key to get from the ParameterDict
@@ -780,32 +821,28 @@ class ParameterDict(Module):
         return self[key] if key in self else default
 
     def fromkeys(self, keys: Iterable[str], default: Optional[Any] = None) -> 'ParameterDict':
-        r"""Return a new ParameterDict with the keys provided
+        r"""Return a new ParameterDict with the keys provided.
 
         Args:
             keys (iterable, string): keys to make the new ParameterDict from
             default (Parameter, optional): value to set for all keys
         """
-        return ParameterDict(((k, default) for k in keys))
+        return ParameterDict((k, default) for k in keys)
 
     def keys(self) -> Iterable[str]:
-        r"""Return an iterable of the ParameterDict keys.
-        """
+        r"""Return an iterable of the ParameterDict keys."""
         return self._keys.keys()
 
     def items(self) -> Iterable[Tuple[str, Any]]:
-        r"""Return an iterable of the ParameterDict key/value pairs.
-        """
+        r"""Return an iterable of the ParameterDict key/value pairs."""
         return ((k, self[k]) for k in self._keys)
 
     def values(self) -> Iterable[Any]:
-        r"""Return an iterable of the ParameterDict values.
-        """
+        r"""Return an iterable of the ParameterDict values."""
         return (self[k] for k in self._keys)
 
     def update(self, parameters: Union[Mapping[str, Any], 'ParameterDict']) -> None:
-        r"""Update the :class:`~torch.nn.ParameterDict` with the key-value pairs from a
-        mapping or an iterable, overwriting existing keys.
+        r"""Update the :class:`~torch.nn.ParameterDict` with key-value pairs from ``parameters``, overwriting existing keys.
 
         .. note::
             If :attr:`parameters` is an ``OrderedDict``, a :class:`~torch.nn.ParameterDict`, or
@@ -845,7 +882,10 @@ class ParameterDict(Module):
         for k, p in self.items():
             if isinstance(p, torch.Tensor):
                 size_str = 'x'.join(str(size) for size in p.size())
-                device_str = '' if not p.is_cuda else ' (GPU {})'.format(p.get_device())
+                if p.device.type in ["cuda", torch._C._get_privateuse1_backend_name()]:
+                    device_str = f' ({p.device})'
+                else:
+                    device_str = ''
                 parastr = '{} containing: [{} of size {}{}]'.format(
                     "Parameter" if isinstance(p, Parameter) else "Tensor",
                     torch.typename(p), size_str, device_str)
@@ -868,6 +908,6 @@ class ParameterDict(Module):
         copy.update(self)
         return copy
 
-    def __ior__(self, other : 'ParameterDict') -> 'ParameterDict':
+    def __ior__(self, other : 'ParameterDict') -> Self:
         self.update(other)
         return self
