@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 from typing import Optional
 
 import torch
@@ -68,7 +69,7 @@ class Embedding(Module):
         >>> # an Embedding module containing 10 tensors of size 3
         >>> embedding = nn.Embedding(10, 3)
         >>> # a batch of 2 samples of 4 indices each
-        >>> input = torch.LongTensor([[1,2,4,5],[4,3,2,9]])
+        >>> input = torch.LongTensor([[1, 2, 4, 5], [4, 3, 2, 9]])
         >>> # xdoctest: +IGNORE_WANT("non-deterministic")
         >>> embedding(input)
         tensor([[[-0.0251, -1.6902,  0.7172],
@@ -84,7 +85,7 @@ class Embedding(Module):
 
         >>> # example with padding_idx
         >>> embedding = nn.Embedding(10, 3, padding_idx=0)
-        >>> input = torch.LongTensor([[0,2,0,5]])
+        >>> input = torch.LongTensor([[0, 2, 0, 5]])
         >>> embedding(input)
         tensor([[[ 0.0000,  0.0000,  0.0000],
                  [ 0.1535, -2.0309,  0.9315],
@@ -107,6 +108,7 @@ class Embedding(Module):
                 [-0.7895, -0.7089, -0.0364],
                 [ 0.6778,  0.5803,  0.2678]], requires_grad=True)
     """
+
     __constants__ = ['num_embeddings', 'embedding_dim', 'padding_idx', 'max_norm',
                      'norm_type', 'scale_grad_by_freq', 'sparse']
 
@@ -117,14 +119,15 @@ class Embedding(Module):
     norm_type: float
     scale_grad_by_freq: bool
     weight: Tensor
+    freeze: bool
     sparse: bool
 
     def __init__(self, num_embeddings: int, embedding_dim: int, padding_idx: Optional[int] = None,
                  max_norm: Optional[float] = None, norm_type: float = 2., scale_grad_by_freq: bool = False,
-                 sparse: bool = False, _weight: Optional[Tensor] = None,
+                 sparse: bool = False, _weight: Optional[Tensor] = None, _freeze: bool = False,
                  device=None, dtype=None) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
-        super(Embedding, self).__init__()
+        super().__init__()
         self.num_embeddings = num_embeddings
         self.embedding_dim = embedding_dim
         if padding_idx is not None:
@@ -138,12 +141,13 @@ class Embedding(Module):
         self.norm_type = norm_type
         self.scale_grad_by_freq = scale_grad_by_freq
         if _weight is None:
-            self.weight = Parameter(torch.empty((num_embeddings, embedding_dim), **factory_kwargs))
+            self.weight = Parameter(torch.empty((num_embeddings, embedding_dim), **factory_kwargs),
+                                    requires_grad=not _freeze)
             self.reset_parameters()
         else:
             assert list(_weight.shape) == [num_embeddings, embedding_dim], \
                 'Shape of weight does not match num_embeddings and embedding_dim'
-            self.weight = Parameter(_weight)
+            self.weight = Parameter(_weight, requires_grad=not _freeze)
 
         self.sparse = sparse
 
@@ -179,7 +183,7 @@ class Embedding(Module):
     def from_pretrained(cls, embeddings, freeze=True, padding_idx=None,
                         max_norm=None, norm_type=2., scale_grad_by_freq=False,
                         sparse=False):
-        r"""Creates Embedding instance from given 2-dimensional FloatTensor.
+        r"""Create Embedding instance from given 2-dimensional FloatTensor.
 
         Args:
             embeddings (Tensor): FloatTensor containing weights for the Embedding.
@@ -212,18 +216,17 @@ class Embedding(Module):
             num_embeddings=rows,
             embedding_dim=cols,
             _weight=embeddings,
+            _freeze=freeze,
             padding_idx=padding_idx,
             max_norm=max_norm,
             norm_type=norm_type,
             scale_grad_by_freq=scale_grad_by_freq,
             sparse=sparse)
-        embedding.weight.requires_grad = not freeze
         return embedding
 
 
 class EmbeddingBag(Module):
-    r"""Computes sums or means of 'bags' of embeddings, without instantiating the
-    intermediate embeddings.
+    r"""Compute sums or means of 'bags' of embeddings, without instantiating the intermediate embeddings.
 
     For bags of constant length, no :attr:`per_sample_weights`, no indices equal to :attr:`padding_idx`,
     and with 2D inputs, this class
@@ -277,8 +280,8 @@ class EmbeddingBag(Module):
         >>> # an EmbeddingBag module containing 10 tensors of size 3
         >>> embedding_sum = nn.EmbeddingBag(10, 3, mode='sum')
         >>> # a batch of 2 samples of 4 indices each
-        >>> input = torch.tensor([1,2,4,5,4,3,2,9], dtype=torch.long)
-        >>> offsets = torch.tensor([0,4], dtype=torch.long)
+        >>> input = torch.tensor([1, 2, 4, 5, 4, 3, 2, 9], dtype=torch.long)
+        >>> offsets = torch.tensor([0, 4], dtype=torch.long)
         >>> # xdoctest: +IGNORE_WANT("non-deterministic")
         >>> embedding_sum(input, offsets)
         tensor([[-0.8861, -5.4350, -0.0523],
@@ -287,7 +290,7 @@ class EmbeddingBag(Module):
         >>> # Example with padding_idx
         >>> embedding_sum = nn.EmbeddingBag(10, 3, mode='sum', padding_idx=2)
         >>> input = torch.tensor([2, 2, 2, 2, 4, 3, 2, 9], dtype=torch.long)
-        >>> offsets = torch.tensor([0,4], dtype=torch.long)
+        >>> offsets = torch.tensor([0, 4], dtype=torch.long)
         >>> embedding_sum(input, offsets)
         tensor([[ 0.0000,  0.0000,  0.0000],
                 [-0.7082,  3.2145, -2.6251]])
@@ -299,6 +302,7 @@ class EmbeddingBag(Module):
                 padding_idx=embedding.padding_idx,
                 mode='sum')
     """
+
     __constants__ = ['num_embeddings', 'embedding_dim', 'max_norm', 'norm_type',
                      'scale_grad_by_freq', 'mode', 'sparse', 'include_last_offset',
                      'padding_idx']
@@ -320,7 +324,7 @@ class EmbeddingBag(Module):
                  include_last_offset: bool = False, padding_idx: Optional[int] = None,
                  device=None, dtype=None) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
-        super(EmbeddingBag, self).__init__()
+        super().__init__()
         self.num_embeddings = num_embeddings
         self.embedding_dim = embedding_dim
         self.max_norm = max_norm
@@ -401,14 +405,14 @@ class EmbeddingBag(Module):
         s += ', mode={mode}'
         if self.padding_idx is not None:
             s += ', padding_idx={padding_idx}'
-        return s.format(**self.__dict__)
+        return s.format(**{k: repr(v) for k, v in self.__dict__.items()})
 
     @classmethod
     def from_pretrained(cls, embeddings: Tensor, freeze: bool = True, max_norm: Optional[float] = None,
                         norm_type: float = 2., scale_grad_by_freq: bool = False,
                         mode: str = 'mean', sparse: bool = False, include_last_offset: bool = False,
                         padding_idx: Optional[int] = None) -> 'EmbeddingBag':
-        r"""Creates EmbeddingBag instance from given 2-dimensional FloatTensor.
+        r"""Create EmbeddingBag instance from given 2-dimensional FloatTensor.
 
         Args:
             embeddings (Tensor): FloatTensor containing weights for the EmbeddingBag.
